/* ######################################################################### */
/* #                                                                       # */
/* #                                                                       # */
/* #                Copyright (c) 2012 Sintesio Org.                       # */
/* #                                                                       # */
/* #                                                                       # */
/* # File Name    : DIAMETERCodec.java                                     # */
/* #                                                                       # */
/* # File Code    :                                                        # */
/* #                                                                       # */
/* # Description  : Implementation of DIAMETER Codec class.                # */
/* #                                                                       # */
/* # Revision     : 1.0                                                    # */
/* #                                                                       # */
/* # Project Name :                                                        # */
/* #                                                                       # */
/* # Date         : Apr 2012                                               # */
/* #                                                                       # */
/* # Author       : Sintesio                                               # */
/* #                                                                       # */
/* # Translation  : javac                                                  # */
/* #                                                                       # */
/* # Remarks      : Implementation of DIAMETER TTCN-3 TCI part.            # */
/* #                                                                       # */
/* ######################################################################### */

package com.iskratel.ttcn.tci.codec;

//import gov.nist.javax.sip.message.SIPMessage;
//import gov.nist.javax.sip.parser.StringMsgParser;

import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.text.ParseException;
import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.StringTokenizer;
import java.util.Vector;
import java.io.LineNumberReader;
import java.io.StringReader;
import java.lang.reflect.Method;
import java.math.BigInteger;
import java.net.URI;

import com.testingtech.ttcn.logging.RTLoggingConstants;
import com.testingtech.ttcn.tci.codec.base.AbstractBaseCodec;
//import com.testingtech.util.logging.LogMessage.Log;

import com.testingtech.util.logging.LogMessage.Log;

import de.tu_berlin.cs.uebb.muttcn.runtime.RB;
import de.tu_berlin.cs.uebb.ttcn.runtime.BitString;
import de.tu_berlin.cs.uebb.ttcn.runtime.ForeignMessage;

import org.etsi.ttcn.tci.BooleanValue;
import org.etsi.ttcn.tci.CharstringValue;
import org.etsi.ttcn.tci.UniversalCharstringValue;
import org.etsi.ttcn.tci.OctetstringValue;
import org.etsi.ttcn.tci.IntegerValue;
import org.etsi.ttcn.tci.RecordOfValue;
import org.etsi.ttcn.tci.RecordValue;
import org.etsi.ttcn.tci.TciCDProvided;
import org.etsi.ttcn.tci.TciCDRequired;
import org.etsi.ttcn.tci.TciTMProvided;
import org.etsi.ttcn.tci.TciTypeClass;
import org.etsi.ttcn.tci.Type;
import org.etsi.ttcn.tci.UnionValue;
import org.etsi.ttcn.tci.Value;
import org.etsi.ttcn.tci.EnumeratedValue;
import org.etsi.ttcn.tri.TriMessage;

import com.iskratel.diameter.parser.*;
import com.iskratel.ttcn.tri.HexDump;


/**<!--#####################################################################-->
 * 
 * Class <b>DIAMETER Codec</b> is extending <b>AbstractBaseCodec</b> class
 * and provides DIAMETER Codec and Decodec.
 * <p>
 * @author SINTESIO
 * 
 * <!--##################################################################--> */  
public class DIAMETERCodec extends AbstractBaseCodec implements TciCDProvided {

  private TciCDRequired typeServer;
  private TciTMProvided management;

  private Type booleanType;
  private Type octetstringType;
  private Type charstringType;
  private Type universalcharstringType;
  private Type integerType;
  private Type bigintegerType;
	
  private byte[] encodedMsg = null ;
  private int UsedBitsInByte;
  private boolean padding = true;
  
  /*used for decoding of unsupported AVPs inside messages*/
  RecordOfValue rov_any_AVP;
  
  /** Command code of  Request/Answer */
  public static final int Code_CE=257;// Capabilities-Exchange
  public static final int Code_RA=258;// Re-Auth
  public static final int Code_AC=271;// Accounting
  public static final int Code_AS=274;// Abort-Session
  public static final int Code_ST=275;// Session-Termination
  public static final int Code_DW=280;// Device-Watchdog
  public static final int Code_DP=282;// Disconnect-Peer
  
  /* Codes on Cx */
  public static final int Code_UA=300;// User-Authorization
  public static final int Code_SA=301;// Server-Assignment-Request        SAR    (284 in RFC4740 or 301 in TS 129 229)
  public static final int Code_LI=302;// Location-Info-Request            LIR    (285 in RFC4740 or 302 in TS 129 229)
  public static final int Code_MA=303;// Multimedia-Auth-Request          MAR    (286 in RFC4740 or 303 in TS 129 229)
  public static final int Code_RT=304;// Registration-Termination-Request RTR    (287 in RFC4740 or 304 in TS 129 229)
  public static final int Code_PP=305;// Push-Profile-Request             PPR    (288 in RFC4740 or 305 in TS 129 229)

  /* Codes on Sh/Dh interface  */
  public static final int Code_UD=306;// UDR User-Data-Request AS->HSS                UDR    (306)
  public static final int Code_PU=307;// PUR Profile-Update-Request AS->HSS           PUR    (307)
  public static final int Code_SN=308;// SNR Subscribe-Notifications-Request AS->HSS  SNR    (308)
  public static final int Code_PN=309;// PNR Profile-Notification-Request HSS->AS     PNR    (309)
  
  /* Codes on Rx interface  */
  public static final int Code_AA=265;// Auth-Accounting
  
  /* Codes on Gx interface - RFC4006 and TS 129 212  */
  public static final int Code_CC=272;// Credit-Control 

  /* Codes on S6a interface - TS 129 272  */
  public static final int Code_UL=316;  // Update-Location-Request                   ULR    (316)- Ref: TS 129 272 7.2.3-4
  public static final int Code_CL=317;  // Cancel-Location-Request                   CLR    (317)- Ref: TS 129 272 7.2.7-8
  public static final int Code_AI=318;  // Authentication-Information-Request        AIR    (318)- Ref: TS 129 272 7.2.5-6
  public static final int Code_ID=319;  // Insert-Subscriber-Data-Request            IDR    (319)- Ref: TS 129 272 7.2.9-10
  public static final int Code_DS=320;  // Delete-Subscriber-Data-Request            DSR    (320)- Ref: TS 129 272 7.2.11-12
  public static final int Code_PUE=321; // Purge-UE-Request                          PUR    (321)- Ref: TS 129 272 7.2.13-14
  public static final int Code_RS=322;  // Reset-Request                             DSR    (322)- Ref: TS 129 272 7.2.15-16
  public static final int Code_NO=323;  // Notify-Request                            NOR    (323)- Ref: TS 129 272 7.2.17-18


  /*RFC3588 AVP codes*/
  private static final int AVP_Accounting_Realtime_Required_AVP = 483; 			//Ref: RFC3588 9.8.7.  Accounting-Realtime-Required AVP
  private static final int AVP_Accounting_Record_Number_AVP = 485;				    //Ref: RFC3588 9.8.3.  Accounting-Record-Number AVP
  private static final int AVP_Accounting_Record_Type_AVP = 480; 					//Ref: RFC3588 9.8.1.  Accounting-Record-Type AVP
  private static final int AVP_Accounting_Sub_Session_Id_AVP = 287;				//Ref: RFC3588 9.8.6.  Accounting-Sub-Session-Id AVP
  private static final int AVP_Acct_Application_Id_AVP = 259; 					    //Ref: RFC3588 6.9.  Acct-Application-Id AVP
  private static final int AVP_Acct_Interim_Interval_AVP = 85; 					//Ref: RFC3588 9.8.2.  Acct-Interim-Interval
  private static final int AVP_Acct_Multi_Session_Id_AVP = 50; 					//Ref: RFC3588 9.8.5.  Acct-Multi-Session-Id AVP
  private static final int AVP_Acct_Session_Id_AVP = 44; 							//Ref: RFC3588 9.8.4.  Acct-Session-Id AVP 
  private static final int AVP_Auth_Application_Id_AVP = 258; 					    //Ref: RFC3588 6.8.  Auth-Application-Id AVP
  private static final int AVP_Class_AVP = 25; 									//Ref: RFC3588 8.20.  Class AVP
  private static final int AVP_Destination_Host_AVP = 293; 						//Ref: RFC3588 6.5.  Destination-Host AVP
  private static final int AVP_Destination_Realm_AVP = 283; 						//Ref: RFC3588 6.6.  Destination-Realm AVP
  private static final int AVP_Disconnect_Cause_AVP = 273; 						//Ref: RFC3588 5.4.3.  Disconnect-Cause AVP
  private static final int AVP_Error_Message_AVP = 281; 							//Ref: RFC3588 7.3.  Error-Message AVP
  private static final int AVP_Error_Reporting_Host_AVP = 294; 					//Ref: RFC3588 7.4.  Error-Reporting-Host AVP
  private static final int AVP_Event_Timestamp_AVP = 55; 							//Ref: RFC3588 8.21.  Event-Timestamp AVP
  private static final int AVP_Failed_AVP = 279; 									//Ref: RFC3588 7.5.  Failed-AVP AVP
  private static final int AVP_Firmware_Revision_AVP = 267; 						//Ref: RFC3588 5.3.4.  Firmware-Revision AVP
  private static final int AVP_Host_IP_Address_AVP = 257; 						    //Ref: RFC3588 5.3.5.  Host-IP-Address AVP
  private static final int AVP_Inband_Security_Id_AVP = 299; 						//Ref: RFC3588 6.10.  Inband-Security-Id AVP
  private static final int AVP_Origin_Host_AVP = 264; 							    //Ref: RFC3588 6.3.  Origin-Host AVP
  private static final int AVP_Origin_Realm_AVP = 296; 							//Ref: RFC3588 6.4.  Origin-Realm AVP
  private static final int AVP_Origin_State_Id_AVP = 278; 						    //Ref: RFC3588 8.16.  Origin-State-Id AVP
  private static final int AVP_Product_Name_AVP = 269; 							//Ref: RFC3588 5.3.7.  Product-Name AVP
  private static final int AVP_Proxy_Host_AVP = 280; 								//Ref: RFC3588 6.7.3.  Proxy-Host AVP
  private static final int AVP_Proxy_Info_AVP = 284; 								//Ref: RFC3588 6.7.2.  Proxy-Info AVP
  private static final int AVP_Proxy_State_AVP = 33; 								//Ref: RFC3588 6.7.4.  Proxy-State AVP
  private static final int AVP_Re_Auth_Request_Type_AVP = 285; 					//Ref: RFC3588 8.12.  Re-Auth-Request-Type AVP
  private static final int AVP_Redirect_Host_AVP = 292; 							//Ref: RFC3588 6.12.  Redirect-Host AVP
  private static final int AVP_redirect_Max_Cache_Time_AVP = 262; 				//Ref: RFC3588  redirect_Max_Cache_Time_AVP is not defined. 
  private static final int AVP_Redirect_Host_Usage_AVP = 261; 					    //Ref: RFC3588 6.13.  Redirect-Host-Usage AVP
  private static final int AVP_Result_Code_AVP = 268; 							    //Ref: RFC3588 7.1.  Result-Code AVP
  private static final int AVP_Route_Record_AVP = 282; 							//Ref: RFC3588 6.7.1.  Route-Record AVP
  private static final int AVP_Session_Id_AVP = 263; 								//Ref: RFC3588 8.8.  Session-Id AVP
  private static final int AVP_Supported_Vendor_Id_AVP = 265; 					    //Ref; RFC3588 5.3.6.  Supported-Vendor-Id AVP
  private static final int AVP_Termination_Cause_AVP = 295; 						//Ref: RFC3588 8.15.  Termination-Cause AVP
  private static final int AVP_User_Name_AVP = 1; 								    //Ref: RFC3588 8.14.  User-Name AVP
  private static final int AVP_Vendor_Id_AVP = 266; 								//Ref: RFC3588 5.3.3.  Vendor-Id AVP
  private static final int AVP_Vendor_Specific_Appl_Id_AVP = 260; 				    //Ref: RFC3588 6.11.  Vendor-Specific-Application-Id AVP
  /*end RFC3588 AVP codes*/
  
  //RFC4740 AVP codes{
//  private static final int AVP_Auth_Grace_Period_AVP = 276; 						//Ref: RFC3588 8.10.  Auth-Grace-Period AVP								
  private static final int AVP_Auth_Session_State_AVP = 277; 						//Ref: RFC3588 8.11.  Auth-Session-State AVP							
//  private static final int AVP_Authorization_Lifetime_AVP = 291; 					//Ref: RFC3588 8.9.  Authorization-Lifetime AVP								
//  private static final int AVP_Redirect_Max_Cache_Time_AVP = 262;					//Ref: RFC3588 6.14.  Redirect-Max-Cache-Time AVP							
//  private static final int AVP_SIP_Accounting_Information_AVP = 368; 				//Ref: RFC4740 9.1.  SIP-Accounting-Information AVP								
//  private static final int AVP_SIP_Accounting_Server_URI_AVP = 369; 				//Ref: RFC4740 9.1.1.  SIP-Accounting-Server-URI AVP								
//  private static final int AVP_SIP_AOR_AVP = 122; 								//Ref: RFC4740 9.8.  SIP-AOR AVP							
  private static final int AVP_SIP_Auth_Data_Item_RFC_AVP = 376; 						//Ref: RFC4740 9.5.  SIP-Auth-Data-Item AVP
  private static final int AVP_SIP_Item_Number_AVP = 378;
  private static final int AVP_SIP_Authenticate_RFC_AVP = 379; 						//Ref: RFC4740 9.5.3.  SIP-Authenticate AVP								
//  private static final int AVP_SIP_Authentication_Info_AVP = 381;  				//Ref: RFC4740 9.5.5.  SIP-Authentication-Info AVP							
  private static final int AVP_SIP_Authentication_Scheme_RFC_AVP = 377; 				//Ref: RFC4740 9.5.1.  SIP-Authentication-Scheme AVP							
  private static final int AVP_SIP_Authorization_RFC_AVP = 380;  						//Ref: RFC4740 9.5.4.  SIP-Authorization AVP							
//  private static final int AVP_SIP_Credit_Control_Server_URI_AVP = 370; 			//Ref: RFC4740 9.1.2.  SIP-Credit-Control-Server-URI AVP								
//  private static final int AVP_SIP_Deregistration_Reason_AVP = 383; 				//Ref: RFC4740 9.5.5.  SIP-Authentication-Info AVP							
//  private static final int AVP_SIP_Item_Number_AVP = 378; 						//Ref: RFC4740 9.5.2 SIP-Item-Number AVP							
//  private static final int AVP_SIP_MandatoryCapability_AVP = 373; 				//Ref: RFC4740 9.3.1.  SIP-Mandatory-Capability AVP								
  private static final int AVP_SIP_Method_AVP = 393; 								//Ref: RFC4740 9.14.  SIP-Method AVP								
  private static final int AVP_SIP_Number_Auth_Items_RFC_AVP = 382; 					//Ref: RFC4740 9.6.  SIP-Number-Auth-Items AVP								
//  private static final int AVP_SIP_OptionalCapability_AVP = 374; 					//Ref: RFC4740 9.3.2.  SIP-Optional-Capability AVP								
//  private static final int AVP_SIP_Reason_Code_AVP = 384; 						//Ref: RFC4740 9.7.1.  SIP-Reason-Code AVP							
//  private static final int AVP_SIP_Reason_Info_AVP = 385;  						//Ref: RFC4740 9.7.2.  SIP-Reason-Info AVP							
//  private static final int AVP_SIP_Server_Assignment_Type_AVP = 375; 				//Ref: RFC4740 9.4.  SIP-Server-Assignment-Type AVP								
//  private static final int AVP_SIP_Server_Capabilities_AVP = 372; 				//Ref: RFC4740 9.3.  SIP-Server-Capabilities AVP								
//  private static final int AVP_SIP_Server_URI_AVP = 371; 							//Ref: RFC4740 9.2 SIP-Server-URI AVP							
//  private static final int AVP_SIP_Supported_User_Data_Type_AVP = 388; 			//Ref: RFC4740 9.12.1.  SIP-User-Data-Type AVP								
//  private static final int AVP_SIP_User_Authorization_Type_AVP = 387; 			//Ref: RFC4740 9.10.  SIP-User-Authorization-Type AVP						
//  private static final int AVP_SIP_User_Data_Already_Available_AVP = 392; 		//Ref: RFC4740 9.13.  SIP-User-Data-Already-Available AVP								
//  private static final int AVP_SIP_User_Data_AVP = 389; 							//Ref: RFC4740 9.12.  SIP-User-Data AVP								
//  private static final int AVP_SIP_User_Data_Contents_AVP = 391; 					//Ref: RFC4740 9.12.2.  SIP-User-Data-Contents AVP								
//  private static final int AVP_SIP_User_Data_Type_AVP = 390; 						//Ref: RFC4740 9.12.1.  SIP-User-Data-Type AVP								
//  private static final int AVP_SIP_Visited_Network_Id_AVP = 386; 					//Ref: RFC4740 9.9.  SIP-Visited-Network-Id AVP							
  //end of RFC4740 AVP codes
  
  private static final int AVP_Digest_Realm_AVP = 104;
  private static final int AVP_Digest_Nonce_AVP = 105;
  private static final int AVP_Digest_Domain_AVP = 119;
  private static final int AVP_Digest_Opaque_AVP = 116;
  private static final int AVP_Digest_Stale_AVP = 120;
  private static final int AVP_Digest_Algorithm_AVP = 111;
  private static final int AVP_Digest_QoP_AVP = 110;
  private static final int AVP_Digest_HA1_AVP = 121;
  private static final int AVP_Digest_Auth_Param_AVP = 117;
  private static final int AVP_Digest_Username_AVP = 115;
  private static final int AVP_Digest_URI_AVP = 109;
  private static final int AVP_Digest_Response_AVP = 103;
  private static final int AVP_Digest_CNonce_AVP = 113;
  private static final int AVP_Digest_Nonce_Count_AVP = 114;
  private static final int AVP_Digest_Method_AVP = 108;
  private static final int AVP_Digest_Entity_Body_Hash_AVP = 112;
  private static final int AVP_Digest_Nextnonce_AVP = 107;
  private static final int AVP_Digest_Response_Auth_AVP = 106;


  //RFC4005 AVP codes
  private static final int AVP_Called_Station_Id_AVP = 30;
  private static final int AVP_Framed_IP_Address_AVP  = 8;
  private static final int AVP_Framed_IPv6_Prefix_AVP = 97;
  private static final int  AVP_Framed_Interface_Id_AVP = 96;
  //end of RFC4005 AVP codes
    
  //RFC4006 AVP codes
//  private static final int AVP_Rating_Group_AVP = 433; 							//Ref: RFC4006 8.29. Rating-Group AVP								
private static final int AVP_Tariff_Time_Change_AVP = 451; 						//Ref: RFC4006 8.20. Tariff-Time-Change AVP								
//  private static final int AVP_CC_Correlation_Id_AVP = 411; 						//Ref: RFC4006 8.1.  CC-Correlation-Id AVP								
private static final int AVP_CC_Input_Octets_AVP = 412; 						//Ref: RFC4006 8.24. CC-Input-Octets AVP
private static final int AVP_CC_Money_AVP = 413; 								//Ref: RFC4006 8.22. CC-Money AVP								
private static final int AVP_CC_Output_Octets_AVP = 414; 						//Ref: RFC4006 8.25. CC-Output-Octets AVP								
  private static final int AVP_CC_Request_Number_AVP = 415; 						//Ref: RFC4006 8.2.  CC-Request-Number AVP								
  private static final int AVP_CC_Request_Type_AVP = 416; 						//Ref: RFC4006 8.3.  CC-Request-Type AVP								
private static final int AVP_CC_Service_Specific_Units_AVP = 417; 				//Ref: RFC4006 8.26. CC-Service-Specific-Units AVP								
private static final int AVP_CC_Session_Failover_AVP = 418; 					//Ref: RFC4006 8.4.  CC-Session-Failover AVP								
//  private static final int AVP_vendIdCC_Sub_Session_Id_AVP = 419; 						//Ref: RFC4006 8.5.  CC-Sub-Session-Id AVP								
private static final int AVP_CC_Time_AVP = 420; 								//Ref: RFC4006 8.21. CC-Time AVP								
private static final int AVP_CC_Total_Octets_AVP = 421; 						//Ref: RFC4006 8.23. CC-Total-Octets AVP								
private static final int AVP_CC_Unit_Type_AVP = 454; 							//Ref: RFC4006 8.32. CC-Unit-Type AVP								
//  private static final int AVP_vendIdCheck_Balance_Result_AVP = 422; 					//Ref: RFC4006 8.6.  Check-Balance-Result AVP 12.8.  Check-Balance-Result AVP								
private static final int AVP_Cost_Information_AVP = 423; 						//Ref: RFC4006 8.7.  Cost-Information AVP								
private static final int AVP_Cost_Unit_AVP = 424; 								//Ref: RFC4006 8.12. Cost-Unit AVP								
private static final int AVP_Credit_Control_Failure_Handling_AVP = 427; 		//Ref: RFC4006 8.14. Credit-Control-Failure-Handling AVP								
private static final int AVP_Currency_Code_AVP = 425; 							//Ref: RFC4006 8.11. Currency-Code AVP								
private static final int AVP_Direct_Debiting_Failure_Handling_AVP = 428; 		//Ref: RFC4006 8.15. Direct-Debiting-Failure-Handling AVP								
private static final int AVP_Exponent_AVP = 429; 								//Ref: RFC4006 8.9.  Exponent AVP								
private static final int AVP_Filter_Id_AVP = 11; 								//Ref: RFC4005 6.7.  Filter-Id AVP								
private static final int AVP_Final_Unit_Action_AVP = 449; 						//Ref: RFC4006 8.35. Final-Unit-Action AVP								
private static final int AVP_Final_Unit_Indication_AVP = 430; 					//Ref: RFC4006 8.34. Final-Unit-Indication AVP								
private static final int AVP_G_S_U_Pool_Identifier_AVP = 453; 					//Ref: RFC4006 8.31. G-S-U-Pool-Identifier AVP								
private static final int AVP_G_S_U_Pool_Reference_AVP = 457; 					//Ref: RFC4006 8.30. G-S-U-Pool-Reference AVP								
private static final int AVP_Granted_Service_Unit_AVP = 431; 					//Ref: RFC4006 8.17. Granted-Service-Unit AVP								
//  private static final int AVP_vendIdMultiple_Services_Credit_Control_AVP = 456; 		//Ref: RFC4006 8.16. Multiple-Services-Credit-Control AVP								
private static final int AVP_Multiple_Services_Indicator_AVP = 455; 			//Ref: RFC4006 8.40. Multiple-Services-Indicator AVP								
  private static final int AVP_Redirect_Address_Type_AVP = 433; 					//Ref: RFC4006 8.38. Redirect-Address-Type AVP								
  private static final int AVP_Redirect_Server_Address_AVP = 435; 				//Ref: RFC4006 8.39. Redirect-Server-Address AVP								
  private static final int AVP_Redirect_Server_AVP = 434; 						//Ref: RFC4006 8.37. Redirect-Server AVP								
//  private static final int AVP_vendIdRequested_Action_AVP = 436; 						//Ref: RFC4006 8.41. Requested-Action AVP								
private static final int AVP_Requested_Service_Unit_AVP = 437; 					//Ref: RFC4006 8.18. Requested-Service-Unit AVP								
private static final int AVP_Restriction_Filter_Rule_AVP = 438; 				//Ref: RFC4006 8.36. Restriction-Filter-Rule AVP								
  private static final int AVP_Service_Context_Id_AVP = 461; 						//Ref: RFC4006 8.42. Service-Context-Id AVP								
  private static final int AVP_Service_Identifier_AVP = 439; 						//Ref: RFC4006 8.28. Service-Identifier AVP								
  private static final int AVP_Requested_Action_AVP = 436;
  //  private static final int AVP_vendIdService_Parameter_Info_AVP = 440; 					//Ref: RFC4006 8.43. Service-Parameter-Info AVP								
//  private static final int AVP_vendIdService_Parameter_Type_AVP = 441; 					//Ref: RFC4006 8.44. Service-Parameter-Type AVP								
//  private static final int AVP_vendIdService_Parameter_Value_AVP = 442; 				//Ref: RFC4006 8.45. Service-Parameter-Value AVP								
  private static final int AVP_Subscription_Id_AVP = 443; 						//Ref: RFC4006 8.46. Subscription-Id AVP								
  private static final int AVP_Subscription_Id_Data_AVP = 444; 					//Ref: RFC4006 								
  private static final int AVP_Subscription_Id_Type_AVP = 450; 					//Ref: RFC4006 8.47. Subscription-Id-Type AVP								
  private static final int AVP_Tariff_Change_Usage_AVP = 452;//Ref: RFC4006 8.27. Tariff-Change-Usage AVP 
  private static final int AVP_Rating_Group_AVP = 433;								
private static final int AVP_Unit_Value_AVP = 445; 								//Ref: RFC4006 8.8.  Unit-Value AVP								
private static final int AVP_Used_Service_Unit_AVP = 446; 						//Ref: RFC4006 8.19. Used-Service-Unit AVP								
//  private static final int AVP_vendIdUser_Equipment_Info_AVP = 458; 					//Ref: RFC4006 8.49. User-Equipment-Info AVP								
//  private static final int AVP_vendIdUser_Equipment_Info_Type_AVP = 459; 				//Ref: RFC4006 8.50. User-Equipment-Info-Type AVP								
//  private static final int AVP_vendIdUser_Equipment_Info_Value_AVP = 460; 				//Ref: RFC4006 8.51. User-Equipment-Info-Value AVP								
private static final int AVP_Validity_Time_AVP = 448; 							//Ref: RFC4006 8.33. Validity-Time AVP								
private static final int AVP_Value_Digits_AVP = 447; 							//Ref: RFC4006 8.10. Value-Digits AVP								
  /*end of RFC4006 AVP codes*/
    
  //TS129_229 AVP codes
//  private static final int AVP_vendIdCharging_Information_AVP = 61; 							//Ref: ETSI TS 129 229 Charging_Information_AVP
//  private static final int AVP_vendIdPrimary_Event_Charging_Function_Name_AVP = 619; 			//Ref: ETSI TS 129 229 Primary_Event_Charging_Function_Name_AVP
//  private static final int AVP_vendIdSecondary_Event_Charging_Function_Name_AVP = 620; 		//Ref: ETSI TS 129 229 Secondary_Event_Charging_Function_Name_AVP
//  private static final int AVP_vendIdPrimary_Charging_Collection_Function_Name_AVP = 621; 	//Ref: ETSI TS 129 229 Primary_Charging_Collection_Function_Name_AVP
//  private static final int AVP_vendIdSecondary_Charging_Collection_Function_Name_AVP = 622; 	//Ref: ETSI TS 129 229 Secondary_Charging_Collection_Function_Name_AVP
  private static final int AVP_Charging_Rule_Remove_AVP = 1002;
  private static final int AVP_Charging_Rule_Install_AVP = 1001;
  private static final int AVP_Charging_Rule_Report_AVP = 1018;
  private static final int AVP_Routing_Rule_Install_AVP = 1081;
  private static final int AVP_Routing_Rule_Remove_AVP = 1075;
  private static final int AVP_QoS_Information_AVP = 1016;
  private static final int AVP_Server_Capabilities_AVP = 603;							//Ref: ETSI TS 129 229 Server_Capabilities_AVP
//  private static final int AVP_vendIdMandatory_Capability_AVP = 604;							//Ref: ETSI TS 129 229 Mandatory_Capability_AVP
//  private static final int AVP_vendIdOptional_Capability_AVP = 605;							//Ref: ETSI TS 129 229 Optional_Capability_AVP
  private static final int AVP_IP_CAN_Type_AVP = 1027;
  private static final int AVP_RAT_Type_AVP = 1032;                                 //Ref: ETSI TS 129 212 RAT_Type_AVP
  private static final int AVP_Event_Trigger_AVP = 1006;
  private static final int AVP_Session_Release_Cause_AVP = 1045;
  private static final int AVP_Charging_Rule_Name_AVP = 1005;
  private static final int AVP_AN_GW_Address_AVP = 1050;
 
  private static final int AVP_Experimental_Result_AVP = 297;
  private static final int AVP_Experimental_Result_Code_AVP = 298;
  private static final int AVP_User_Data_AVP = 606;
  
  /*end of TS129_229 AVP codes*/
  
  //tS129_329
  private static final int AVP_Public_Identity_AVP = 601;
  private static final int AVP_Server_Name_AVP = 602;
  private static final int AVP_Feature_List_ID_AVP = 629; //    Ref: 3GPP TS 29.229, clause 6.3.30
  private static final int AVP_Feature_List_AVP = 630;             //    Ref: 3GPP TS 29.229, clause 6.3.31
  private static final int AVP_Supported_Features_AVP = 628;                           //    Ref: 3GPP TS 29.329    6.3.11 Supported-Features AVP (See 3GPP TS 29.229, clause 6.3.29)
  /*End TS129_329*/

  //Sh AVP codes

  ////private static final int AVP_Public_Identity_AVP = 601;
  private static final int AVP_User_Identity_AVP = 700;
  private static final int AVP_MSISDN_AVP= 701;
  private static final int AVP_User_Data_Sh_AVP= 702;
  private static final int AVP_Data_Reference_AVP = 703;
  private static final int AVP_Service_Indication_AVP = 704;
  private static final int AVP_Subs_Req_Type_AVP =705; 
  private static final int AVP_Requested_Domain_AVP = 706;
  private static final int AVP_Current_Location_AVP = 707;
  private static final int AVP_Identity_Set_AVP = 708;
  private static final int AVP_Expiry_Time_AVP= 709;
  private static final int AVP_Send_Data_Indication_AVP = 710;
  private static final int AVP_Repository_Data_ID_AVP = 715;									
  private static final int AVP_Sequence_Number_AVP = 716;
  ////private static final int AVP_Server_Name_AVP = 602;
  ////private static final int AVP_Supported_Features_AVP = 628; 
  ////private static final int AVP_Feature_List_ID_AVP = 629;
  ////private static final int AVP_Feature_List_AVP = 630;
  private static final int AVP_Wildcarded_PSI_AVP = 634;
  private static final int AVP_Wildcarded_IMPU_AVP = 636;
  private static final int AVP_DSAI_Tag_AVP = 711;
  ////private static final int AVP_Experimental_Result_Code_AVP = 298;
  ////private static final int AVP_Experimental_Result_AVP = 297; 
  ////private static final int AVP_User_Data_AVP = 606;

  
  //TS129_215 AVP codes
  private static final int AVP_QoS_Rule_Name_AVP = 1054;
  private static final int AVP_Rule_Failure_Code_AVP = 1031;
  private static final int AVP_PDN_Connection_ID_AVP = 1065;
  private static final int AVP_QoS_Rule_Report_AVP = 1055;
  private static final int AVP_QoS_Rule_Install_AVP = 1051;
  private static final int AVP_Multiple_BBERF_Action_AVP = 2204;
  private static final int AVP_Session_Linking_Indicator_AVP = 1064;
  private static final int AVP_Subses_Decision_Info_AVP = 2200;
  private static final int AVP_Subses_Enforcement_Info_AVP = 2201;
  private static final int AVP_Subsession_Id_AVP = 2202;
  private static final int AVP_Subsession_Operation_AVP = 2203;
  //End TS129_215 AVP codes
  
   //TS129_272 AVP codes
   private static final int AVP_ULR_Flags_AVP = 1405;                                //ref: 7.3.7 ETSI TS 129 272
   private static final int AVP_ULA_Flags_AVP = 1406;                                //ref: 7.3.8
   private static final int AVP_Visited_PLMN_Id_AVP = 1407;                          //ref: 7.3.9
   private static final int AVP_UE_SRVCC_Capability_AVP = 1615;                      //ref: 7.3.130
   private static final int AVP_SGSN_Number_AVP = 1489;  							//ref: 7.3.102
   private static final int AVP_Homogeneous_Support_AVP = 1493;  					//ref: 7.3.107
   
   private static final int AVP_Context_Identifier_AVP = 1423;                      //ref: 7.3.27
   private static final int AVP_Service_Selection_AVP  = 493;                       //ref: 7.3.36
   private static final int AVP_MIP_Home_Agent_Address_AVP = 334;                   //ref: 7.3.42 - IETF RFC 4004 7.4
   private static final int AVP_MIP_Home_Agent_Host_AVP = 348;                      //ref: 7.3.43 - IETF RFC 4004 7.11
   private static final int AVP_MIP6_Home_Link_Prefix_AVP = 125;                    //ref:IETF RFC 5447 4.2.4
   private static final int AVP_MIP6_Agent_Info_AVP    = 486;                       //ref: 7.3.45
   private static final int AVP_Visited_Network_Identifier_AVP = 600;               //ref: 7.3.105 - TS 129 229 6.3.1
   private static final int AVP_SIP_Authentication_Context_AVP = 611;
   private static final int AVP_UAR_Flags_AVP = 637;
   private static final int AVP_User_Authorization_Type_AVP = 623;
   private static final int AVP_Server_Assignment_Type_AVP = 614;
   private static final int AVP_User_Data_Already_Available_AVP = 624;
   private static final int AVP_SIP_Digest_Authenticate_AVP = 635;
   private static final int AVP_SCSCF_Restoration_Info_AVP = 639;
   private static final int AVP_Restoration_Info_AVP = 649;
   private static final int AVP_Path_AVP = 640;
   private static final int AVP_Contact_AVP = 641;
   private static final int AVP_Subscription_Info_AVP = 642;
   private static final int AVP_Originating_Request_AVP = 652;
   private static final int AVP_Deregistration_Reason_AVP = 615;
   private static final int AVP_Reason_Code_AVP = 616;
   private static final int AVP_Reason_Info_AVP = 617;
   private static final int AVP_Specific_APN_Info_AVP  = 1472;                      //ref: 7.3.82
   private static final int AVP_Active_APN_AVP = 1612;                              //ref: 7.3.127
   private static final int AVP_Error_Diagnostic_AVP = 1614;						//ref: 7.3.128;
   private static final int AVP_Subscriber_Status_AVP = 1424;
   private static final int AVP_Operator_Determined_Barring_AVP = 1425;
   private static final int AVP_HPLMN_ODB_AVP = 1418;
   private static final int AVP_Access_Restriction_Data_AVP = 1426;
   private static final int AVP_APN_OI_Replacement_AVP = 1427;
   private static final int AVP_Max_Requested_Bandwidth_UL_AVP = 516;
   private static final int AVP_Max_Requested_Bandwidth_DL_AVP = 515;
   private static final int AVP_AMBR_AVP = 1435;
   
   private static final int AVP_All_APN_Conf_Included_Id_AVP = 1428;
   private static final int AVP_APN_Configuration_Profile_AVP = 1429;
   private static final int AVP_PDN_Type_AVP = 1456;
   private static final int AVP_QoS_Class_Identifier_AVP=1028;
   private static final int AVP_Priority_Level_AVP = 1046;
   private static final int AVP_Allocation_Retention_Priority_AVP =1034;
   private static final int AVP_EPS_Subscribed_QoS_Profile_AVP = 1431;
   private static final int AVP_APN_Configuration_AVP = 1430;
   private static final int AVP_GMLC_Address_AVP = 2405;                            //ref: 7.3.109 - TS 129 173 6.4.7
   //....
   private static final int AVP_Complete_DL_Included_Indicator_AVP = 1468;
   private static final int AVP_QoS_Subscribed_AVP = 1404;
   private static final int AVP_PDP_Type_AVP = 1470;
   private static final int AVP_PDP_Context_AVP = 1469; 
   private static final int AVP_GPRS_Subscription_Data_AVP = 1467;
   private static final int AVP_Subscription_Data_AVP = 1400;                       //ref: 7.3.2
   private static final int AVP_Immediate_Response_Preferred_AVP = 1412;            //ref: 7.3.16
   private static final int AVP_Re_synchronization_Info_AVP = 1411;                 //ref: 7.3.15
   private static final int AVP_Req_EUTRAN_Auth_Info_AVP = 1408;                    //ref: 7.3.11
   private static final int AVP_Req_UTRAN_GERAN_Auth_Info_AVP = 1409;               //ref: 7.3.12
  
   private static final int AVP_Item_Number_AVP = 1419;
   private static final int AVP_RAND_AVP = 1447;
   private static final int AVP_XRES_AVP = 1448;
   private static final int AVP_AUTN_AVP = 1449;
   private static final int AVP_KASME_AVP = 1450;
   private static final int AVP_E_UTRAN_Vector_AVP = 1414;
   private static final int AVP_SIP_Authentication_Scheme_3GPP_AVP = 608;
   private static final int AVP_SIP_Authenticate_3GPP_AVP = 609;
   private static final int AVP_SIP_Authorization_3GPP_AVP = 610;
   private static final int AVP_SIP_Number_Auth_Items_3GPP_AVP = 607;
   private static final int AVP_SIP_Auth_Data_Item_3GPP_AVP = 612;//ETSI TS 129 229 V10.5.0 (2013-04) Clause 6.3.13 SIP-Auth-Data-Item AVP
   private static final int AVP_Identity_with_Emergency_Registration_AVP = 651;
   private static final int AVP_Confidentiality_Key_AVP = 625;
   private static final int AVP_Integrity_Key_AVP = 626;
   private static final int AVP_Line_Identifier_AVP = 500;
   private static final int AVP_UTRAN_Vector_AVP = 1415; 
   private static final int AVP_SRES_AVP = 1454;
   private static final int AVP_Kc_AVP = 1453;
   private static final int AVP_GERAN_Vector_AVP = 1416;
   private static final int AVP_Authentication_Info_AVP = 1413;
   
   private static final int AVP_Cancellation_Type_AVP = 1420;
   private static final int AVP_CLR_Flags_AVP = 1638;                               //ref: 7.3.152
   private static final int AVP_IDR_Flags_AVP = 1490;                               //ref: 7.3.103
   private static final int AVP_DSR_Flags_AVP = 1421;                               //ref: 7.3.25
   private static final int AVP_DSA_Flags_AVP = 1422;                               //ref: 7.3.26
   private static final int AVP_PUER_Flags_AVP= 1635;                               //ref: 7.3.149
   private static final int AVP_PUEA_Flags_AVP= 1442;                               //ref: 7.3.48
   private static final int AVP_IDA_Flags_AVP = 1441;                               //ref: 7.3.47
   private static final int AVP_NOR_Flags_AVP = 1443;                               //ref: 7.3.49
   private static final int AVP_IMS_Voice_PS_Sessions_Support_AVP = 1492;
   private static final int AVP_Last_UE_Activity_Time_AVP = 1494;
   private static final int AVP_EPS_User_State_AVP = 1495;
   private static final int AVP_EPS_Location_Information_AVP = 1496;
   //end TS129_272 AVP codes
  
    
  //TS132_299 AVP codes
   private static final int AVP_Charging_Information_AVP = 618;
   private static final int AVP_Primary_Event_Charging_Function_Name_AVP = 619;
   private static final int AVP_Secondary_Event_Charging_Function_Name_AVP = 620;
   private static final int AVP_Primary_Charging_Collection_Function_Name_AVP = 621;
   private static final int AVP_Secondary_Charging_Collection_Function_Name_AVP = 622;
//  private static final int AVP_User_CSG_Information_AVP = 2319; 		//Ref: ETSI TS 132 299 User_CSG_Information_AVP
//  private static final int AVP_CSG_Id_AVP = 1437; 						//Ref: ETSI TS 132 299 CSG_Id_AVP
//  private static final int AVP_CSG_Access_Mode_AVP = 2317; 				//Ref: ETSI TS 132 299 CSG_Access_Mode_AVP
//  private static final int AVP_CSG_Membership_Indication_AVP = 2318;	//Ref: ETSI TS 132 299 CSG_Membership_Indication_AVP
  private static final int AVP_Service_Information_AVP = 873;			//Ref: ETSI TS 132 299 Service_Information_AVP
  private static final int AVP_PS_Information_AVP = 874;				//Ref: ETSI TS 132 299 PS_Information_AVP
  private static final int AVP_WLAN_Information_AVP = 875;				//Ref: ETSI TS 132 299 WLAN_Information_AVP
  private static final int AVP_IMS_Information_AVP = 876;				//Ref: ETSI TS 132 299 IMS_Information_AVP
//  private static final int AVP_MMS_Information_AVP = 877;				//Ref: ETSI TS 132 299 MMS_Information_AVP
//  private static final int AVP_LCS_Information_AVP = 878;				//Ref: ETSI TS 132 299 LCS_Information_AVP
//  private static final int AVP_PoC_Information_AVP = 879;				//Ref: ETSI TS 132 299 PoC_Information_AVP
//  private static final int AVP_MBMS_Information_AVP = 880;				//Ref: ETSI TS 132 299 MBMS_Information_AVP
  private static final int AVP_Service_Generic_Information_AVP = 1256;	//Ref: ETSI TS 132 299 Service_Generic_Information_AVP
  private static final int AVP_Event_Type_AVP = 825;					//Ref: ETSI TS 132 299 Event_Type_AVP
  private static final int AVP_Event_AVP = 823;							//Ref: ETSI TS 132 299 Event_AVP
  private static final int AVP_Expires_AVP = 888;						//Ref: ETSI TS 132 299 Expires_AVP
  private static final int AVP_Role_Of_Node_AVP = 829;					//Ref: ETSI TS 132 299 Role_Of_Node_AVP
  private static final int AVP_Node_Functionality_AVP = 862;			//Ref: ETSI TS 132 299 Node_Functionality_AVP
  private static final int AVP_User_Session_ID_AVP = 830;				//Ref: ETSI TS 132 299 User_Session_ID_AVP
  private static final int AVP_Calling_Party_Address_AVP = 831;			//Ref: ETSI TS 132 299 Calling_Party_Address_AVP
  private static final int AVP_Called_Party_Address_AVP = 832;			//Ref: ETSI TS 132 299 Called_Party_Address_AVP
  private static final int AVP_Called_Asserted_Identity_AVP = 1250;		//Ref: ETSI TS 132 299 Called_Asserted_Identity_AVP
  private static final int AVP_Alternate_Charged_Party_Address_AVP = 1280;//Ref: ETSI TS 132 299 Alternate_Charged_Party_Address_AVP
  private static final int AVP_Requested_Party_Address_AVP = 1251;		//Ref: ETSI TS 132 299 Requested_Party_Address_AVP
  private static final int AVP_Associated_URI_AVP = 856;				//Ref: ETSI TS 132 299 Associated_URI_AVP
  private static final int AVP_Time_Stamps_AVP = 833;					//Ref: ETSI TS 132 299 Time_Stamps_AVP
  private static final int AVP_SIP_Request_Timestamp_AVP = 834;			//Ref: ETSI TS 132 299 SIP_Request_Timestamp_AVP
  private static final int AVP_SIP_Response_Timestamp_AVP = 835;		//Ref: ETSI TS 132 299 SIP_Response_Timestamp_AVP
  private static final int AVP_Application_Server_Information_AVP = 850;//Ref: ETSI TS 132 299 Application_Server_Information_AVP
  private static final int AVP_Application_Server_AVP = 836;			//Ref: ETSI TS 132 299 Application_Server_AVP
  private static final int AVP_Application_Provided_Called_Party_Address_AVP = 837;		//Ref: ETSI TS 132 299 Application_Provided_Called_Party_Address_AVP
  private static final int AVP_Inter_Operator_Identifier_AVP = 838;		//Ref: ETSI TS 132 299 Inter_Operator_Identifier_AVP
  private static final int AVP_Originating_IOI_AVP = 839;				//Ref: ETSI TS 132 299 Originating_IOI_AVP
  private static final int AVP_Terminating_IOI_AVP = 840;				//Ref: ETSI TS 132 299 Terminating_IOI_AVP
  private static final int AVP_IMS_Charging_Identifier_AVP = 841;		//Ref: ETSI TS 132 299 IMS_Charging_Identifier_AVP
  private static final int AVP_SDP_Session_Description_AVP = 842;		//Ref: ETSI TS 132 299 SDP_Session_Description_AVP
  private static final int AVP_SDP_Media_Component_AVP = 844;				//Ref: ETSI TS 132 299 SDP_Media_Name_AVP
//  private static final int AVP_SDP_Media_Name_AVP = 844;				//Ref: ETSI TS 132 299 SDP_Media_Name_AVP
//  private static final int AVP_SDP_Media_Description_AVP = 845;			//Ref: ETSI TS 132 299 SDP_Media_Description_AVP
//  private static final int AVP_Media_Initiator_Flag_AVP = 882;			//Ref: ETSI TS 132 299 Media_Initiator_Flag_AVP
//  private static final int AVP_Media_Initiator_Party_AVP = 1288;		//Ref: ETSI TS 132 299 Media_Initiator_Party_AVP
//  private static final int AVP_Authorized_QoS_AVP = 849;				//Ref: ETSI TS 132 299 Authorized_QoS_AVP
  private static final int AVP_Served_Party_IP_Address_AVP = 848;		//Ref: ETSI TS 132 299 Served_Party_IP_Address_AVP
  private static final int AVP_Trunk_Group_ID_AVP = 851;				//Ref: ETSI TS 132 299 Trunk_Group_ID_AVP
  private static final int AVP_Incoming_Trunk_Group_ID_AVP = 852;		//Ref: ETSI TS 132 299 Incoming_Trunk_Group_ID_AVP
  private static final int AVP_Outgoing_Trunk_Group_ID_AVP = 853;		//Ref: ETSI TS 132 299 Outgoing_Trunk_Group_ID_AVP
  
  private static final int AVP_Delivery_Status_AVP =2104;
  private static final int AVP_Application_Session_ID_AVP=2103;
  private static final int AVP_Application_Service_Type_AVP=2102;
  private static final int AVP_Application_Server_ID_AVP=2101;
  
  private static final int AVP_Reporting_Reason_AVP=872;
  private static final int AVP_Time_Quota_Threshold_AVP=868;
  private static final int AVP_Volume_Quota_Threshold_AVP=869;
  private static final int AVP_Unit_Quota_Threshold_AVP=1226;
  private static final int AVP_Quota_Holding_Time_AVP=871;
  private static final int AVP_Quota_Consumption_Time_AVP=881;
  private static final int AVP_Time_Quota_Type_AVP=1271;
  private static final int AVP_Base_Time_Interval_AVP=1265;
  private static final int AVP_Time_Quota_Mechanism_AVP=1270;
  private static final int AVP_Envelope_Start_Time_AVP=1269;
  private static final int AVP_Envelope_End_Time_AVP=1267;
  private static final int AVP_Envelope_AVP=1266;
  private static final int AVP_Envelope_Reporting_AVP=1268;
  private static final int AVP_Trigger_AVP=1264;
  private static final int AVP_Trigger_Type_AVP=870;
  private static final int AVP_Refund_Information_AVP=2022;
  private static final int AVP_Multiple_Services_Credit_Control_AVP = 456;
  
  private static final int AVP_Offline_Charging_AVP = 1278;
  private static final int AVP_Low_Balance_Indication_AVP=2020;
  private static final int AVP_Remaining_Balance_AVP = 2021;
  //end of TS132_299 AVP codes
    
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] command_codes ={
	   /* Ref: RFC3588 p.3.1 */
	
		  Code_CE,  // Capabilities-Exchange-Request 	CER  (257)
		  Code_CE,  // Capabilities-Exchange-Answer 	CEA  (257)
		  Code_RA,  // Re-Auth-Request 				RAR  (258)
		  Code_RA,  // Re-Auth-Answer 				RAA  (258)
	    /**/	
		  Code_AC,  // Accounting-Request				ACR(271)
		  Code_AC,  // Accounting-Answer				ACA (271)
	      /**/
		  Code_AS,  // Abort-Session-Request			ASR (274)
		  Code_AS,  // Abort-Sesion-Answer			ASA (274)
		  Code_ST,  // Session-Termination-Request	STR (275)
		  Code_ST,  // Session-Termination-Answer		STA (275)  		    
		/**/
		  Code_DW,  // Device-Watchdog-Request		DWR (280)
		  Code_DW,  // Device-Watchdog-Answer			DWA (280)
		  Code_DP,  // Disconnect-Peer-Request			DPR (282)
		  Code_DP,  // Disconnect-Peer-Answer			DPA (282) 
		
		/* Codes on Cx */
		  Code_UA,   	// User-Authorization-Request	UAR	300
		  Code_UA,   	// User-Authorization-Answer	UAA	300
		  Code_SA,   	// Server-Assignment-Request	SAR	301
		  Code_SA,   	// Server-Assignment-Answer		SAA	301
		  Code_LI,   	// Location-Info-Request		LIR	302
		  Code_LI,   	// Location-Info-Answer			LIA	302
		  Code_MA,   	// Multimedia-Auth-Request		MAR	303
		  Code_MA,   	// Multimedia-Auth-Answer		MAA	303
		  Code_RT,  	// Registration-Termination-Request	RTR	304
		  Code_RT,  	// Registration-Termination-Answer	RTA	304
		  Code_PP,  	// Push-Profile-Request			PPR	305
		  Code_PP,  	// Push-Profile-Answer			PPA	305	
		
		/* Codes on Sh/Dh interface  */
		  Code_UD,	//	UDR User-Data-Request AS->HSS 	306
		  Code_UD,	//	UDA User-Data-Answer HSS->AS	306		
		  Code_PU, 	//	PUR Profile-Update-Request AS->HSS	307
		  Code_PU,	//	PUA Profile-Update-Answer HSS->AS	307
		  Code_SN,	//	SNR Subscribe-Notifications-Request AS->HSS	308
		  Code_SN,	//	SNA Subscribe-Notifications-Answer HSS->AS	308
		  Code_PN,	//	PNR Profile-Notification-Request HSS->AS	309
		  Code_PN,	//	PNA Profile-Notification-Answer AS->HSS		309
		
		/* Codes on Rx interface  */
		  Code_AA,	//	AAR Auth-Accounting-Request  	265
		  Code_AA,	//	AAA Auth-Accounting-Request 	265
		  
		/*RFC4006*/
		  Code_CC,  //	CCR Credit-Control-Request    272
		  Code_CC,  //	CCA Credit-Control-Answer     272
		
        /* Codes on S6a interface - TS 129 272  */
		  Code_UL,  // Update-Location-Request                   ULR    (316)- Ref: TS 129 272 7.2.3
		  Code_UL,  // Update-Location-Answer 			         ULA    (316)- Ref: TS 129 272 7.2.4
		  Code_CL,  // Cancel-Location-Request                   CLR    (317)- Ref: TS 129 272 7.2.7
		  Code_CL,  // Cancel-Location-Answer                    CLA    (317)- Ref: TS 129 272 7.2.8
		  Code_AI,  // Authentication-Information-Request        AIR    (318)- Ref: TS 129 272 7.2.5
		  Code_AI,  // Authentication-Information-Answer         AIA    (318)- Ref: TS 129 272 7.2.6
		  Code_ID,  // Insert-Subscriber-Data-Request            IDR    (319)- Ref: TS 129 272 7.2.9
		  Code_ID,  // Insert-Subscriber-Data-Answer             IDA    (319)- Ref: TS 129 272 7.2.10
		  Code_DS,  // Delete-Subscriber-Data-Request            DSR    (320)- Ref: TS 129 272 7.2.11
		  Code_DS,  // Delete-Subscriber-Data-Answer             DSA    (320)- Ref: TS 129 272 7.2.12
		  Code_PUE, // Purge-UE-Request                          PUR    (321)- Ref: TS 129 272 7.2.13
		  Code_PUE, // Purge-UE-Answer                           PUA    (321)- Ref: TS 129 272 7.2.14
		  Code_RS,  // Reset-Request                             DSR    (322)- Ref: TS 129 272 7.2.15
		  Code_RS,  // Reset-Answer                              DSA    (322)- Ref: TS 129 272 7.2.16
		  Code_NO,  // Notify-Request                            NOR    (323)- Ref: TS 129 272 7.2.17
		  Code_NO   // Notify-Answer                             NOA    (323)- Ref: TS 129 272 7.2.18	  
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] result_codes ={
		0,		//          DIAMETER_DUMMY_E (0),
	         
		1001,	//          DIAMETER_MULTI_ROUND_AUTH_E (1001),
		         
		2001,	//          DIAMETER_SUCCESS_E (2001),
		2002,	//          DIAMETER_LIMITED_SUCCESS_E (2002),
		        
		3001,	//          DIAMETER_COMMAND_UNSUPPORTED_E (3001),
		3002,	//          DIAMETER_UNABLE_TO_DELIVER_E (3002),			
		3003,	//          DIAMETER_REALM_NOT_SERVED_E (3003),
		3004,	//          DIAMETER_TOO_BUSY_E (3004),
		3005,	//          DIAMETER_LOOP_DETECTED_E (3005),
		3006,	//          DIAMETER_REDIRECT_INDICATION_E (3006),
		3007,	//          DIAMETER_APPLICATION_UNSUPPORTED_E (3007),
		3008,	//          DIAMETER_INVALID_HDR_BITS_E (3008),
		3009,	//          DIAMETER_INVALID_AVP_BITS_E (3009),
		3010,	//          DIAMETER_UNKNOWN_PEER_E (3010),
		       
		4001,   //          DIAMETER_AUTHENTICATION_REJECTED_E (4001),
		4002,	//          DIAMETER_OUT_OF_SPACE_E (4002),			
		4003,	//          DIAMETER_ELECTION_LOST_E (4003),
		        
		5001,	//          DIAMETER_AVP_UNSUPPORTED_E (5001),
		5002,	//          DIAMETER_UNKNOWN_SESSION_ID_E (5002),
		5003,	//          DIAMETER_AUTHORIZATION_REJECTED_E (5003),
		5004,	//          DIAMETER_INVALID_AVP_VALUE_E (5004),
		5005,	//          DIAMETER_MISSING_AVP_E (5005),
		5006,	//          DIAMETER_RESOURCES_EXCEEDED_E (5006),
		5007,	//          DIAMETER_CONTRADICTING_AVPS_E (5007),
		5008,	//          DIAMETER_AVP_NOT_ALLOWED_E (5008),
		5009,	//          DIAMETER_AVP_OCCURS_TOO_MANY_TIMES_E (5009),
		5010,	//          DIAMETER_NO_COMMON_APPLICATION_E (5010),
		5011,	//          DIAMETER_UNSUPPORTED_VERSION_E (5011),
		5012,	//          DIAMETER_UNABLE_TO_COMPLY_E (5012),
		5013,	//          DIAMETER_INVALID_BIT_IN_HEADER_E (5013),
		5014,	//          DIAMETER_INVALID_AVP_LENGTH_E (5014),
		5015,	//          DIAMETER_INVALID_MESSAGE_LENGTH_E (5015),
		5016,	//          DIAMETER_INVALID_AVP_BIT_COMBO_E (5016),
		5017	//          DIAMETER_NO_COMMON_SECURITY_E (5017)		  
  };
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Redirect_Host_Usage={
		0,//	    DONT_CACHE_E (0),
		1,//		ALL_SESSION_E (1),
		2,//		ALL_REALM_E (2),
		3,//		REALM_AND_APPLICATION_E (3),
		4,//		ALL_APPLICATION_E (4),
		5,//		ALL_HOST_E (5),
		6 //		ALL_USER_E  (6)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Acc_Record={
		1,//		EVENT_RECORD_E (1),                   
		2,//		START_RECORD_E  (2),                  
		3,//		INTERIM_RECORD_E (3),               
		4 //		STOP_RECORD_E    (4)                 
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Acc_Realtime_Req={
		1,//		DELIVER_AND_GRANT_E (1),
		2,//		GRANT_AND_STORE_E (2),
		3 //		GRANT_AND_LOSE_E (3)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Termination_Cause={
		1,//		DIAMETER_LOGOUT_E (1),
		2,//		DIAMETER_SERVICE_NOT_PROVIDED_E (2),
		3,//		DIAMETER_BAD_ANSWER_E (3),
		4,//			DIAMETER_ADMINISTRATIVE_E (4),
		5,//		DIAMETER_LINK_BROKEN_E (5),
		6,//		DIAMETER_AUTH_EXPIRED_E (6),
		7,//		DIAMETER_USER_MOVED_E (7),
		8 //		DIAMETER_SESSION_TIMEOUT_E (8)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Disconnect_Cause={
		0,//		REBOOTING_E (0),
		1,//		BUSY_E (1),
		2 //		DO_NOT_WANT_TO_TALK_TO_YOU_E (2)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Role_Of_Node={
		0,//		ORIGINATING_ROLE_E (0),
        1,//		TERMINATING_ROLE_E (1),
        2,//		PROXY_ROLE_E (2),
        3 //		B2BUA_ROLE_E (3)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Node_Functionality={
		0,//		S_CSCF_E (0),
        1,//		P_CSCF_E (1),
        2,//		I_CSCF_E (2),
        3,//		MRFC_E (3),
        4,//		MGCF_E (4),
        5,//		BGCF_E (5),
        6 //		AS_E (6)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] CC_Request={
		// Ref: RFC4006 8.3.  CC-Request-Type AVP
		1,//	INITIAL_REQUEST_E (1),
		2,//	UPDATE_REQUEST_E (2),
		3,//	TERMINATION_REQUEST_E (3),
		4 // 	EVENT_REQUEST_E  (4)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Requested_Action={
		// Ref: RFC4006 8.41.  Requested-Action AVP
		0,//	DIRECT_DEBITING_E (0),
		1,//	REFUND_ACCOUNT_E (1),
		2,//	CHECK_BALANCE_E (2),
		3 //	PRICE_ENQUIRY_E (3)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Multi_Serv_Indi={
		// Ref: RFC4006 8.40.  Multiple-Services-Indicator AVP
		0,//	MULTIPLE_SERVICES_NOT_SUPPORTED_E (0),
		1 //	MULTIPLE_SERVICES_SUPPORTED_E     (1)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] User_Equipment_Info={
		//Ref: RFC4006 8.50.  User-Equipment-Info-Type AVP
		0,//	IMEISV_E (0),
		1,//	MAC_E (1),
		2,//	EUI64_E (2),
		3 //	MODIFIED_EUI64_E (3)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] CC_Session_Failover={
		// Ref: RFC4006 8.4.  CC-Session-Failover AVP
		0,//	FAILOVER_NOT_SUPPORTED_E (0),
		1 //	FAILOVER_SUPPORTED_E  (1)
	};
    
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] CC_Failure_Handling={
		//Ref: RFC4006 8.14.  Credit-Control-Failure-Handling AVP
		0,//	TERMINATE_E (0),
		1,//	CONTINUE_E  (1),
		2 //	RETRY_AND_TERMINATE_E (2)
	};

  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] DD_Failure_Handling={
		//Ref: RFC4006 8.15.  Direct-Debiting-Failure-Handling AVP
		0,//	TERMINATE_OR_BUFFER_E (0),
		1 //	CONTINUE_E (1)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Tariff_Change_Usage={
		// Ref: RFC4006 8.27.  Tariff-Change-Usage AVP
		0,//	UNIT_BEFORE_TARIFF_CHANGE_E (0),
		1,//	UNIT_AFTER_TARIFF_CHANGE_E (1),
	 	2 //  	UNIT_INDETERMINATE_E (2)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] CC_Unit_Type={
		//Ref: RFC4006 8.31.  G-S-U-Pool-Identifier AVP
		0,//	TIME_E (0),
		1,//	MONEY_E (1),
		2,//	TOTAL_OCTETS_E (2),
		3,//	INPUT_OCTETS_E (3),
		4,//	OUTPUT_OCTETS_E (4),
		5 //	SERVICE_SPECIFIC_UNITS_E (5)
	};  

  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[]Final_Unit_Action_Type={
		// Ref: RFC4006 8.35.  Final-Unit-Action AVP
		0,//	TERMINATE_E (0),
		1,//	REDIRECT_E (1),
		2 //	RESTRICT_ACCESS_E (2)				
	};			
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Redirect_Address_Type={
		// Ref: RFC4006 8.38.  Redirect-Address-Type AVP
		0,//	IPV4_ADDRESS_E (0),
		1,//	IPV6_ADDRESS_E (1),
		2,//	URL_E (2),
		3 //	SIP_URI_E (3)
	};
  
  //NOTE: The order of constants in the following array is very important.
  //It must exactly match order specified in TTCN-3 module.
  private final int[] Auth_Session_State_Type={
      0,//      STATE_MAINTAINED_E (0),
      1,//      NO_STATE_MAINTAINED_E (1)
  };
  
  private final int[] RAT_Type={
      // Ref: ETSI TS 129 212 5.3.31
      0,// WLAN_E (0),
      1,//VIRTUAL_E (1),
      1000,//UTRAN_E (1000),
      1001,//GERAN_E (1001),
      1002,//GAN_E (1002),
      1003,//HSPA_EVOLUTION_E (1003),
      1004,//EUTRAN_E (1004),
      2000,//CDMA2000_IX_E (2000),
      2002,//(2002),
      2003//EHRPD_E(2003)
  };
  
  //129.329 -sh
  private final int[] Experimental_Result_Type={
		  2001,//DIAMETER_FIRST_REGISTRATION_E (2001),
          2002,//DIAMETER_SUBSEQUENT_REGISTRATION_E (2002),
          2003,//DIAMETER_UNREGISTERED_SERVICE_E (2003),
          2004,//DIAMETER_SUCCESS_SERVER_NAME_NOT_STORED_E (2004),
          4181,//DIAMETER_AUTHENTICATION_DATA_UNAVAILABLE_E (4181),
          5001,//DIAMETER_ERROR_USER_UNKNOWN_E (5001),
          5002,//DIAMETER_ERROR_IDENTITIES_DONT_MATCH_E (5002),
          5003,//DIAMETER_ERROR_IDENTITY_NOT_REGISTERED_E (5003),
          5004,//DIAMETER_ERROR_ROAMING_NOT_ALLOWED_E (5004),
          5005,//DIAMETER_ERROR_IDENTITY_ALREADY_REGISTERED_E (5005),
          5006,//DIAMETER_ERROR_AUTH_SCHEME_NOT_SUPPORTED_E (5006),
          5007,//DIAMETER_ERROR_IN_ASSIGNMENT_TYPE_E (5007),
          5008,//DIAMETER_ERROR_TOO_MUCH_DATA_E (5008),
          5009,//DIAMETER_ERROR_NOT_SUPPORTED_USER_DATA_E (5009),
          // Ref: ETSI TS 129 214 5.5
          5061,//INVALID_SERVICE_INFORMATION_E (5061),
          5062,//FILTER_RESTRICTIONS_E (5062),
          5063,//REQUESTED_SERVICE_NOT_AUTHORIZED_E (5063),
          5064,//DUPLICATED_AF_SESSION_E (5064),
          5065,//IP_CAN_SESSION_NOT_AVAILABLE_E (5065),
          5066,//UNAUTHORIZED_NON_EMERGENCY_SESSION_E (5066),
          5067,//UNAUTHORIZED_SPONSORED_DATA_CONNECTIVITY_E (5067),
          5140,//DIAMETER_ERROR_INITIAL_PARAMETERS_E (5140),
          5141,//DIAMETER_ERROR_TRIGGER_EVENT_E (5141),
          5142,//DIAMETER_PCC_RULE_EVENT_E (5142),
          5143,//DIAMETER_ERROR_BEARER_NOT_AUTHORIZED_E (5143),
          5144,//DIAMETER_ERROR_TRAFFIC_MAPPING_INFO_REJECTED_E (5144),
          5147,//DIAMETER_ERROR_CONFLICTING_REQUEST_E (5147),
          // Ref: 3GPP TS 29.215 
          5470,//DIAMETER_ERROR_SUBSESSION_E (5470),
          5420,//DIAMETER_ERROR_UNKNOWN_EPS_SUBSCRIPTION_E (5420),//(9990), // Set the correct error code value - See ETSI TS 129 272 Clause 5.2.1.1.3 Detailed behaviour of the HSS
          5421,//DIAMETER_ERROR_RAT_NOT_ALLOWED_E (5421),//(9991), // Set the correct error code value - See ETSI TS 129 272 Clause 5.2.1.1.3 Detailed behaviour of the HSS
          5422,//DIAMETER_ERROR_EQUIPMENT_UNKNOWN_E (5422),
          5423//DIAMETER_ERROR_UNKOWN_SERVING_NODE_E (5423),

          //unknown  
  };
  
  private final int[] Cancellation_Type_Code_Type ={
      0, //MME_UPDATE_PROCEDURE_E        (0),
      1,//SGSN_UPDATE_PROCEDURE_E        (1),
      2,//SUBSCRIPTION_WITHDRAWAL_E    (2),
      3,//UPDATE_PROCEDURE_IWF_E        (3),
      4//INITIAL_ATTACH_PROCEDURE_E    (4)
  };
  
  //*End 129.329 -sh
  
  //129_272
  private final int[] UE_SRVCC_Capability_Type={
	  0, //UE_SRVCC_NOT_SUPPORTED_E (0),
      1  //UE_SRVCC_SUPPORTED_E     (1)  
  };
  
  private final int[] Homogeneous_Support_Type={
	  0,//  NOT_SUPPORTED_E (0),
      1 //     SUPPORTED_E         (1) 
  };
  
  private final int[] Error_Diagnostic_Code_Type={
	  0,//	  GPRS_DATA_SUBSCRIBED_E        (0),
      1,//    NO_GPRS_DATA_SUBSCRIBED_E     (1),
      2,//    ODB_ALL_APN_E                 (2),
      3,//    ODB_HPLMN_APN_E               (3),
      4 //   ODB_VPLMN_APN_E               (4)	  
  };
  
  //End 129_272
  
  /**<!--###################################################################-->
   *
   * Constructor method <b>DefaultCodec</b> creates new object and prepares RB.
   * <p>
   * @param rb needed to resolve specific information (e.g. which logging to 
   *           use)
   *         
   * <!--################################################################--> */  
  public DIAMETERCodec(RB rb) {
    super(rb);
	typeServer = RB.getTciCDRequired();
	octetstringType = typeServer.getOctetstring();
	charstringType = typeServer.getCharstring();
	universalcharstringType = typeServer.getUniversalCharstring();
	integerType = typeServer.getInteger();
	bigintegerType = typeServer.getFloat();
	booleanType = typeServer.getBoolean();
	management = RB.getTciTMProvided();	
  }
  
  /**<!--###################################################################-->
   *
   * Method <b>encode</b> codes a message according to the native encoding
   * rules.
   * <p>
   * @param template value should be encoded.
   * <p>
   * @return <code>TriMessage</code> represents the collected byteArray
   *         message.
   * 
   * <!--################################################################--> */
   public TriMessage encode(Value template) {
       // The template is given to the AbstractBaseCodec which handles the basic encoding
       //TriMessage encodedMessage = super.encode(template);
       //return encodedMessage;
    
		Type msgType = template.getType();
		// RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1, null, RTLoggingConstants.RT_LOG_INFO, "DIAMETER codec: encode:  TypeClass:" + msgType +msgType.getTypeClass());
		// RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1, null, RTLoggingConstants.RT_LOG_INFO, "DIAMETER codec: encode:  Template:" + template);
		 
		 final ByteArrayOutputStream out = new ByteArrayOutputStream();
		 Class<? extends DIAMETERCodec> c = this.getClass();
		 
		 if(msgType.getName().toLowerCase().contains("diameteruri")){
			 try {
			     Method m=null;
			     m = c.getDeclaredMethod("encode_"+msgType.getName(), new Class[] { ByteArrayOutputStream.class , RecordValue.class   });
			     m.setAccessible(true);
			     m.invoke(this,out,template);
			     return new ForeignMessage(RB, out.toByteArray());
			 }
			 catch(Exception e){
			     System.out.println( "AVP TYPE = " +  msgType.getName() + " NOT supported!!!");
			     System.out.println( "TEMPLATE = " +  template.toString());
			     return new ForeignMessage(RB, out.toByteArray());
			 }
		 }
		 if(msgType.getName().toLowerCase().contains("avp")){			 
			
			 try {
				 Method m=null;
				 
				 switch (msgType.getTypeClass()) {
				     case TciTypeClass.RECORD   : m = c.getDeclaredMethod("encode_"+msgType.getName(), new Class[] { ByteArrayOutputStream.class , RecordValue.class   });break;
				     case TciTypeClass.RECORD_OF: m = c.getDeclaredMethod("encode_"+msgType.getName(), new Class[] { ByteArrayOutputStream.class , RecordOfValue.class });break;				 
//				     case TciTypeClass.SET_OF   : 
//				    	 if(msgType.getName().toLowerCase().contains("qos_rule_install_avp_2")){
//				    	 m = c.getDeclaredMethod("encode_"+"QoS_Rule_Install_AVP", new Class[] { ByteArrayOutputStream.class , RecordOfValue.class });}
//				    	 else if(msgType.getName().toLowerCase().contains("subses_enforcement_info_avp_3")){
//					     m = c.getDeclaredMethod("encode_"+"Subses_Enforcement_Info_AVP", new Class[] { ByteArrayOutputStream.class , RecordOfValue.class });}
//
//				    	 template = (RecordOfValue)template; 
//				    	 System.out.println(m.toString());
//				    	 System.out.println(template.toString()+template.getType());
//				    	 break;
				 }   

			     //System.out.println( "AVP TYPE = " +  msgType.getName() + " supported!!!");
			     //System.out.println( "TEMPLATE = " +  template.toString());
			     m.setAccessible(true);
			     m.invoke(this,out,template);
			     //HexDump.printHexArray(out.toByteArray());
			     return new ForeignMessage(RB, out.toByteArray());
			 }
			 //catch(NoSuchMethodException e)
			 catch(Exception e){
			     System.out.println( "AVP TYPE = " +  msgType.getName() + " NOT supported!!!");
			     System.out.println( "TEMPLATE = " +  template.toString());
			     return new ForeignMessage(RB, out.toByteArray());
			 }
			 //return null;	 
		 }

		switch (msgType.getTypeClass()) {
		
			case TciTypeClass.INTEGER:
			case TciTypeClass.FLOAT:
			case TciTypeClass.BITSTRING:
			case TciTypeClass.HEXSTRING:
			case TciTypeClass.OCTETSTRING:
				              encode_Octetstring(out, (OctetstringValue) template);
				              return new ForeignMessage(RB, out.toByteArray());
			case TciTypeClass.CHARSTRING:
				              encode_Charstring(out, (CharstringValue) template);
				              return new ForeignMessage(RB, out.toByteArray());
			case TciTypeClass.UNIVERSAL_CHARSTRING:
			case TciTypeClass.RECORD_OF:
			case TciTypeClass.ENUMERATED:
				              encode_IntegerValue(out,((EnumeratedValue)template).getInt());
				              return new ForeignMessage(RB, out.toByteArray());
			case TciTypeClass.UNION:
				              return encode_DIAMETERMessage(out, (UnionValue) template);
			case TciTypeClass.BOOLEAN:
			case TciTypeClass.VERDICT:
				              //return null;
			case TciTypeClass.RECORD:
							  return encode_DIAMETERMessage(out, (RecordValue) template);
			default:
				//throw new TciException("Unknown TypeClass");
			    RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1, null, RTLoggingConstants.RT_LOG_INFO, "DIAMETER codec: encode: Unknown TypeClass" + msgType + msgType.getTypeClass());	
			}
		return null; 
  }
  
  public TriMessage encode_DIAMETERMessage(ByteArrayOutputStream out, RecordValue value_) {
	    //RB.getTciTLProvided().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: Encoding DIAMETER message");
	    // RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1, null, RTLoggingConstants.RT_LOG_INFO,	"DIAMETERCodec: Encoding DIAMETER message:"+value_);
		    
		RecordValue header=(RecordValue)value_.getField("header");
		int cmdcode=command_codes[((IntegerValue) header.getField("cmdcode")).getInt()];
		RecordValue cmdflags = (RecordValue) header.getField("cmdflags");
		BitString r_a=(BitString)(cmdflags.getField("r_bit"));
		int BitVal = r_a.getBit(0);
	
		String messagebodyvar= MessageVariant(cmdcode,(BitVal ==0) ? false:true);//MessageVariant(parsedMessage.commandCode,parsedMessage.flagRequest);
		String messagevar = messagebodyvar + "MSG";
		String bodyvar = messagebodyvar + "Body";
		
		try {
			if ((messagevar == null)) {
				//return;
			}
			if (messagevar.equals("cER_MSG") && !value_.notPresent()) {
				encode_CER_MSG(out, (RecordValue) value_);
			} 
			if (messagevar.equals("cEA_MSG") && !value_.notPresent()) {
				encode_CEA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("rAR_MSG") && !value_.notPresent()) {
				encode_RAR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("rAA_MSG") && !value_.notPresent()) {
				encode_RAA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("aCR_MSG") && !value_.notPresent()) {
				encode_ACR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("aCA_MSG") && !value_.notPresent()) {
				encode_ACA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("aSR_MSG") && !value_.notPresent()) {
				encode_ASR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("aSA_MSG") && !value_.notPresent()) {
				encode_ASA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("sTR_MSG") && !value_.notPresent()) {
				encode_STR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("sTA_MSG") && !value_.notPresent()) {
				encode_STA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("dWR_MSG") && !value_.notPresent()) {
				encode_DWR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("dWA_MSG") && !value_.notPresent()) {
				encode_DWA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("dPR_MSG") && !value_.notPresent()) {
				encode_DPR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("dPA_MSG") && !value_.notPresent()) {
				encode_DPA_MSG(out, (RecordValue) value_);
			}
			
    	    //Cx interface
			if (messagevar.equals("uAR_MSG") && !value_.notPresent()) {
				encode_UAR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("uAA_MSG") && !value_.notPresent()) {
				encode_UAA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("sAR_MSG") && !value_.notPresent()) {
				encode_SAR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("sAA_MSG") && !value_.notPresent()) {
				encode_SAA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("lIR_MSG") && !value_.notPresent()) {
				encode_LIR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("lIA_MSG") && !value_.notPresent()) {
				encode_LIA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("mAR_MSG") && !value_.notPresent()) {
				encode_MAR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("mAA_MSG") && !value_.notPresent()) {
				encode_MAA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("rTR_MSG") && !value_.notPresent()) {
				encode_RTR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("rTA_MSG") && !value_.notPresent()) {
				encode_RTA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("pPR_MSG") && !value_.notPresent()) {
				encode_PPR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("pPA_MSG") && !value_.notPresent()) {
				encode_PPA_MSG(out, (RecordValue) value_);
			}
    	        	    
    	    //Sh interface
			if (messagevar.equals("uDR_MSG") && !value_.notPresent()) {
				encode_UDR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("uDA_MSG") && !value_.notPresent()) {
				encode_UDA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("pUR_MSG") && !value_.notPresent()) {
				encode_PUR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("pUA_MSG") && !value_.notPresent()) {
				encode_PUA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("sNR_MSG") && !value_.notPresent()) {
				encode_SNR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("sNA_MSG") && !value_.notPresent()) {
				encode_SNA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("pNR_MSG") && !value_.notPresent()) {
				encode_PNR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("pNA_MSG") && !value_.notPresent()) {
				encode_PNA_MSG(out, (RecordValue) value_);
			}
    	    
			//Rx interface
			if (messagevar.equals("aAR_MSG") && !value_.notPresent()) {
				encode_aAR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("aAA_MSG") && !value_.notPresent()) {
				encode_aAA_MSG(out, (RecordValue) value_);
			}
			
			//RFC4006
    	    if (messagevar.equals("cCR_MSG") && !value_.notPresent()) {
				encode_CCR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("cCA_MSG") && !value_.notPresent()) {
				encode_CCA_MSG(out, (RecordValue) value_);
			}
    	   
            /* Codes on S6a interface - TS 129 272  */
			if (messagevar.equals("uLR_MSG") && !value_.notPresent()) {
				encode_ULR_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("uLA_MSG") && !value_.notPresent()) {
				encode_ULA_MSG(out, (RecordValue) value_);
			}
			if (messagevar.equals("cLR_MSG") && !value_.notPresent()) {
				encode_CLR_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("cLA_MSG") && !value_.notPresent()) {
				encode_CLA_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("aIR_MSG") && !value_.notPresent()) {
				encode_AIR_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("aIA_MSG") && !value_.notPresent()) {
				encode_AIA_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("iDR_MSG") && !value_.notPresent()) {
				encode_IDR_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("iDA_MSG") && !value_.notPresent()) {
				encode_IDA_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("dSR_MSG") && !value_.notPresent()) {
				encode_DSR_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("dSA_MSG") && !value_.notPresent()) {
				encode_DSA_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("pUER_MSG") && !value_.notPresent()) {
				encode_PUER_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("pUEA_MSG") && !value_.notPresent()) {
				encode_PUEA_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("rSR_MSG") && !value_.notPresent()) {
				encode_RSR_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("rSA_MSG") && !value_.notPresent()) {
				encode_RSA_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("nOR_MSG") && !value_.notPresent()) {
				encode_NOR_MSG(out, (RecordValue) value_);
			}
            if (messagevar.equals("nOA_MSG") && !value_.notPresent()) {
				encode_NOA_MSG(out, (RecordValue) value_);
			}
           			
    	    //.....
			if (messagevar.equals("uNKNOWN_MSG") && !value_.notPresent()) {
				//encode_UNKNOWN_MSG(out, (RecordValue) value.getVariant("uNKNOWN_MSG"));
			}
			
			//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1, null, RTLoggingConstants.RT_LOG_INFO,	"DIAMETERCodec: Encoding DIAMETER message:"+out.size());
			//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1, null, RTLoggingConstants.RT_LOG_INFO,	"DIAMETERCodec: Encoding DIAMETER message:"+HexDump.BufferToString(out.toByteArray()));

		}
		catch (Exception e) {
			e.printStackTrace();
		}	
		return new ForeignMessage(RB, out.toByteArray());
}
  
  public TriMessage encode_DIAMETERMessage(ByteArrayOutputStream out,	UnionValue/*RecordValue*/ value) {
		//RB.getTciTLProvided().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: Encoding DIAMETER message");
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1, null, RTLoggingConstants.RT_LOG_INFO,	"DIAMETERCodec: Encoding DIAMETER message");
	    //RB.getTciTLProvided_Latest().tliInfo("", System.currentTimeMillis(), "", -1, null, RTLoggingConstants.RT_LOG_INFO,	"DIAMETERCodec: Encoding DIAMETER message");
		try {
			if ((value == null) | value.notPresent()) {
				//return;
			}
			if (value.getPresentVariantName().equals("cER_MSG") && !value.getVariant("cER_MSG").notPresent()) {
				encode_CER_MSG(out, (RecordValue) value.getVariant("cER_MSG"));
			} 
			if (value.getPresentVariantName().equals("cEA_MSG") && !value.getVariant("cEA_MSG").notPresent()) {
				encode_CEA_MSG(out, (RecordValue) value.getVariant("cEA_MSG"));
			}
			if (value.getPresentVariantName().equals("rAR_MSG") && !value.getVariant("rAR_MSG").notPresent()) {
				encode_RAR_MSG(out, (RecordValue) value.getVariant("rAR_MSG"));
			}
			if (value.getPresentVariantName().equals("rAA_MSG") && !value.getVariant("rAA_MSG").notPresent()) {
				encode_RAA_MSG(out, (RecordValue) value.getVariant("rAA_MSG"));
			}
			if (value.getPresentVariantName().equals("aCR_MSG") && !value.getVariant("aCR_MSG").notPresent()) {
				encode_ACR_MSG(out, (RecordValue) value.getVariant("aCR_MSG"));
			}
			if (value.getPresentVariantName().equals("aCA_MSG") && !value.getVariant("aCA_MSG").notPresent()) {
				encode_ACA_MSG(out, (RecordValue) value.getVariant("aCA_MSG"));
			}
			if (value.getPresentVariantName().equals("aSR_MSG") && !value.getVariant("aSR_MSG").notPresent()) {
				encode_ASR_MSG(out, (RecordValue) value.getVariant("aSR_MSG"));
			}
			if (value.getPresentVariantName().equals("aSA_MSG") && !value.getVariant("aSA_MSG").notPresent()) {
				encode_ASA_MSG(out, (RecordValue) value.getVariant("aSA_MSG"));
			}
			if (value.getPresentVariantName().equals("sTR_MSG") && !value.getVariant("sTR_MSG").notPresent()) {
				encode_STR_MSG(out, (RecordValue) value.getVariant("sTR_MSG"));
			}
			if (value.getPresentVariantName().equals("sTA_MSG") && !value.getVariant("sTA_MSG").notPresent()) {
				encode_STA_MSG(out, (RecordValue) value.getVariant("sTA_MSG"));
			}
			if (value.getPresentVariantName().equals("dWR_MSG") && !value.getVariant("dWR_MSG").notPresent()) {
				encode_DWR_MSG(out, (RecordValue) value.getVariant("dWR_MSG"));
			}
			if (value.getPresentVariantName().equals("dWA_MSG") && !value.getVariant("dWA_MSG").notPresent()) {
				encode_DWA_MSG(out, (RecordValue) value.getVariant("dWA_MSG"));
			}
			if (value.getPresentVariantName().equals("dPR_MSG") && !value.getVariant("dPR_MSG").notPresent()) {
				encode_DPR_MSG(out, (RecordValue) value.getVariant("dPR_MSG"));
			}
			if (value.getPresentVariantName().equals("dPA_MSG") && !value.getVariant("dPA_MSG").notPresent()) {
				encode_DPA_MSG(out, (RecordValue) value.getVariant("dPA_MSG"));
			}
			
	   	    //Cx interface
			if (value.getPresentVariantName().equals("uAR_MSG") && !value.getVariant("uAR_MSG").notPresent()) {
				encode_UAR_MSG(out, (RecordValue) value.getVariant("uAR_MSG"));
			}
			if (value.getPresentVariantName().equals("uAA_MSG") && !value.getVariant("uAA_MSG").notPresent()) {
				encode_UAA_MSG(out, (RecordValue) value.getVariant("uAA_MSG"));
			}
			if (value.getPresentVariantName().equals("sAR_MSG") && !value.getVariant("sAR_MSG").notPresent()) {
				encode_SAR_MSG(out, (RecordValue) value.getVariant("sAR_MSG"));
			}
			if (value.getPresentVariantName().equals("sAA_MSG") && !value.getVariant("sAA_MSG").notPresent()) {
				encode_SAA_MSG(out, (RecordValue) value.getVariant("sAA_MSG"));
			}
			if (value.getPresentVariantName().equals("lIR_MSG") && !value.getVariant("lIR_MSG").notPresent()) {
				encode_LIR_MSG(out, (RecordValue) value.getVariant("lIR_MSG"));
			}
			if (value.getPresentVariantName().equals("lIA_MSG") && !value.getVariant("lIA_MSG").notPresent()) {
				encode_LIA_MSG(out, (RecordValue) value.getVariant("lIA_MSG"));
			}
			if (value.getPresentVariantName().equals("mAR_MSG") && !value.getVariant("mAR_MSG").notPresent()) {
				encode_MAR_MSG(out, (RecordValue) value.getVariant("mAR_MSG"));
			}
			if (value.getPresentVariantName().equals("mAA_MSG") && !value.getVariant("mAA_MSG").notPresent()) {
				encode_MAA_MSG(out, (RecordValue) value.getVariant("mAA_MSG"));
			}
			if (value.getPresentVariantName().equals("rTR_MSG") && !value.getVariant("rTR_MSG").notPresent()) {
				encode_RTR_MSG(out, (RecordValue) value.getVariant("rTR_MSG"));
			}
			if (value.getPresentVariantName().equals("rTA_MSG") && !value.getVariant("rTA_MSG").notPresent()) {
				encode_RTA_MSG(out, (RecordValue) value.getVariant("rTA_MSG"));
			}
			if (value.getPresentVariantName().equals("pPR_MSG") && !value.getVariant("pPR_MSG").notPresent()) {
				encode_PPR_MSG(out, (RecordValue) value.getVariant("pPR_MSG"));
			}
			if (value.getPresentVariantName().equals("pPA_MSG") && !value.getVariant("pPA_MSG").notPresent()) {
				encode_PPA_MSG(out, (RecordValue) value.getVariant("pPA_MSG"));
			}
    	        	    
    	    //Sh interface
			if (value.getPresentVariantName().equals("uDR_MSG") && !value.getVariant("uDR_MSG").notPresent()) {
				encode_UDR_MSG(out, (RecordValue) value.getVariant("uDR_MSG"));
			}
			if (value.getPresentVariantName().equals("uDA_MSG") && !value.getVariant("uDA_MSG").notPresent()) {
				encode_UDA_MSG(out, (RecordValue) value.getVariant("uDA_MSG"));
			}
			if (value.getPresentVariantName().equals("pUR_MSG") && !value.getVariant("pUR_MSG").notPresent()) {
				encode_PUR_MSG(out, (RecordValue) value.getVariant("pUR_MSG"));
			}
			if (value.getPresentVariantName().equals("pUA_MSG") && !value.getVariant("pUA_MSG").notPresent()) {
				encode_PUA_MSG(out, (RecordValue) value.getVariant("pUA_MSG"));
			}
			if (value.getPresentVariantName().equals("sNR_MSG") && !value.getVariant("sNR_MSG").notPresent()) {
				encode_SNR_MSG(out, (RecordValue) value.getVariant("sNR_MSG"));
			}
			if (value.getPresentVariantName().equals("sNA_MSG") && !value.getVariant("sNA_MSG").notPresent()) {
				encode_SNA_MSG(out, (RecordValue) value.getVariant("sNA_MSG"));
			}
			if (value.getPresentVariantName().equals("pNR_MSG") && !value.getVariant("pNR_MSG").notPresent()) {
				encode_PNR_MSG(out, (RecordValue) value.getVariant("pNR_MSG"));
			}
			if (value.getPresentVariantName().equals("pNA_MSG") && !value.getVariant("pNA_MSG").notPresent()) {
				encode_PNA_MSG(out, (RecordValue) value.getVariant("pNA_MSG"));
			}
			
			//Rx interface
			if (value.getPresentVariantName().equals("aAR_MSG") && !value.getVariant("aAR_MSG").notPresent()) {
				encode_aAR_MSG(out, (RecordValue) value.getVariant("aAR_MSG"));
			}
			if (value.getPresentVariantName().equals("aAA_MSG") && !value.getVariant("aAA_MSG").notPresent()) {
				encode_aAA_MSG(out, (RecordValue) value.getVariant("aAA_MSG"));
			}		

		    //RFC4006
    	    if (value.getPresentVariantName().equals("cCR_MSG") && !value.getVariant("cCR_MSG").notPresent()) {
				encode_CCR_MSG(out, (RecordValue) value.getVariant("cCR_MSG"));
			}
			if (value.getPresentVariantName().equals("cCA_MSG") && !value.getVariant("cCA_MSG").notPresent()) {
				encode_CCA_MSG(out, (RecordValue) value.getVariant("cCA_MSG"));
			}

            /* Codes on S6a interface - TS 129 272  */
			if (value.getPresentVariantName().equals("uLR_MSG") && !value.getVariant("uLR_MSG").notPresent()) {
				encode_ULR_MSG(out, (RecordValue) value.getVariant("uLR_MSG"));
			}
			if (value.getPresentVariantName().equals("uLA_MSG") && !value.getVariant("uLA_MSG").notPresent()) {
				encode_ULA_MSG(out, (RecordValue) value.getVariant("uLA_MSG"));
			}
			if (value.getPresentVariantName().equals("cLR_MSG") && !value.getVariant("cLR_MSG").notPresent()) {
				encode_CLR_MSG(out, (RecordValue) value.getVariant("cLR_MSG"));
			}
            if (value.getPresentVariantName().equals("cLA_MSG") && !value.getVariant("cLA_MSG").notPresent()) {
				encode_CLA_MSG(out, (RecordValue) value.getVariant("cLA_MSG"));
			}
            if (value.getPresentVariantName().equals("aIR_MSG") && !value.getVariant("aIR_MSG").notPresent()) {
				encode_AIR_MSG(out, (RecordValue) value.getVariant("aIR_MSG"));
			}
            if (value.getPresentVariantName().equals("aIA_MSG") && !value.getVariant("aIA_MSG").notPresent()) {
				encode_AIA_MSG(out, (RecordValue) value.getVariant("aIA_MSG"));
			}
            if (value.getPresentVariantName().equals("iDR_MSG") && !value.getVariant("iDR_MSG").notPresent()) {
				encode_IDR_MSG(out, (RecordValue) value.getVariant("iDR_MSG"));
			}
            if (value.getPresentVariantName().equals("iDA_MSG") && !value.getVariant("iDA_MSG").notPresent()) {
				encode_IDA_MSG(out, (RecordValue) value.getVariant("iDA_MSG"));
			}
            if (value.getPresentVariantName().equals("dSR_MSG") && !value.getVariant("dSR_MSG").notPresent()) {
				encode_DSR_MSG(out, (RecordValue) value.getVariant("dSR_MSG"));
			}
            if (value.getPresentVariantName().equals("dSA_MSG") && !value.getVariant("dSA_MSG").notPresent()) {
				encode_DSA_MSG(out, (RecordValue) value.getVariant("dSA_MSG"));
			}
            if (value.getPresentVariantName().equals("pUER_MSG") && !value.getVariant("pUER_MSG").notPresent()) {
				encode_PUER_MSG(out, (RecordValue) value.getVariant("pUER_MSG"));
			}
            if (value.getPresentVariantName().equals("pUEA_MSG") && !value.getVariant("pUEA_MSG").notPresent()) {
				encode_PUEA_MSG(out, (RecordValue) value.getVariant("pUEA_MSG"));
			}
            if (value.getPresentVariantName().equals("rSR_MSG") && !value.getVariant("rSR_MSG").notPresent()) {
				encode_RSR_MSG(out, (RecordValue) value.getVariant("rSR_MSG"));
			}
            if (value.getPresentVariantName().equals("rSA_MSG") && !value.getVariant("rSA_MSG").notPresent()) {
				encode_RSA_MSG(out, (RecordValue) value.getVariant("rSA_MSG"));
			}
            if (value.getPresentVariantName().equals("nOR_MSG") && !value.getVariant("nOR_MSG").notPresent()) {
				encode_NOR_MSG(out, (RecordValue) value.getVariant("nOR_MSG"));
			}
            if (value.getPresentVariantName().equals("nOA_MSG") && !value.getVariant("nOA_MSG").notPresent()) {
				encode_NOA_MSG(out, (RecordValue) value.getVariant("nOA_MSG"));
			}
			
			if (value.getPresentVariantName().equals("uNKNOWN_MSG") && !value.getVariant("uNKNOWN_MSG").notPresent()) {
				//encode_UNKNOWN_MSG(out, (RecordValue) value.getVariant("uNKNOWN_MSG"));
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}	
		return new ForeignMessage(RB, out.toByteArray());
  } 
	
	/*Encoding Of Message Types*/
	private void encode_CER_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_CER_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		
		RecordValue Body = (RecordValue)value.getField("cER_Body");
		
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
        encode_Host_IP_Address_AVP(out, (RecordOfValue)Body.getField("host_IP_Address"));
		encode_Vendor_Id_AVP(out, (RecordValue)Body.getField("vendor_Id"));
		encode_Product_Name_AVP(out,(RecordValue)Body.getField("product_Name"));
		//encode_Origin_State_Id_AVP(out,(RecordOfValue)Body.getField("origin_State_Id"));
		encode_Origin_State_Id_AVP(out,(RecordValue)Body.getField("origin_State_Id"));
		encode_Supported_Vendor_Id_AVP(out,(RecordOfValue)Body.getField("supported_Vendor_Id"));
		encode_Auth_Application_Id_AVP(out,(RecordOfValue)Body.getField("auth_Application_Id"));
		encode_Inband_Security_Id_AVP(out,(RecordOfValue)Body.getField("inband_Security_Id"));
		encode_Acct_Application_Id_AVP(out,(RecordOfValue)Body.getField("acct_Application_Id"));
		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordOfValue)Body.getField("vendor_Specific_Application_Id"));
		encode_Firmware_Revision_AVP(out,(RecordValue)Body.getField("firmware_Revision"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_CEA_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_CEA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		
		RecordValue Body = (RecordValue)value.getField("cEA_Body");
		
		encode_Result_Code_AVP(out, (RecordValue)Body.getField("result_Code"));
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
        encode_Host_IP_Address_AVP(out, (RecordOfValue)Body.getField("host_IP_Address"));
		encode_Vendor_Id_AVP(out, (RecordValue)Body.getField("vendor_Id"));		
		encode_Product_Name_AVP(out,(RecordValue)Body.getField("product_Name"));
		encode_Origin_State_Id_AVP(out,(RecordValue)Body.getField("origin_State_Id"));
		encode_Error_Message_AVP(out,(RecordValue)Body.getField("error_Message"));
		encode_Failed_AVP(out,(RecordOfValue)Body.getField("failed"));
		encode_Supported_Vendor_Id_AVP(out,(RecordOfValue)Body.getField("supported_Vendor_Id"));
		encode_Auth_Application_Id_AVP(out,(RecordOfValue)Body.getField("auth_Application_Id"));
		encode_Inband_Security_Id_AVP(out,(RecordOfValue)Body.getField("inband_Security_Id"));
		encode_Acct_Application_Id_AVP(out,(RecordOfValue)Body.getField("acct_Application_Id"));
		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordOfValue)Body.getField("vendor_Specific_Application_Id"));	
		encode_Firmware_Revision_AVP(out,(RecordValue)Body.getField("firmware_Revision"));
	
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));		
	}

	private void encode_RAR_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_RAR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("rAR_Body");
		
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
		encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
		encode_Auth_Application_Id_AVP(out,(RecordValue)Body.getField("auth_Application_Id"));
		//Rx encode_Specific_Action_AVP(out,(RecordOfValue)Body.getField("specific_action"));
		encode_Re_Auth_Request_Type_AVP(out,(RecordValue)Body.getField("re_Auth_Request_Type"));
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));
		//Rx encode_Access_Network_Charging_Identifier_AVP(out,(RecordOfValue)Body.getField("acc_nc_ident"));
		//Rx encode_Access_Network_Charging_Address_AVP(out, (RecordValue)Body.getField("acc_nc_addr"));
		//Rx encode_Flows_AVP(out,(RecordOfValue)Body.getField("flows"));
		//Rx encode_Subscription_Id_AVP(out,(RecordOfValue)Body.getField("subscription_Id"));
		//Rx encode_Abort_Cause_AVP(out, (RecordValue)Body.getField("abort_Cause"));
		//Rx encode_IP_CAN_Type_AVP(out, (RecordValue)Body.getField("ip_Can"));
		//Rx encode_RAT_Type_AVP(out, (RecordValue)Body.getField("rat"));
		//Rx encode_Sponsored_Connectivity_Data_AVP(out, (RecordValue)Body.getField("sponsored_Conectivity_Data"));
	    encode_Origin_State_Id_AVP(out, (RecordValue)Body.getField("origin_State_Id"));
	    encode_Class_AVP(out,(RecordOfValue)Body.getField("class"));
	    encode_Session_Release_Cause_AVP(out, (RecordValue)Body.getField("session_Release_Cause"));// optional,    // TS 129 212 & TS129 215
//        set of Event_Trigger_AVP               event_Trigger optional,            // TS 129 212 & TS129 215
//        Event_Report_Indication_AVP            event_Report_Indication optional,  // TS 129 212
//        set of Charging_Rule_Remove_AVP        charging_Rule_Remove optional,     // TS 129 212
//        set of Charging_Rule_Install_AVP       charging_Rule_Install optional,    // TS 129 212
//        Default_EPS_Bearer_QoS_AVP             default_EPS_Bearer_QoS optional,   // TS 129 212
        encode_QoS_Information_AVP(out, (RecordOfValue)Body.getField("qoS_Information"));// optional,          // TS 129 212 & TS129 215
//        Revalidation_Time_AVP                  revalidation_Time optional,        // TS 129 212
//        set of Usage_Monitoring_Information_AVP  usage_Monitoring_Information optional,    // TS 129 212
        encode_QoS_Rule_Install_AVP(out, (RecordOfValue)Body.getField("qoS_Rule_Install"));// optional,         //S9 (TS129 215) only
//        set of QoS_Rule_Remove_AVP             qoS_Rule_Remove optional,          //S9 (TS129 215) only
        encode_Subses_Decision_Info_AVP(out, (RecordOfValue)Body.getField("subsession_Decision_Info"));// optional, //S9 (TS129 215) only
        encode_AN_GW_Addr_AVP(out, (RecordOfValue)Body.getField("aN_GW_Address"));//0..2 optional,            //S9 (TS129 215) only
        encode_Multiple_BBERF_Action_AVP(out, (RecordValue)Body.getField("multiple_BBERF_Action"));// optional,    //S9 (TS129 215) only
	    encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));
	    encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record"));
	    
	    encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_RAA_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_RAA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("rAA_Body");
		
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));
		//Rx encode_Media_Component_Description_AVP(out,(RecordOfValue)Body.getField(" media_Component_Description"));
		//Rx encode_Service_URN_AVP(out,(RecordValue)Body.getField("service_URN"));
		encode_Origin_State_Id_AVP(out,(RecordValue)Body.getField("origin_State_Id"));
        encode_Class_AVP(out,(RecordOfValue)Body.getField("class"));
//        IP_CAN_Type_AVP                        iP_CAN_Type optional,              //TS 129 212 & TS129 215
//        RAT_Type_AVP                           rAT_Type optional,                 //TS 129 212 & TS129 215
        encode_AN_GW_Addr_AVP(out, (RecordOfValue)Body.getField("aN_GW_Address"));// optional,      //TS 129 212 & TS129 215
//        ThreeGPP_SGSN_MCC_MNC_AVP              threeGPP_SGSN_MCC_MNC optional,    //TS 129 212 & TS129 215
//        ThreeGPP_SGSN_Address_AVP              threeGPP_SGSN_Address optional,    //TS 129 212 & TS129 215
//        ThreeGPP_SGSN_IPv6_Address_AVP         threeGPP_SGSN_IPv6_Address optional,//TS 129 212 & TS129 215
//        RAI_AVP                                rAI optional,                      //TS 129 212 & TS129 215
//        ThreeGPP_User_Location_Info_AVP        threeGPP_User_Location_Info optional,//TS 129 212 & TS129 215
//        ThreeGPP_MS_TimeZone_AVP               threeGPP_MS_TimeZone optional,     //TS 129 212 & TS129 215
//        set of Charging_Rule_Report_AVP        charging_Rule_Report optional,     //TS 129 212
        encode_Error_Message_AVP(out,(RecordValue)Body.getField("error_Message"));
		encode_Error_Reporting_Host_AVP(out,(RecordValue)Body.getField("error_Reporting_Host"));
		encode_Failed_AVP(out,(RecordOfValue)Body.getField("failed"));
		encode_Redirect_Host_AVP(out,(RecordOfValue)Body.getField("redirect_Host"));
        encode_Redirect_Host_Usage_AVP(out,(RecordValue)Body.getField("redirect_Host_Usage"));
        encode_Redirect_Max_Cache_Time_AVP(out,(RecordValue)Body.getField("redirect_Max_Cache_Time"));
//        Framed_IP_Address_AVP                  framed_IP_Address optional,        //S9 (TS129 215) only
//        Framed_IPv6_Prefix_AVP                 framed_IPv6_Prefix optional,       //S9 (TS129 215) only
//        ThreeGPP2_BSID_AVP                     threeGPP2_BSID optional,           //S9 (TS129 215) only
        encode_QoS_Information_AVP(out, (RecordValue)Body.getField("qoS_Information"));// optional,          //S9 (TS129 215) only
        encode_QoS_Rule_Report_AVP(out, (RecordOfValue)Body.getField("qoS_Rule_Report"));// optional,          //S9 (TS129 215) only
        encode_Subses_Enforcement_Info_AVP(out, (RecordOfValue)Body.getField("subsesion_Enforcement_Info"));// optional, //S9 (TS129 215) only
//        User_CSG_Information_AVP               user_CSG_Information optional,     //S9 (TS129 215) only
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type")); 	
	}

	private void encode_ACR_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_ACR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));

		RecordValue Body = (RecordValue)value.getField("aCR_Body");
		
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
		encode_Accounting_Record_Type_AVP(out, (RecordValue)Body.getField("accounting_Record_Type"));
		encode_Accounting_Record_Number_AVP(out, (RecordValue)Body.getField("accounting_Record_Number"));
		encode_Acct_Application_Id_AVP(out,(RecordValue)Body.getField("acct_Application_Id"));
		encode_Vendor_Specific_Appl_Id_AVP(out, (RecordValue)Body.getField("vendor_Specific_Application_Id"));
		encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
		//Sh encode_Supported_Features_AVP(out, (RecordOfValue)Body.getField("supported_Features"));
		encode_Accounting_Sub_Session_Id_AVP(out, (RecordValue)Body.getField("accounting_Sub_Session_Id"));
		encode_Acct_Session_Id_AVP(out, (RecordValue)Body.getField("acct_Session_Id"));
		encode_Acct_Multi_Session_Id_AVP(out, (RecordValue)Body.getField("acct_Multi_Session_Id"));
		encode_Acct_Interim_Interval_AVP(out, (RecordValue)Body.getField("acct_Interim_Interval"));
		encode_Accounting_Realtime_Required_AVP(out, (RecordValue)Body.getField("accounting_Realtime_Required"));
		encode_Origin_State_Id_AVP(out, (RecordValue)Body.getField("origin_State_Id"));
		encode_Event_Timestamp_AVP(out, (RecordValue)Body.getField("event_Timestamp"));
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		encode_Service_Context_Id_AVP(out, (RecordValue)Body.getField("service_Context_Id"));
        encode_Service_Information_AVP(out, (RecordValue)Body.getField("service_Information"));

		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}

	private void encode_ACA_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_ACA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("aCA_Body");

		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Accounting_Record_Type_AVP(out, (RecordValue)Body.getField("accounting_Record_Type"));
		encode_Accounting_Record_Number_AVP(out, (RecordValue)Body.getField("accounting_Record_Number"));
		encode_Acct_Application_Id_AVP(out,(RecordValue)Body.getField("acct_Application_Id"));
		encode_Vendor_Specific_Appl_Id_AVP(out, (RecordValue)Body.getField("vendor_Specific_Application_Id"));
		encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
		encode_Accounting_Sub_Session_Id_AVP(out, (RecordValue)Body.getField("accounting_Sub_Session_Id"));
		encode_Acct_Session_Id_AVP(out, (RecordValue)Body.getField("acct_Session_Id"));
		encode_Acct_Multi_Session_Id_AVP(out, (RecordValue)Body.getField("acct_Multi_Session_Id"));
		encode_Error_Reporting_Host_AVP(out,(RecordValue)Body.getField("error_Reporting_Host"));
		encode_Acct_Interim_Interval_AVP(out, (RecordValue)Body.getField("acct_Interim_Interval"));
		encode_Accounting_Realtime_Required_AVP(out, (RecordValue)Body.getField("accounting_Realtime_Required"));
		encode_Origin_State_Id_AVP(out, (RecordValue)Body.getField("origin_State_Id"));
		encode_Event_Timestamp_AVP(out, (RecordValue)Body.getField("event_Timestamp"));
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));		
	}

	private void encode_ASR_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_ASR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("aSR_Body");

		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
		encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
		encode_Auth_Application_Id_AVP(out,(RecordValue)Body.getField("auth_Application_Id"));
		//Rx encode_Abort_Cause_AVP(out,(RecordValue)Body.getField("abort_cause"));
		encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
		encode_Origin_State_Id_AVP(out, (RecordValue)Body.getField("origin_State_Id"));
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_ASA_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_ASA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("aSA_Body");
		
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
		encode_Origin_State_Id_AVP(out, (RecordValue)Body.getField("origin_State_Id"));
		encode_Error_Message_AVP(out, (RecordValue)Body.getField("error_Message"));
		encode_Error_Reporting_Host_AVP(out, (RecordValue)Body.getField("error_Reporting_Host"));
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));
		encode_Redirect_Host_AVP(out, (RecordOfValue)Body.getField("redirect_Host"));
		encode_Redirect_Host_Usage_AVP(out, (RecordValue)Body.getField("redirect_Host_Usage"));
		encode_Redirect_Max_Cache_Time_AVP(out, (RecordValue)Body.getField("redirect_Max_Cache_Time"));
        encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_STR_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_STR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("sTR_Body");

		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
		encode_Auth_Application_Id_AVP(out,(RecordValue)Body.getField("auth_Application_Id"));
		encode_Termination_Cause_AVP(out,(RecordValue)Body.getField("termination_Cause"));
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));
		encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));
		encode_Class_AVP(out,(RecordOfValue)Body.getField("class"));
		encode_Origin_State_Id_AVP(out,(RecordValue)Body.getField("origin_State_Id"));
		encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("poute_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_STA_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_STA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("sTA_Body");

		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));
		encode_Class_AVP(out,(RecordOfValue)Body.getField("class"));
		encode_Error_Message_AVP(out, (RecordValue)Body.getField("error_Message"));
		encode_Error_Reporting_Host_AVP(out, (RecordValue)Body.getField("error_Reporting_Host"));
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));
		//Rx encode_Sponsored_Connectivity_Data_AVP(out, (RecordValue)Body.getField("sponsored_Connectivity_Data"));
		encode_Origin_State_Id_AVP(out,(RecordValue)Body.getField("origin_State_Id"));
		encode_Redirect_Host_AVP(out, (RecordOfValue)Body.getField("redirect_Host"));
		encode_Redirect_Host_Usage_AVP(out, (RecordValue)Body.getField("redirect_Host_Usage"));
		encode_Redirect_Max_Cache_Time_AVP(out, (RecordValue)Body.getField("redirect_Max_Cache_Time"));
        encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}

	private void encode_DWR_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_DWR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		
		RecordValue Body = (RecordValue)value.getField("dWR_Body");

		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Origin_State_Id_AVP(out,(RecordValue)Body.getField("origin_State_Id"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_DWA_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_DWA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
	    
		RecordValue Body = (RecordValue)value.getField("dWA_Body");

		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		
		encode_Error_Message_AVP(out, (RecordValue)Body.getField("error_Message"));
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));
		encode_Origin_State_Id_AVP(out,(RecordValue)Body.getField("origin_State_Id"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_DPR_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_DPR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		
		RecordValue Body = (RecordValue)value.getField("dPR_Body");

		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Disconnect_Cause_AVP(out,(RecordValue)Body.getField("disconnect_Cause"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_DPA_MSG(ByteArrayOutputStream out, RecordValue value) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_DPA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
	    
		RecordValue Body = (RecordValue)value.getField("dPA_Body");

		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		
		encode_Error_Message_AVP(out, (RecordValue)Body.getField("error_Message"));
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	/*CxDx*/
	private void encode_UAR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_UAR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("uAR_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//         Auth_Application_Id_AVP                auth_Application_Id optional,           //required only RFC4740
         encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
         encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
         encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
         encode_Destination_Realm_AVP(out,(RecordValue)Body.getField("destination_Realm"));//,                      //RFC4740 & TS129 229
//         SIP_AOR_AVP                            sIP_AOR optional,                       //required only inRFC4740
         encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));// optional,              //RFC4740 & TS129 229
         encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));// optional,                     //RFC4740 & required in TS129 229
//         SIP_Visited_Network_Id_AVP             sIP_Visited_Network_Id optional,        //required only in RFC4740
//         SIP_User_Authorization_Type_AVP        sIP_User_Authorization_Type optional,   //required only in RFC4740
         encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only in TS129 229
         encode_Public_Identity_AVP(out,(RecordValue)Body.getField("public_Identity"));// optional,               //required only in TS129 229
         encode_Visited_Network_Identifier_AVP(out,(RecordValue)Body.getField("visited_Network_Identifier"));// optional,            //required only in TS129 229
         encode_User_Authorization_Type_AVP(out,(RecordValue)Body.getField("user_Authorization_Type"));// optional,       //only in TS129 229
         encode_UAR_Flags_AVP(out,(RecordValue)Body.getField("uAR_Flags"));// optional,                     //only in TS129 229
         encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
         encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
         encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_UAA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_UAA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("uAA_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));// optional,                   //RFC4740 & TS129 229
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,           //only TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //required in RFC4740 & TS129 229
//        SIP_Server_URI_AVP                     sIP_Server_URI optional,                //RFC4740
//        SIP_Server_Capabilities_AVP            sIP_Server_Capabilities optional,       //RFC4740
//        Authorization_Lifetime_AVP             authorization_Lifetime optional,        //RFC4740
//        Auth_Grace_Period_AVP                  auth_Grace_Period optional,             //RFC4740
        encode_Redirect_Host_AVP(out,(RecordValue)Body.getField("redirect_Host"));// optional,                 //RFC4740
//        Redirect_Host_Usage_AVP                redirect_Host_Usage optional,           //RFC4740
//        Redirect_Max_Cache_Time_AVP            redirect_Max_Cache_Time optional,       //RFC4740
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
		encode_Server_Name_AVP(out,(RecordValue)Body.getField("server_Name"));// optional,                   //only TS129 229
		encode_Server_Capabilities_AVP(out,(RecordValue)Body.getField("server_Capabilities"));// optional,           //only TS129 229
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));//optional,                        //only TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_SAR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_SAR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("sAR_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740 
        encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
        encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
        encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
        encode_Destination_Realm_AVP(out,(RecordValue)Body.getField("destination_Realm"));//,                      //RFC4740 & TS129 229
//        SIP_Server_Assignment_Type_AVP         sIP_Server_Assignment_Type optional,    //required only in RFC4740 
//        SIP_User_Data_Already_Available_AVP    sIP_User_Data_Already_Available optional,//required only in RFC4740 
        encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));// optional,              //RFC4740 & TS129 229
        encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));// optional,                     //RFC4740 & TS129 229
//        SIP_Server_URI_AVP                     sIP_Server_URI optional,                //only in RFC4740 
//        set of SIP_Supported_User_Data_Type_AVP sIP_Supported_User_Data_Type optional, //only in RFC4740 
//        set of SIP_AOR_AVP                     sIP_AOR optional,                       //only in RFC4740
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only in TS129 229
        encode_Public_Identity_AVP(out,(RecordOfValue)Body.getField("public_Identity"));// optional,               //only in TS129 229
//        Wildcarded_Public_Identity_AVP         wildcarded_Public_Identity optional,    //only in TS129 229
        encode_Server_Name_AVP(out,(RecordValue)Body.getField("server_Name"));// optional,                   //required only in TS129 229
        encode_Server_Assignment_Type_AVP(out,(RecordValue)Body.getField("server_Assignment_Type"));// optional,        //required only in TS129 229
        encode_User_Data_Already_Available_AVP(out,(RecordValue)Body.getField("user_Data_Already_Available"));// optional,   //required only in TS129 229
//        SCSCF_Restoration_Info_AVP             sCSCF_Restoration_Info optional,        //only in TS129 229
//        Multiple_Registration_Indication_AVP   multiple_Registration_Indication optional,//only in TS129 229
//        Session_Priority_AVP                   session_Priority optional,              //only in TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_SAA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_SAA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("sAA_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740 
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));// optional,                   //RFC4740 & TS129 229
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,           //only TS129 229
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
//        set of SIP_User_Data_AVP               sIP_User_Data optional,                 //required only in RFC4740
//        SIP_Accounting_Information_AVP         sIP_Accounting_Information optional,    //required only in RFC4740
//        set of SIP_Supported_User_Data_Type_AVP sIP_Supported_User_Data_Type optional, //only in RFC4740
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));// optional,                     //RFC4740 & TS129 229
//        Auth_Grace_Period_AVP                  auth_Grace_Period optional,             //only in RFC4740
//        Authorization_Lifetime_AVP             authorization_Lifetime optional,        //only in RFC4740
		encode_Redirect_Host_AVP(out,(RecordValue)Body.getField("redirect_Host"));// optional,                 //only in RFC4740
//        Redirect_Host_Usage_AVP                redirect_Host_Usage optional,           //only in RFC4740
//        Redirect_Max_Cache_Time_AVP            redirect_Max_Cache_Time optional,       //only in RFC4740
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
		encode_User_Data_AVP(out,(RecordValue)Body.getField("user_Data"));// optional,                     //only TS129 229
	    encode_Charging_Information_AVP(out,(RecordValue)Body.getField("charging_Information"));// optional,          //only TS129 229
//        Associated_Identities_AVP              associated_Identities optional,         //only TS129 229
//        Loose_Route_Indication_AVP             loose_Route_Indication optional,        //only TS129 229
		//TODO:....encode_SCSCF_Restoration_Info_AVP(out,(RecordOfValue)Body.getField("sCSCF_Restoration_Info"));// optional,        //only TS129 229
//        Associated_Registered_Identities_AVP   associated_Registered_Identities optional, //only TS129 229
        encode_Server_Name_AVP(out,(RecordValue)Body.getField("server_Name"));// optional,                   //only TS129 229
//        Wildcarded_Public_Identity_AVP         wildcarded_Public_Identity optional,    //only TS129 229
//        Priviledged_Sender_Indication_AVP      priviledged_Sender_Indication optional, //only TS129 229
        encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));// optional,                        //only TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_LIR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_LIR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("lIR_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740 
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
		encode_Destination_Realm_AVP(out,(RecordValue)Body.getField("destination_Realm"));//,                      //RFC4740 & TS129 229
//        SIP_AOR_AVP                            sIP_AOR optional,                       //required only in RFC4740
		encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));// optional,              //RFC4740 & TS129 229
		encode_Originating_Request_AVP(out,(RecordValue)Body.getField("originating_Request"));// optional,           //only TS129 229
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
		encode_Public_Identity_AVP(out,(RecordValue)Body.getField("public_Identity"));// optional,               //required only in TS129 229
		encode_User_Authorization_Type_AVP(out,(RecordValue)Body.getField("user_Authorization_Type"));// optional,       //only TS129 229
//        Session_Priority_AVP                   session_Priority optional,              //only TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_LIA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_LIA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("lIA_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740 
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));// optional,                   //RFC4740 & TS129 229
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,           //only TS129 229
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
//        SIP_Server_URI_AVP                     sIP_Server_URI optional,                //only in RFC4740
//        SIP_Server_Capabilities_AVP            sIP_Server_Capabilities optional,       //only in RFC4740
//        Auth_Grace_Period_AVP                  auth_Grace_Period optional,             //only in RFC4740
//        Authorization_Lifetime_AVP             authorization_Lifetime optional,        //only in RFC4740
		encode_Redirect_Host_AVP(out,(RecordValue)Body.getField("redirect_Host"));// optional,                 //only in RFC4740
//        Redirect_Host_Usage_AVP                redirect_Host_Usage optional,           //only in RFC4740
//        Redirect_Max_Cache_Time_AVP            redirect_Max_Cache_Time optional,       //only in RFC4740
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
		encode_Server_Name_AVP(out,(RecordValue)Body.getField("server_Name"));// optional,                   //only TS129 229
		encode_Server_Capabilities_AVP(out,(RecordValue)Body.getField("server_Capabilities"));// optional,           //only TS129 229
//        Wildcarded_Public_Identity_AVP         wildcarded_Public_Identity optional,    //only TS129 229
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));//optional,                        //only TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}

	private void encode_MAR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_MAR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("mAR_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740 
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
		encode_Destination_Realm_AVP(out,(RecordValue)Body.getField("destination_Realm"));//,                      //RFC4740 & TS129 229
//        SIP_AOR_AVP                            sIP_AOR optional,                       //required in RFC4740
//        SIP_Method_AVP                         sIP_Method optional,                    //required in RFC4740
		encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));// optional,              //RFC4740 & TS129 229
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));// optional,                     //RFC4740 & required in TS129 229
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
		encode_Public_Identity_AVP(out,(RecordValue)Body.getField("public_Identity"));// optional,               //required only in TS129 229
//        SIP_Server_URI_AVP                     sIP_Server_URI optional,                //only in RFC4740
		encode_SIP_Number_Auth_Items_AVP(out,(RecordValue)Body.getField("sIP_Number_Auth_Items"));// optional,         //RFC4740 & required in TS129 229
		encode_SIP_Auth_Data_Item_AVP(out,(UnionValue)Body.getField("sIP_Auth_Data_Item"));// optional,            //required in TS129 229
        encode_Server_Name_AVP(out,(RecordValue)Body.getField("server_Name"));// optional,                   //only TS129 229
        encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));// optional,                        //only TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_MAA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_MAA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("mAA_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740 
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));// optional,                   //RFC4740 & TS129 229
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,           //only TS129 229
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));// optional,                     //RFC4740 & TS129 229
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
        encode_Public_Identity_AVP(out,(RecordValue)Body.getField("public_Identity"));// optional,               //required only in TS129 229
//        SIP_AOR_AVP                            sIP_AOR optional,                       //only in RFC4740
		encode_SIP_Number_Auth_Items_AVP(out,(RecordValue)Body.getField("sIP_Number_Auth_Items"));// optional,         //RFC4740 & TS129 229
		encode_SIP_Auth_Data_Item_AVP(out,(RecordOfValue)Body.getField("sIP_Auth_Data_Item"));// optional,            //TS129 229
//        Auth_Grace_Period_AVP                  auth_Grace_Period optional,             //only in RFC4740
//        Authorization_Lifetime_AVP             authorization_Lifetime optional,        //only in RFC4740
//        Redirect_Host_AVP                      redirect_Host optional,                 //only in RFC4740
//        Redirect_Host_Usage_AVP                redirect_Host_Usage optional,           //only in RFC4740
//        Redirect_Max_Cache_Time_AVP            redirect_Max_Cache_Time optional,       //only in RFC4740
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));// optional,                        //only TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_Server_Name_AVP(out,(RecordValue)Body.getField("server_Name"));// optional,                  //only TS129 229
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_RTR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_RTR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("rTR_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740 
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
		encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));//,                       //RFC4740 & TS129 229
		encode_Destination_Realm_AVP(out,(RecordValue)Body.getField("destination_Realm"));//,                      //RFC4740 & TS129 229
//        SIP_Deregistration_Reason_AVP          sIP_Deregistration_Reason optional,     //required only in RFC4740
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));// optional,                     //RFC4740 & required in TS129 229
//        set of SIP_AOR_AVP                     sIP_AOR optional,                       //only in RFC4740
//        Associated_Identities_AVP              associated_Identities optional,         //only TS129 229
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
        encode_Public_Identity_AVP(out,(RecordOfValue)Body.getField("public_Identity"));// optional,               //only TS129 229
		encode_Deregistration_Reason_AVP(out,(RecordValue)Body.getField("deregistration_Reason"));// optional,         //required only in TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_RTA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_RTA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("rTA_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740 
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));// optional,                   //RFC4740 & TS129 229
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,           //only TS129 229
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
//        Authorization_Lifetime_AVP             authorization_Lifetime optional,        //only in RFC4740
//        Auth_Grace_Period_AVP                  auth_Grace_Period optional,             //only in RFC4740
		encode_Redirect_Host_AVP(out,(RecordValue)Body.getField("redirect_Host"));// optional,                 //only in RFC4740
//        Redirect_Host_Usage_AVP                redirect_Host_Usage optional,           //only in RFC4740
//        Redirect_Max_Cache_Time_AVP            redirect_Max_Cache_Time optional,       //only in RFC4740
//        Associated_Identities_AVP              associated_Identities optional,         //only TS129 229
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
        encode_Identity_with_Emergency_Registration_AVP(out,(RecordOfValue)Body.getField("identity_with_Emergency_Registration"));// optional,  //only TS129 229
        encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));// optional,                        //only TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_PPR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_PPR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		RecordValue Body = (RecordValue)value.getField("pPR_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740 
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
		encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));//,                       //RFC4740 & TS129 229
		encode_Destination_Realm_AVP(out,(RecordValue)Body.getField("destination_Realm"));//,                      //RFC4740 & TS129 229
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));// ,                             //RFC4740 & TS129 229
//        set of SIP_User_Data_AVP               sIP_User_Data optional,                 //only in RFC4740
//        SIP_Accounting_Information_AVP         sIP_Accounting_Information optional,    //only in RFC4740
//        Authorization_Lifetime_AVP             authorization_Lifetime optional,        //only in RFC4740
//        Auth_Grace_Period_AVP                  auth_Grace_Period optional,             //only in RFC4740
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
		encode_User_Data_AVP(out,(RecordValue)Body.getField("user_Data"));// optional,                     //only TS129 229
		encode_Charging_Information_AVP(out,(RecordValue)Body.getField("charging_Information"));// optional,          //only TS129 229
		encode_SIP_Auth_Data_Item_AVP(out,(UnionValue)Body.getField("sIP_Auth_Data_Item"));// optional,            //only TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_PPA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_PPA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("pPA_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
//        Auth_Application_Id_AVP                auth_Application_Id optional,           //required only in RFC4740 
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));// optional,                   //RFC4740 & TS129 229
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,           //only TS129 229
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
		encode_Redirect_Host_AVP(out,(RecordValue)Body.getField("redirect_Host"));// optional,                 //only in RFC4740
//        Redirect_Host_Usage_AVP                redirect_Host_Usage optional,           //only in RFC4740
//        Redirect_Max_Cache_Time_AVP            redirect_Max_Cache_Time optional,       //only in RFC4740
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
        encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));// optional,                        //only TS129 229
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info")); //optional,                    //RFC4740
        encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record")); //optional,                  //RFC4740
        encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	/*Sh*/
	private void encode_UDR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_UDR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("uDR_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional,//required only in TS129 229
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//,                     //RFC4740 & TS129 229
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//,                            //RFC4740 & TS129 229
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//,                           //RFC4740 & TS129 229
		encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));//,                       //RFC4740 & TS129 229
		encode_Destination_Realm_AVP(out,(RecordValue)Body.getField("destination_Realm"));//,                      //RFC4740 & TS129 229
		encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,            //only TS129 229
        encode_User_Identity_AVP(out,(RecordValue)Body.getField("user_Identity"));//requi.
		encode_Wildcarded_PSI_AVP(out,(RecordValue)Body.getField("wildcarded_PSI"));//optional,
		encode_Wildcarded_IMPU_AVP(out,(RecordValue)Body.getField("wildcarded_IMPU"));// optional,
		encode_Server_Name_AVP(out,(RecordValue)Body.getField("server_Name"));// optional,
		encode_Service_Indication_AVP(out,(RecordOfValue)Body.getField("service_Indication"));// optional,
		encode_Data_Reference_AVP(out,(RecordOfValue)Body.getField("data_Reference"));//requi.
		encode_Identity_Set_AVP(out,(RecordOfValue)Body.getField("identity_Set"));// optional,
		encode_Requested_Domain_AVP(out,(RecordValue)Body.getField("requested_Domain"));// optional,
		encode_Current_Location_AVP(out,(RecordValue)Body.getField("current_Location"));// optional,
		encode_DSAI_Tag_AVP(out,(RecordOfValue)Body.getField("dSAI_Tag"));// optional,
		encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));// optional,
		encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record"));// optional,
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));// optional
	}
	
	private void encode_UDA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_UDA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("uDA_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));//optional
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//requi.
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//requi.
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//requi.
		encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,
		encode_Wildcarded_PSI_AVP(out,(RecordValue)Body.getField("wildcarded_PSI"));// optional,
		encode_Wildcarded_IMPU_AVP(out,(RecordValue)Body.getField("wildcarded_IMPU"));// optional,
		encode_User_Data_AVP(out,(RecordValue)Body.getField("user_Data"));// optional,
		encode_Redirect_Host_AVP(out,(RecordValue)Body.getField("redirect_Host"));// optional,                 //RFC4740
		encode_Redirect_Host_Usage_AVP(out,(RecordValue)Body.getField("redirect_Host_Usage"));// optional,           //RFC4740
		encode_Redirect_Max_Cache_Time_AVP(out,(RecordValue)Body.getField("redirect_Max_Cache_Time"));// optional,       //RFC4740
		encode_Failed_AVP(out,(RecordOfValue)Body.getField("failed"));// optional,
		encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));// optional,
		encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record"));// optional,
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));// optional
	}

	private void encode_PUR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_PUR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("pUR_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//requi.
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//requi.
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//requi.
		encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));//,                       //RFC4740 & TS129 229
		encode_Destination_Realm_AVP(out,(RecordValue)Body.getField("destination_Realm"));//,                      //RFC4740 & TS129 229
		encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,
		encode_User_Identity_AVP(out,(RecordValue)Body.getField("user_Identity"));//requi.
		encode_Wildcarded_PSI_AVP(out,(RecordValue)Body.getField("wildcarded_PSI"));//optional,
		encode_Wildcarded_IMPU_AVP(out,(RecordValue)Body.getField("wildcarded_IMPU"));// optional,
		encode_Data_Reference_AVP(out,(RecordOfValue)Body.getField("data_Reference"));//requi.		
		encode_User_Data_AVP(out,(RecordValue)Body.getField("user_Data"));//Requi.
		encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));// optional,
		encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record"));// optional,
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));// optional
		
	}
	
	private void encode_PUA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_PUA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("pUA_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));//optional
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//requi.
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//requi.
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//requi.
		encode_Wildcarded_PSI_AVP(out,(RecordValue)Body.getField("wildcarded_PSI"));// optional,
		encode_Wildcarded_IMPU_AVP(out,(RecordValue)Body.getField("wildcarded_IMPU"));// optional,
        encode_Repository_Data_ID_AVP(out,(RecordValue)Body.getField("repository_Data_Id"));// optional,
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,
        encode_Redirect_Host_AVP(out,(RecordValue)Body.getField("redirect_Host"));// optional,                 //RFC4740
		encode_Redirect_Host_Usage_AVP(out,(RecordValue)Body.getField("redirect_Host_Usage"));// optional,           //RFC4740
		encode_Redirect_Max_Cache_Time_AVP(out,(RecordValue)Body.getField("redirect_Max_Cache_Time"));// optional,       //RFC4740
		encode_Failed_AVP(out,(RecordOfValue)Body.getField("failed"));// optional,
		encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));// optional,
		encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record"));// optional,
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));// optional
	}
	
	private void encode_SNR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_SNR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("sNR_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//requi.
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//requi.
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//requi.
		encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));//,                       //RFC4740 & TS129 229
		encode_Destination_Realm_AVP(out,(RecordValue)Body.getField("destination_Realm"));//,                      //RFC4740 & TS129 229
		encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,
		encode_User_Identity_AVP(out,(RecordValue)Body.getField("user_Identity"));//requi.
		encode_Wildcarded_PSI_AVP(out,(RecordValue)Body.getField("wildcarded_PSI"));//optional,
		encode_Wildcarded_IMPU_AVP(out,(RecordValue)Body.getField("wildcarded_IMPU"));// optional,
        encode_Service_Indication_AVP(out,(RecordOfValue)Body.getField("service_Indication"));// optional,
        encode_Send_Data_Indication_AVP(out,(RecordValue)Body.getField("send_Data_Indication"));//  optional,
        encode_Server_Name_AVP(out,(RecordValue)Body.getField("server_Name"));// optional,
        encode_Subs_Req_Type_AVP(out,(RecordValue)Body.getField("subs_Req_Type"));
		encode_Data_Reference_AVP(out,(RecordOfValue)Body.getField("data_Reference"));//requi.
		encode_Identity_Set_AVP(out,(RecordOfValue)Body.getField("identity_Set"));// optional,
        encode_Expiry_Time_AVP(out,(RecordValue)Body.getField("expiry_Time")); //optional,
		encode_DSAI_Tag_AVP(out,(RecordOfValue)Body.getField("dSAI_Tag"));// optional,
		encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));// optional,
		encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record"));// optional,
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));// optional
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));// optional                    //RFC4740 & required in TS129 229
	}
	
	private void encode_SNA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_SNA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("sNA_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//requi.
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));//optional
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//requi.
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//requi.
		encode_Wildcarded_PSI_AVP(out,(RecordValue)Body.getField("wildcarded_PSI"));// optional,
		encode_Wildcarded_IMPU_AVP(out,(RecordValue)Body.getField("wildcarded_IMPU"));// optional,
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,
        encode_User_Data_AVP(out,(RecordValue)Body.getField("user_Data"));// optional,
        encode_Expiry_Time_AVP(out,(RecordValue)Body.getField("expiry_Time")); //optional,
        encode_Redirect_Host_AVP(out,(RecordValue)Body.getField("redirect_Host"));// optional,                 //RFC4740
		encode_Redirect_Host_Usage_AVP(out,(RecordValue)Body.getField("redirect_Host_Usage"));// optional,           //RFC4740
		encode_Redirect_Max_Cache_Time_AVP(out,(RecordValue)Body.getField("redirect_Max_Cache_Time"));// optional,       //RFC4740
		encode_Failed_AVP(out,(RecordOfValue)Body.getField("failed"));// optional,
		encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));// optional,
		encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record"));// optional,
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));// optional
	}
	
	private void encode_PNR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_PNR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));

		RecordValue Body = (RecordValue)value.getField("pNR_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//requi.
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//requi.
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//requi.
		encode_Destination_Host_AVP(out,(RecordValue)Body.getField("destination_Host"));//,                       //RFC4740 & TS129 229
		encode_Destination_Realm_AVP(out,(RecordValue)Body.getField("destination_Realm"));//,                      //RFC4740 & TS129 229
		encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,
        encode_User_Identity_AVP(out,(RecordValue)Body.getField("user_Identity"));//requi.
        encode_Wildcarded_PSI_AVP(out,(RecordValue)Body.getField("wildcarded_PSI"));// optional,
      	encode_Wildcarded_IMPU_AVP(out,(RecordValue)Body.getField("wildcarded_IMPU"));// optional,
        encode_User_Data_AVP(out,(RecordValue)Body.getField("user_Data"));// optional,
        encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));// optional,
		encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record"));// optional,
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));// optional
	}
	
	private void encode_PNA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_PNA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("pNA_Body");

		encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));//optional
		encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));//requi.
		encode_Origin_Host_AVP(out,(RecordValue)Body.getField("origin_Host"));//requi.
		encode_Origin_Realm_AVP(out,(RecordValue)Body.getField("origin_Realm"));//requi.
		encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional,
        encode_Redirect_Host_AVP(out,(RecordValue)Body.getField("redirect_Host"));// optional,                 //RFC4740
		encode_Redirect_Host_Usage_AVP(out,(RecordValue)Body.getField("redirect_Host_Usage"));// optional,           //RFC4740
		encode_Redirect_Max_Cache_Time_AVP(out,(RecordValue)Body.getField("redirect_Max_Cache_Time"));// optional,       //RFC4740
		encode_Failed_AVP(out,(RecordOfValue)Body.getField("failed"));// optional,
		encode_Proxy_Info_AVP(out,(RecordOfValue)Body.getField("proxy_Info"));// optional,
		encode_Route_Record_AVP(out,(RecordOfValue)Body.getField("route_Record"));// optional,
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));// optional
		
	}
	
	/*Rx*/
	private void encode_aAR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_AAR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		
		RecordValue Body = (RecordValue)value.getField("aAR_Body");

		//TODO:....
	}
	
	private void encode_aAA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_AAA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		
		RecordValue Body = (RecordValue)value.getField("aAA_Body");

		//TODO:....
	}
	
	/*RFC4006*/
	private void encode_CCR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_CCR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("cCR_Body");

		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));		
				
		encode_Auth_Application_Id_AVP(out,(RecordValue)Body.getField("auth_Application_Id"));
		encode_Service_Context_Id_AVP(out, (RecordValue)Body.getField("service_Context_Id"));
		encode_CC_Request_AVP(out, (RecordValue)Body.getField("cC_Request_Type"));	
		encode_CC_Request_Number_AVP(out, (RecordValue)Body.getField("cC_Request_Number"));
		encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
		encode_User_Name_AVP(out,(RecordValue)Body.getField("user_Name"));
//		CC_Sub_Session_Id_AVP 			cC_Sub_Session_Id optional,			//RFC4006
		encode_Acct_Multi_Session_Id_AVP(out, (RecordValue)Body.getField("acct_Multi_Session_Id"));
		encode_Origin_State_Id_AVP(out,(RecordValue)Body.getField("origin_State_Id"));
		encode_Event_Timestamp_AVP(out, (RecordValue)Body.getField("event_Timestamp"));

		encode_Subscription_Id_AVP(out,(RecordOfValue)Body.getField("subscription_Id"));
		encode_Service_Identifier_AVP(out, (RecordValue)Body.getField("service_Identifier"));// optional,		//RFC4006
		encode_Termination_Cause_AVP(out,(RecordValue)Body.getField("termination_Cause"));
		encode_Requested_Service_Unit_AVP(out,(RecordValue)Body.getField("requested_Service_Unit"));// optional,	//RFC4006
		encode_Requested_Action_AVP(out,(RecordValue)Body.getField("requested_Action"));
		encode_Used_Service_Unit_AVP(out,(RecordOfValue)Body.getField("used_Service_Unit"));// optional,			//RFC4006
		encode_Multiple_Services_Indicator_AVP(out,(RecordValue)Body.getField("multiple_Services_Indicator"));
		encode_Multiple_Services_Credit_Control_AVP(out,(RecordOfValue)Body.getField("multiple_Services_Credit_Control"));
//		set of Service_Parameter_Info_AVP service_Parameter_Info optional,	//RFC4006
//		CC_Correlation_Id_AVP 			cC_Correlation_Id optional,			//RFC4006
		
		encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));// optional"));				//TS129 212 only
//        Network_Request_Support_AVP		network_Request_Support optional,			//TS129 212 only
//        set of Packet_Filter_Information_AVP	packet_Filter_Information optional,	//TS129 212 only
//        Packet_Filter_Operation_AVP		packet_Filter_Operation optional,			//TS129 212 only
//        Bearer_Identifier_AVP			bearer_Identifier optional,					//TS129 212 only
//        Bearer_Operation_AVP			bearer_Operation optional,					//TS129 212 only
        encode_Framed_IP_Address_AVP(out,(RecordValue)Body.getField("framed_IP_Address"));// optional,					//TS129 212 only
        encode_Framed_IPv6_Prefix_AVP(out,(RecordValue)Body.getField("framed_IPv6_Prefix"));// optional,				//TS129 212 only
        encode_IP_CAN_Type_AVP(out,(RecordValue)Body.getField("iP_CAN_Type"));// optional,						//TS129 212 only
//        ThreeGPP_RAT_Type_AVP			threeGPP_RAT_Type optional,					//TS129 212 only
        encode_RAT_Type_AVP(out,(RecordValue)Body.getField("rat_Type"));// optional,							//TS129 212 only
        encode_QoS_Information_AVP(out,(RecordValue)Body.getField("qoS_Information"));// optional,					//TS129 212 only
//        QoS_Negotiation_AVP				qoS_Negotiation optional,					//TS129 212 only
//        QoS_Upgrade_AVP					qoS_Upgrade optional,						//TS129 212 only
//        Default_EPS_Bearer_QoS_AVP		default_EPS_Bearer_QoS optional,			//TS129 212 only
        encode_AN_GW_Addr_AVP(out,(RecordOfValue)Body.getField("aN_GW_Address"));//0..2 optional,			//TS129 212 only
//        ThreeGPP_SGSN_MCC_MNC_AVP		threeGPP_SGSN_MCC_MNC optional,				//TS129 212 only
//        ThreeGPP_SGSN_Address_AVP		threeGPP_SGSN_Address optional,				//TS129 212 only
//        ThreeGPP_SGSN_IPv6_Address_AVP	threeGPP_SGSN_IPv6_Address optional,		//TS129 212 only
//        RAI_AVP							rAI optional,								//TS129 212 only
//        ThreeGPP_User_Location_Info_AVP	threeGPP_User_Location_Info optional,		//TS129 212 only
//        ThreeGPP_MS_TimeZone_AVP		threeGPP_MS_TimeZone optional,				//TS129 212 only
//        Called_Station_Id_AVP			called_Station_Id optional,					//TS129 212 only
//        PDN_Connection_ID_AVP			pDN_Connection_ID optional,					//TS129 212 only
//        Bearer_Usage_AVP				bearer_Usage optional,						//TS129 212 only
//        Online_AVP						online optional,							//TS129 212 only
//        Offline_AVP						offline optional,							//TS129 212 only
//        set of TFT_Packet_Filter_Information_AVP	tFT_Packet_Filter_Information optional,		//TS129 212 only
        encode_Charging_Rule_Report_AVP(out,(RecordOfValue)Body.getField("charging_Rule_Report"));// optional,				//TS129 212 only
        encode_Event_Trigger_AVP(out,(RecordOfValue)Body.getField("event_Trigger"));// optional,						//TS129 212 only
//        Event_Report_Indication_AVP		event_Report_Indication optional,			//TS129 212 only
//        Access_Network_Charging_Address_AVP	access_Network_Charging_Address optional,//TS129 212 only
//        set of Access_Network_Charging_Identifier_Gx_AVP	access_Network_Charging_Identifier_Gx optional,		//TS129 212 only
//        set of CoA_Information_AVP		coA_Information optional,					//TS129 212 only
//        set of Usage_Monitoring_Information_AVP	usage_Monitoring_Information optional,//TS129 212 only
//        Routing_Rule_Install_AVP		routing_Rule_Install optional,				//TS129 212 only
//        Routing_Rule_Remove_AVP			routing_Rule_Remove optional,				//TS129 212 only
//        Maximum_Bandwidth_AVP			maximum_Bandwidth optional,					//TS129 212 only
//        Logical_Access_ID_AVP			logical_Access_ID optional,					//TS129 212 only
//        Physical_Access_ID_AVP			physical_Access_ID optional,				//TS129 212 only
//        Session_Linking_Indicator_AVP          session_Linking_Indicator optional,//Gxx TS129 212 only
        encode_QoS_Rule_Report_AVP(out,(RecordOfValue)Body.getField("qoS_Rule_Report"));// optional,          //S9 (TS129 215) & Gxx (TS129 212)
        encode_Subses_Enforcement_Info_AVP(out,(RecordOfValue)Body.getField("subsesion_Enforcement_Info"));// optional, //S9 (TS129 215) only
//        ThreeGPP2_BSID_AVP                     threeGPP2_BSID optional,           //S9 (TS129 215) & Gxx (TS129 212)
        encode_Multiple_BBERF_Action_AVP(out,(RecordValue)Body.getField("multiple_BBERF_Action"));// optional,    //S9 (TS129 215) only
//        User_CSG_Information_AVP               user_CSG_Information optional,     //S9 (TS129 215) & Gxx (TS129 212)
		
		encode_User_Equipment_Info_AVP(out,(RecordValue)Body.getField("user_Equipment_Info"));
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		encode_Service_Information_AVP(out,(RecordValue)Body.getField("service_Information"));

		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_CCA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_CCA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("cCA_Body");		
		
		encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
		encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Auth_Application_Id_AVP(out,(RecordValue)Body.getField("auth_Application_Id"));
		encode_CC_Request_AVP(out, (RecordValue)Body.getField("cC_Request_Type"));	
		encode_CC_Request_Number_AVP(out, (RecordValue)Body.getField("cC_Request_Number"));
//        User_Name_AVP                          user_Name optional,                //RFC4006
		encode_CC_Session_Failover_AVP(out, (RecordValue)Body.getField("cC_Session_Failover"));
//        CC_Sub_Session_Id_AVP                  cC_Sub_Session_Id optional,        //RFC4006
//        Acct_Multi_Session_Id_AVP              acct_Multi_Session_Id optional,    //RFC4006
//        Origin_State_Id_AVP                    origin_State optional,             //RFC4006 & TS129 212 & TS129 215
//        Event_Timestamp_AVP                    event_Timestamp optional,          //RFC4006
//        Granted_Service_Unit_AVP               granted_Service_Unit optional,     //RFC4006
		encode_Multiple_Services_Credit_Control_AVP(out, (RecordOfValue)Body.getField("multiple_Services_Credit_Control"));
		encode_Cost_Information_AVP(out, (RecordValue)Body.getField("cost_Information"));
//        Final_Unit_Indication_AVP              final_Unit_Indication optional,    //RFC4006
//        Check_Balance_Result_AVP               check_Balance_Result optional,     //RFC4006
//        Low_Balance_Indication_AVP             low_Balance_Indication optional,   //TS132 299 only
		encode_Remaining_Balance_AVP(out, (RecordValue)Body.getField("remaining_Balance"));// optional,        //TS132 299 only
		encode_Credit_Control_Failure_Handling_AVP(out, (RecordValue)Body.getField("credit_Control_Failure_Handling"));
		encode_Direct_Debiting_Failure_Handling_AVP(out, (RecordValue)Body.getField("direct_Debiting_Failure_Handling"));
        encode_Validity_Time_AVP(out, (RecordValue)Body.getField("validity_Time"));// optional,            //RFC4006
		encode_Redirect_Host_AVP(out, (RecordOfValue)Body.getField("redirect_Host"));
		encode_Redirect_Host_Usage_AVP(out, (RecordValue)Body.getField("redirect_Host_Usage"));
		encode_Redirect_Max_Cache_Time_AVP(out, (RecordValue)Body.getField("redirect_Max_Cache_Time"));
		encode_Experimental_Result_AVP(out, (RecordValue)Body.getField("experimental_Result"));// optional,	//TS129 212 only
//        set of Supported_Features_AVP          supported_Features optional,       //TS129 212 & TS129 215
//        Bearer_Control_Mode_AVP                bearer_Control_Mode optional,      //TS129 212 & TS129 215
//        set of Event_Trigger_AVP               event_Trigger optional,            //TS129 212 & TS129 215
//        set of Charging_Rule_Remove_AVP        charging_Rule_Remove optional,     //TS129 212 only
//        set of Charging_Rule_Install_AVP       charging_Rule_Install optional,    //TS129 212 only
//        Charging_Information_AVP               charging_Information optional,     //TS129 212 & TS129 215
//        Online_AVP                             online optional,                   //TS129 212 only
//        Offline_AVP                            offline optional,                  //TS129 212 only
//        set of QoS_Information_AVP             qoS_Information optional,          //TS129 212 & TS129 215
//        Revalidation_Time_AVP                  revalidation_Time optional,        //TS129 212 only
//        Default_EPS_Bearer_QoS_AVP             default_EPS_Bearer_QoS optional,   //TS129 212 only
//        Bearer_Usage_AVP                       bearer_Usage optional,             //TS129 212 only
//        ThreeGPP_User_Location_Info_AVP        threeGPP_User_Location_Info optional,             //TS129 212 only
//        set of Usage_Monitoring_Information_AVP    usage_Monitoring_Information optional,        //TS129 212 only
//        set of CSG_Information_Reporting_AVP   cSG_Information_Reporting optional,//TS129 212 & TS129 215
//        User_CSG_Information_AVP               user_CSG_Information optional,     //TS129 212 only
//        Error_Message_AVP                      error_Message optional,            //TS129 212 & TS129 215
//        Error_Reporting_Host_AVP               error_Reporting_Host optional,     //TS129 212 & TS129 215
		encode_QoS_Rule_Install_AVP(out, (RecordOfValue)Body.getField("qoS_Rule_Install"));// optional,         //S9 (TS129 215) only
//        set of QoS_Rule_Remove_AVP             qoS_Rule_Remove optional,          //S9 (TS129 215) only
		encode_Subses_Decision_Info_AVP(out, (RecordOfValue)Body.getField("subsession_Decision_Info"));// optional, //S9 (TS129 215) only
        encode_AN_GW_Addr_AVP(out, (RecordOfValue)Body.getField("aN_GW_Address"));// optional,
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));
        encode_Service_Information_AVP(out, (RecordValue)Body.getField("service_Information"));// optional,      //TS132 299 only
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}

	/*S6a*/
	private void encode_ULR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_ULR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("uLR_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out, (RecordValue)Body.getField("vendor_Specific_Application_Id"));// optional"));
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));

        encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
        encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
        encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        //Terminal_Information_AVP terminal_Information optional,
        encode_RAT_Type_AVP(out, (RecordValue)Body.getField("rat_Type"));
        encode_ULR_Flags_AVP(out, (RecordValue)Body.getField("ulr_Flags"));
        encode_UE_SRVCC_Capability_AVP(out, (RecordValue)Body.getField("ue_SRVCC_Capability"));
        encode_Visited_PLMN_Id_AVP(out, (RecordValue)Body.getField("visited_PLMN_Id"));
        encode_SGSN_Number_AVP(out, (RecordValue)Body.getField("sgsn_Number"));
        encode_Homogeneous_Support_AVP(out, (RecordValue)Body.getField("homogeneous_Support"));
        encode_GMLC_Address_AVP(out, (RecordValue)Body.getField("gmlc_Address"));
        encode_Active_APN_AVP(out, (RecordValue)Body.getField("active_APN"));
        encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_ULA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_ULA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("uLA_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
        encode_Experimental_Result_AVP(out, (RecordValue)Body.getField("experimental_Result"));
        encode_Error_Diagnostic_AVP(out, (RecordValue)Body.getField("error_Diagnostic"));
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_ULA_Flags_AVP(out, (RecordValue)Body.getField("ula_Flags"));
        encode_Subscription_Data_AVP(out, (RecordValue)Body.getField("subscription_Data"));
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_CLR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_CLR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("cLR_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
		encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_Cancellation_Type_AVP(out, (RecordValue)Body.getField("cancellation_Type"));
        encode_CLR_Flags_AVP(out, (RecordValue)Body.getField("cLR_Flags")); //optional
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_CLA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_CLA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("cLA_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));       
        encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
        encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}

	private void encode_AIR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_AIR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("aIR_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
		encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_Req_EUTRAN_Auth_Info_AVP(out, (RecordValue)Body.getField("req_EUTRAN_Auth_Info"));// optional,
        encode_Req_UTRAN_GERAN_Auth_Info_AVP(out, (RecordValue)Body.getField("req_UTRAN_GERAN_Auth_Info"));// optional,
        encode_Visited_PLMN_Id_AVP(out, (RecordValue)Body.getField("visited_PLMN_Id"));
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));

		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_AIA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_AIA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("aIA_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
        encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,
        encode_Error_Diagnostic_AVP(out,(RecordValue)Body.getField("error_Diagnostic"));// optional,
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
 //       encode_Authentication_Info_AVP(out,(RecordValue)Body.getField("authentication_Info"));// optional,
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}

	private void encode_IDR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_IDR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("iDR_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
		encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_Subscription_Data_AVP(out, (RecordValue)Body.getField("subscription_Data"));
        encode_IDR_Flags_AVP(out, (RecordValue)Body.getField("iDR_Flags"));// optional,
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));

		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));		
	}
	
	private void encode_IDA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_IDA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("iDA_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
        encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
        encode_IMS_Voice_PS_Sessions_Support_AVP(out, (RecordValue)Body.getField("iMS_Voice_PS_Sessions_Support"));// optional,
        encode_Last_UE_Activity_Time_AVP(out, (RecordValue)Body.getField("last_UE_Activity_Time_AVP"));// optional,
        encode_RAT_Type_AVP(out, (RecordValue)Body.getField("rat_Type"));
        encode_IDA_Flags_AVP(out, (RecordValue)Body.getField("iDA_Flags"));// optional,
//        EPS_User_State_AVP ePS_User_State optional,
//        EPS_Location_Information_AVP ePS_Location_Information optional,
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_DSR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_DSR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("dSR_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
		encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_DSR_Flags_AVP(out,(RecordValue)Body.getField("dSR_Flags"));
//        set of Context_Identifier_AVP context_Identifier optional,
//        Trace_Reference_AVP trace_Reference optional,
//        set of TS_Code_AVP tS_Code optional,
//        set of SS_Code_AVP sS_Code optional,
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_DSA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_DSA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("dSA_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
        encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,
        encode_Error_Diagnostic_AVP(out,(RecordValue)Body.getField("error_Diagnostic"));// optional,
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
        encode_DSA_Flags_AVP(out,(RecordValue)Body.getField("dSA_Flags"));// optional,
		encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}

	private void encode_PUER_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_PUER_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("pUER_Body");
		
        encode_Vendor_Specific_Appl_Id_AVP(out, (RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
		encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
        encode_PUER_Flags_AVP(out, (RecordValue)Body.getField("pUER_Flags"));// optional,
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_PUEA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_PUEA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("pUEA_Body");
		
        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
        encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
        encode_PUEA_Flags_AVP(out, (RecordValue)Body.getField("pUEA_Flags"));// optional,
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));// optional,
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));

		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_RSR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_RSR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("rSR_Body");
		
        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_User_Id_AVP(out,(RecordOfValue)Body.getField("user_Id"));// optional,
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_RSA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_RSA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("rSA_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out, (RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
        encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,
        encode_Auth_Session_State_AVP(out,(RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
        encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));// optional,
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_NOR_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_NOR_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("nOR_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out,(RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
		encode_Destination_Host_AVP(out, (RecordValue)Body.getField("destination_Host"));
		encode_Destination_Realm_AVP(out, (RecordValue)Body.getField("destination_Realm"));
		encode_User_Name_AVP(out, (RecordValue)Body.getField("user_Name"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        //Terminal_Information_AVP terminal_Information optional,
        encode_MIP6_Agent_Info_AVP(out, (RecordValue)Body.getField("mIP6_Agent_Info"));// optional,
        encode_Visited_Network_Identifier_AVP(out, (RecordValue)Body.getField("visited_Network_Identifier"));// optional,
        encode_Context_Identifier_AVP(out, (RecordValue)Body.getField("context_Identifier"));// optional,
        encode_Service_Selection_AVP(out, (RecordValue)Body.getField("service_Selection"));// optional,
        //Alert_Reason_AVP alert_Reason optional,
        //UE_SRVCC_Capability_AVP uE_SRVCC_Capability optional,
        encode_NOR_Flags_AVP(out, (RecordValue)Body.getField("nOR_Flags"));// optional,
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	private void encode_NOA_MSG(ByteArrayOutputStream out, RecordValue value){
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_NOA_MSG");
		
		encode_DiameterHeader(out,(RecordValue)value.getField("header"));
		encode_Session_Id_AVP(out,(RecordValue)value.getField("session_Id"));
		
		RecordValue Body = (RecordValue)value.getField("nOA_Body");

        encode_Vendor_Specific_Appl_Id_AVP(out, (RecordValue)Body.getField("vendor_Specific_Application_Id"));
        encode_Result_Code_AVP(out,(RecordValue)Body.getField("result_Code"));
        encode_Experimental_Result_AVP(out,(RecordValue)Body.getField("experimental_Result"));// optional,
        encode_Auth_Session_State_AVP(out, (RecordValue)Body.getField("auth_Session_State"));
        encode_Origin_Host_AVP(out, (RecordValue)Body.getField("origin_Host"));
		encode_Origin_Realm_AVP(out, (RecordValue)Body.getField("origin_Realm"));
        encode_Supported_Features_AVP(out,(RecordOfValue)Body.getField("supported_Features"));
        encode_Failed_AVP(out, (RecordOfValue)Body.getField("failed"));// optional,
		encode_Proxy_Info_AVP(out, (RecordOfValue)Body.getField("proxy_Info"));
		encode_Route_Record_AVP(out, (RecordOfValue)Body.getField("route_Record"));
		
		encode_AVP_Type(out,(RecordOfValue)Body.getField("aVP_Type"));
	}
	
	
	
	/*Encoding Of Parameter Types*/
	/*RFC3588 Types */
	private void encode_DiameterHeader (ByteArrayOutputStream out, RecordValue header) {
		//RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: encode_DiameterHeader");
	
		byte[] cmd_flags = new byte[1];
		
		int version=((IntegerValue) header.getField("version")).getInt();
		int msglen=((IntegerValue) header.getField("msglen")).getInt();
		encode_IntegerValue(out, (((version&0x000000ff)<<24) | (msglen & 0x00ffffff)));
		
		RecordValue cmdflags = (RecordValue) header.getField("cmdflags");
		UsedBitsInByte = 0;
		BitStringToBuffer((BitString)cmdflags.getField("r_bit"),cmd_flags);
		BitStringToBuffer((BitString)cmdflags.getField("p_bit"),cmd_flags);
		BitStringToBuffer((BitString)cmdflags.getField("e_bit"),cmd_flags);
		BitStringToBuffer((BitString)cmdflags.getField("t_bit"),cmd_flags);
		BitStringToBuffer((BitString)cmdflags.getField("reserved_bits"),cmd_flags);

		int flags=cmd_flags[0];
		int cmdcode=command_codes[((IntegerValue) header.getField("cmdcode")).getInt()];
		//encode_IntegerValue(out, command_codes[((IntegerValue) header.getField("cmdcode")).getInt()]);
		encode_IntegerValue(out, (((flags&0x000000ff)<<24) | (cmdcode & 0x00ffffff)));
		
		encode_IntegerValue(out,(IntegerValue) header.getField("applid"));
		
		if (!header.getField("hopid").notPresent()) {
			encode_IntegerValue(out,(IntegerValue) header.getField("hopid"));
		}
		
		if (!header.getField("endid").notPresent()) {
			encode_IntegerValue(out,(IntegerValue) header.getField("endid"));
		}
		return;
	}
	
	private void encode_AVPHeader (ByteArrayOutputStream out, RecordValue avp) {
		
		byte[] avp_flags = new byte[1];
		
		encode_IntegerValue(out,(IntegerValue) avp.getField("aVP_Code"));

		RecordValue avpflags = (RecordValue) avp.getField("aVP_flags");
		UsedBitsInByte = 0;
		BitStringToBuffer((BitString)avpflags.getField("v_bit"),avp_flags);
		BitStringToBuffer((BitString)avpflags.getField("m_bit"),avp_flags);
		BitStringToBuffer((BitString)avpflags.getField("p_bit"),avp_flags);
		BitStringToBuffer((BitString)avpflags.getField("reserved_bits"),avp_flags);

		int flags=avp_flags[0];
		int avplen=((IntegerValue)avp.getField("aVP_len")).getInt();
		
		encode_IntegerValue(out, (((flags&0x000000ff)<<24) | (avplen & 0x00ffffff)));

		if (!avp.getField("aVP_vid").notPresent()) {
			encode_IntegerValue(out,(IntegerValue) avp.getField("aVP_vid"));
		}
		return;
	}
	
	private void encode_IPFilterRule_Type_AVP (ByteArrayOutputStream out, RecordValue rv) {
	
		if ((rv == null) | rv.notPresent()) {return;}
		
		//UTF8String	action_,
      	//UTF8String	direction,
      	//UTF8String	proto,
      	//IP_AddressMask src, //including optional ports
      	//IP_AddressMask dst, //including optional ports
      	//UTF8String options optional	
		return;
	}
	
	private void encode_DiameterIdentity (ByteArrayOutputStream out, CharstringValue data) {
		
		encode_Charstring(out,data);
		return;
	}
	
	private void encode_DiameterURI(ByteArrayOutputStream out, RecordValue rv){
        
		ByteArrayOutputStream uri = new ByteArrayOutputStream();
		
        if((rv.getField("scheme")!=null) && !(rv.getField("scheme").notPresent())){
        	write(uri,((CharstringValue)rv.getField("scheme")).getString().toString());// ("aaa", "aaas"),
        }
        if((rv.getField("diameterId_FQDN")!=null) && !(rv.getField("diameterId_FQDN").notPresent())){
        	write(uri, ((CharstringValue)rv.getField("diameterId_FQDN")).getString().toString());
        }
        
		encode_IntegerValue   (uri, (IntegerValue)rv.getField("portField"));// optional,
		
        if((rv.getField("transport")!=null) && !(rv.getField("transport").notPresent())){
            write(uri, ((CharstringValue)rv.getField("transport")).getString().toString());// ("tcp", "sctp", "udp") optional,
        }
        if((rv.getField("protocol") !=null) && !(rv.getField("protocol").notPresent())){
        	write(uri, ((CharstringValue)rv.getField("protocol")).getString().toString());// ("diameter", "radius", "tacacs+") optional
        }
		
        //padding=false;
        //encode_Charstring(out, (CharstringValue)rv.getField("scheme"));
		//encode_Charstring(out, (CharstringValue)rv.getField("diameterId_FQDN"));
		//encode_IntegerValue   (out, (IntegerValue)rv.getField("portField"));
		//encode_Charstring(out, (CharstringValue)rv.getField("transport"));
		//encode_Charstring(out, (CharstringValue)rv.getField("protocol"));
		//padding=true;
        write_avp_padding(uri,uri.size());
        write(out,uri.toByteArray());
        return;    
    }
	
	private void encode_Origin_Host_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_DiameterIdentity(out, (CharstringValue)rv.getField("aVP_Data"));
		return;
	}
	
	private void encode_Origin_Realm_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_DiameterIdentity(out, (CharstringValue)rv.getField("aVP_Data"));
		return;
	}

	private void encode_IPV4Addr (ByteArrayOutputStream out, RecordOfValue data) {
		
		byte[] version=new byte[]{0x00, 0x01};
		byte[] padded=new byte[]{0x00, 0x00};
		
		write(out,version);
		
		int len = data.getLength();
		for (int i = 0; i < len; i++) {
			IntegerValue val=(IntegerValue) data.getField(i);
			//byte[] x=new byte[] {(byte) val.getInt()};
			write(out,new byte[]{(byte) val.getInt()});	
		}

		write(out,padded);
		return;
	}
	
	private void encode_IPV6Addr (ByteArrayOutputStream out, RecordOfValue data) {
		
		byte[] version=new byte[]{0x00, 0x02};
		byte[] padded=new byte[]{0x00, 0x00};
        
		write(out,version);
		
		int len = data.getLength();
		for (int i = 0; i < len; i++) {
			int val=((IntegerValue) data.getField(i)).getInt();
			byte[] x=new byte[] {(byte)(val>>>8) , (byte)val};
			write(out,x);		
		}
		
		write(out,padded);
		return;
	}
			
	private void encode_Address (ByteArrayOutputStream out, RecordValue data) {
		
		int version;
		OctetstringValue family=(OctetstringValue)data.getField("addressFamily");
		//encodeOctetstring(out, family);
		//int len = family.getLength();
		//for (int i=1;i<=len;i++){
		//	version = family.getOctet(i);
		//}
        
		UnionValue diam_address = (UnionValue)data.getField("ipAddress");	
		if (diam_address.getPresentVariantName().equals("ipv4")) {
			encode_IPV4Addr(out, (RecordOfValue) diam_address.getVariant("ipv4"));
		}else if (diam_address.getPresentVariantName().equals("ipv6")){
			encode_IPV6Addr(out, (RecordOfValue) diam_address.getVariant("ipv6"));
		}			
		return;
	}
	
	private void encode_Host_IP_Address_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
		    encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		    encode_Address(out, (RecordValue)rv.getField("aVP_Data"));
		}
		return;
	}
	
	private void encode_Vendor_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
		return;
	}
	
    private void encode_Product_Name_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data") );
		//encode_Charstring(out,(CharstringValue) rv.getField("aVP_Data") );
		return;
	}
    
    private void encode_Origin_State_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
		return;
	}
	
    private void encode_Origin_State_Id_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    	
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
			encode_Origin_State_Id_AVP(out,rv);
		}
		return;
	}
	
    private void encode_Supported_Vendor_Id_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    	
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
		}
		return;
    }
    
    private void encode_Auth_Application_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
  
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
		return;
    }
    
    private void encode_Auth_Application_Id_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    	
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
		}
		return;
    }
    
    private void encode_Inband_Security_Id_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    	
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
		}
		return;
    }

    private void encode_Acct_Application_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}

		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
		return;
    }
    
    private void encode_Acct_Application_Id_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    	
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
		}
		return;
    }

    private void encode_Vendor_Specific_Appl_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		
		RecordOfValue VId=(RecordOfValue)rv.getField("vendor_Id");
		if ((VId == null) | VId.notPresent()) {}
		else{
			int lenVId = VId.getLength();
			for (int j = 0; j < lenVId; j++) {
				encode_Vendor_Id_AVP(out,(RecordValue)VId.getField(j));
			}
		}
		
		encode_Auth_Application_Id_AVP(out, (RecordOfValue)rv.getField("auth_Application_Id"));
		encode_Acct_Application_Id_AVP(out, (RecordOfValue)rv.getField("acct_Application_Id"));
		return;
    }
    
    private void encode_Vendor_Specific_Appl_Id_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    	
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
			encode_Vendor_Specific_Appl_Id_AVP(out,rv);	
		}
		return;
    }
    
	private void encode_Firmware_Revision_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
		return;
	}
	
	private void encode_Result_Code_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(result_codes[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;
	}
	
	private void encode_Error_Message_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data") );
		encode_Charstring(out,(CharstringValue) rv.getField("aVP_Data") );
		return;
	}

    private void encode_Failed_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
    	   encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		   encode_AVP_Type(out,(RecordOfValue) rv.getField("aVP") );
		}
		return;
	}
	
	private void encode_Session_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
	
	private void encode_Destination_Realm_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_DiameterIdentity(out, (CharstringValue)rv.getField("aVP_Data"));
		return;
	}
	
	private void encode_Destination_Host_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_DiameterIdentity(out, (CharstringValue)rv.getField("aVP_Data"));
		return;
	}
	
	private void encode_Re_Auth_Request_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
	    encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
	    return;		
	}
	

	
	private void encode_User_Name_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
    		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data") );
		return;	
	}
	
    private void encode_User_Name_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_User_Name_AVP(out, rv);
		}
		return;
	}
	
	private void encode_Proxy_Host_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_DiameterIdentity(out, (CharstringValue)rv.getField("aVP_Data"));
		return;
	}
	
	private void encode_Proxy_State_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    return;	
	}
	
    private void encode_Proxy_Info_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
    	   encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	   encode_Proxy_Host_AVP(out,(RecordValue) rv.getField("proxy_Host") );
    	   encode_Proxy_State_AVP(out,(RecordValue) rv.getField("proxy_State") );
    	   encode_AVP_Type(out,(RecordOfValue) rv.getField("aVP") );
		}
		return;
	}
    
	private void encode_Route_Record_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_DiameterIdentity(out, (CharstringValue)rv.getField("aVP_Data"));
		return;
	}
	
	private void encode_Route_Record_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
		if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
			encode_Route_Record_AVP(out,rv);
		}
	    return;	
	}
	
	private void encode_Error_Reporting_Host_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_DiameterIdentity(out, (CharstringValue)rv.getField("aVP_Data"));
		return;
	}
	
	private void encode_Redirect_Host_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_Address(out, (RecordValue)rv.getField("aVP_Data"));
		encode_DiameterIdentity(out, (CharstringValue)rv.getField("aVP_Data"));
		return;
	}
	
	private void encode_Redirect_Host_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
		if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
			encode_Redirect_Host_AVP(out,rv);
		}
	    return;	
	}
	
	private void encode_Redirect_Host_Usage_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Redirect_Host_Usage[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;
	}
	
    private void encode_Redirect_Max_Cache_Time_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_Integer64Value(out, (IntegerValue)rv.getField("aVP_Data"));
		encode_IntegerValue(out, (IntegerValue)rv.getField("aVP_Data"));
		return;
	}
    
	private void encode_Accounting_Record_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Acc_Record[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;
	}
    
	private void encode_Accounting_Record_Number_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
		return;
	}
	
    private void encode_Accounting_Sub_Session_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Integer64Value(out, (IntegerValue)rv.getField("aVP_Data"));
		return;
	}	

	private void encode_Acct_Session_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    return;	
	}
	
	private void encode_Acct_Multi_Session_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Charstring(out,(CharstringValue) rv.getField("aVP_Data"));
	    return;	
	}
	
	private void encode_Acct_Interim_Interval_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
	    return;	
	}
	
	private void encode_Accounting_Realtime_Required_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Acc_Realtime_Req[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
	    return;	
	}
	
    private void encode_Event_Timestamp_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_Integer64Value(out, (IntegerValue)rv.getField("aVP_Data"));
		encode_IntegerValue(out, (IntegerValue)rv.getField("aVP_Data"));
		return;
	}
    
	private void encode_Termination_Cause_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Termination_Cause[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
	    return;	
	}
	
	private void encode_Class_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    return;	
	}
	
	private void encode_Class_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
		if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
			encode_Class_AVP(out,rv);
		}

	    return;	
	}
	
	private void encode_Disconnect_Cause_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Disconnect_Cause[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
	    return;	
	}
	/*End RFC3588 Types */
	
	/*Rx Types */
	/*End Rx Types */
	
	/*RFC4005*/
	private void encode_Called_Station_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
	    return;	
	}
    
	private void encode_Framed_IP_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    return;	
	}
    
	private void encode_Framed_IPv6_Prefix_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    return;	
	}
	
	private void encode_Framed_Interface_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
			
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    return;	
	}
	/*End RFC4005*/
	
	/*RFC4006 Types*/
    private void encode_Service_Context_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_Charstring(out,(CharstringValue) rv.getField("aVP_Data"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
	    return;	
	}
    
    private void encode_CC_Request_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(CC_Request[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_CC_Request_Number_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_Subscription_Id_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;
	}
    
    private void encode_Subscription_Id_Data_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Subscription_Id_AVP(ByteArrayOutputStream out, RecordValue rv){

		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Subscription_Id_Type_AVP(out,(RecordValue)rv.getField("subscription_Id_Type"));
        encode_Subscription_Id_Data_AVP(out,(RecordValue)rv.getField("subscription_Id_Data"));
        return;
	}
    
    private void encode_Subscription_Id_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Subscription_Id_AVP(out, rv);
		}
		return;
	}
    
	private void encode_Service_Identifier_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
	    return;	
	}
	
    private void encode_Service_Identifier_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Service_Identifier_AVP(out, rv);
		}
		return;

	}	
	
	private void encode_CC_Time_AVP(ByteArrayOutputStream out, RecordValue rv){
		
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
	    return;	
	}
    
    private void encode_Value_Digits_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Integer64Value(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Exponent_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Currency_Code_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Unit_Value_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Value_Digits_AVP(out, (RecordValue)rv.getField("value_Digits"));
    	encode_Exponent_AVP(out, (RecordValue)rv.getField("exponent"));
    	return;
    }
    
    private void encode_CC_Money_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Unit_Value_AVP(out, (RecordValue)rv.getField("unit_Value")); 
    	encode_Currency_Code_AVP(out, (RecordValue)rv.getField("currency_Code"));
    	return;
    }
    
    private void encode_CC_Total_Octets_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Integer64Value(out,(((IntegerValue) rv.getField("aVP_Data"))));
    	return;
    }
    
    private void encode_CC_Input_Octets_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Integer64Value(out,(((IntegerValue) rv.getField("aVP_Data"))));
    	return;
    }
    
    private void encode_CC_Output_Octets_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Integer64Value(out,(((IntegerValue) rv.getField("aVP_Data"))));
    	return;
    }
    
    private void encode_CC_Service_Specific_Units_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Integer64Value(out,(((IntegerValue) rv.getField("aVP_Data"))));
    	return;
    }
    
    private void encode_Requested_Service_Unit_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_CC_Time_AVP(out, (RecordValue)rv.getField("cC_Time"));
    	encode_CC_Money_AVP(out, (RecordValue)rv.getField("cC_Money"));
    	encode_CC_Total_Octets_AVP(out, (RecordValue)rv.getField("cC_Total_Octets"));
    	encode_CC_Input_Octets_AVP(out, (RecordValue)rv.getField("cC_Input_Octets"));
    	encode_CC_Output_Octets_AVP(out, (RecordValue)rv.getField("cC_Output_Octets"));
    	encode_CC_Service_Specific_Units_AVP(out, (RecordValue)rv.getField("cC_Service_Specific_Units"));
    	encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));
    	return;
    }
    
    private void encode_Requested_Action_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Requested_Action[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Tariff_Change_Usage_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(Tariff_Change_Usage[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Used_Service_Unit_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_Tariff_Time_Change_AVP(out, (RecordValue)rv.getField("tariff_Time_Change"));
    	encode_Reporting_Reason_AVP(out, (RecordValue)rv.getField("reporting_Reason"));// optional, // TS 132 299
        encode_Tariff_Change_Usage_AVP(out, (RecordValue)rv.getField("tariff_Change_Usage"));// optional,
    	encode_CC_Time_AVP(out, (RecordValue)rv.getField("cC_Time"));
    	encode_CC_Money_AVP(out, (RecordValue)rv.getField("cC_Money"));
    	encode_CC_Total_Octets_AVP(out, (RecordValue)rv.getField("cC_Total_Octets"));
    	encode_CC_Input_Octets_AVP(out, (RecordValue)rv.getField("cC_Input_Octets"));
    	encode_CC_Output_Octets_AVP(out, (RecordValue)rv.getField("cC_Output_Octets"));
    	encode_CC_Service_Specific_Units_AVP(out, (RecordValue)rv.getField("cC_Service_Specific_Units"));
    	//encode_Event_Charging_TimeStamp_AVP(out, (RecordValue)rv.getField("event_Charging_TimeStamp"));// optional,
    	encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));
    	return;
    }
    
    private void encode_Used_Service_Unit_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Used_Service_Unit_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Multiple_Services_Indicator_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Multi_Serv_Indi[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Tariff_Time_Change_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data"))));
    	return;
    }
    
    private void encode_Granted_Service_Unit_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Tariff_Time_Change_AVP(out, (RecordValue)rv.getField("tariff_Time_Change"));
    	encode_CC_Time_AVP(out, (RecordValue)rv.getField("cC_Time"));
    	encode_CC_Money_AVP(out, (RecordValue)rv.getField("cC_Money"));
    	encode_CC_Total_Octets_AVP(out, (RecordValue)rv.getField("cC_Total_Octets"));
    	encode_CC_Input_Octets_AVP(out, (RecordValue)rv.getField("cC_Input_Octets"));
    	encode_CC_Output_Octets_AVP(out, (RecordValue)rv.getField("cC_Output_Octets"));
    	encode_CC_Service_Specific_Units_AVP(out, (RecordValue)rv.getField("cC_Service_Specific_Units"));
    	encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));
    	return;
    }
    
    private void encode_Rating_Group_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
	    return;	
	}
    
    private void encode_G_S_U_Pool_Identifier_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
	    return;	
	}
    
    private void encode_CC_Unit_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(CC_Unit_Type[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_G_S_U_Pool_Reference_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_G_S_U_Pool_Identifier_AVP(out, (RecordValue)rv.getField("g_S_U_Pool_Identifier"));
		encode_CC_Unit_Type_AVP(out, (RecordValue)rv.getField("cC_Unit_Type"));
		encode_Unit_Value_AVP(out, (RecordValue)rv.getField("unit_Value"));
	    return;	
	}
    
    private void encode_G_S_U_Pool_Reference_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_G_S_U_Pool_Reference_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Validity_Time_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
	    return;	
	}
    
    private void encode_Final_Unit_Action_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Final_Unit_Action_Type[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Restriction_Filter_Rule_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IPFilterRule_Type_AVP(out, (RecordValue)rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_Restriction_Filter_Rule_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Restriction_Filter_Rule_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Filter_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_UniversalCharstring(out,(((UniversalCharstringValue) rv.getField("aVP_Data"))));
    	return;
    }
    
    private void encode_Filter_Id_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Filter_Id_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Redirect_Address_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Redirect_Address_Type[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Redirect_Server_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_UniversalCharstring(out,(((UniversalCharstringValue) rv.getField("aVP_Data"))));
    	return;
    }
    
    private void encode_Redirect_Server_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Redirect_Address_Type_AVP(out, (RecordValue)rv.getField("redirect_Address_Type"));
    	encode_Redirect_Server_Address_AVP(out, (RecordValue)rv.getField("redirect_Server_Address"));	
    	return;
    }
    
    private void encode_Final_Unit_Indication_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Final_Unit_Action_AVP(out, (RecordValue)rv.getField("final_Unit_Action"));
    	encode_Restriction_Filter_Rule_AVP(out, (RecordOfValue)rv.getField("restriction_Filter_Rule"));
    	encode_Filter_Id_AVP(out, (RecordOfValue)rv.getField("filter_Id"));
    	encode_Redirect_Server_AVP(out, (RecordValue)rv.getField("redirect_Server"));
	    return;	
	}
    
    private void encode_Time_Quota_Threshold_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Volume_Quota_Threshold_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Unit_Quota_Threshold_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Quota_Holding_Time_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
	    private void encode_Reporting_Reason_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
	    	return;
	    }
    
	    private void encode_Reporting_Reason_AVP(ByteArrayOutputStream out, RecordOfValue rov){
	    	
	    	if ((rov == null) | rov.notPresent()) {return;}
	    
			int len = rov.getLength();
			for (int i = 0; i < len; i++) {
			   RecordValue rv=(RecordValue) rov.getField(i);
			   encode_Reporting_Reason_AVP(out, rv);
			}
	    	return;
	    }
	    
	    private void encode_Trigger_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	
	    	RecordOfValue rov = (RecordOfValue)rv.getField("trigger_Type");
	    	
	    	if ((rov == null) | rov.notPresent()) {return;}
		    
			int len = rov.getLength();
			for (int i = 0; i < len; i++) {
			   RecordValue rvi=(RecordValue) rov.getField(i);
			   encode_AVPHeader(out, (RecordValue)rvi.getField("aVP_Header"));
			   encode_IntegerValue(out,(((IntegerValue) rvi.getField("aVP_Data")).getInt()));
			}
			
	    	return;
	    }    
	    
	    private void encode_Refund_Information_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    	return;
	    }
    
    private void encode_Multiple_Services_Credit_Control_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Multi_Serv_Indi[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
    	encode_Granted_Service_Unit_AVP(out, (RecordValue)rv.getField("granted_Service_Unit"));
    	encode_Requested_Service_Unit_AVP(out, (RecordValue)rv.getField("requested_Service_Unit"));
    	encode_Used_Service_Unit_AVP(out, (RecordOfValue)rv.getField("used_Service_Unit"));
    	encode_Tariff_Change_Usage_AVP(out, (RecordValue)rv.getField("tariff_Change_Usage"));
    	encode_Service_Identifier_AVP(out, (RecordOfValue)rv.getField("service_Identifier"));
    	encode_Rating_Group_AVP(out, (RecordValue)rv.getField("rating_Group"));
    	encode_G_S_U_Pool_Reference_AVP(out, (RecordOfValue)rv.getField("g_S_U_Pool_Reference"));
    	encode_Validity_Time_AVP(out, (RecordValue)rv.getField("validity_Time"));
    	encode_Result_Code_AVP(out, (RecordValue)rv.getField("result_Code"));
    	encode_Final_Unit_Indication_AVP(out, (RecordValue)rv.getField("final_Unit_Indication"));
    	encode_Time_Quota_Threshold_AVP(out, (RecordValue)rv.getField("time_Quota_Threshold"));// optional, // TS 132 299
    	encode_Volume_Quota_Threshold_AVP(out, (RecordValue)rv.getField("volume_Quota_Threshold"));// optional, // TS 132 299 
        encode_Unit_Quota_Threshold_AVP(out, (RecordValue)rv.getField("unit_Quota_Threshold"));// optional, // TS 132 299
        encode_Quota_Holding_Time_AVP(out, (RecordValue)rv.getField("quota_Holding_Time"));// optional, // TS 132 299
        encode_Quota_Consumption_Time_AVP(out, (RecordValue)rv.getField("quota_Consumption_Time"));// optional, // TS 132 299
        encode_Reporting_Reason_AVP(out, (RecordOfValue)rv.getField("reporting_Reason"));// optional, // TS 132 299
        encode_Trigger_AVP(out, (RecordValue)rv.getField("trigger_"));// optional, // TS 132 299
        //encode_PS_Furnish_Charging_Information_AVP(out, (RecordValue)rv.getField("ps_Furnish_Charging_Information"));// optional, // TS 132 299
        encode_Refund_Information_AVP(out, (RecordValue)rv.getField("refund_Information"));// optional, // TS 132 299
        //encode_AF_Correlation_Information_AVP(out, (RecordOfValue)rv.getField("aF_Correlation_Information"));// optional, // TS 132 299
        encode_Envelope_AVP(out, (RecordOfValue)rv.getField("envelope"));// optional, // TS 132 299
        encode_Envelope_Reporting_AVP(out, (RecordValue)rv.getField("envelope_Reporting"));// optional, // TS 132 299
        encode_Time_Quota_Mechanism_AVP(out, (RecordValue)rv.getField("time_Quota_Mechanism"));// optional, // TS 132 299
        //encode_Service_Specific_Info_AVP(out, (RecordOfValue)rv.getField("service_Specific_Info"));// optional, // TS 132 299
        encode_QoS_Information_AVP(out, (RecordValue)rv.getField("qoS_Information"));// optional, // TS 132 299
    	encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));
    	return;
    }
    
    private void encode_Multiple_Services_Credit_Control_AVP(ByteArrayOutputStream out, RecordOfValue rov){
    	
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Multiple_Services_Credit_Control_AVP(out, rv);
		}
    	return;
    }
    
    private void encode_Remaining_Balance_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Unit_Value_AVP(out, (RecordValue)rv.getField("unit_Value"));
        encode_Currency_Code_AVP(out, (RecordValue)rv.getField("currency_Code"));
    	return;
    }
    
    private void encode_User_Equipment_Info_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(User_Equipment_Info[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_User_Equipment_Info_Value_AVP(ByteArrayOutputStream out, RecordValue rv){
	    
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
		return;
    }
    
    
    private void encode_User_Equipment_Info_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_User_Equipment_Info_Type_AVP(out, (RecordValue)rv.getField("user_Equipment_Info_Type"));
        encode_User_Equipment_Info_Value_AVP(out, (RecordValue)rv.getField("user_Equipment_Info_Value"));
    	return;
    }
    
    private void encode_CC_Session_Failover_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(CC_Session_Failover[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Cost_Unit_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_UniversalCharstring(out,(((UniversalCharstringValue) rv.getField("aVP_Data"))));
    	return;
    }
    
    private void encode_Cost_Information_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Unit_Value_AVP(out, (RecordValue)rv.getField("unit_Value"));
        encode_Currency_Code_AVP(out, (RecordValue)rv.getField("currency_Code"));
        encode_Cost_Unit_AVP(out, (RecordValue)rv.getField("cost_Unit"));
    	return;
    }
    
    private void encode_Credit_Control_Failure_Handling_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(CC_Failure_Handling[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Direct_Debiting_Failure_Handling_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(DD_Failure_Handling[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
	/*End RFC4006 AVPs*/
    
    /*RFC4740 AVPs*/
    private void encode_Auth_Session_State_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Auth_Session_State_Type[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;	
	}
    
    private void encode_SIP_Authentication_Context_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
     	if ((rv == null) | rv.notPresent()) {return;}
     	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
		return;	
	}
    
    private void encode_UAR_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
		
       	if ((rv == null) | rv.notPresent()) {return;}
       	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;	
	}
    
    private void encode_User_Authorization_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;	
	}
    
    private void encode_SIP_Server_Assignment_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;	
	}
    
    private void encode_Server_Assignment_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_SIP_Server_Assignment_Type_AVP(out,rv);
    	return;
    }
    
    private void encode_User_Data_Already_Available_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;	
	}
    
    private void encode_SIP_Digest_Authenticate_AVP(ByteArrayOutputStream out, RecordValue rv){

    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));//SIP-Digest-Authenticate ::= < AVP Header: 635 >
        encode_Digest_Realm_AVP(out, (RecordValue)rv.getField("digest_Realm"));//,//requ.
        encode_Digest_Algorithm_AVP(out, (RecordValue)rv.getField("digest_Algorithm"));// optional,
        encode_Digest_QoP_AVP(out, (RecordValue)rv.getField("digest_QoP"));// optional,
        encode_Digest_HA1_AVP(out, (RecordValue)rv.getField("digest_HA1"));// optional,
        //set of AVP_Type                 aVP_Type optional
        return;
    }
    
    private void encode_Originating_Request_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;	
	}
    
		private void encode_Reason_Code_AVP(ByteArrayOutputStream out, RecordValue rv){
			
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
			return;	
		}
		
		private void encode_Reason_Info_AVP(ByteArrayOutputStream out, RecordValue rv){
			
			if ((rv == null) | rv.notPresent()) {return;}
			
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
			return;
		}

    private void encode_Deregistration_Reason_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Reason_Code_AVP(out, (RecordValue)rv.getField("reason_Code"));//,//requi.
        encode_Reason_Info_AVP(out, (RecordValue)rv.getField("reason_Info"));// optional,
        //set of AVP_Type         aVP_Type optional
		return;
    }    
    
	    private void encode_SIP_Authentication_Scheme_RFC_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		    return;	
		}
	    
	    private void encode_SIP_Authentication_Scheme_3GPP_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    
    private void encode_SIP_Authentication_Scheme_AVP(ByteArrayOutputStream out, UnionValue uv){
    
    	if ((uv == null) | uv.notPresent()) {return;}
    	
    	if (uv.getPresentVariantName().equals("sIP_Authentication_Scheme_RFC_AVP") && !uv.getVariant("sIP_Authentication_Scheme_RFC_AVP").notPresent()) {
    		encode_SIP_Authentication_Scheme_RFC_AVP(out, (RecordValue) uv.getVariant("sIP_Authentication_Scheme_RFC_AVP"));
		} 
		if (uv.getPresentVariantName().equals("sIP_Authentication_Scheme_3GPP_AVP") && !uv.getVariant("sIP_Authentication_Scheme_3GPP_AVP").notPresent()) {
			encode_SIP_Authentication_Scheme_3GPP_AVP(out, (RecordValue) uv.getVariant("sIP_Authentication_Scheme_3GPP_AVP"));
		} 
	    return;	
	}
    
    private void encode_SIP_Item_Number_AVP(ByteArrayOutputStream out, RecordValue rv){
    
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
	    return;	
	}
    
	    private void encode_SIP_Authenticate_RFC_AVP(ByteArrayOutputStream out, RecordValue rv){

	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));//SIP-Authenticate ::= < AVP Header: 379 >
	    	encode_Digest_Realm_AVP(out, (RecordValue)rv.getField("digest_Realm"));//,//requ.
	    	encode_Digest_Nonce_AVP(out, (RecordValue)rv.getField("digest_Nonce"));//,//requ.
	    	encode_Digest_Domain_AVP(out, (RecordValue)rv.getField("digest_Domain"));//  optional,
	    	encode_Digest_Opaque_AVP(out, (RecordValue)rv.getField("digest_Opaque"));// optional,
	    	encode_Digest_Stale_AVP(out, (RecordValue)rv.getField("digest_Stale"));// optional,
	    	encode_Digest_Algorithm_AVP(out, (RecordValue)rv.getField("digest_Algorithm"));// optional,
	    	encode_Digest_QoP_AVP(out, (RecordValue)rv.getField("digest_QoP"));// optional,
	    	encode_Digest_HA1_AVP(out, (RecordValue)rv.getField("digest_HA1"));// optional,
	        //set of Digest_Auth_Param_AVP   digest_Auth_Param  optional,
	        //set of AVP_Type                aVP_Type optional
	        return;
	    }
	    
	    private void encode_SIP_Authenticate_3GPP_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    
    private void encode_SIP_Authenticate_AVP(ByteArrayOutputStream out, UnionValue uv){
        
    	if ((uv == null) | uv.notPresent()) {return;}
    	
    	if (uv.getPresentVariantName().equals("sIP_Authenticate_RFC_AVP") && !uv.getVariant("sIP_Authenticate_RFC_AVP").notPresent()) {
    		encode_SIP_Authenticate_RFC_AVP(out, (RecordValue) uv.getVariant("sIP_Authenticate_RFC_AVP"));
		} 
		if (uv.getPresentVariantName().equals("sIP_Authenticate_3GPP_AVP") && !uv.getVariant("sIP_Authenticate_3GPP_AVP").notPresent()) {
			encode_SIP_Authenticate_3GPP_AVP(out, (RecordValue) uv.getVariant("sIP_Authenticate_3GPP_AVP"));
		} 
	    return;	
	}
      
	   private void encode_SIP_Authorization_RFC_AVP(ByteArrayOutputStream out, RecordValue rv){
	      
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));//SIP-Authenticate ::= < AVP Header: 379 >
	    	encode_Digest_Username_AVP(out, (RecordValue)rv.getField("digest_Username"));//,//requi.
	    	encode_Digest_Realm_AVP(out, (RecordValue)rv.getField("digest_Realm"));//,
	    	encode_Digest_Nonce_AVP(out, (RecordValue)rv.getField("digest_Nonce"));//,
	    	encode_Digest_URI_AVP(out, (RecordValue)rv.getField("digest_URI"));//,
	    	encode_Digest_Response_AVP(out, (RecordValue)rv.getField("digest_Response"));//,//requi.
	    	encode_Digest_Algorithm_AVP(out, (RecordValue)rv.getField("digest_Algorithm"));// optional,
	    	encode_Digest_CNonce_AVP(out, (RecordValue)rv.getField("digest_CNonce"));// optional,
	    	encode_Digest_Opaque_AVP(out, (RecordValue)rv.getField("digest_Opaque"));// optional,
	    	encode_Digest_QoP_AVP(out, (RecordValue)rv.getField("digest_QoP"));// optional,
	    	encode_Digest_Nonce_Count_AVP(out, (RecordValue)rv.getField("digest_Nonce_Count"));// optional,
	    	encode_Digest_Method_AVP(out, (RecordValue)rv.getField("digest_Method"));// optional,
	    	encode_Digest_Entity_Body_Hash_AVP(out, (RecordValue)rv.getField("digest_Entity_Body_Hash"));// optional,
            //set of Digest_Auth_Param_AVP     digest_Auth_Param  optional,
            //set of AVP_Type                 aVP_Type optional
	        return;
	    }
	    
	    private void encode_SIP_Authorization_3GPP_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
		    return;	
		}

    private void encode_SIP_Authorization_AVP(ByteArrayOutputStream out, UnionValue uv){
    
		if ((uv == null) | uv.notPresent()) {return;}
		
		if (uv.getPresentVariantName().equals("sIP_Authorization_RFC_AVP") && !uv.getVariant("sIP_Authorization_RFC_AVP").notPresent()) {
			encode_SIP_Authorization_RFC_AVP(out, (RecordValue) uv.getVariant("sIP_Authorization_RFC_AVP"));
		} 
		if (uv.getPresentVariantName().equals("sIP_Authorization_3GPP_AVP") && !uv.getVariant("sIP_Authorization_3GPP_AVP").notPresent()) {
			encode_SIP_Authorization_3GPP_AVP(out, (RecordValue) uv.getVariant("sIP_Authorization_3GPP_AVP"));
		} 
        return;	
    }
    
	    private void encode_Identity_with_Emergency_Registration_AVP(ByteArrayOutputStream out, RecordValue rv){
			
			if ((rv == null) | rv.notPresent()) {return;}
			
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_User_Name_AVP(out, (RecordValue)rv.getField("user_Name"));
			encode_Public_Identity_AVP(out, (RecordValue)rv.getField("public_Identity"));
			//set of AVP_Type             aVP_Type optional
		    return;	
		}
    
    private void encode_Identity_with_Emergency_Registration_AVP(ByteArrayOutputStream out, RecordOfValue rov){
    	
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Identity_with_Emergency_Registration_AVP(out, rv);
		}
    	return;
    }
    
		private void encode_Confidentiality_Key_AVP(ByteArrayOutputStream out, RecordValue rv){
			
			if ((rv == null) | rv.notPresent()) {return;}
			
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
		    return;	
		}

		private void encode_Integrity_Key_AVP(ByteArrayOutputStream out, RecordValue rv){
			
			if ((rv == null) | rv.notPresent()) {return;}
			
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
		    return;	
		}
		
		private void encode_Line_Identifier_AVP(ByteArrayOutputStream out, RecordValue rv){
			
			if ((rv == null) | rv.notPresent()) {return;}
			
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
		    return;	
		}
	
    private void encode_SIP_Auth_Data_Item_RFC_AVP(ByteArrayOutputStream out, RecordValue rv){
            
    	if ((rv == null) | rv.notPresent()) {return;}
    	
        encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));//
		encode_SIP_Authentication_Scheme_AVP(out, (UnionValue)rv.getField("sIP_Authentication_Scheme"));// optional,
		encode_SIP_Item_Number_AVP(out, (RecordValue)rv.getField("sIP_Item_Number"));// optional,
        encode_SIP_Authenticate_AVP(out, (UnionValue)rv.getField("sIP_Authenticate"));// optional,
        encode_SIP_Authorization_AVP(out, (UnionValue)rv.getField("sIP_Authorization"));// optional,
        //encode_SIP_Authentication_Info_AVP(out, (RecordValue)rv.getField("sIP_Authentication_Info");// optional,
        //set of AVP_Type                 aVP_Type optional
        return;       
    }	
    
    private void encode_SIP_Auth_Data_Item_3GPP_AVP(ByteArrayOutputStream out, RecordValue rv){
        
	   	if ((rv == null) | rv.notPresent()) {return;}
	   	
	    encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));//
		encode_SIP_Authentication_Scheme_AVP(out, (UnionValue)rv.getField("sIP_Authentication_Scheme"));// optional,
		encode_SIP_Item_Number_AVP(out, (RecordValue)rv.getField("sIP_Item_Number"));// optional,
        encode_SIP_Authenticate_AVP(out, (UnionValue)rv.getField("sIP_Authenticate"));// optional,
        encode_SIP_Authorization_AVP(out, (UnionValue)rv.getField("sIP_Authorization"));// optional,
		encode_SIP_Authentication_Context_AVP(out, (RecordValue)rv.getField("sIP_Authentication_Context"));// optional,
        encode_SIP_Digest_Authenticate_AVP(out, (RecordValue)rv.getField("sIP_Digest_Authenticate"));// optional,
        encode_Confidentiality_Key_AVP(out, (RecordValue)rv.getField("confidentiality_Key"));// optional,
        encode_Integrity_Key_AVP(out, (RecordValue)rv.getField("integrity_Key"));// optional,
        encode_Framed_IP_Address_AVP(out, (RecordValue)rv.getField("framed_IP_Address"));// optional,
        encode_Framed_IPv6_Prefix_AVP(out, (RecordValue)rv.getField("framed_IPv6_Prefix"));// optional,
        encode_Framed_Interface_Id_AVP(out, (RecordValue)rv.getField("framed_Interface_Id"));// optional,
        encode_Line_Identifier_AVP(out, (RecordValue)rv.getField("line_Identifier"));// optional,
        //set of AVP_Type                     aVP_Type optional
    }
		
    private void encode_SIP_Auth_Data_Item_AVP(ByteArrayOutputStream out, UnionValue uv){
		
		if ((uv == null) | uv.notPresent()) {return;}
		
		if (uv.getPresentVariantName().equals("sIP_Auth_Data_Item_RFC_AVP") && !uv.getVariant("sIP_Auth_Data_Item_RFC_AVP").notPresent()) {
			encode_SIP_Auth_Data_Item_RFC_AVP(out, (RecordValue) uv.getVariant("sIP_Auth_Data_Item_RFC_AVP"));
		} 
		if (uv.getPresentVariantName().equals("sIP_Auth_Data_Item_3GPP_AVP") && !uv.getVariant("sIP_Auth_Data_Item_3GPP_AVP").notPresent()) {
			encode_SIP_Auth_Data_Item_3GPP_AVP(out, (RecordValue) uv.getVariant("sIP_Auth_Data_Item_3GPP_AVP"));
		} 
		return;
	}
    
    private void encode_SIP_Auth_Data_Item_AVP(ByteArrayOutputStream out, RecordOfValue rov){
    	
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   UnionValue uv=(UnionValue) rov.getField(i);
		   encode_SIP_Auth_Data_Item_AVP(out, uv);
		}
    	return;
    }
    
    private void encode_SIP_Method_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_SIP_Number_Auth_Items_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;	
    }    
    /*End RFC4740 AVPs*/
    
    /* ******** Digest AVPs ********* */
    	private void encode_Digest_Realm_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}

    	private void encode_Digest_Nonce_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}

    	private void encode_Digest_Domain_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Opaque_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Stale_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Algorithm_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_QoP_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_HA1_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Auth_Param_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Username_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_URI_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Response_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_CNonce_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Nonce_Count_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Method_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Entity_Body_Hash_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Nextnonce_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    	
    	private void encode_Digest_Response_Auth_AVP(ByteArrayOutputStream out, RecordValue rv){
	        
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		    return;	
		}
    /* ************ end of Digest AVPs************ */
    
    
    /*Sh-TS29_329*/
    private void encode_Public_Identity_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Public_Identity_AVP(ByteArrayOutputStream out, RecordOfValue rov){
    	
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Public_Identity_AVP(out, rv);
		}
    	return;
    }
    
    private void encode_User_Identity_AVP(ByteArrayOutputStream out, RecordValue rv){
        
            
        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		
		encode_Public_Identity_AVP(out,(RecordValue)rv.getField("public_Identity_AVP"));
        encode_MSISDN_AVP(out,(RecordValue)rv.getField("mSISDN_AVP"));
        encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Data"));//optional
        return;
    }
        

    private void encode_MSISDN_AVP(ByteArrayOutputStream out, RecordValue rv){
        
        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Octetstring(out, (OctetstringValue)rv.getField("aVP_Data"));
        return;
    }
        

    private void encode_Data_Reference_AVP(ByteArrayOutputStream out, RecordValue rv){
      
        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        //encode_Data_Reference_Type(out, (OctetstringValue)rv.getField("aVP_Data"));
        encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
        return;
    }
    
    private void encode_Data_Reference_AVP(ByteArrayOutputStream out, RecordOfValue rov){

    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Data_Reference_AVP(out, rv);
		}
    	return;
	}
    
       
    private void encode_Service_Indication_AVP(ByteArrayOutputStream out, RecordValue rv){

        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Octetstring(out, (OctetstringValue)rv.getField("aVP_Data"));
        return;
    }
    
    private void encode_Service_Indication_AVP(ByteArrayOutputStream out, RecordOfValue rov){

    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Service_Indication_AVP(out, rv);
		}
    	return;
	}
    
    private void encode_Subs_Req_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
        
        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
        return;
    }
   
    private void encode_Requested_Domain_AVP(ByteArrayOutputStream out, RecordValue rv){
        
        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
        return;
    }
  
    private void encode_Current_Location_AVP(ByteArrayOutputStream out, RecordValue rv){
      
        if ((rv == null) | rv.notPresent()) {return;}
  	
	    encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
        return;
  }
    
    private void encode_Identity_Set_AVP(ByteArrayOutputStream out, RecordValue rv){
        
        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
        return;
    }
    
    private void encode_Identity_Set_AVP(ByteArrayOutputStream out, RecordOfValue rov){

    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Identity_Set_AVP(out, rv);
		}
    	return;
	}
    
    private void encode_Expiry_Time_AVP(ByteArrayOutputStream out, RecordValue rv){
        
        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
        return;
    }
    
    private void encode_Send_Data_Indication_AVP(ByteArrayOutputStream out, RecordValue rv){
        
        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
        return;
    }
    
    private void encode_Sequence_Number_AVP(ByteArrayOutputStream out, RecordValue rv){
        
        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
        return;
    }
    
    private void encode_Repository_Data_ID_AVP(ByteArrayOutputStream out, RecordValue rv){
        
        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Service_Indication_AVP(out, (RecordValue)rv.getField("service_Indication"));
        encode_Sequence_Number_AVP(out, (RecordValue)rv.getField("sequence_Number"));
        encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Data"));//optional
    
        return;
    } 
    
    
    private void encode_Experimental_Result_Code_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(Experimental_Result_Type[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_Experimental_Result_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Vendor_Id_AVP(out, (RecordValue)rv.getField("vendor_Id"));
        encode_Experimental_Result_Code_AVP(out, (RecordValue)rv.getField("experimental_Result_Code"));
    	return;
    }
    
    private void encode_Server_Name_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Server_Name_AVP(ByteArrayOutputStream out, RecordOfValue rov){

    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Server_Name_AVP(out, rv);
		}
    	return;
	}
    
    private void encode_Feature_List_ID_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;	
    }
    
    private void encode_Feature_List_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data") );
    	return;
    }
    
    private void encode_Supported_Features_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Vendor_Id_AVP(out, (RecordValue)rv.getField("vendor_Id"));
        encode_Feature_List_ID_AVP(out, (RecordValue)rv.getField("feature_list_Id")); //optional,
        encode_Feature_List_AVP(out, (RecordValue)rv.getField("feature_list"));
        //encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Data"));//optional
		return;
	}
    
    private void encode_Supported_Features_AVP(ByteArrayOutputStream out, RecordOfValue rov){

    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Supported_Features_AVP(out, rv);
		}
    	return;
	}
    
    private void encode_Wildcarded_PSI_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}

    private void encode_Wildcarded_IMPU_AVP(ByteArrayOutputStream out, RecordValue rv){
	
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
    }
    
    private void encode_DSAI_Tag_AVP(ByteArrayOutputStream out, RecordValue rv){

        if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Octetstring(out, (OctetstringValue)rv.getField("aVP_Data"));
        return;
    }
    
    private void encode_DSAI_Tag_AVP(ByteArrayOutputStream out, RecordOfValue rov){

    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_DSAI_Tag_AVP(out, rv);
		}
    	return;
	}
    
    private void encode_User_Data_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
		return;
	}
    /*End Sh-TS29_329 AVPs*/
    
    /*TS129_229*/
    private void encode_Mandatory_Capability_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Mandatory_Capability_AVP(ByteArrayOutputStream out, RecordOfValue rov){

    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Mandatory_Capability_AVP(out, rv);
		}
    	return;
	}
 
    private void encode_Optional_Capability_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Optional_Capability_AVP(ByteArrayOutputStream out, RecordOfValue rov){

    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Optional_Capability_AVP(out, rv);
		}
    	return;
	}
    
    private void encode_IP_CAN_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());      
    	return;
    }
    
    private void encode_RAT_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(RAT_Type[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
    	//System.out.println("RAT:"+ (/*(IntegerValue)  */   rv.getField("aVP_Data").toString()));
    	//System.out.println("RAT:"+ ((IntegerValue) rv.getField("aVP_Data")).getInt());
    	//int v = RAT_Type[((IntegerValue)rv.getField("aVP_Data")).getInt()];
    	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());    
    	return;
    }
    
    private void encode_EventTrigger_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());      
    	return;
    }
    
    private void encode_Event_Trigger_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_EventTrigger_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Session_Release_Cause_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());      
    	return;
    }
    
    private void encode_PDN_Connection_ID_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));      
    	return;
    }
    
    
    private void encode_AN_GW_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_AN_GW_Addr_AVP(out,rv);
    	return;
    }
    
    private void encode_AN_GW_Addr_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Address(out,(RecordValue) rv.getField("aVP_Data"));      
    	return;
    }
    
    private void encode_AN_GW_Address_AVP(ByteArrayOutputStream out, RecordOfValue rov){
    	
    	encode_AN_GW_Addr_AVP(out, rov);
    	return;
    }
    
    private void encode_AN_GW_Addr_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_AN_GW_Addr_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Charging_Rule_Install_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
//            set of Charging_Rule_Definition_AVP        charging_Rule_Definition optional,
//            set of Charging_Rule_Name_AVP            charging_Rule_Name optional,
//            set of Charging_Rule_Base_Name_AVP        charging_Rule_Base_Name optional,
//            Bearer_Identifier_AVP                    bearer_Identifier optional,
//            Rule_Activation_Time_AVP                rule_Activation_Time optional,
//            Rule_DeActivation_Time_AVP                rule_DeActivation_Time optional,
//            Resource_Allocation_Notification_AVP    resource_Allocation_Notification optional,
//            Charging_Correlation_Indicator_AVP        charging_Correlation_Indicator optional,
//            set of AVP_Type                         aVP_Type optional
    	return;
    }
    
    private void encode_Charging_Rule_Install_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Charging_Rule_Install_AVP(out, rv);
		}
		return;
	}
       

    private void encode_Charging_Rule_Remove_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
//        set of Charging_Rule_Name_AVP        charging_Rule_Name optional,
//        set of Charging_Rule_Base_Name_AVP    charging_Rule_Base_Name optional,
//        set of AVP_Type                     aVP_Type optional
    	return;
    }
    
    private void encode_Charging_Rule_Remove_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Charging_Rule_Remove_AVP(out, rv);
		}
		return;
	}
    
	    private void encode_Charging_Rule_Name_AVP(ByteArrayOutputStream out, RecordOfValue rov){
	    	
	    	if ((rov == null) | rov.notPresent()) {return;}
	    	
	    	int len = rov.getLength();
			for (int i = 0; i < len; i++) {
			   RecordValue rv=(RecordValue) rov.getField(i);
			   encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	    encode_Octetstring(out,((OctetstringValue) rv.getField("aVP_Data")));
			}
	    	return;
	    }

	private void encode_Charging_Rule_Report_AVP(ByteArrayOutputStream out, RecordValue rv){
	    
		if ((rv == null) | rv.notPresent()) {return;}
		
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Charging_Rule_Name_AVP(out, (RecordOfValue)rv.getField("charging_Rule_Name")); //optional,
	//	encode_Charging_Rule_Base_Name_AVP(out, (RecordOfValue)rv.getField("charging_Rule_Base_Name"));// optional,
	//    Bearer_Identifier_AVP                    bearer_Identifier optional,
	    encode_PCC_Rule_Status_AVP(out, (RecordValue)rv.getField("pCC_Rule_Status"));// optional,
	    encode_Rule_Failure_Code_AVP(out, (RecordValue)rv.getField("rule_Failure_Code"));// optional,
	//    Final_Unit_Indication_AVP                final_Unit_Indication optional,
	//    set of AVP_Type                         aVP_Type optional
	    return;
	}	
    	
	private void encode_Charging_Rule_Report_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
		if ((rov == null) | rov.notPresent()) {return;}
	
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Charging_Rule_Report_AVP(out, rv);
		}
		return;
	}
    
        private void encode_QoS_Class_Identifier_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
        	return;
        }
        
        private void encode_Max_Requested_Bandwidth_UL_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
        	return;
        }

        private void encode_Max_Requested_Bandwidth_DL_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
        	return;
        }
 
       private void encode_Guaranteed_Bitrate_UL_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
    	   if ((rv == null) | rv.notPresent()) {return;}
    	   
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
        	return;
        }

        private void encode_Guaranteed_Bitrate_DL_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
        	return;
        }
        
        private void encode_Bearer_Identifier_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
        	return;
        }
        
	        private void encode_Priority_Level_AVP(ByteArrayOutputStream out, RecordValue rv){
	        	
	        	if ((rv == null) | rv.notPresent()) {return;}
	        	
	        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
	        	return;
	        }
        
	        private void encode_Pre_emption_Capability_AVP(ByteArrayOutputStream out, RecordValue rv){
	        	
	        	if ((rv == null) | rv.notPresent()) {return;}
	        	
	        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
	        	return;
	        }
	        
	        private void encode_Pre_emption_Vulnerability_AVP(ByteArrayOutputStream out, RecordValue rv){
	        	
	        	if ((rv == null) | rv.notPresent()) {return;}
	        	
	        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
	        	return;
	        }
        
        
        private void encode_Allocation_Retention_Priority_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_Priority_Level_AVP(out, (RecordValue)rv.getField("priority_Level"));
        	encode_Pre_emption_Capability_AVP(out, (RecordValue)rv.getField("pre_emption_Capability"));// optional,
        	encode_Pre_emption_Vulnerability_AVP(out, (RecordValue)rv.getField("pre_emption_Vulnerability"));// optional
        	return;
        }
        
       private void encode_APN_Aggregate_Max_Bitrate_UL_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
    	   if ((rv == null) | rv.notPresent()) {return;}
    	   
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
        	return;
        }

        private void encode_APN_Aggregate_Max_Bitrate_DL_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
        	return;
        }

    private void encode_QoS_Information_AVP(ByteArrayOutputStream out, RecordValue rv){
        
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_QoS_Class_Identifier_AVP(out, (RecordValue)rv.getField("qoS_Class_Identifier"));// optional,
        encode_Max_Requested_Bandwidth_UL_AVP(out, (RecordValue)rv.getField("max_Requested_Bandwidth_UL"));// optional,
        encode_Max_Requested_Bandwidth_DL_AVP(out, (RecordValue)rv.getField("max_Requested_Bandwidth_DL"));// optional,
        encode_Guaranteed_Bitrate_UL_AVP(out, (RecordValue)rv.getField("guaranteed_Bitrate_UL"));// optional,
        encode_Guaranteed_Bitrate_DL_AVP(out, (RecordValue)rv.getField("guaranteed_Bitrate_DL"));// optional,
        encode_Bearer_Identifier_AVP(out, (RecordValue)rv.getField("bearer_Identifier"));// optional,
        encode_Allocation_Retention_Priority_AVP(out, (RecordValue)rv.getField("allocation_Retention_Priority"));// optional,
        encode_APN_Aggregate_Max_Bitrate_UL_AVP(out, (RecordValue)rv.getField("aPN_Aggregate_Max_Bitrate_UL"));// optional,
        encode_APN_Aggregate_Max_Bitrate_DL_AVP(out, (RecordValue)rv.getField("aPN_Aggregate_Max_Bitrate_DL"));// optional,
        /*set of*/ encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));
        return;
    }
    
    private void encode_QoS_Information_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_QoS_Information_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Routing_Rule_Install_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        //    set of Routing_Rule_Definition_AVP    routing_Rule_Definition optional,
        //    set of AVP_Type                     aVP_Type optional
        return;
    }
      
    private void encode_Routing_Rule_Remove_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        //set of Routing_Rule_Identifier_AVP    routing_Rule_Identifier optional,
        //set of AVP_Type                     aVP_Type optional
        return;
    }
    
	    private void encode_QoS_Rule_Name_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    	return;
	    }
    
        private void encode_QoS_Rule_Name_AVP(ByteArrayOutputStream out, RecordOfValue rov){
    		
        	if ((rov == null) | rov.notPresent()) {return;}
        
    		int len = rov.getLength();
    		for (int i = 0; i < len; i++) {
    		   RecordValue rv=(RecordValue) rov.getField(i);
    		   encode_QoS_Rule_Name_AVP(out, rv);
    		}
    		return;
    	}
        
	    private void encode_QoS_Rule_Base_Name_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
	    	return;
	    }
    	
    	private void encode_QoS_Rule_Base_Name_AVP(ByteArrayOutputStream out, RecordOfValue rov){
    		
    		if ((rov == null) | rov.notPresent()) {return;}
        
    		int len = rov.getLength();
    		for (int i = 0; i < len; i++) {
    		   RecordValue rv=(RecordValue) rov.getField(i);
    		   encode_QoS_Rule_Base_Name_AVP(out, rv);
    		}
    		return;
    	}   
    	
        private void encode_PCC_Rule_Status_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
        	return;
        }
        
        private void encode_Rule_Failure_Code_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_IntegerValue(out,((IntegerValue) rv.getField("aVP_Data")).getInt());
        	return;
        }
    
    private void encode_QoS_Rule_Report_AVP(ByteArrayOutputStream out, RecordValue rv){
        
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_QoS_Rule_Name_AVP(out, (RecordOfValue)rv.getField("qoS_Rule_Name"));//					optional,
        encode_QoS_Rule_Base_Name_AVP(out, (RecordOfValue)rv.getField("qoS_Rule_Base_Name"));//				optional,
        encode_PCC_Rule_Status_AVP(out, (RecordValue)rv.getField("pCC_Rule_Status"));//					optional,
        encode_Rule_Failure_Code_AVP(out, (RecordValue)rv.getField("rule_Failure_Code"));//				optional,
        //encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));
        return;
    }
    
    private void encode_QoS_Rule_Report_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_QoS_Rule_Report_AVP(out, rv);
		}
		return;
	}
    
    private void encode_QoS_Rule_Install_AVP(ByteArrayOutputStream out, RecordValue rv){
        
    	if ((rv == null) | rv.notPresent()) {return;}
        
    	//System.out.println(rv.toString());
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));//
        //encode_QoS_Rule_Definition_AVP(out, (RecordOfValue)rv.getField("qoS_Rule_Definition"));//	qoS_Rule_Definition			optional,
        encode_QoS_Rule_Name_AVP(out, (RecordOfValue)rv.getField("qoS_Rule_Name"));//					optional,
        //encode_QoS_Rule_Base_Name_AVP(out, (RecordOfValue)rv.getField("qoS_Rule_Base_Name"));//				optional,
        //encode_Tunnel_Information_AVP(out, (RecordValue)rv.getField("tunnel_Information"));//				optional,
        //encode_ANw_Charging_Id_Value_AVP(out, (RecordValue)rv.getField("acc_Net_Charging_Id_Val"));//			optional,
        //encode_Resource_Alloc_Notification_AVP(out, (RecordValue)rv.getField("resource_Alloc_Notification"));//		optional,
        //encode_Rule_Activation_Time_AVP(out, (RecordValue)rv.getField("rule_Activation_Time"));//			optional,
        //encode_Rule_DeActivation_Time_AVP(out, (RecordValue)rv.getField("rule_DeActivation_Time"));//			optional,
        //encode_ThreeGPP_GGSN_Address_AVP(out, (RecordValue)rv.getField("threeGPP_GGSN_Address"));//			optional,
        //encode_ThreeGPP_GGSN_IPv6_Address_AVP(out, (RecordValue)rv.getField("threeGPP_GGSN_IPv6_Address"));//		optional,
        //encode_AN_GW_Addr_AVP(out, (RecordOfValue)rv.getField("aN_GW_Address"));//					optional,
        //encode_UDP_Source_Port_AVP(out, (RecordValue)rv.getField("uDP_Source_Port"));//					optional,
        ///*set of*/ encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));
                   
    	return;
    }
    
    private void encode_QoS_Rule_Install_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_QoS_Rule_Install_AVP(out, rv);
		}
		return;
	}
    
    private void encode_QoS_Rule_Remove_AVP(ByteArrayOutputStream out, RecordValue rv){
        
    	if ((rv == null) | rv.notPresent()) {return;}									//ref: TS129 212 5a.3.2
        
        encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));//
        encode_QoS_Rule_Name_AVP(out, (RecordOfValue)rv.getField("qoS_Rule_Name"));//					optional,
        encode_QoS_Rule_Base_Name_AVP(out, (RecordOfValue)rv.getField("qoS_Rule_Base_Name"));//				optional,
        //encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));
        return;
    }
    
    private void encode_QoS_Rule_Remove_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_QoS_Rule_Remove_AVP(out, rv);
		}
		return;
	}
    
    private void encode_QoS_Rule_Definition_AVP(ByteArrayOutputStream out, RecordValue rv){
        
    	if ((rv == null) | rv.notPresent()) {return;}
       
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));//
    	encode_QoS_Rule_Name_AVP(out, (RecordValue)rv.getField("qoS_Rule_Name"));//
        //encode_Flow_Information_AVP(out, (RecordOfValue)rv.getField("flow_Information"));//				optional,
        encode_QoS_Information_AVP(out, (RecordValue)rv.getField("qoS_Information"));//					optional,
        //encode_Precedence_AVP(out, (RecordValue)rv.getField("precedence"));//						optional,
        //encode_Required_Access_Info_AVP(out, (RecordOfValue)rv.getField("required_Access_Info"));//			optional,
        //encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));    
    	return;
    }
    
    private void encode_QoS_Rule_Definition_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_QoS_Rule_Definition_AVP(out, rv);
		}
		return;
	}
        
    private void encode_Server_Capabilities_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Mandatory_Capability_AVP(out, (RecordOfValue)rv.getField("Mandatory_Capability"));
        encode_Optional_Capability_AVP(out, (RecordOfValue)rv.getField("Optional_Capability"));
        encode_Server_Name_AVP(out, (RecordOfValue)rv.getField("Server_Name"));
        encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));
		return;
    }
    /*End TS129_229*/
    
    /*TS129_215*/
	    private void encode_Session_Linking_Indicator_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_IntegerValue(out,(((IntegerValue) rv.getField("session_Linking_Indicator")).getInt()));
	    	return;
	    }
        
    private void encode_Subses_Decision_Info_AVP(ByteArrayOutputStream out, RecordValue rv) {									//ref: TS129 215 5.3.1
        
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Subsession_Id_AVP(out, (RecordValue)rv.getField("subsession_Id"));//,
//        set length (0..2) of AN_GW_Addr_AVP	aN_GW_Address					optional,
        encode_Result_Code_AVP(out, (RecordValue)rv.getField("result_Code"));						//optional,
        encode_Experimental_Result_AVP(out, (RecordValue)rv.getField("experimental_Result"));			//	optional,
        encode_Charging_Rule_Remove_AVP(out, (RecordOfValue)rv.getField("charging_Rule_Remove"));//			optional,
        encode_Charging_Rule_Install_AVP(out, (RecordOfValue)rv.getField("charging_Rule_Install"));//			optional,
        encode_QoS_Rule_Install_AVP(out, (RecordOfValue)rv.getField("qoS_Rule_Install"));//				optional,
        encode_QoS_Rule_Remove_AVP(out, (RecordOfValue)rv.getField("qoS_Rule_Remove"));//					optional,
//        Default_EPS_Bearer_QoS_AVP       	default_EPS_Bearer_QoS			optional,
//        set of Usage_Monitoring_Info_AVP    usage_Monitoring_Information	optional,
        encode_Session_Release_Cause_AVP(out, (RecordValue)rv.getField("session_Release_Cause"));//			optional,
//        Bearer_Control_Mode_AVP            	bearer_Control_Mode				optional,
//        set of Event_Trigger_AVP        	event_Trigger					optional,
//        Revalidation_Time_AVP            	revalidation_Time				optional,
//        Online_AVP                        	online							optional,
//        Offline_AVP                        	offline							optional,
//        QoS_Information_AVP                	qoS_Information					optional,
//        set of AVP_Type                    	aVP_Type						optional
        return;
    }
    
    private void encode_Subses_Decision_Info_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Subses_Decision_Info_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Subses_Enforcement_Info_AVP(ByteArrayOutputStream out, RecordValue rv) {									//ref: TS129 215 5.3.1
        
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Subsession_Id_AVP(out, (RecordValue)rv.getField("subsession_Id"));//,
        encode_Subsession_Operation_AVP(out, (RecordValue)rv.getField("subsession_Operation"));//			optional,
        encode_AN_GW_Addr_AVP(out, (RecordOfValue)rv.getField("aN_GW_Address"));//					optional,
//        Bearer_Identifier_AVP            	bearer_Identifier				optional,
//        Bearer_Operation_AVP            	bearer_Operation				optional,
//        set of Packet_Filter_Info_AVP		packet_Filter_Information		optional,
//        Packet_Filter_Operation_AVP        	packet_Filter_Operation			optional,
        encode_QoS_Information_AVP(out, (RecordValue)rv.getField("qoS_Information"));//					optional,
//        Framed_IP_Address_AVP            	framed_IP_Address				optional,
//        Framed_IPv6_Prefix_AVP            	framed_IPv6_Prefix				optional,
//        set of CoA_Information_AVP        	coA_Information					optional,
        encode_Called_Station_Id_AVP(out, (RecordValue)rv.getField("called_Station_Id"));//				optional,
        encode_PDN_Connection_ID_AVP(out, (RecordValue)rv.getField("pDN_Connection_ID"));//				optional,
//        Bearer_Usage_AVP                	bearer_Usage					optional,
//        set of TFT_Packet_Filter_Info_AVP   tFT_Packet_Filter_Information	optional,
//        Online_AVP                        	online							optional,
//        Offline_AVP                        	offline							optional,
        encode_Result_Code_AVP(out, (RecordValue)rv.getField("result_Code"));//						optional,
        encode_Experimental_Result_AVP(out, (RecordValue)rv.getField("experimental_Result"));//				optional,
//        set of Charging_Rule_Report_AVP    	charging_Rule_Report			optional,
        encode_QoS_Rule_Report_AVP(out, (RecordOfValue)rv.getField("qoS_Rule_Report"));//					optional,
//        IP_CAN_Type_AVP                    	iP_CAN_Type						optional,
//        RAT_Type_AVP                    	rAT_Type						optional,
//        ThreeGPP_SGSN_MCC_MNC_AVP        	threeGPP_SGSN_MCC_MNC			optional,
//        ThreeGPP_SGSN_Address_AVP        	threeGPP_SGSN_Address			optional,
//        ThreeGPP_SGSN_IPv6_Address_AVP    	threeGPP_SGSN_IPv6_Address		optional,
//        RAI_AVP                            	rAI								optional,
//        ThreeGPP_User_Location_Info_AVP    	threeGPP_User_Location_Info		optional,
//        ThreeGPP2_BSID_AVP					threeGPP2_BSID					optional,
//        User_CSG_Information_AVP			user_CSG_Information			optional,
//        Default_EPS_Bearer_QoS_AVP       	default_EPS_Bearer_QoS			optional,
//        Network_Request_Support_AVP        	network_Request_Support			optional,
        encode_Routing_Rule_Install_AVP(out, (RecordValue)rv.getField("routing_Rule_Install"));//			optional,
        encode_Routing_Rule_Remove_AVP(out, (RecordValue)rv.getField("routing_Rule_Remove"));//				optional,
//        Logical_Access_ID_AVP            	logical_Access_ID				optional,
//        Physical_Access_ID_AVP            	physical_Access_ID				optional,
//        set of Usage_Monitoring_Info_AVP    usage_Monitoring_Information	optional,
        encode_Multiple_BBERF_Action_AVP(out, (RecordValue)rv.getField("multiple_BBERF_Action"));//			optional,
        encode_Event_Trigger_AVP(out, (RecordOfValue)rv.getField("event_Trigger"));//					optional,
//        Access_Netw_Charging_Addr_AVP    	access_Network_Charging_Address optional,
//        set of Access_Netw_Charg_Id_Gx_AVP  access_Network_Charging_Id_Gx	optional,
        encode_Session_Linking_Indicator_AVP(out, (RecordValue)rv.getField("session_Linking_Indicator"));//		optional,
//        set of AVP_Type                    	aVP_Type						optional
        return;
    }
    
    private void encode_Subses_Enforcement_Info_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Subses_Enforcement_Info_AVP(out, rv);
		}
		return;
	}
    
	    private void encode_Subsession_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
	    	return;
	    }
	    
	    private void encode_Subsession_Operation_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_IntegerValue(out,(((IntegerValue) rv.getField("subsession_Operation")).getInt()));
	    	return;
	    }
    
    
	    
    private void encode_Multiple_BBERF_Action_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("multiple_BBERF_Action")).getInt()));
    	return;
    }
    /*End TS129_215*/
    
    //TS129_272 AVP codes
    private void encode_ULR_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_ULA_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_Visited_PLMN_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_UE_SRVCC_Capability_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(UE_SRVCC_Capability_Type[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_SGSN_Number_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_Homogeneous_Support_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(Homogeneous_Support_Type[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
    	return;
    }
    
    private void encode_GMLC_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Address (out, (RecordValue)rv.getField("aVP_Data"));
    	return;
    }
    
	    private void encode_Context_Identifier_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
	    	return;
	    }
	    
	    private void encode_Service_Selection_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
	    	return;
	    }
    
		    private void encode_MIP_Home_Agent_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
		    	
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		    	encode_Address (out, (RecordValue)rv.getField("aVP_Data"));
		    	return;
		    }
		    
		    private void encode_MIP_Home_Agent_Host_AVP(ByteArrayOutputStream out, RecordValue rv){
		    	
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		    	encode_Destination_Realm_AVP(out,(RecordValue)rv.getField("destination_Realm"));
	            encode_Destination_Host_AVP(out, (RecordValue)rv.getField("destination_Host"));		    	
		    	encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));//optional
		    	return;
		    }
		    
		    private void encode_MIP6_Home_Link_Prefix_AVP(ByteArrayOutputStream out, RecordValue rv){
		    	
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
		    	return;
		    }
	    
	    private void encode_MIP6_Agent_Info_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
            encode_MIP_Home_Agent_Address_AVP(out,(RecordValue)rv.getField("mip_Home_Agent_Address1"));//            optional,
            encode_MIP_Home_Agent_Address_AVP(out,(RecordValue)rv.getField("mip_Home_Agent_Address2"));//            optional,
            encode_MIP_Home_Agent_Host_AVP(out, (RecordValue)rv.getField("mip_Home_Agent_Host"));//                optional,
            encode_MIP6_Home_Link_Prefix_AVP(out, (RecordValue)rv.getField("mip6_Home_Link_Prefix"));//              optional,
            encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));//optional
	    	return;
	    }
	    
	    private void encode_Visited_Network_Identifier_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    	return;
	    }
	    
	    private void encode_Specific_APN_Info_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_Service_Selection_AVP(out, (RecordValue)rv.getField("service_Selection"));//optional
            encode_MIP6_Agent_Info_AVP(out,(RecordValue)rv.getField("mip6_Agent_Info"));
            encode_Visited_Network_Identifier_AVP(out, (RecordValue)rv.getField("visited_Network_Identifier"));//optional,
            encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));//optional
	    	return;
	    }
	    
    private void encode_Active_APN_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Context_Identifier_AVP(out, (RecordValue)rv.getField("context_Identifier"));
        encode_Service_Selection_AVP(out, (RecordValue)rv.getField("service_Selection"));//optional
        encode_MIP6_Agent_Info_AVP(out, (RecordValue)rv.getField("mip6_Agent_Info"));//optional,
        encode_Visited_Network_Identifier_AVP(out, (RecordValue)rv.getField("visited_Network_Identifier"));//optional,
        encode_Specific_APN_Info_AVP(out, (RecordValue)rv.getField("specific_APN_Info"));//                optional,
        encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));//     optional
    	return;
    }
    
    private void encode_Error_Diagnostic_AVP(ByteArrayOutputStream out, RecordValue rv){

    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(Error_Diagnostic_Code_Type[((IntegerValue) rv.getField("error_Diagnostic_Code")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("error_Diagnostic_Code")).getInt()));
    	return;
    }
    
	    private void encode_Subscriber_Status_AVP(ByteArrayOutputStream out, RecordValue rv){
	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	//encode_IntegerValue(out,(Error_Diagnostic_Code_Type[((IntegerValue) rv.getField("error_Diagnostic_Code")).getInt()]));
	    	encode_IntegerValue(out,(((IntegerValue) rv.getField("subscriber_Status_Code")).getInt()));
	    	return;
	    }
	    
	    private void encode_Operator_Determined_Barring_AVP(ByteArrayOutputStream out, RecordValue rv){
	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
	    	return;
	    }
	    
        private void encode_Regional_Subscription_Zone_Code_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
        	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    	return;
	    }
	    
	    private void encode_Regional_Subscription_Zone_Code_AVP(ByteArrayOutputStream out, RecordOfValue rov){
	    	
	    	if ((rov == null) | rov.notPresent()) {return;}
	    	
			int len = rov.getLength();
			for (int i = 0; i < len; i++) {
			   RecordValue rv=(RecordValue) rov.getField(i);
			   //encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			   encode_Regional_Subscription_Zone_Code_AVP(out,rv);
			}
	    	return;
	    }
	    
	    private void encode_Access_Restriction_Data_AVP(ByteArrayOutputStream out, RecordValue rv){
	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
	    	return;
	    }
	    
	    private void encode_APN_OI_Replacement_AVP(ByteArrayOutputStream out, RecordValue rv){
	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
	    	return;
	    }
	    
	    private void encode_GMLC_Number_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    	return;
	    }
	    
	    private void encode_GMLC_Number_AVP(ByteArrayOutputStream out, RecordOfValue rov){
	    	
	    	if ((rov == null) | rov.notPresent()) {return;}

			int len = rov.getLength();
			for (int i = 0; i < len; i++) {
			   RecordValue rv=(RecordValue) rov.getField(i);
			   encode_GMLC_Number_AVP(out,rv);
			}
	    	return;
	    }
    
	    private void encode_SS_Code_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    	return;
	    }
	    
	    private void encode_SS_Status_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    	return;
	    }	    
	    
	    private void encode_LCS_Info_AVP(ByteArrayOutputStream out, RecordValue rv){
	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	        encode_GMLC_Number_AVP(out, (RecordOfValue)rv.getField("gMLC_Number"));//                        optional,
//	          set of LCS_PrivacyException_AVP        lCS_PrivacyException            optional,
//	          set of MO_LR_AVP                    mO_LR                            optional,
//	          set of AVP_Type                     aVP_Type                        optional
            return;
	    }
	    
	        private void encode_TS_Code_AVP(ByteArrayOutputStream out, RecordValue rv){   
	    	
	        	if ((rv == null) | rv.notPresent()) {return;}
	        	
	        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
				encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
				return;
	        }
	        	
        	private void encode_TS_Code_AVP(ByteArrayOutputStream out, RecordOfValue rov){
	    	
        		if ((rov == null) | rov.notPresent()) {return;}
	
				int len = rov.getLength();
				for (int i = 0; i < len; i++) {
				   RecordValue rv=(RecordValue) rov.getField(i);
				   encode_TS_Code_AVP(out,rv);
				}
		    	return;
		    }
	    
	    private void encode_Teleservice_List_AVP(ByteArrayOutputStream out, RecordValue rv){
	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_TS_Code_AVP(out, (RecordOfValue)rv.getField("tS_Code"));
            //set of AVP_Type                           aVP_Type                    optional	      
            return;
	    }
	    
	    
	    private void  encode_Call_Barring_Info_AVP(ByteArrayOutputStream out, RecordValue rv){
	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_SS_Code_AVP(out, (RecordValue)rv.getField("sS_Code"));
            encode_SS_Status_AVP(out, (RecordValue)rv.getField("sS_Status"));
//            set of AVP_Type                     aVP_Type                        optional	      
            return;
	    }
	    
	    private void encode_Call_Barring_Info_AVP(ByteArrayOutputStream out, RecordOfValue rov){
	    	
	    	if ((rov == null) | rov.notPresent()) {return;}

			int len = rov.getLength();
			for (int i = 0; i < len; i++) {
			   RecordValue rv=(RecordValue) rov.getField(i);
			   //encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			   encode_Call_Barring_Info_AVP(out,rv);
			}
	    	return;
	    }
	    
	    
		    private void encode_All_APN_Conf_Included_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
	
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		    	//encode_IntegerValue(out,(Error_Diagnostic_Code_Type[((IntegerValue) rv.getField("error_Diagnostic_Code")).getInt()]));
		    	encode_IntegerValue(out,(((IntegerValue) rv.getField("all_APN_Conf_Included_Indicator")).getInt()));
		    	return;
		    }
		    
		    private void encode_APN_Configuration_AVP(ByteArrayOutputStream out, RecordValue rv) {                                        //ref: 7.3.35
                
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		        encode_Context_Identifier_AVP(out, (RecordValue)rv.getField("context_Identifier"));
                //set length (2) of Served_Party_IP_Address_AVP    served_Party        optional,
                encode_PDN_Type_AVP(out, (RecordValue)rv.getField("pDN_Type"));
                encode_Service_Selection_AVP(out, (RecordValue)rv.getField("service_Selection"));
//                EPS_Subscribed_QoS_Profile_AVP        ePS_Subscribed_QoS_Profile        optional,
//                VPLMN_Dynamic_Address_Allowed_AVP    vPLMN_Dynamic_Address_Allowed    optional,
                encode_MIP6_Agent_Info_AVP(out, (RecordValue)rv.getField("mIP6_Agent_Info"));//                    optional,
//                Visited_Network_Identifier_AVP        visited_Network_Identifier        optional,
//                PDN_GW_Allocation_Type_AVP            pDN_GW_Allocation_Type            optional,
//                ThreeGPP_Charging_Char_AVP            threeGPP_Charging_Char            optional,
//                AMBR_AVP                            aMBR                            optional,
//                set of Specific_APN_Info_AVP        specific_APN_Info                optional,
//                APN_OI_Replacement_AVP                aPN_OI_Replacement                optional,
//                SIPTO_Permission_AVP                sIPTO_Permission                optional,
//                LIPA_Permission_AVP                    lIPA_Permission                    optional,
                encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));//                        optional
            }
		    
		    
		    private void encode_APN_Configuration_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		    	
		    	if ((rov == null) | rov.notPresent()) {return;}

				int len = rov.getLength();
				for (int i = 0; i < len; i++) {
				   RecordValue rv=(RecordValue) rov.getField(i);
				   //encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
				   encode_APN_Configuration_AVP(out,rv);
				}
		    	return;
		    }
	    
	    private void encode_APN_Configuration_Profile_AVP(ByteArrayOutputStream out, RecordValue rv){
	
	    	if ((rv == null) | rv.notPresent()) {return;}     
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	        encode_Context_Identifier_AVP(out, (RecordValue)rv.getField("context_Identifier"));
	        encode_All_APN_Conf_Included_Id_AVP(out, (RecordValue)rv.getField("all_APN_Conf_Included_Id"));
	        encode_APN_Configuration_AVP(out, (RecordOfValue)rv.getField("aPN_Configuration"));
	        encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));  
	        return;
	    }
    
    private void encode_PDN_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("pDN_Type_Code")).getInt()));
    	return;
    }
    
    private void encode_Complete_DL_Included_Indicator_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("complete_DL_Included_Indicator")).getInt()));
    	return;
    }
    
    private void encode_PDP_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_QoS_Subscribed_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_PDP_Context_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Context_Identifier_AVP(out, (RecordValue)rv.getField("context_Identifier"));
        encode_PDP_Type_AVP(out, (RecordValue)rv.getField("pDP_Type"));
//         PDP_Address_AVP                        pDP_Address                        optional,
        encode_QoS_Subscribed_AVP(out, (RecordValue)rv.getField("qoS_Subscribed"));
//         VPLMN_Dynamic_Address_Allowed_AVP    vPLMN_Dynamic_Address_Allowed    optional,
        encode_Service_Selection_AVP(out, (RecordValue)rv.getField("service_Selection"));
//         ThreeGPP_Charging_Char_AVP            threeGPP_Charging_Char            optional,
//         Ext_PDP_Type_AVP                    ext_PDP_Type                    optional,
//         Ext_PDP_Address_AVP                    ext_PDP_Address                    optional,
//         AMBR_AVP                            aMBR                            optional,
//         SIPTO_Permission_AVP                sIPTO_Permission                optional,
//         LIPA_Permission_AVP                    lIPA_Permission                    optional,
//         set of AVP_Type                     aVP_Type                        optional
    	return;
    }
    
    private void encode_PDP_Context_AVP(ByteArrayOutputStream out, RecordOfValue rov){
    	
    	if ((rov == null) | rov.notPresent()) {return;}
    	
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   //encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		   encode_PDP_Context_AVP(out,rv);
		} 	
    	return;
    }
    
    private void encode_GPRS_Subscription_Data_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Complete_DL_Included_Indicator_AVP(out, (RecordValue)rv.getField("complete_Data_List_Included_Indicator"));
        encode_PDP_Context_AVP(out, (RecordOfValue)rv.getField("pDP_Context"));
       // set of AVP_Type                     aVP_Type                        optional
    	return;
    }
    
	    private void encode_CSG_Id_AVP(ByteArrayOutputStream out, RecordValue rv){
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
	    	return;
	    }
    
    private void encode_CSG_Subscription_Data_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_CSG_Id_AVP(out, (RecordValue)rv.getField("cSG_Id"));
//        Expiration_Date_AVP                    expiration_Date                    optional,
//        set of Service_Selection_AVP        service_Selection                optional,
//        set of AVP_Type                     aVP_Type                        optional
        return;	
    }
	    
    private void encode_Subscription_Data_AVP(ByteArrayOutputStream out, RecordValue rv){

    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_Subscriber_Status_AVP(out, (RecordValue)rv.getField("subscriber_Status"));//                optional,
//    	encode_MSISDN_AVP(out, (RecordValue)rv.getField("mSISDN"));//                             optional,
//    	encode_STN_SR_AVP(out, (RecordValue)rv.getField("sTN_SR"));//                             optional,
//    	encode_ICS_Indicator_AVP(out, (RecordValue)rv.getField("iCS_Indicator"));//                     optional,
//    	encode_Network_Access_Mode_AVP(out, (RecordValue)rv.getField("network_Access_Mode"));//                 optional,
    	encode_Operator_Determined_Barring_AVP(out, (RecordValue)rv.getField("operator_Determined_Barring"));//         optional,
//    	encode_HPLMN_ODB_AVP(out, (RecordValue)rv.getField("hPLMN_ODB"));//                         optional,
        encode_Regional_Subscription_Zone_Code_AVP(out, (RecordOfValue)rv.getField("zone_Code"));//     optional,
        encode_Access_Restriction_Data_AVP(out, (RecordValue)rv.getField("access_Restriction_Data"));//             optional,
        encode_APN_OI_Replacement_AVP(out, (RecordValue)rv.getField("aPN_OI_Replacement"));//                 optional,
        encode_LCS_Info_AVP(out, (RecordValue)rv.getField("lCS_Info"));//                         optional,
        encode_Teleservice_List_AVP(out, (RecordValue)rv.getField("teleservice_List"));//                 optional,
        encode_Call_Barring_Info_AVP(out, (RecordOfValue)rv.getField("call_Barring_Info"));//                 optional,
//        encode_ThreeGPP_Charging_Char_AVP(out, (RecordValue)rv.getField("threeGPP_Charging_Char"));//             optional,
//        encode_AMBR_AVP(out, (RecordValue)rv.getField("aMBR"));//                             optional,
        encode_APN_Configuration_Profile_AVP(out, (RecordValue)rv.getField("aPN_Configuration_Profile"));//         optional,
//        encode_RAT_Freq_Sel_Priority_ID_AVP(out, (RecordValue)rv.getField("rAT_Freq_Sel_Priority_ID"));//         optional,
//        encode_Trace_Data_AVP(out, (RecordValue)rv.getField("trace_Data"));//                         optional,
        encode_GPRS_Subscription_Data_AVP(out, (RecordValue)rv.getField("gPRS_Subscription_Data"));//             optional,
        encode_CSG_Subscription_Data_AVP(out, (RecordValue)rv.getField("cSG_Subscription_Data"));//             optional,
//        encode_Roaming_Rest_Unsupp_Feature_AVP(out, (RecordValue)rv.getField("roaming_Rest_Unsupp_Feature"));//         optional,
//        encode_RAU_TAU_Timer_AVP(out, (RecordValue)rv.getField("subscribed_Perio_RAUTAU_Timer"));//     optional,
//        encode_MPS_Priority_AVP(out, (RecordValue)rv.getField("mPS_Priority"));//                     optional,
//        encode_VPLMN_LIPA_Allowed_AVP(out, (RecordValue)rv.getField("vPLMN_LIPA_Allowed"));//                 optional,
//        encode_Relay_Node_Indicator_AVP(out, (RecordValue)rv.getField("relay_Node_Indicator"));//             optional,
//        encode_MDT_User_Consent_AVP(out, (RecordValue)rv.getField("mDT_User_Consent"));//                 optional,
//        encode_AVP_Type(out, (RecordOfValue)rv.getField("aVP_Type"));//                         optional
    	return;
    }
    
    private void encode_Number_Of_Requested_Vectors_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
        private void encode_Immediate_Response_Preferred_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
        	return;
        }
        
        private void encode_Re_synchronization_Info_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	    	encode_Octetstring(out,(OctetstringValue) rv.getField("aVP_Data"));
	    	return;
        }
    
    private void encode_Req_EUTRAN_Auth_Info_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));	
    	encode_Number_Of_Requested_Vectors_AVP(out, (RecordValue)rv.getField("number_Of_Requested_Vectors"));//        optional,
        encode_Immediate_Response_Preferred_AVP(out, (RecordValue)rv.getField("immediate_Response_Preferred")); //   optional,
        encode_Re_synchronization_Info_AVP(out, (RecordValue)rv.getField("re_synchronization_Info"));      //      optional,
        //set of AVP_Type                     aVP_Type                        optional
    	return;
    }

    private void encode_Req_UTRAN_GERAN_Auth_Info_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));  	
    	encode_Number_Of_Requested_Vectors_AVP(out, (RecordValue)rv.getField("number_Of_Requested_Vectors"));//        optional,
        encode_Immediate_Response_Preferred_AVP(out, (RecordValue)rv.getField("immediate_Response_Preferred")); //   optional,
        encode_Re_synchronization_Info_AVP(out, (RecordValue)rv.getField("re_synchronization_Info"));      //      optional,
        //set of AVP_Type                     aVP_Type                        optional
    	return;
    }
    
    private void encode_Cancellation_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(Cancellation_Type_Code_Type[((IntegerValue) rv.getField("cancellation_Type")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("cancellation_Type")).getInt()));
    	return;
    }
    
    private void encode_CLR_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_IDR_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_DSR_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_DSA_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_PUER_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_PUEA_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_IDA_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_NOR_Flags_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_IMS_Voice_PS_Sessions_Support_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
    	//encode_IntegerValue(out,(Cancellation_Type_Code_Type[((IntegerValue) rv.getField("iMS_Voice_Over_PS_Sessions_Supported")).getInt()]));
    	encode_IntegerValue(out,(((IntegerValue) rv.getField("iMS_Voice_Over_PS_Sessions_Supported")).getInt()));
    	return;
    }
    
    private void encode_Last_UE_Activity_Time_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
    	return;
    }
    
    private void encode_User_Id_AVP(ByteArrayOutputStream out, RecordOfValue rov){
    	
    	if ((rov == null) | rov.notPresent()) {return;}

		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	       encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		}
    	return;
    }
    //*End TS129_272 AVP codes
    
    
    /*TS129_299 AVP codes*/
        private void encode_Primary_Event_Charging_Function_Name_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	//encode_DiameterURI(out, (RecordValue)rv.getField("aVP_Data"));
        	encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
        	return;
        }
        private void encode_Secondary_Event_Charging_Function_Name_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	//encode_DiameterURI(out, (RecordValue)rv.getField("aVP_Data"));
        	encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
        	return;
        }
        private void encode_Primary_Charging_Collection_Function_Name_AVP(ByteArrayOutputStream out, RecordValue rv){
        	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	//encode_DiameterURI(out, (RecordValue)rv.getField("aVP_Data"));
        	encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
        	return;
        }
        private void encode_Secondary_Charging_Collection_Function_Name_AVP(ByteArrayOutputStream out, RecordValue rv){
         	
        	if ((rv == null) | rv.notPresent()) {return;}
        	
        	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        	//encode_DiameterURI(out, (RecordValue)rv.getField("aVP_Data"));
        	encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
        	return;
        }
    
    private void encode_Charging_Information_AVP(ByteArrayOutputStream out, RecordValue rv){
    	
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));//<AVP header: 618 ....>
        encode_Primary_Event_Charging_Function_Name_AVP(out, (RecordValue)rv.getField("primary_Event_Charging_Function_Name"));// optional,
        encode_Secondary_Event_Charging_Function_Name_AVP(out, (RecordValue)rv.getField("secondary_Event_Charging_Function_Name"));// optional,
        encode_Primary_Charging_Collection_Function_Name_AVP(out, (RecordValue)rv.getField("primary_Charging_Collection_Function_Name"));// optional,
        encode_Secondary_Charging_Collection_Function_Name_AVP(out, (RecordValue)rv.getField("secondary_Charging_Collection_Function_Name"));// optional,
        //set of AVP_Type  
        return;
    }
    
    //*End TS129_299 AVP codes
    
    /*TS132_299*/
		    private void encode_Quota_Consumption_Time_AVP(ByteArrayOutputStream out, RecordValue rv) {
		        
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
				encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
		    	return;
		    }
		    
			    private void encode_Time_Quota_Type_AVP(ByteArrayOutputStream out, RecordValue rv) {
			        
			    	if ((rv == null) | rv.notPresent()) {return;}
			    	
			    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
					encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
			    	return;
			    }
			    
			    private void encode_Base_Time_Interval_AVP(ByteArrayOutputStream out, RecordValue rv) {
			        
			    	if ((rv == null) | rv.notPresent()) {return;}
			    	
			    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
					encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
			    	return;
			    }
		    
		    private void encode_Time_Quota_Mechanism_AVP(ByteArrayOutputStream out, RecordValue rv) {
                
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
                encode_Time_Quota_Type_AVP(out, (RecordValue)rv.getField("time_Quota_Type")); //optional,
                encode_Base_Time_Interval_AVP(out, (RecordValue)rv.getField("base_Time_Interval"));// optional 
            }
		    
		    private void encode_Envelope_Start_Time_AVP(ByteArrayOutputStream out, RecordValue rv) {
		        
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
				encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
		    	return;
		    }
		    
		    private void encode_Envelope_End_Time_AVP(ByteArrayOutputStream out, RecordValue rv) {
		        
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
				encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
		    	return;
		    }
		    
		    private void encode_Envelope_AVP(ByteArrayOutputStream out, RecordValue rv) {
		        
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));//
		    	encode_Envelope_Start_Time_AVP(out, (RecordValue)rv.getField("envelope_Start_Time"));//,
		    	encode_Envelope_End_Time_AVP(out, (RecordValue)rv.getField("envelope_End_Time"));// optional,
		    	encode_CC_Total_Octets_AVP(out, (RecordValue)rv.getField("cC_Total_Octets"));// optional,
		    	encode_CC_Input_Octets_AVP(out, (RecordValue)rv.getField("cC_Input_Octets"));// optional,
		    	encode_CC_Output_Octets_AVP(out, (RecordValue)rv.getField("cC_Output_Octets"));// optional,
		    	encode_CC_Service_Specific_Units_AVP(out, (RecordValue)rv.getField("cC_Service_Specific_Units"));// optional,
                //set of AVP_Type                 aVP_Type optional
		    	return;
		    }
		    
		    private void encode_Envelope_AVP(ByteArrayOutputStream out, RecordOfValue rov){
				 
		    	if ((rov == null) | rov.notPresent()) {return;}
		    
				int len = rov.getLength();
				for (int i = 0; i < len; i++) {
				   RecordValue rv=(RecordValue) rov.getField(i);
				   encode_Envelope_AVP(out, rv);
				}
				return;
			}
		    
		    private void encode_Envelope_Reporting_AVP(ByteArrayOutputStream out, RecordValue rv) {
		        
		    	if ((rv == null) | rv.notPresent()) {return;}
		    	
		    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
				encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
		    	return;
		    }
		    
	    private void encode_Offline_Charging_AVP(ByteArrayOutputStream out, RecordValue rv) {
	    	
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
	    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
	        encode_Quota_Consumption_Time_AVP(out, (RecordValue)rv.getField("quota_Consumption_Time"));// optional,
	        encode_Time_Quota_Mechanism_AVP(out, (RecordValue)rv.getField("time_Quota_Mechanism"));// optional,
	        encode_Envelope_Reporting_AVP(out, (RecordValue)rv.getField("envelope_Reporting"));// optional,
	        encode_Multiple_Services_Credit_Control_AVP(out, (RecordOfValue)rv.getField("multiple_Services_Credit_Control"));// optional,
	        //set of AVP_Type                             aVP_Type optional
	        return;
	    }
    
    private void encode_PS_Information_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
//		 ThreeGPP_Charging_Id_AVP                       threeGPP_Charging_Id optional,
//         PDN_Connection_Charging_ID_AVP                 pdn_Connection_Charging_ID optional,
//         Node_Id_AVP                                    node_Id optional,
//         ThreeGPP_PDP_Type_AVP                          threeGPP_PDP_Type optional,
//         set of PDP_Address_AVP                         pdp_Address optional,
//         PDP_Address_Prefix_Length_AVP                  pdp_Address_Prefix_Length optional,
//         Dynamic_Address_Flag_AVP                       dynamic_Address_Flag optional,
//         Dynamic_Address_Flag_Extension_AVP             dynamic_Address_Flag_Extension optional,
//         QoS_Information_AVP                            qos_Information optional,
//         set of SGSN_Address_AVP                        sgsn_Address optional,
//         set of GGSN_Address_AVP                        ggsn_Address optional,
//         set of SGW_Address_AVP                         sgw_Address optional,
//         CG_Address_AVP                                 cg_Address optional,
//         Serving_Node_Type_AVP                          serving_Node_Type optional,
//         SGW_Change_AVP                                 sgw_Change optional,
//         ThreeGPP_IMSI_MCC_MNC_AVP                      threeGPP_IMSI_MCC_MNC optional,
//         IMSI_Unauthenticated_Flag_AVP                  imsi_Unauthenticated_Flag optional,
//         ThreeGPP_GGSN_MCC_MNC_AVP                      threeGPP_GGSN_MCC_MNC optional,
//         ThreeGPP_NSAPI_AVP                             threeGPP_NSAPI optional,
//         Called_Station_Id_AVP                          called_Station_Id optional,
//         ThreeGPP_Session_Stop_Indicator_AVP            threeGPP_Session_Stop_Indicator optional,
//         ThreeGPP_Selection_Mode_AVP                    threeGPP_Selection_Mode optional,
//         ThreeGPP_Charging_Characteristics_AVP          threeGPP_Charging_Characteristics optional,
//         Charging_Characteristics_Selection_Mode_AVP    charging_Characteristics_Selection_Mode optional,
//         ThreeGPP_SGSN_MCC_MNC_AVP                      threeGPP_SGSN_MCC_MNC optional,
//         ThreeGPP_MS_TimeZone_AVP                       threeGPP_MS_TimeZone optional,
//         Charging_Rule_Base_Name_AVP                    charging_Rule_Base_Name optional,
//         ThreeGPP_User_Location_Info_AVP                threeGPP_User_Location_Info optional,
//         User_CSG_Information_AVP                       user_CSG_Information optional,
//         ThreeGPP2_BSID_AVP                             threeGPP2_BSID optional,
//         ThreeGPP_RAT_Type_AVP                          threeGPP_RAT_Type optional,
//         PS_Furnish_Charging_Information_AVP            ps_Furnish_Charging_Information optional,
//         PDP_Context_Type_AVP                           pdp_Context_Type optional,
         encode_Offline_Charging_AVP(out, (RecordValue)rv.getField("offline_Charging"));// optional,
//         set of Traffic_Data_Volumes_AVP                traffic_Data_Volumes optional,
//         set of Service_Data_Container_AVP              service_Data_Container optional,
//         User_Equipment_Info_AVP                        user_Equipment_Info optional,
//         Terminal_Information_AVP                       terminal_Information optional,
//         Start_Time_AVP                                 start_Time optional,
//         Stop_Time_AVP                                  stop_Time optional,
//         Change_Condition_AVP                           change_Condition optional,
//         Diagnostics_AVP                                diagnostics optional,
//         Low_Priority_Indicator_AVP                     low_Priority_Indicator optional
        return;
	}
    
    private void encode_WLAN_Information_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
//		  WLAN_Session_Id_AVP							wLAN_Session_Id optional,
//        PDG_Address_AVP								pDG_Address optional,
//        PDG_Charging_Id_AVP							pDG_Charging_Id optional,
//        WAG_Address_AVP								wAG_Address optional,
//        WAG_PLMN_Id_AVP			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));					wAG_PLMN_Id optional,
//        WLAN_Radio_Container_AVP						wLAN_Radio_Container optional,
//        WLAN_UE_Local_IPAddress_AVP					wLAN_UE_Local_IPAddress optional
        return;
	}
    
    private void encode_Event_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Expires_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Event_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_SIP_Method_AVP(out, (RecordValue)rv.getField("sIP_Method"));
		encode_Event_AVP(out, (RecordValue)rv.getField("event"));
		encode_Expires_AVP(out, (RecordValue)rv.getField("expires"));
		return;
	}
    
    private void encode_Role_Of_Node_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Role_Of_Node[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;
	}
    
    private void encode_Node_Functionality_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		//encode_IntegerValue(out,(Node_Functionality[((IntegerValue) rv.getField("aVP_Data")).getInt()]));
		encode_IntegerValue(out,(((IntegerValue) rv.getField("aVP_Data")).getInt()));
		return;
	}
    
    private void encode_User_Session_ID_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Calling_Party_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Calling_Party_Address_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Calling_Party_Address_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Called_Party_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Called_Asserted_Identity_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Called_Asserted_Identity_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Called_Asserted_Identity_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Alternate_Charged_Party_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Requested_Party_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Associated_URI_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Associated_URI_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Associated_URI_AVP(out, rv);
		}
		return;
	}
    
    private void encode_SIP_Request_Timestamp_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_SIP_Response_Timestamp_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Time_Stamps_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_SIP_Request_Timestamp_AVP(out, (RecordValue)rv.getField("sIP_Request_Timestamp"));
		encode_SIP_Response_Timestamp_AVP(out, (RecordValue)rv.getField("sIP_Response_Timestamp")); 
		return;
	}
    
    private void encode_Application_Server_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
   
    private void encode_Application_Provided_Called_Party_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Application_Provided_Called_Party_Address_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Application_Provided_Called_Party_Address_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Application_Server_Information_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Application_Server_AVP(out, (RecordValue)rv.getField("application_Server"));
        encode_Application_Provided_Called_Party_Address_AVP(out, (RecordOfValue)rv.getField("application_Provided_Called_Party_Address"));
		return;
	}
    
    private void encode_Application_Server_Information_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Application_Server_Information_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Originating_IOI_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Terminating_IOI_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
        
    private void encode_Inter_Operator_Identifier_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Originating_IOI_AVP(out, (RecordValue)rv.getField("originating_IOI"));
        encode_Terminating_IOI_AVP(out, (RecordValue)rv.getField("terminating_IOI"));
		return;
	}
    
    private void encode_Inter_Operator_Identifier_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_Inter_Operator_Identifier_AVP(out, rv);
		}
		return;
	}
    
    private void encode_IMS_Charging_Identifier_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_SDP_Session_Description_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_SDP_Session_Description_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_SDP_Session_Description_AVP(out, rv);
		}
		return;
	}
    
    private void encode_SDP_Media_Component_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
//		encode_SDP_Media_Name_AVP(out, (RecordValue)rv.getField("sDP_Media_Name"));
//        encode_SDP_Media_Description_AVP(out, (RecordOfValue)rv.getField("sDP_Media_Description"));
//        encode_Media_Initiator_Flag_AVP(out, (RecordValue)rv.getField("media_Initiator_Flag"));
//        encode_Media_Initiator_Party_AVP(out, (RecordValue)rv.getField("media_Initiator_Party"));
//        encode_Authorized_QoS_AVP(out, (RecordValue)rv.getField("authorized_QoS"));
//        encode_ThreeGPP_Charging_Id_AVP(out, (RecordValue)rv.getField("threeGPP_Charging_Id"));
//        encode_Access_Network_Charging_Identifier_Value_AVP(out, (RecordValue)rv.getField("access_Network_Charging_Identifier_Value"));
		return;
	}
    
    private void encode_SDP_Media_Component_AVP(ByteArrayOutputStream out, RecordOfValue rov){
		 
    	if ((rov == null) | rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
		   RecordValue rv=(RecordValue) rov.getField(i);
		   encode_SDP_Media_Component_AVP(out, rv);
		}
		return;
	}
    
    private void encode_Served_Party_IP_Address_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Address(out,(RecordValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Incoming_Trunk_Group_ID_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Outgoing_Trunk_Group_ID_AVP(ByteArrayOutputStream out, RecordValue rv){
		 
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
		return;
	}
    
    private void encode_Trunk_Group_ID_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Incoming_Trunk_Group_ID_AVP(out, (RecordValue)rv.getField("incoming_Trunk_Group_ID"));
        encode_Outgoing_Trunk_Group_ID_AVP(out, (RecordValue)rv.getField("outgoing_Trunk_Group_ID"));
		return;
	}
    
    private void encode_IMS_Information_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Event_Type_AVP(out, (RecordValue)rv.getField("event_Type"));
        encode_Role_Of_Node_AVP(out, (RecordValue)rv.getField("role_Of_Node"));
        encode_Node_Functionality_AVP(out, (RecordValue)rv.getField("node_Functionality"));
        encode_User_Session_ID_AVP(out, (RecordValue)rv.getField("user_Session_Id"));
//        Outgoing_Session_Id_AVP                     outgoing_Session_Id optional,
//        Session_Priority_AVP                        session_Priority optional,
        encode_Calling_Party_Address_AVP(out, (RecordOfValue)rv.getField("calling_Party_Address"));
        encode_Called_Party_Address_AVP(out, (RecordValue)rv.getField("called_Party_Address"));
        encode_Called_Asserted_Identity_AVP(out, (RecordOfValue)rv.getField("called_Asserted_Identity"));
//        Number_Portability_Routing_Information_AVP  number_Portability_Routing_Information optional,
//        Carrier_Select_Routing_Information_AVP      carrier_Select_Routing_Information optional,
        encode_Alternate_Charged_Party_Address_AVP(out, (RecordValue)rv.getField("alternate_Charged_Party_Address"));
        encode_Requested_Party_Address_AVP(out, (RecordValue)rv.getField("requested_Party_Address"));
        encode_Associated_URI_AVP(out, (RecordOfValue)rv.getField("associated_URI"));
        encode_Time_Stamps_AVP(out, (RecordValue)rv.getField("time_Stamps"));
        encode_Application_Server_Information_AVP(out, (RecordOfValue)rv.getField("application_Server_Information"));
        encode_Inter_Operator_Identifier_AVP(out, (RecordOfValue)rv.getField("inter_Operator_Identifier"));
        encode_IMS_Charging_Identifier_AVP(out, (RecordValue)rv.getField("ims_Charging_Identifier"));
        encode_SDP_Session_Description_AVP(out, (RecordOfValue)rv.getField("sdp_Session_Description"));
        encode_SDP_Media_Component_AVP(out, (RecordOfValue)rv.getField("sdp_Media_Component"));
        encode_Served_Party_IP_Address_AVP(out, (RecordValue)rv.getField("served_Party_IP_Address"));
        encode_Server_Capabilities_AVP(out, (RecordValue)rv.getField("server_Capabilities"));
        encode_Trunk_Group_ID_AVP(out, (RecordValue)rv.getField("trunk_Group_ID"));
//        Bearer_Service_AVP                          bearer_Service optional,
//        Service_Id_AVP                              service_Id optional,
//        set of Service_Specific_Info_AVP            service_Specific_Info optional,
//        set of Message_Body_AVP                     message_Body optional,
//        Cause_Code_AVP                              cause_Code optional,
//        Access_Network_Information_AVP              access_Network_Information optional,
//        set of Early_Media_Description_AVP          early_Media_Description optional, 
//        IMS_Communication_Service_Identifier_AVP    ims_Communication_Service_Identifier optional, 
//        Online_Charging_Flag_AVP                    online_Charging_Flag optional,
//        Real_Time_Tariff_Information_AVP            real_Time_Tariff_Information optional,
//        Account_Expiration_AVP                      account_Expiration optional,
//        Initial_IMS_Charging_Identifier_AVP         initial_IMS_Charging_Identifier optional, 
//        IMS_Emergency_Indicator_AVP                 ims_Emergency_Indicator optional
        return;
	}
    
	    private void encode_Application_Server_ID_AVP(ByteArrayOutputStream out, RecordValue rv){
			
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
			return;
		}
    
	    private void encode_Application_Service_Type_AVP(ByteArrayOutputStream out, RecordValue rv){
			
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
			return;
		}
	    
	    private void encode_Application_Session_ID_AVP(ByteArrayOutputStream out, RecordValue rv){
			
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			//encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
			encode_IntegerValue(out,(IntegerValue) rv.getField("aVP_Data"));
			return;
		}
	    
	    private void encode_Delivery_Status_AVP(ByteArrayOutputStream out, RecordValue rv){
			
	    	if ((rv == null) | rv.notPresent()) {return;}
	    	
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_UniversalCharstring(out,(UniversalCharstringValue) rv.getField("aVP_Data"));
			return;
		}
	    
    private void encode_Service_Generic_Information_AVP(ByteArrayOutputStream out, RecordValue rv) {
        
    	if ((rv == null) | rv.notPresent()) {return;}
    	
    	encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
        encode_Application_Server_ID_AVP(out, (RecordValue)rv.getField("application_Server_ID"));// optional,
        encode_Application_Service_Type_AVP(out, (RecordValue)rv.getField("application_Service_Type"));// optional,
        encode_Application_Session_ID_AVP(out, (RecordValue)rv.getField("application_Session_ID"));// optional,
        encode_Delivery_Status_AVP(out, (RecordValue)rv.getField("delivery_Status"));// optional
        return;
    }
    
    private void encode_Service_Information_AVP(ByteArrayOutputStream out, RecordValue rv){
		
    	if ((rv == null) | rv.notPresent()) {return;}
    	
		encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
		encode_Subscription_Id_AVP(out, (RecordOfValue)rv.getField("subscription_Id"));
//        AoC_Information_AVP             aoc_Information optional,
        encode_PS_Information_AVP(out, (RecordValue)rv.getField("ps_Information"));
        encode_WLAN_Information_AVP(out, (RecordValue)rv.getField("wlan_Information"));
        encode_IMS_Information_AVP(out, (RecordValue)rv.getField("ims_Information"));
//        MMS_Information_AVP             mms_Information optional,
//        LCS_Information_AVP             lcs_Information optional,
//        PoC_Information_AVP             poc_Information optional,
//        MBMS_Information_AVP            mbms_Information optional,
//        SMS_Information_AVP             sms_Information optional,
//        MMTel_Information_AVP           mmtel_Information optional,
        encode_Service_Generic_Information_AVP(out, (RecordValue)rv.getField("service_Generic_Information"));// optional,
//        IM_Information_AVP              im_Information optional,
//        DCD_Information_AVP             dcd_Information optional
	    return;	
	}
    /*End TS132_299 AVPs*/

	
	private void encode_AVP_Type_Data(ByteArrayOutputStream out, UnionValue uv){
		
		if ((uv == null) || uv.notPresent()) {return;}
		
		try{ 
		if (uv.getPresentVariantName().equals("oCTETSTRING")) {
			 encode_Octetstring(out,(OctetstringValue) uv.getVariant("oCTETSTRING"));
		 }
		 if (uv.getPresentVariantName().equals("cHARSTRING")) {
			 encode_Charstring(out,(CharstringValue) uv.getVariant("cHARSTRING"));
		 }
		 if (uv.getPresentVariantName().equals("iNTEGER32")) {
			 encode_IntegerValue(out,(IntegerValue) uv.getVariant("iNTEGER32"));
		 }
		 if (uv.getPresentVariantName().equals("iNTEGER64")) {
			 encode_Integer64Value(out,(IntegerValue) uv.getVariant("iNTEGER64"));
		 }
		 if (uv.getPresentVariantName().equals("uNSIGNED32")) {
			 encode_IntegerValue(out,(IntegerValue) uv.getVariant("uNSIGNED32"));
		 }
		 if (uv.getPresentVariantName().equals("uNSIGNED64")) {
			 encode_Integer64Value(out,(IntegerValue) uv.getVariant("uNSIGNED64"));
		 }
		}catch(Exception ex){
			
		}
		 //TODO: Check and encode set of ....
		 // FLOAT32     fLOAT32,
 	     // FLOAT64     fLOAT64,
		 // GROUPED     gROUPED
		 return;
	}
	
    private void encode_AVP_Type(ByteArrayOutputStream out, RecordOfValue rov){
		
    	if ((rov == null) || rov.notPresent()) {return;}
    
		int len = rov.getLength();
		for (int i = 0; i < len; i++) {
			RecordValue rv=(RecordValue) rov.getField(i);
			
			encode_AVPHeader(out, (RecordValue)rv.getField("aVP_Header"));
			encode_AVP_Type_Data(out,(UnionValue) rv.getField("aVP_Data") );
		}		
		return;
    }
	
	private void encode_Charstring(ByteArrayOutputStream out, CharstringValue value) {
		
		if ((value == null) || value.notPresent()) {return;}

		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < value.getLength(); i++) {
			sb.append(value.getChar(i));
		}
		write(out, sb.toString());
		
		if (padding)
		    write_avp_padding(out,value.getLength());
	}
	
	private void encode_UniversalCharstring(ByteArrayOutputStream out, UniversalCharstringValue value) {
		
		if ((value == null) || value.notPresent()) {return;}
     
		StringBuffer sb = new StringBuffer();
        sb.append(value.getString());
		write(out, sb.toString());
		if (padding)
		    write_avp_padding(out,sb.length());
	}
	
	private void encode_Octetstring(ByteArrayOutputStream out, OctetstringValue value) {
		
		if ((value == null) || value.notPresent()) {return;}

		byte[] OctetVal= new byte[value.getLength()];

		for (int i = 0; i < value.getLength(); i++) {
			OctetVal[i]=(byte)value.getOctet(i);
		}
		write(out, OctetVal);
		if (padding)
		    write_avp_padding(out,value.getLength());
	}
	
	private void encode_IntegerValue(ByteArrayOutputStream out, IntegerValue value) {
		
		if ((value == null) || value.notPresent()) {return;}
		
		int ivalue=value.getInt(); 
		byte[] IntVal= new byte[]{(byte)(ivalue >>> 24),
                                  (byte)(ivalue >>> 16),
                                  (byte)(ivalue >>> 8),
                                  (byte)ivalue};	
		write(out,IntVal);
	}
	
	private void encode_IntegerValue(ByteArrayOutputStream out, int value) {
		
		int ivalue=value;
		byte[] IntVal= new byte[]{(byte)(ivalue >>> 24),
                                  (byte)(ivalue >>> 16),
                                  (byte)(ivalue >>> 8),
                                  (byte)ivalue};
		write(out,IntVal);
	}
	
	private void encode_Integer64Value(ByteArrayOutputStream out, IntegerValue value) {
		
		if ((value == null) || value.notPresent()) {return;}
		
		//byte[] IntVal= new byte[8];
		//IntVal = String.valueOf(value).getBytes();
		
		int ivalue=value.getInt();
		byte[] IntVal64= new byte[]{
				(byte)((ivalue& 0xffffffffL) >>> 56),
				(byte)((ivalue& 0xffffffffL) >>> 48),
                (byte)((ivalue& 0xffffffffL) >>> 40),
                (byte)((ivalue& 0xffffffffL) >>> 32),
				(byte)((ivalue& 0xffffffffL) >>> 24),
                (byte)((ivalue& 0xffffffffL) >>> 16),
                (byte)((ivalue& 0xffffffffL) >>> 8),
                (byte)(ivalue& 0xffffffffL)};
		
		write(out,IntVal64);
		
	}
	
	private void encode_Integer64Value(ByteArrayOutputStream out, long ivalue) {
		
		//byte[] IntVal= new byte[8];
		//IntVal = String.valueOf(ivalue).getBytes();
		
		byte[] IntVal64= new byte[]{
				(byte)((ivalue& 0xffffffffL) >>> 56),
				(byte)((ivalue& 0xffffffffL) >>> 48),
                (byte)((ivalue& 0xffffffffL) >>> 40),
                (byte)((ivalue& 0xffffffffL) >>> 32),
				(byte)((ivalue& 0xffffffffL) >>> 24),
                (byte)((ivalue& 0xffffffffL) >>> 16),
                (byte)((ivalue& 0xffffffffL) >>> 8),
                (byte)(ivalue& 0xffffffffL)};
		
		write(out,IntVal64);
		//System.out.println(ivalue+" "+IntVal64);
	}
	
    private void write_avp_padding(ByteArrayOutputStream out, int avp_data_len) {

		final byte[] NullVal = {0x00};
		
		if ((avp_data_len % 4)!=0){
			for (int i = 0; i < 4-((avp_data_len % 4)); i++) {	
				write(out, NullVal);
			}
		}
	}
	
	private void write(ByteArrayOutputStream out, String outString) {
		
		try {
			out.write(outString.getBytes());
		} catch (IOException iex) {
			return;
		}
		return;
	}
	
	private void write(ByteArrayOutputStream out, byte [] outString) {
		
		try {
			out.write(outString);
		} catch (IOException iex) {
			return;
		}
		return;
	}
	
	private int BitStringToBuffer(BitString value, byte[] buf/*,int UsedBitsInByte*/) {
		
		int BitVal;
		int j = 0;/*BufCount*/
		int i = 0;/*IntReg*/

		for (i = 0; i < value.length(); i++)
		//while(i<value.length()) do
		{
			{
				buf[j] = (byte) (buf[j] * 2);
				BitVal = value.getBit(i);
				
				if (BitVal == 1) {
					buf[j]++;
				}
				if ((((UsedBitsInByte + i + 1) % 8) == 0) && (i > 0)) {
					j++;
				}
			}
		}
		UsedBitsInByte = UsedBitsInByte + i % 8;

		return j;
	}

   /**<!--###################################################################-->
    *
    * Method <b>decode</b> decodes a received message according to the native 
    * decoding rules.
    * <p>
    * @param rcvdMessage message that shall be decoded.
    * @param decodingHypothesis the decoding hypothesis against which the
    *        message will be decoded.
    * <p>
    * @return the received message decoded according to the decoding hypothesis
    *         or <code>null</code> if message was not of type
    *         <code>decodingHypothesis</code>.
    * 
    * <!--################################################################--> */
    public Value decode(TriMessage rcvdMessage, Type decodingHypothesis) {
        DiameterMessage Message = null;
	    // Get the received byte array 
        encodedMsg = rcvdMessage.getEncodedMessage();
        //RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_ERROR, "DIAMETERCodec: decode"+rcvdMessage);
        //RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_ERROR, "DIAMETERCodec: decode:"+HexDump.toString(encodedMsg));

	    // Set the bitpos to 0 (this is needed to know the actual bit position from the byte array)
	    //RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_ERROR, "DIAMETERCodec: hypothesis:"+decodingHypothesis.getTypeClass());
	    bitpos = 0;
	    // The the encoded message is null, nothing can be done, so return null 
	    if(encodedMsg.length == 0) return null;
	
        // Check which type class the decoding hypothesis is of
        switch (decodingHypothesis.getTypeClass()) {
            case TciTypeClass.BOOLEAN:
	    	  	BooleanValue booleanValue = (BooleanValue) booleanType.newInstance();
				if (encodedMsg[0] == 0x00) {booleanValue.setBoolean(false);}
				else {booleanValue.setBoolean(true);}
				return booleanValue;
            case TciTypeClass.UNION:
    	  	    //UnionValue NewHypotesisu = (UnionValue) decodingHypothesis.newInstance();
	    		try {
	    			// Now we parse the received String instead of the byteArray
	    			Message = Codec.decodeDiameterMessage(encodedMsg,0);
	    		    //RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: message decode UNION:\n"+Message.toString());
	    		} catch (DiameterMessageDecodeException ex) {
	    		    RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_ERROR, "DIAMETERCodec: exception:"+ex.getMessage());
	    			return null;
	    		}
	    	  	return decode_DIAMETERMessage(Message,	decodingHypothesis);
	    	  	
	    	  	// Fill the NewHypotesis
	    	  	//return createNewHypotesis(NewHypotesis,encodedMsg);
	    	  	//return NewHypotesisu;
	        case TciTypeClass.RECORD:
	            //RecordValue NewHypotesis = (RecordValue) decodingHypothesis.newInstance();
		  		try {
		  			// Now we parse the received String instead of the byteArray
		  			Message = Codec.decodeDiameterMessage(encodedMsg,0);
		  		   // RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: message decode RECORD:\n"+Message.toString());
		  		} catch (DiameterMessageDecodeException ex) {
		  		    RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_ERROR, "DIAMETERCodec: exception:"+ex.getMessage());
		  			return null;
		  		}
		  	  	return decode_DIAMETERMessage(Message,	decodingHypothesis);
	    	  
	    	    // Fill the NewHypotesis
	            //return createNewHypotesis(NewHypotesis,encodedMsg);
	            //return NewHypotesis;
        //} else {
        //  return null;
        //}
            default:
                return super.decode(rcvdMessage,decodingHypothesis);
        }   
    }
  
    public Value decode_DIAMETERMessage(DiameterMessage parsedMessage,	Type type){
	    
	    //RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec:  decode_DIAMETERMessage:"+type);
	    //RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec:  parsed:"+parsedMessage.toString());
	  //System.out.println("parsed:"+parsedMessage.toString());
	  
	    switch (type.getTypeClass()){
		    case TciTypeClass.UNION:
	  		    
			    UnionValue uDM = (UnionValue) type.newInstance();
			  
  		        String messagebodyvar=MessageVariant(parsedMessage.commandCode,parsedMessage.flagRequest);
			    String messagevar = messagebodyvar + "MSG";
			    String bodyvar = messagebodyvar + "Body";
			    
			    RecordValue DM = (RecordValue) uDM.getVariant(messagevar).getType().newInstance();	  		  
			    RecordValue DMBody = (RecordValue) DM.getField(bodyvar).getType().newInstance();
			
			    decode_DiameterHeader(DM,parsedMessage);
			    decode_Session_Id_AVP(DM,parsedMessage);
			    decode_DiameterAVPs(DMBody,parsedMessage);
			    //decode_DiameterAVPs_(DMBody,parsedMessage);
			  
			    DM.setField(bodyvar, DMBody);
			    uDM.setVariant(messagevar, DM);
			    //logError(uDM.toString());
			    return uDM;
			  
		    case TciTypeClass.RECORD:
			    //RecordValue rDM = (RecordValue) type.newInstance();
			    //decode_DiameterHeader(rDM,parsedMessage);
			    return null;//rDM;
		    default:return null;
	    }
    }
  
  private String MessageVariant(int code, boolean r_a){
	  
	  String variant="uNKNOWN_"/*"uNKNOWN_MSG"*/;
	  if(r_a){
		  switch(code){
		    case Code_CE: variant="cER_"/*"cER_MSG"*/; break;
		    case Code_RA: variant="rAR_"/*"rAR_MSG"*/; break;
		    case Code_AC: variant="aCR_"/*"aCR_MSG"*/; break;
		    case Code_AS: variant="aSR_"/*"aSR_MSG"*/; break;
		    case Code_ST: variant="sTR_"/*"sTR_MSG"*/; break;
		    case Code_DW: variant="dWR_"/*"dWR_MSG"*/; break;
		    case Code_DP: variant="dPR_"/*"dPR_MSG"*/; break;
		    
		    /* Codes on Cx */
		    case Code_UA: variant="uAR_"/*"uAR_MSG"*/; break;
		    case Code_SA: variant="sAR_"/*"sAR_MSG"*/; break;   	
		    case Code_LI: variant="lIR_"/*"lIR_MSG"*/; break;   	
		    case Code_MA: variant="mAR_"/*"mAR_MSG"*/; break;
		    case Code_RT: variant="rTR_"/*"rTR_MSG"*/; break;
		    case Code_PP: variant="pPR_"/*"pPR_MSG"*/; break;
		    
		    /* Codes on Sh/Dh interface  */
		    case Code_UD: variant="uDR_"/*"uDR_MSG"*/; break;// UDR User-Data-Request AS->HSS                UDR    (306)
		    case Code_PU: variant="pUR_"/*"pUR_MSG"*/; break;// PUR Profile-Update-Request AS->HSS           PUR    (307)
		    case Code_SN: variant="sNR_"/*"sNR_MSG"*/; break;// SNR Subscribe-Notifications-Request AS->HSS  SNR    (308)
		    case Code_PN: variant="pNR_"/*"pNR_MSG"*/; break;// PNR Profile-Notification-Request HSS->AS     PNR    (309)
		    		    
		    //Rx
		    case Code_AA: variant="aAR_"/*"aAR_MSG"*/; break;
		    //4006
		    case Code_CC: variant="cCR_"/*"cCR_MSG"*/; break;
		    
		    /* Codes on S6a interface - TS 129 272  */
		    case Code_UL: variant="uLR_"/*"uLR_MSG"*/; break;  // Update-Location-Request                   ULR    (316)- Ref: TS 129 272 7.2.3-4
		    case Code_CL: variant="cLR_"/*"cLR_MSG"*/; break;  // Cancel-Location-Request                   CLR    (317)- Ref: TS 129 272 7.2.7-8
		    case Code_AI: variant="aIR_"/*"aIR_MSG"*/; break;  // Authentication-Information-Request        AIR    (318)- Ref: TS 129 272 7.2.5-6
		    case Code_ID: variant="iDR_"/*"iDR_MSG"*/; break;  // Insert-Subscriber-Data-Request            IDR    (319)- Ref: TS 129 272 7.2.9-10
		    case Code_DS: variant="dSR_"/*"dSR_MSG"*/; break;  // Delete-Subscriber-Data-Request            DSR    (320)- Ref: TS 129 272 7.2.11-12
		    case Code_PUE: variant="pUER_"/*"pUER_MSG"*/; break; // Purge-UE-Request                          PUR    (321)- Ref: TS 129 272 7.2.13-14
		    case Code_RS: variant="rSR_"/*"rSR_MSG"*/; break;  // Reset-Request                             DSR    (322)- Ref: TS 129 272 7.2.15-16
		    case Code_NO: variant="nOR_"/*"nOR_MSG"*/; break;  // Notify-Request                            NOR    (323)- Ref: TS 129 272 7.2.17-18
		  }
	  }else{
		  switch(code){
		    case Code_CE: variant="cEA_"/*"cEA_MSG"*/; break;
		    case Code_RA: variant="rAA_"/*"rAA_MSG"*/; break;
		    case Code_AC: variant="aCA_"/*"aCA_MSG"*/; break;
		    case Code_AS: variant="aSA_"/*"aSA_MSG"*/; break;
		    case Code_ST: variant="sTA_"/*"sTA_MSG"*/; break;
		    case Code_DW: variant="dWA_"/*"dWA_MSG"*/; break;
		    case Code_DP: variant="dPA_"/*"dPA_MSG"*/; break;
		    //CxDx
		    case Code_UA: variant="uAA_"/*"uAA_MSG"*/; break;
		    case Code_SA: variant="sAA_"/*"sAA_MSG"*/; break;   	
		    case Code_LI: variant="lIA_"/*"lIA_MSG"*/; break;   	
		    case Code_MA: variant="mAA_"/*"mAA_MSG"*/; break;
		    case Code_RT: variant="rTA_"/*"rTA_MSG"*/; break;
		    case Code_PP: variant="pPA_"/*"pPA_MSG"*/; break;
		    
		    /* Codes on Sh/Dh interface  */
		    case Code_UD: variant="uDA_"/*"uDA_MSG"*/; break;// UDR User-Data-Request AS->HSS                UDR    (306)
		    case Code_PU: variant="pUA_"/*"pUA_MSG"*/; break;// PUR Profile-Update-Request AS->HSS           PUR    (307)
		    case Code_SN: variant="sNA_"/*"sNA_MSG"*/; break;// SNR Subscribe-Notifications-Request AS->HSS  SNR    (308)
		    case Code_PN: variant="pNA_"/*"pNA_MSG"*/; break;// PNR Profile-Notification-Request HSS->AS     PNR    (309)
		    
		    //Rx
		    case Code_AA: variant="aAA_"/*"aAA_MSG"*/; break;
		    //4006
		    case Code_CC: variant="cCA_"/*"cCA_MSG"*/; break;
		    
		    /* Codes on S6a interface - TS 129 272  */
		    case Code_UL: variant="uLA_"/*"uLA_MSG"*/; break;  // Update-Location-Request                   ULR    (316)- Ref: TS 129 272 7.2.3-4
		    case Code_CL: variant="cLA_"/*"cLA_MSG"*/; break;  // Cancel-Location-Request                   CLR    (317)- Ref: TS 129 272 7.2.7-8
		    case Code_AI: variant="aIA_"/*"aIA_MSG"*/; break;  // Authentication-Information-Request        AIR    (318)- Ref: TS 129 272 7.2.5-6
		    case Code_ID: variant="iDA_"/*"iDA_MSG"*/; break;  // Insert-Subscriber-Data-Request            IDR    (319)- Ref: TS 129 272 7.2.9-10
		    case Code_DS: variant="dSA_"/*"dSA_MSG"*/; break;  // Delete-Subscriber-Data-Request            DSR    (320)- Ref: TS 129 272 7.2.11-12
		    case Code_PUE: variant="pUEA_"/*"pUEA_MSG"*/; break; // Purge-UE-Request                          PUR    (321)- Ref: TS 129 272 7.2.13-14
		    case Code_RS: variant="rSA_"/*"rSA_MSG"*/; break;  // Reset-Request                             DSR    (322)- Ref: TS 129 272 7.2.15-16
		    case Code_NO: variant="nOA_"/*"nOA_MSG"*/; break;  // Notify-Request                            NOR    (323)- Ref: TS 129 272 7.2.17-18
		  }
	  } 
	  return variant;
  }
  
  private void decode_DiameterHeader(RecordValue rv_DiameterM,	DiameterMessage msg){
    
	RecordValue DH = (RecordValue) rv_DiameterM.getField("header").getType().newInstance();		
	
    IntegerValue version = (IntegerValue)DH.getField("version").getType().newInstance();
	version.setInt(1);
	DH.setField("version", version);
	
	IntegerValue msglen = (IntegerValue)DH.getField("msglen").getType().newInstance();
	msglen.setInt(msg.length);
	DH.setField("msglen", msglen);
		
	RecordValue cmdf = (RecordValue)DH.getField("cmdflags").getType().newInstance();
	byte[] buffer = new byte[] {msg.flags};
	UsedBitsInByte = 0;
    cmdf.setField("r_bit", BufferToBitString(buffer, 1)); //Request
    cmdf.setField("p_bit", BufferToBitString(buffer, 1)); //Proxiable
    cmdf.setField("e_bit", BufferToBitString(buffer, 1)); //Error
    cmdf.setField("t_bit", BufferToBitString(buffer, 1)); //Potentialy re-transmited message
    cmdf.setField("reserved_bits", BufferToBitString(buffer, 4)); //reserved_bits
	DH.setField("cmdflags", cmdf);
	
	EnumeratedValue cmd = (EnumeratedValue) DH.getField("cmdcode").getType().newInstance();
    cmd.setEnum(CommandCodeEnumerated(msg.commandCode,msg.flagRequest));
    DH.setField("cmdcode", cmd);
	
	IntegerValue applid = (IntegerValue)DH.getField("applid").getType().newInstance();
	applid.setInt(msg.applicationID);
	DH.setField("applid", applid);
	
	IntegerValue hopid = (IntegerValue)DH.getField("hopid").getType().newInstance();
	hopid.setInt(msg.hopByHopID);
	DH.setField("hopid", hopid);
	
	//IntegerValue endid = (IntegerValue)DH.getField("endid").getType().newInstance();
	//endid.setBigInt(BigInteger.valueOf((long)  msg.endToEndID));
	//DH.setField("endid", endid);
	DH.setField("endid", mkInt((long)msg.endToEndID));
   
	rv_DiameterM.setField("header", DH);
  }
  
  private String CommandCodeEnumerated(int code, boolean r_a){
	  
	  String codeEnum="";
	  if(r_a){
		  switch(code){
		    case Code_CE: codeEnum="CER_E"; break;
		    case Code_RA: codeEnum="RAR_E"; break;
		    case Code_AC: codeEnum="ACR_E"; break;
		    case Code_AS: codeEnum="ASR_E"; break;
		    case Code_ST: codeEnum="STR_E"; break;
		    case Code_DW: codeEnum="DWR_E"; break;
		    case Code_DP: codeEnum="DPR_E"; break;
            //CxDx
		    case Code_UA: codeEnum="UAR_E"; break;
		    case Code_SA: codeEnum="SAR_E"; break;
		    case Code_LI: codeEnum="LIR_E"; break;
		    case Code_MA: codeEnum="MAR_E"; break;
		    case Code_RT: codeEnum="RTR_E"; break;
		    case Code_PP: codeEnum="PPR_E"; break;
		    
		    /* Codes on Sh/Dh interface  */
		    case Code_UD: codeEnum="UDR_E"; break;// UDR User-Data-Request AS->HSS                UDR    (306)
		    case Code_PU: codeEnum="PUR_E"; break;// PUR Profile-Update-Request AS->HSS           PUR    (307)
		    case Code_SN: codeEnum="SNR_E"; break;// SNR Subscribe-Notifications-Request AS->HSS  SNR    (308)
		    case Code_PN: codeEnum="PNR_E"; break;// PNR Profile-Notification-Request HSS->AS     
		    
		    //Rx
		    case Code_AA: codeEnum="AAR_E"; break;
		    //4006
		    case Code_CC: codeEnum="CCR_E"; break;
		    
		    //s6a
		    case Code_UL: codeEnum="ULR_E"; break;  // Update-Location-Request                   ULR    (316)- Ref: TS 129 272 7.2.3-4
		    case Code_CL: codeEnum="CLR_E"; break;  // Cancel-Location-Request                   CLR    (317)- Ref: TS 129 272 7.2.7-8
		    case Code_AI: codeEnum="AIR_E"; break;  // Authentication-Information-Request        AIR    (318)- Ref: TS 129 272 7.2.5-6
		    case Code_ID: codeEnum="IDR_E"; break;  // Insert-Subscriber-Data-Request            IDR    (319)- Ref: TS 129 272 7.2.9-10
		    case Code_DS: codeEnum="DSR_E"; break;  // Delete-Subscriber-Data-Request            DSR    (320)- Ref: TS 129 272 7.2.11-12
		    case Code_PUE: codeEnum="PUER_E"; break; // Purge-UE-Request                          PUR    (321)- Ref: TS 129 272 7.2.13-14
		    case Code_RS: codeEnum="RSR_E"; break;  // Reset-Request                             DSR    (322)- Ref: TS 129 272 7.2.15-16
		    case Code_NO: codeEnum="NOR_E"; break;  // Notify-Request                            NOR    (323)- Ref: TS 129 272 7.2.17-18
		  }
	  }else{
		  switch(code){
		    case Code_CE: codeEnum="CEA_E"; break;
		    case Code_RA: codeEnum="RAA_E"; break;
		    case Code_AC: codeEnum="ACA_E"; break;
		    case Code_AS: codeEnum="ASA_E"; break;
		    case Code_ST: codeEnum="STA_E"; break;
		    case Code_DW: codeEnum="DWA_E"; break;
		    case Code_DP: codeEnum="DPA_E"; break;
		    //CxDx
		    case Code_UA: codeEnum="UAA_E"; break;
		    case Code_SA: codeEnum="SAA_E"; break;
		    case Code_LI: codeEnum="LIA_E"; break;
		    case Code_MA: codeEnum="MAA_E"; break;
		    case Code_RT: codeEnum="RTA_E"; break;
		    case Code_PP: codeEnum="PPA_E"; break;
		    
		    /* Codes on Sh/Dh interface  */
		    case Code_UD: codeEnum="UDA_E"; break;// UDR User-Data-Request AS->HSS                UDR    (306)
		    case Code_PU: codeEnum="PUA_E"; break;// PUR Profile-Update-Request AS->HSS           PUR    (307)
		    case Code_SN: codeEnum="SNA_E"; break;// SNR Subscribe-Notifications-Request AS->HSS  SNR    (308)
		    case Code_PN: codeEnum="PNA_E"; break;// PNR Profile-Notification-Request HSS->AS 
		    //Rx
		    case Code_AA: codeEnum="AAA_E"; break;
		    //4006
		    case Code_CC: codeEnum="CCA_E"; break;
		    
		    //S6a
		    case Code_UL: codeEnum="ULA_E"; break;  // Update-Location-Request                   ULR    (316)- Ref: TS 129 272 7.2.3-4
		    case Code_CL: codeEnum="CLA_E"; break;  // Cancel-Location-Request                   CLR    (317)- Ref: TS 129 272 7.2.7-8
		    case Code_AI: codeEnum="AIA_E"; break;  // Authentication-Information-Request        AIR    (318)- Ref: TS 129 272 7.2.5-6
		    case Code_ID: codeEnum="IDA_E"; break;  // Insert-Subscriber-Data-Request            IDR    (319)- Ref: TS 129 272 7.2.9-10
		    case Code_DS: codeEnum="DSA_E"; break;  // Delete-Subscriber-Data-Request            DSR    (320)- Ref: TS 129 272 7.2.11-12
		    case Code_PUE: codeEnum="PUEA_E"; break; // Purge-UE-Request                          PUR    (321)- Ref: TS 129 272 7.2.13-14
		    case Code_RS: codeEnum="RSA_E"; break;  // Reset-Request                             DSR    (322)- Ref: TS 129 272 7.2.15-16
		    case Code_NO: codeEnum="NOA_E"; break;  // Notify-Request                            NOR    (323)- Ref: TS 129 272 7.2.17-1
		  }
	  } 
	  return codeEnum;
  }
  
    private void decode_AVPHeader (RecordValue rvAvp, AVP avp) {
		// RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec: AVP:\n"+avp.getStringData());

		RecordValue AvpH = (RecordValue)rvAvp.getField("aVP_Header").getType().newInstance();
		
		IntegerValue avpcode = (IntegerValue)AvpH.getField("aVP_Code").getType().newInstance();
		avpcode.setInt(avp.code);
		AvpH.setField("aVP_Code", avpcode);
		
		RecordValue avpflags = (RecordValue) AvpH.getField("aVP_flags").getType().newInstance();
		byte[] buffer = new byte[] {(byte) ((((avp.flag_vendor_specific?1:0)<<7) | ((avp.flag_mandatory?1:0)<<6) | ((avp.flag_protected?1:0)<<5)) & 0xFF)};
		UsedBitsInByte = 0;
		avpflags.setField("v_bit", BufferToBitString(buffer, 1)); //
		avpflags.setField("m_bit", BufferToBitString(buffer, 1)); //
		avpflags.setField("p_bit", BufferToBitString(buffer, 1)); //
		avpflags.setField("reserved_bits", BufferToBitString(buffer, 5)); //reserved_bits
		AvpH.setField("aVP_flags", avpflags);

		IntegerValue avplen = (IntegerValue)AvpH.getField("aVP_len").getType().newInstance();
		avplen.setInt(avp.length);
		AvpH.setField("aVP_len", avplen);
		
		if(avp.vendor_id!=0){
		   IntegerValue avpvid = (IntegerValue)AvpH.getField("aVP_vid").getType().newInstance();
		   avpvid.setInt(avp.vendor_id);
		   AvpH.setField("aVP_vid", avpvid);
		}

		rvAvp.setField("aVP_Header",AvpH);
		return;
    }
	
    private void decode_DiameterAVPs_(RecordValue rv_,	DiameterMessage msg){
    	
    	rov_any_AVP = (RecordOfValue) rv_.getField("aVP_Type").getType().newInstance();
    	System.out.println("decode_DiameterAVPs_");
    	String[] fields = rv_.getFieldNames();
    	for(int i=0;i<fields.length;i++){
    		Value v = (Value)rv_.getField(fields[i]);
    		
    		String scv =null;
    		switch (v.getType().getTypeClass()){
    		    case TciTypeClass.RECORD:scv="RECORD";break;
    		    case TciTypeClass.RECORD_OF:scv="RECORD_OF";break;
    		    case TciTypeClass.SET:scv="SET";break;
    		    case TciTypeClass.SET_OF:scv="SET_OF";break;
    		    case TciTypeClass.UNION:scv="UNION";break;
    		    
 		    	default:scv="ANY";break;
    		}	
    		System.out.println(scv+" : "+fields[i] + ": "+v.toString());
    	}
    	
    	Vector<AVP> avps=msg.avps;
    	for(int i=0;i<avps.size();i++){
    		System.out.println(avps.get(i).code);
    	}
    	
    	if (rov_any_AVP.getLength()>0){
    		rv_.setField("aVP_Type", rov_any_AVP);
		}
    }
    private void decode_DiameterAVPs(RecordValue rv_DiameterM,	DiameterMessage msg){
		 
	    //RB.getTciTLProvidedV321TT().tliRT("", System.currentTimeMillis(), "", -1,	null, RTLoggingConstants.RT_LOG_INFO, "DIAMETERCodec:  decode_DiameterAVPs:"+rv_DiameterM + msg);
	    rov_any_AVP = (RecordOfValue) rv_DiameterM.getField("aVP_Type").getType().newInstance();
	 
	    /*RFC3588 AVPs*/
		decode_Origin_Host_AVP( rv_DiameterM,	 msg);
		decode_Origin_Realm_AVP( rv_DiameterM,	 msg);
		decode_Host_IP_Address_AVP( rv_DiameterM,	 msg);
		decode_Vendor_Id_AVP( rv_DiameterM,	 msg);
		decode_Product_Name_AVP( rv_DiameterM,	 msg);
		decode_Origin_State_Id_AVP( rv_DiameterM,	 msg);
		decode_Supported_Vendor_Id_AVP( rv_DiameterM,	 msg);
		decode_Auth_Application_Id_AVP( rv_DiameterM,	 msg);
		decode_Inband_Security_Id_AVP( rv_DiameterM,	 msg);
		decode_Acct_Application_Id_AVP( rv_DiameterM,	 msg);
		decode_Vendor_Specific_Appl_Id_AVP( rv_DiameterM,	 msg);
		decode_Firmware_Revision_AVP( rv_DiameterM,	 msg);
		decode_Result_Code_AVP( rv_DiameterM,	 msg);
		decode_Error_Message_AVP( rv_DiameterM,	 msg);
		decode_Failed_AVP( rv_DiameterM,	 msg);
		decode_Destination_Realm_AVP( rv_DiameterM,	 msg);
		decode_Destination_Host_AVP( rv_DiameterM,	 msg);
		decode_Re_Auth_Request_Type_AVP( rv_DiameterM,	 msg);	
		decode_User_Name_AVP( rv_DiameterM,	 msg);
		//decode_Proxy_Host_AVP( rv_DiameterM,	 msg);
		//decode_Proxy_State_AVP( rv_DiameterM,	 msg);
		decode_Proxy_Info_AVP( rv_DiameterM,	 msg);
		decode_Route_Record_AVP( rv_DiameterM,	 msg);
		decode_Error_Reporting_Host_AVP( rv_DiameterM,	 msg);
		decode_Redirect_Host_AVP( rv_DiameterM,	 msg);
		decode_Redirect_Host_Usage_AVP( rv_DiameterM,	 msg);
		decode_redirect_Max_Cache_Time_AVP( rv_DiameterM,	 msg);
		decode_Accounting_Record_Type_AVP( rv_DiameterM,	 msg);
		decode_Accounting_Record_Number_AVP( rv_DiameterM,	 msg);
		decode_Accounting_Sub_Session_Id_AVP( rv_DiameterM,	 msg);
		decode_Acct_Session_Id_AVP( rv_DiameterM,	 msg);
		decode_Acct_Multi_Session_Id_AVP( rv_DiameterM,	 msg);
		decode_Acct_Interim_Interval_AVP( rv_DiameterM,	 msg);
		decode_Accounting_Realtime_Required_AVP( rv_DiameterM,	 msg);
		decode_Event_Timestamp_AVP( rv_DiameterM,	 msg);
		decode_Termination_Cause_AVP( rv_DiameterM,	 msg);
		decode_Class_AVP( rv_DiameterM,	 msg);
		decode_Disconnect_Cause_AVP( rv_DiameterM,	 msg);
	    /*End RFC3588 AVPs*/
		
		/*RFC4740*/
		decode_Auth_Session_State_AVP(rv_DiameterM, msg);
		decode_SIP_Number_Auth_Items_AVP(rv_DiameterM, msg);
		decode_SIP_Auth_Data_Item_AVP(rv_DiameterM, msg);
		/*End RFC4740 AVPs*/
		
		/*RFC4005*/
		decode_Framed_IP_Address_AVP(rv_DiameterM, msg);
		decode_Framed_IPv6_Prefix_AVP(rv_DiameterM, msg);
		/*End RFC4005 AVPs*/
		
		/*RFC4006*/
		decode_Service_Context_Id_AVP(rv_DiameterM,	 msg);
		decode_CC_Request_Type_AVP(rv_DiameterM,	 msg);
		decode_CC_Request_Number_AVP(rv_DiameterM,	 msg);
		//decode_CC_Sub_Session_Id_AVP(rv_DiameterM,	 msg);
		decode_Subscription_Id_AVP(rv_DiameterM,	 msg);
		decode_Service_Identifier_AVP(rv_DiameterM,	 msg);
		decode_Requested_Service_Unit_AVP(rv_DiameterM,	 msg);
		decode_Requested_Action_AVP(rv_DiameterM,	 msg);
        decode_Used_Service_Unit_AVP(rv_DiameterM,	 msg);// 	used_Service_Unit optional,			//RFC4006
		decode_Multiple_Services_Indicator_AVP(rv_DiameterM,	 msg);
		decode_Multiple_Services_Credit_Control_AVP(rv_DiameterM,	 msg);
//			set of Service_Parameter_Info_AVP service_Parameter_Info optional,	//RFC4006
//			CC_Correlation_Id_AVP 			cC_Correlation_Id optional,			
		//decode_User_Equipment_Info_AVP(rv_DiameterM,	 msg);
		decode_CC_Session_Failover_AVP(rv_DiameterM,	 msg);
		decode_Cost_Information_AVP(rv_DiameterM,	 msg);
//	    	Check_Balance_Result_AVP 		check_Balance_Result optional,  //RFC4006
		decode_Low_Balance_Indication_AVP(rv_DiameterM,	 msg);  
		decode_Remaining_Balance_AVP(rv_DiameterM,	 msg);
		decode_Credit_Control_Failure_Handling_AVP(rv_DiameterM,	 msg);
		decode_Direct_Debiting_Failure_Handling_AVP(rv_DiameterM,	 msg);			
		/*End RFC4006 AVPs*/
		
		//sh-29.329
		decode_Public_Identity_AVP(rv_DiameterM,	 msg);
		decode_User_Identity_AVP(rv_DiameterM,	 msg);
		decode_Data_Reference_AVP(rv_DiameterM,	 msg);  
		decode_Service_Indication_AVP(rv_DiameterM,	 msg);
		decode_Subs_Req_Type_AVP(rv_DiameterM,	 msg);
		decode_Requested_Domain_AVP(rv_DiameterM,	 msg);
	    decode_Current_Location_AVP(rv_DiameterM,	 msg);
		decode_Identity_Set_AVP(rv_DiameterM,	 msg);
		decode_Expiry_Time_AVP(rv_DiameterM,	 msg);    
		decode_Send_Data_Indication_AVP(rv_DiameterM,	 msg);
		decode_Repository_Data_ID_AVP(rv_DiameterM,	 msg);
		decode_Wildcarded_PSI_AVP(rv_DiameterM,	 msg);
	    decode_Wildcarded_IMPU_AVP(rv_DiameterM,	 msg);
		decode_DSAI_Tag_AVP(rv_DiameterM,	 msg);
		decode_Server_Name_AVP(rv_DiameterM,	 msg);
		decode_Supported_Features_AVP(rv_DiameterM,	 msg);
		decode_Experimental_Result_AVP(rv_DiameterM,	 msg);
		decode_User_Data_AVP(rv_DiameterM,	 msg);
		/*End Sh*/
		
		//TS129_215 AVP codes
		decode_Routing_Rule_Install_AVP(rv_DiameterM,	 msg);
		decode_Routing_Rule_Remove_AVP(rv_DiameterM,	 msg);
		decode_QoS_Rule_Report_AVP(rv_DiameterM,	 msg);
		decode_QoS_Rule_Install_AVP(rv_DiameterM,	 msg);
		decode_Multiple_BBERF_Action_AVP(rv_DiameterM,	 msg);
		decode_Subses_Decision_Info_AVP(rv_DiameterM,	 msg);
		decode_Subses_Enforcement_Info_AVP(rv_DiameterM,	 msg);
		
		//TS129_229 
		decode_Visited_Network_Identifier_AVP(rv_DiameterM,	 msg);
		decode_UAR_Flags_AVP(rv_DiameterM,	 msg);
		decode_User_Authorization_Type_AVP(rv_DiameterM,	 msg);
		decode_Server_Assignment_Type_AVP_AVP(rv_DiameterM,	 msg);
		decode_User_Data_Already_Available_AVP(rv_DiameterM,	 msg);
		decode_SCSCF_Restoration_Info_AVP(rv_DiameterM,	 msg);
		decode_Originating_Request_AVP(rv_DiameterM,	 msg);
		decode_Deregistration_Reason_AVP(rv_DiameterM,	 msg);
		decode_Identity_with_Emergency_Registration_AVP(rv_DiameterM,	 msg);
		
		//TS129_272 AVP codes
		decode_ULR_Flags_AVP(rv_DiameterM,	 msg);
		decode_ULA_Flags_AVP(rv_DiameterM,	 msg);
		decode_UE_SRVCC_Capability_AVP(rv_DiameterM,	 msg);
		decode_SGSN_Number_AVP(rv_DiameterM,	 msg);
		decode_Homogeneous_Support_AVP(rv_DiameterM,	 msg);
		decode_Error_Diagnostic_AVP(rv_DiameterM,	 msg);
		decode_GMLC_Address_AVP(rv_DiameterM,	 msg);
		decode_Active_APN_AVP(rv_DiameterM,	 msg);
		
		decode_Subscription_Data_AVP(rv_DiameterM,	 msg);
		decode_Req_EUTRAN_Auth_Info_AVP(rv_DiameterM,	 msg);
		decode_Req_UTRAN_GERAN_Auth_Info_AVP(rv_DiameterM,	 msg);
		decode_Authentication_Info_AVP(rv_DiameterM,	 msg);
		decode_Cancellation_Type_AVP(rv_DiameterM,	 msg);
		decode_CLR_Flags_AVP(rv_DiameterM,	 msg);
		decode_IDR_Flags_AVP(rv_DiameterM,	 msg);
		decode_DSR_Flags_AVP(rv_DiameterM,	 msg);
		decode_DSA_Flags_AVP(rv_DiameterM,	 msg);
		decode_PUER_Flags_AVP(rv_DiameterM,	 msg);
		decode_PUEA_Flags_AVP(rv_DiameterM,	 msg);
		decode_IDA_Flags_AVP(rv_DiameterM,	 msg);
		decode_NOR_Flags_AVP(rv_DiameterM,	 msg);
		decode_IMS_Voice_PS_Sessions_Support_AVP(rv_DiameterM,	 msg);
		decode_Last_UE_Activity_Time_AVP(rv_DiameterM,	 msg);
		decode_EPS_User_State_AVP(rv_DiameterM,	 msg);
		decode_EPS_Location_Information_AVP(rv_DiameterM,	 msg);
		
		/*TS132_299*/
		decode_Charging_Information_AVP(rv_DiameterM,	 msg);
		decode_Charging_Rule_Report_AVP(rv_DiameterM,	 msg);
		decode_QoS_Information_AVP(rv_DiameterM,	 msg);
		decode_Service_Information_AVP(rv_DiameterM,	 msg);
		decode_Visited_PLMN_Id_AVP(rv_DiameterM,	 msg);
		decode_IP_CAN_Type_AVP(rv_DiameterM,	 msg);
		decode_RAT_Type_AVP(rv_DiameterM,	 msg);
		decode_Event_Trigger_AVP(rv_DiameterM,	 msg);
		decode_Session_Release_Cause_AVP(rv_DiameterM,	 msg);
		decode_AN_GW_Addr_AVP(rv_DiameterM,	 msg);
	
		if (rov_any_AVP.getLength()>0){
			rv_DiameterM.setField("aVP_Type", rov_any_AVP);
		}
	 }
  
	private void decode_DiameterIdentity ( RecordValue rvAvp, AVP avp) {
		
		if(avp.getStringData()!=null)
		    rvAvp.setField("aVP_Data",mkCv(avp.getStringData()));
		return;
	}
	
    private void decode_DiameterURI ( RecordValue rvAvp, AVP avp) {
		
		if(avp.getStringData()!=null){
	
			URI uri=null;	    
			try{
			    uri = new URI(avp.getStringData());
			
			    RecordValue rv= (RecordValue)rvAvp.getField("aVP_Data").getType().newInstance();
			
				//charstring scheme ("aaa", "aaas"),
				rv.setField("scheme",mkCv(uri.getScheme()));
		        //charstring diameterId_FQDN,
				rv.setField("diameterId_FQDN",mkCv(uri.getHost()));
		        //integer portField optional,
		        //charstring transport ("tcp", "sctp", "udp") optional,
		        //charstring protocol ("diameter", "radius", "tacacs+") optional
				
				rvAvp.setField("aVP_Data",rv);
				return;
			}
			catch (Exception ex){
				System.out.println("DiameterURI not in correct format("+avp.getStringData()+")" );
				return;
			}
		}
		return;
	}
	
	private void decode_Origin_Host_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Origin_Host_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue origHost = (RecordValue) rv_DiameterM.getField("origin_Host").getType().newInstance();
			
			decode_AVPHeader( origHost ,avps[0]);
			decode_DiameterIdentity(origHost, avps[0]);
			
			rv_DiameterM.setField("origin_Host",origHost);
		}
		return;
	}
	
	private void decode_Origin_Realm_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Origin_Realm_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue origRealm = (RecordValue) rv_DiameterM.getField("origin_Realm").getType().newInstance();
			
			decode_AVPHeader( origRealm ,avps[0]);
			decode_DiameterIdentity(origRealm, avps[0]);
			
			rv_DiameterM.setField("origin_Realm",origRealm);
		}
		return;
	}

/*	private void decode_IPV4Addr (ByteArrayOutputStream out, RecordValue data) {
		
		return;
	}
	private void decode_IPV6Addr (ByteArrayOutputStream out, RecordValue data) {
		
		return;
	}
*/	
	private void decode_Address (RecordValue data, AVP avp) {
		
		RecordValue rdata = (RecordValue)data.getField("aVP_Data").getType().newInstance();
		
		int version=avp.data[0]<<8|avp.data[1];
		
		OctetstringValue family=(OctetstringValue)rdata.getField("addressFamily").getType().newInstance();
		rdata.setField("addressFamily", createOctetstringValue(avp.data, 2 * 8));

		UnionValue diam_address = (UnionValue)rdata.getField("ipAddress").getType().newInstance();
		if(version==1){
			RecordOfValue IP = (RecordOfValue) diam_address.getVariant("ipv4").getType().newInstance();
			IP.setLength(4);
			for(int i=0;i<4;i++){
				IP.setField(i, mkInt((avp.data[i+2]) & 0xff));
			}
			diam_address.setVariant("ipv4", IP);
		}
		else{
			RecordOfValue IP = (RecordOfValue) diam_address.getVariant("ipv6").getType().newInstance();
			IP.setLength(8);
			for(int i=0;i<8;i++){
				IP.setField(i, mkInt((avp.data[i+2]) & 0xffff));
			}
			diam_address.setVariant("ipv6", IP);
		}
		
		rdata.setField("ipAddress", diam_address);
		data.setField("aVP_Data",rdata);
		
		/*if (udata.getPresentVariantName().equals("ipv4")) {
			//decode_IPV4Addr(out, (RecordValue) data.getVariant("ipv4"));
		}else if (udata.getPresentVariantName().equals("ipv6")){
			//decode_IPV6Addr(out, (RecordValue) data.getVariant("ipv6"));
		}*/
		return;
	}
	
	private void decode_Host_IP_Address_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Host_IP_Address_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
		
			RecordOfValue recOfhostIPAddr = (RecordOfValue) rv_DiameterM.getField("host_IP_Address").getType().newInstance();
			recOfhostIPAddr.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
				
	        	RecordValue rv = (RecordValue) recOfhostIPAddr.getElementType().newInstance();
	        	decode_AVPHeader( rv ,avps[i]);
				decode_Address(rv, avps[i]);//(CharstringValue)rv.getField("aVP_Data"));
				recOfhostIPAddr.setField(i, rv);
				recOfhostIPAddr.setLength(i);
				//recOfhostIPAddr.appendField(rv);
	        }
			rv_DiameterM.setField("host_IP_Address",recOfhostIPAddr);
		}
		//else{
		//	RecordValue hostIpAddr = (RecordValue) rv_DiameterM.getField("host_IP_Address").getType().newInstance();
			
		//	decode_AVPHeader( hostIpAddr ,avps[0]);
		//	decode_DiameterIdentity(hostIpAddr, avps[0]);//(CharstringValue)rv.getField("aVP_Data"));
			
		//	rv_DiameterM.setField("host_IP_Address",hostIpAddr);
		//}
		return;
	}
	
	private void decode_Vendor_Id_AVP(RecordValue rv,	AVP avp_vid){
		
		AVP avp=avp_vid.findChildAVP(AVP_Vendor_Id_AVP);
   	    if (avp==null) return;
		
   	    RecordValue rv_vid = (RecordValue) rv.getField("vendor_Id").getType().newInstance();
	   	
   	    decode_AVPHeader( rv_vid ,avp);
		rv_vid.setField("aVP_Data", mkInt(avp.int_data));
		
		rv.setField("vendor_Id",rv_vid);
		return;
	}
	
	private void decode_Vendor_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Vendor_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue vendor = (RecordValue) rv_DiameterM.getField("vendor_Id").getType().newInstance();
			
			decode_AVPHeader( vendor ,avps[0]);
			vendor.setField("aVP_Data", mkInt(avps[0].int_data));
			
			rv_DiameterM.setField("vendor_Id",vendor);
		}
		return;
	}
	
    private void decode_Product_Name_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Product_Name_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvproduct_Name = (RecordValue) rv_DiameterM.getField("product_Name").getType().newInstance();
        
			decode_AVPHeader( rvproduct_Name ,avps[0]);
        	rvproduct_Name.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
        
        	rv_DiameterM.setField("product_Name",rvproduct_Name);
		}
		return;
	}
	
    private void decode_Origin_State_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Origin_State_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>1){
		
			RecordOfValue recOfOrigin_State = (RecordOfValue) rv_DiameterM.getField("origin_State_Id").getType().newInstance();
			recOfOrigin_State.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
				
	        	RecordValue rv = (RecordValue) recOfOrigin_State.getElementType().newInstance();
	        	
	        	decode_AVPHeader( rv ,avps[i]);
				rv.setField("aVP_Data", mkInt(avps[i].int_data));
			
				recOfOrigin_State.appendField(rv);
	        }
			rv_DiameterM.setField("origin_State_Id",recOfOrigin_State);
		}else{
			RecordValue Origin_State = (RecordValue) rv_DiameterM.getField("origin_State_Id").getType().newInstance();
			
			decode_AVPHeader( Origin_State ,avps[0]);
			Origin_State.setField("aVP_Data", mkInt(avps[0].int_data));
			
			rv_DiameterM.setField("origin_State_Id",Origin_State);
		}
		return;
	}
	
    private void decode_Supported_Vendor_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Supported_Vendor_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			
			RecordOfValue recOfSupported_Vendor = (RecordOfValue) rv_DiameterM.getField("supported_Vendor_Id").getType().newInstance();
			recOfSupported_Vendor.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
				
	        	RecordValue rv = (RecordValue) recOfSupported_Vendor.getElementType().newInstance();
	        	
	        	decode_AVPHeader( rv ,avps[i]);
				rv.setField("aVP_Data", mkInt(avps[i].int_data));
			
				recOfSupported_Vendor.appendField(rv);
	        }
			rv_DiameterM.setField("supported_Vendor_Id",recOfSupported_Vendor);
		}
		else{}
		return;
    }
    
    private void decode_Auth_Application_Id_AVP(RecordValue rv,	AVP avp){
	    
    	decode_AVPHeader( rv ,avp);
		rv.setField("aVP_Data", mkInt(avp.int_data));
		return;
    }
    
    private void decode_Auth_Application_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
    	boolean rv_present=false;
    	
		AVP[] avps=msg.findAVPs(AVP_Auth_Application_Id_AVP);
		if (avps==null) return;
				
		if (avps.length>=1){
			try{	
				RecordValue rvAuth_Application = (RecordValue) (RecordValue) rv_DiameterM.getField("auth_Application_Id").getType().newInstance();
	        	
				decode_AVPHeader( rvAuth_Application ,avps[0]);
				rvAuth_Application.setField("aVP_Data", mkInt(avps[0].int_data));
				
				rv_DiameterM.setField("auth_Application_Id",rvAuth_Application);
				rv_present=true;
				return;
			}catch(Exception ex){
				//System.out.print(ex.getMessage());	
			}
			if(!rv_present){
				try{
					RecordOfValue recOfAuth_Application = (RecordOfValue) rv_DiameterM.getField("auth_Application_Id").getType().newInstance();
					recOfAuth_Application.setLength(0);
			
			        for(int i=0;i<avps.length;i++){			
			        	RecordValue rv = (RecordValue) recOfAuth_Application.getElementType().newInstance();
			        	
			        	decode_AVPHeader( rv ,avps[i]);
						rv.setField("aVP_Data", mkInt(avps[i].int_data));
					
						recOfAuth_Application.appendField(rv);
			        }
					rv_DiameterM.setField("auth_Application_Id",recOfAuth_Application);
					return;
				}catch(Exception ex){
					//System.out.print(ex.getMessage());	
				}	
			}
		}
		return;
    }
    
    private void decode_Inband_Security_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Inband_Security_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			
			RecordOfValue recOfInband_Security = (RecordOfValue) rv_DiameterM.getField("inband_Security_Id").getType().newInstance();
			recOfInband_Security.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
	        	RecordValue rv = (RecordValue) recOfInband_Security.getElementType().newInstance();
	        	
	        	decode_AVPHeader( rv ,avps[i]);
				rv.setField("aVP_Data", mkInt(avps[i].int_data));
			
				recOfInband_Security.appendField(rv);
	        }
			rv_DiameterM.setField("inband_Security_Id",recOfInband_Security);
		}		
		return;
    }
    
    private void decode_Acct_Application_Id_AVP(RecordValue rv,	AVP avp){
    	decode_AVPHeader( rv ,avp);
		rv.setField("aVP_Data", mkInt(avp.int_data));
    } 
    private void decode_Acct_Application_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
    	boolean rv_present=false;
		
    	AVP[] avps=msg.findAVPs(AVP_Acct_Application_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			
			try{
				RecordValue rvAcct_Application = (RecordValue) (RecordValue) rv_DiameterM.getField("acct_Application_Id").getType().newInstance();
	        	
				decode_AVPHeader( rvAcct_Application ,avps[0]);
				rvAcct_Application.setField("aVP_Data", mkInt(avps[0].int_data));
				
				rv_DiameterM.setField("acct_Application_Id",rvAcct_Application);
				rv_present=true;
				return;
	        }catch(Exception ex){
				//System.out.print(ex.getMessage());	
			}
	        
	        if(!rv_present){
				try{
					RecordOfValue recOfAcct_Application = (RecordOfValue) rv_DiameterM.getField("acct_Application_Id").getType().newInstance();
					recOfAcct_Application.setLength(0);
			
			        for(int i=0;i<avps.length;i++){			
			        	RecordValue rv = (RecordValue) recOfAcct_Application.getElementType().newInstance();
			        	
			        	decode_AVPHeader( rv ,avps[i]);
						rv.setField("aVP_Data", mkInt(avps[i].int_data));
					
						recOfAcct_Application.appendField(rv);
			        }
					rv_DiameterM.setField("acct_Application_Id",recOfAcct_Application);
					return;
				}catch(Exception ex){
					//System.out.print(ex.getMessage());	
				}
			}
		}
		return;
    }

    private void decode_Vendor_Specific_Appl_Id_AVP(RecordValue rv_avp,	AVP avp_vSAId){

    	try {avp_vSAId.ungroup();} catch (AVPDecodeException e) {/*/e.printStackTrace();*/	return;}  
		
    	AVP[] vendors=avp_vSAId.findChildAVPs(AVP_Vendor_Id_AVP);
    	if(vendors!=null){
    		RecordOfValue RecOfVId=(RecordOfValue)rv_avp.getField("vendor_Id");
    		RecOfVId.setLength(0);
    	
			for (int j = 0; j < vendors.length; j++) {
				RecordValue rv = (RecordValue) RecOfVId.getElementType().newInstance();
				//decode_Vendor_Id_AVP(rv,vendors[j]);
				decode_AVPHeader( rv ,vendors[j]);
				rv.setField("aVP_Data", mkInt(vendors[j].int_data));
				
				RecOfVId.appendField(rv);
			}
			rv_avp.setField("vendor_Id",RecOfVId);
		}
    	
    	AVP[] AuthAppl=avp_vSAId.findChildAVPs(AVP_Auth_Application_Id_AVP);
    	if(AuthAppl!=null){
	    	RecordOfValue RecOfAuthAppl=(RecordOfValue)rv_avp.getField("auth_Application_Id");
			RecOfAuthAppl.setLength(0);
	    	
			for (int j = 0; j < AuthAppl.length; j++) {
	    	   RecordValue rv = (RecordValue) RecOfAuthAppl.getElementType().newInstance();	
			  
	    	   decode_Auth_Application_Id_AVP(rv,AuthAppl[j]);
			   RecOfAuthAppl.appendField(rv);
	    	}
	    	rv_avp.setField("auth_Application_Id",RecOfAuthAppl);
    	}
    	
        AVP[] AcctAppl=avp_vSAId.findChildAVPs(AVP_Acct_Application_Id_AVP);
        if(AcctAppl!=null){
	        RecordOfValue RecOfAcctAppl=(RecordOfValue)rv_avp.getField("acct_Application_Id");
			RecOfAcctAppl.setLength(0);
	    	
			for (int j = 0; j < AcctAppl.length; j++) {
	     	   RecordValue rv = (RecordValue) RecOfAcctAppl.getElementType().newInstance();	
	 		   
	     	   decode_Acct_Application_Id_AVP(rv,AcctAppl[j]);
	 		   
	 		   RecOfAcctAppl.appendField(rv);
	    	}
	    	rv_avp.setField("acct_Application_Id",RecOfAcctAppl);
        }
        return;
    }
    
    private void decode_Vendor_Specific_Appl_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){

	    boolean rv_present=false;
	    AVP[] avps=msg.findAVPs(AVP_Vendor_Specific_Appl_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			try{
			   RecordValue rv_vSAId = (RecordValue) (RecordValue) rv_DiameterM.getField("vendor_Specific_Application_Id").getType().newInstance();
			   
			   decode_AVPHeader( rv_vSAId ,avps[0]);
			   decode_Vendor_Specific_Appl_Id_AVP(rv_vSAId,avps[0]);
			   
			   rv_DiameterM.setField("vendor_Specific_Application_Id",rv_vSAId);
			   rv_present=true;
			   return;
			}
			catch(Exception ex){
				//System.out.print(ex.getMessage());
			}
			if(!rv_present){
				try{
					RecordOfValue recOfrv_vSAId = (RecordOfValue) rv_DiameterM.getField("vendor_Specific_Application_Id").getType().newInstance();
					recOfrv_vSAId.setLength(0);
		
		        for(int i=0;i<avps.length;i++){			
		        	RecordValue rv = (RecordValue) recOfrv_vSAId.getElementType().newInstance();
		        	
		        	decode_AVPHeader( rv ,avps[i]);
					decode_Vendor_Specific_Appl_Id_AVP(rv,avps[i]);
				
		        	recOfrv_vSAId.appendField(rv);
		        }
				rv_DiameterM.setField("vendor_Specific_Application_Id",recOfrv_vSAId);
				return;
				}catch(Exception ex){
					//System.out.print(ex.getMessage());
				}
			}
		}
		return;
    }
    
	private void decode_Firmware_Revision_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Firmware_Revision_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvFirmware_Revision = (RecordValue) (RecordValue) rv_DiameterM.getField("firmware_Revision").getType().newInstance();
        	
			decode_AVPHeader( rvFirmware_Revision ,avps[0]);
			rvFirmware_Revision.setField("aVP_Data", mkInt(avps[0].int_data));
			
			rv_DiameterM.setField("firmware_Revision",rvFirmware_Revision);
        }
		return;
	}
	
    private void decode_Result_Code_AVP(RecordValue rv_DiameterM,	AVP msg){
		
		String err_enum="";
		
		AVP[] avps=msg.findChildAVPs(AVP_Result_Code_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvResult_Code = (RecordValue) (RecordValue) rv_DiameterM.getField("result_Code").getType().newInstance();
        	
			decode_AVPHeader( rvResult_Code ,avps[0]);
			rvResult_Code.setField("aVP_Data", mkInt(avps[0].int_data));
			
			/*switch(avps[0].int_data){
					         
				case		1001:	err_enum="DIAMETER_MULTI_ROUND_AUTH_E";break;
						         
				case		2001:	err_enum="DIAMETER_SUCCESS_E";break;
				case		2002:	err_enum="DIAMETER_LIMITED_SUCCESS_E";break;
						        
				case		3001:	err_enum="DIAMETER_COMMAND_UNSUPPORTED_E";break;
				case		3002:	err_enum="DIAMETER_UNABLE_TO_DELIVER_E";break;			
				case		3003:	err_enum="DIAMETER_REALM_NOT_SERVED_E";break;
				case		3004:	err_enum="DIAMETER_TOO_BUSY_E";break;
				case		3005:	err_enum="DIAMETER_LOOP_DETECTED_E";break;
				case		3006:	err_enum="DIAMETER_REDIRECT_INDICATION_E";break;
				case		3007:	err_enum="DIAMETER_APPLICATION_UNSUPPORTED_E";break;
				case		3008:	err_enum="DIAMETER_INVALID_HDR_BITS_E";break;
				case		3009:	err_enum="DIAMETER_INVALID_AVP_BITS_E";break;
				case		3010:	err_enum="DIAMETER_UNKNOWN_PEER_E";break;
						       
				case		4001:   err_enum="DIAMETER_AUTHENTICATION_REJECTED_E";break;
				case		4002:	err_enum="DIAMETER_OUT_OF_SPACE_E";break;			
				case		4003:	err_enum="DIAMETER_ELECTION_LOST_E";break;
						        
				case		5001:	err_enum="DIAMETER_AVP_UNSUPPORTED_E";break;
				case		5002:	err_enum="DIAMETER_UNKNOWN_SESSION_ID_E";break;
				case		5003:	err_enum="DIAMETER_AUTHORIZATION_REJECTED_E";break;
				case		5004:	err_enum="DIAMETER_INVALID_AVP_VALUE_E";break;
				case		5005:	err_enum="DIAMETER_MISSING_AVP_E";break;
				case		5006:	err_enum="DIAMETER_RESOURCES_EXCEEDED_E";break;
				case		5007:	err_enum="DIAMETER_CONTRADICTING_AVPS_E";break;
				case		5008:	err_enum="DIAMETER_AVP_NOT_ALLOWED_E";break;
				case		5009:	err_enum="DIAMETER_AVP_OCCURS_TOO_MANY_TIMES_E";break;
				case		5010:	err_enum="DIAMETER_NO_COMMON_APPLICATION_E";break;
				case		5011:	err_enum="DIAMETER_UNSUPPORTED_VERSION_E";break;
				case		5012:	err_enum="DIAMETER_UNABLE_TO_COMPLY_E";break;
				case		5013:	err_enum="DIAMETER_INVALID_BIT_IN_HEADER_E";break;
				case		5014:	err_enum="DIAMETER_INVALID_AVP_LENGTH_E";break;
				case		5015:	err_enum="DIAMETER_INVALID_MESSAGE_LENGTH_E";break;
				case		5016:	err_enum="DIAMETER_INVALID_AVP_BIT_COMBO_E";break;
				case		5017:	err_enum="DIAMETER_NO_COMMON_SECURITY_E";break;
				case        5470:   err_enum="DIAMETER_ERROR_SUBSESSION_E";break;
				default:			err_enum="DIAMETER_DUMMY_E";break;
				
			}
			
			EnumeratedValue err= (EnumeratedValue)rvResult_Code.getField("aVP_Data");
			err.setEnum(err_enum);
			rvResult_Code.setField("aVP_Data", err);*/
			
			rv_DiameterM.setField("result_Code",rvResult_Code);
        }
		return;
	}
	
	private void decode_Result_Code_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		String err_enum="";
		
		AVP[] avps=msg.findAVPs(AVP_Result_Code_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvResult_Code = (RecordValue) (RecordValue) rv_DiameterM.getField("result_Code").getType().newInstance();
        	
			decode_AVPHeader( rvResult_Code ,avps[0]);
			rvResult_Code.setField("aVP_Data", mkInt(avps[0].int_data));
			
			/*switch(avps[0].int_data){
					         
				case		1001:	err_enum="DIAMETER_MULTI_ROUND_AUTH_E";break;
						         
				case		2001:	err_enum="DIAMETER_SUCCESS_E";break;
				case		2002:	err_enum="DIAMETER_LIMITED_SUCCESS_E";break;
						        
				case		3001:	err_enum="DIAMETER_COMMAND_UNSUPPORTED_E";break;
				case		3002:	err_enum="DIAMETER_UNABLE_TO_DELIVER_E";break;			
				case		3003:	err_enum="DIAMETER_REALM_NOT_SERVED_E";break;
				case		3004:	err_enum="DIAMETER_TOO_BUSY_E";break;
				case		3005:	err_enum="DIAMETER_LOOP_DETECTED_E";break;
				case		3006:	err_enum="DIAMETER_REDIRECT_INDICATION_E";break;
				case		3007:	err_enum="DIAMETER_APPLICATION_UNSUPPORTED_E";break;
				case		3008:	err_enum="DIAMETER_INVALID_HDR_BITS_E";break;
				case		3009:	err_enum="DIAMETER_INVALID_AVP_BITS_E";break;
				case		3010:	err_enum="DIAMETER_UNKNOWN_PEER_E";break;
						       
				case		4001:   err_enum="DIAMETER_AUTHENTICATION_REJECTED_E";break;
				case		4002:	err_enum="DIAMETER_OUT_OF_SPACE_E";break;			
				case		4003:	err_enum="DIAMETER_ELECTION_LOST_E";break;
						        
				case		5001:	err_enum="DIAMETER_AVP_UNSUPPORTED_E";break;
				case		5002:	err_enum="DIAMETER_UNKNOWN_SESSION_ID_E";break;
				case		5003:	err_enum="DIAMETER_AUTHORIZATION_REJECTED_E";break;
				case		5004:	err_enum="DIAMETER_INVALID_AVP_VALUE_E";break;
				case		5005:	err_enum="DIAMETER_MISSING_AVP_E";break;
				case		5006:	err_enum="DIAMETER_RESOURCES_EXCEEDED_E";break;
				case		5007:	err_enum="DIAMETER_CONTRADICTING_AVPS_E";break;
				case		5008:	err_enum="DIAMETER_AVP_NOT_ALLOWED_E";break;
				case		5009:	err_enum="DIAMETER_AVP_OCCURS_TOO_MANY_TIMES_E";break;
				case		5010:	err_enum="DIAMETER_NO_COMMON_APPLICATION_E";break;
				case		5011:	err_enum="DIAMETER_UNSUPPORTED_VERSION_E";break;
				case		5012:	err_enum="DIAMETER_UNABLE_TO_COMPLY_E";break;
				case		5013:	err_enum="DIAMETER_INVALID_BIT_IN_HEADER_E";break;
				case		5014:	err_enum="DIAMETER_INVALID_AVP_LENGTH_E";break;
				case		5015:	err_enum="DIAMETER_INVALID_MESSAGE_LENGTH_E";break;
				case		5016:	err_enum="DIAMETER_INVALID_AVP_BIT_COMBO_E";break;
				case		5017:	err_enum="DIAMETER_NO_COMMON_SECURITY_E";break;
				case        5470:   err_enum="DIAMETER_ERROR_SUBSESSION_E";break;
				default:			err_enum="DIAMETER_DUMMY_E";break;
			}
			
			EnumeratedValue err= (EnumeratedValue)rvResult_Code.getField("aVP_Data");
			err.setEnum(err_enum);
			rvResult_Code.setField("aVP_Data", err);*/
			
			rv_DiameterM.setField("result_Code",rvResult_Code);
        }
		return;
	}
	
	private void decode_Error_Message_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Error_Message_AVP);
		if (avps==null) return;
		
		if (useList(rv_DiameterM.getFieldNames(),"error_Message")){
			if (avps.length>1){}
			else{
				RecordValue rvError_Message = (RecordValue) (RecordValue) rv_DiameterM.getField("error_Message").getType().newInstance();
	        	
				decode_AVPHeader( rvError_Message ,avps[0]);
				//rvError_Message.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
				rvError_Message.setField("aVP_Data", mkCv(avps[0].getStringData()));
				
				rv_DiameterM.setField("error_Message",rvError_Message);
	        }
		}
		else{
            RecordValue rv_avp = (RecordValue)rov_any_AVP.getElementType().newInstance();
            decode_AVP_Type(rv_avp,avps[0]);
            rov_any_AVP.appendField(rv_avp);
        }
		return;
	}

    private void decode_Failed_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Failed_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			
			RecordOfValue recOfFailed = (RecordOfValue) rv_DiameterM.getField("failed").getType().newInstance();
			recOfFailed.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
				RecordValue rv = (RecordValue) recOfFailed.getElementType().newInstance();
	        	
				decode_AVPHeader( rv ,avps[i]);
				
	        	RecordOfValue recOfAVPType = (RecordOfValue) rv.getField("aVP").getType().newInstance();
	        	recOfAVPType.setLength(0);
	        	
	        	try{avps[i].ungroup();}catch(Exception ex){return;}
	        	
	        	for (int j=0; j < avps[i].getChildCount();j++){
	        	  AVP childavp = avps[i].getChildAVP(j);
	        	 
	        	  RecordValue rv_c= (RecordValue) recOfAVPType.getElementType().newInstance();
	        	  
	        	  decode_AVPHeader(rv_c ,childavp);
	        	  decode_AVP_Type(rv_c,childavp); 
	        	  	        	  
	        	  recOfAVPType.appendField(rv_c);
	        	}
	        	rv.setField("aVP", recOfAVPType);
				recOfFailed.appendField(rv);
	        }
			rv_DiameterM.setField("failed",recOfFailed);
		}
		else{}
		return;
	}
	
	private void decode_Session_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Session_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvSession_Id = (RecordValue) (RecordValue) rv_DiameterM.getField("session_Id").getType().newInstance();
        	
			decode_AVPHeader( rvSession_Id ,avps[0]);
			rvSession_Id.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
			///rvSession_Id.setField("aVP_Data", mkCv(avps[0].getStringData()));
			rv_DiameterM.setField("session_Id",rvSession_Id);
        }
		return;
	}
	
	private void decode_Destination_Realm_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Destination_Realm_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			//String[] fields = rv_DiameterM.getFieldNames();
			if (useList(rv_DiameterM.getFieldNames(),"destination_Realm")){
				RecordValue destRealm = (RecordValue) rv_DiameterM.getField("destination_Realm").getType().newInstance();
				
				decode_AVPHeader( destRealm ,avps[0]);
				decode_DiameterIdentity(destRealm, avps[0]);
				
				rv_DiameterM.setField("destination_Realm",destRealm);
			}else{
			    RecordValue rv_avp = (RecordValue)rov_any_AVP.getElementType().newInstance();
			    decode_AVP_Type(rv_avp,avps[0]);
			    rov_any_AVP.appendField(rv_avp);
			}
		}
		return;
	}
	
    private void decode_Destination_Realm_AVP(RecordValue rv_DiameterM,	AVP avp_){
		
		AVP avp=avp_.findChildAVP(AVP_Destination_Realm_AVP);
		if (avp==null) return;
			
		RecordValue destRealm = (RecordValue) rv_DiameterM.getField("destination_Realm").getType().newInstance();
			
		decode_AVPHeader( destRealm ,avp);
		decode_DiameterIdentity(destRealm, avp);
			
		rv_DiameterM.setField("destination_Realm",destRealm);
		return;
	}
	
	private void decode_Destination_Host_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Destination_Host_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			//String[] fields = rv_DiameterM.getFieldNames();
			if (useList(rv_DiameterM.getFieldNames(),"destination_Host")){
				RecordValue destHost = (RecordValue) rv_DiameterM.getField("destination_Host").getType().newInstance();
				
			    decode_AVPHeader( destHost ,avps[0]);
			    decode_DiameterIdentity(destHost, avps[0]);
			
			    rv_DiameterM.setField("destination_Host",destHost);
			}else{
			    RecordValue rv_avp = (RecordValue)rov_any_AVP.getElementType().newInstance();
			    decode_AVP_Type(rv_avp,avps[0]);
			    rov_any_AVP.appendField(rv_avp);
			}
		}
		return;
	}
	
    private void decode_Destination_Host_AVP(RecordValue rv_DiameterM,	AVP avp_){
		
		AVP avp=avp_.findChildAVP(AVP_Destination_Host_AVP);
		if (avp==null) return;
		
	    RecordValue destHost = (RecordValue) rv_DiameterM.getField("destination_Host").getType().newInstance();
		
	    decode_AVPHeader( destHost ,avp);
	    decode_DiameterIdentity(destHost, avp);
		
	    rv_DiameterM.setField("destination_Host",destHost);
		return;
	}
	
	private void decode_Re_Auth_Request_Type_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
			
		AVP[] avps=msg.findAVPs(AVP_Re_Auth_Request_Type_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue reAuthReqType = (RecordValue) rv_DiameterM.getField("re_Auth_Request_Type").getType().newInstance();
			
			decode_AVPHeader( reAuthReqType ,avps[0]);
			reAuthReqType.setField("aVP_Data", mkInt(avps[0].int_data));
			
			rv_DiameterM.setField("re_Auth_Request_Type",reAuthReqType);
		}
	    return;		
	}
	
    private void decode_User_Name_AVP(RecordValue rv_,	AVP avp_){
		
		AVP[] avps=avp_.findChildAVPs(AVP_User_Name_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvUser_Name = (RecordValue) (RecordValue) rv_.getField("user_Name").getType().newInstance();
        	
			decode_AVPHeader( rvUser_Name ,avps[0]);
			rvUser_Name.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
			//rvUser_Name.setField("aVP_Data", mkCv(avps[0].getStringData()));
			rv_.setField("user_Name",rvUser_Name);
        }
		return;
	}
	
    private void decode_User_Name_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_User_Name_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvUser_Name = (RecordValue) (RecordValue) rv_DiameterM.getField("user_Name").getType().newInstance();
        	
			decode_AVPHeader( rvUser_Name ,avps[0]);
			rvUser_Name.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
			//rvUser_Name.setField("aVP_Data", mkCv(avps[0].getStringData()));
			rv_DiameterM.setField("user_Name",rvUser_Name);
        }
		return;
	}
	
	private void decode_Proxy_Host_AVP(RecordValue rv_proxy,	AVP pi){
		
		AVP[] proxy_Host=pi.findChildAVPs(AVP_Proxy_Host_AVP);
		if (proxy_Host==null) return;
		
		if (proxy_Host.length>1){} //Should never be more than one AVP
		else{
			RecordValue proxyHost = (RecordValue) rv_proxy.getField("proxy_Host").getType().newInstance();
			
			decode_AVPHeader( proxyHost ,proxy_Host[0]);
			decode_DiameterIdentity(proxyHost, proxy_Host[0]);
			
			rv_proxy.setField("proxy_Host",proxyHost);
		}
		return;
	}
	
	private void decode_Proxy_State_AVP(RecordValue rv_proxy,	AVP pi){
		
		AVP[] proxy_State=pi.findChildAVPs(AVP_Proxy_State_AVP);
		if (proxy_State==null) return;
		
		if (proxy_State.length>1){} //Should never be more than one AVP
		else{
			RecordValue proxyState = (RecordValue) rv_proxy.getField("proxy_State").getType().newInstance();
			
			decode_AVPHeader( proxyState ,proxy_State[0]);
			proxyState.setField("aVP_Data", mkOctetstring(proxy_State[0].data));
						
			rv_proxy.setField("proxy_State",proxyState);
		}
	    return;	
	}
	
    private void decode_Proxy_Info_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
    	AVP[] avps=msg.findAVPs(AVP_Proxy_Info_AVP);
		if (avps==null) return;
    	
        if (avps.length>=1){
			RecordOfValue recOfproxy_Info = (RecordOfValue) rv_DiameterM.getField("proxy_Info").getType().newInstance();
			recOfproxy_Info.setLength(0);
		
			for (int i = 0; i < avps.length; i++) {
			    RecordValue rv = (RecordValue) recOfproxy_Info.getElementType().newInstance();
	        	decode_AVPHeader( rv ,avps[i]);
	
	     		decode_Proxy_Host_AVP(rv,avps[i]);
	     		decode_Proxy_State_AVP(rv,avps[i]);
	     		//decode_AVP_Type(rv,avps[i]);
	    	   
	    	    recOfproxy_Info.appendField(rv);
			}
			rv_DiameterM.setField("proxy_Info",recOfproxy_Info);
        }
		return;
	}
	
	private void decode_Route_Record_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Route_Record_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			RecordOfValue recOfRoute_Record = (RecordOfValue) rv_DiameterM.getField("route_Record").getType().newInstance();
			recOfRoute_Record.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
	        	RecordValue rv = (RecordValue) recOfRoute_Record.getElementType().newInstance();
	        	
	        	decode_AVPHeader( rv ,avps[i]);
				decode_DiameterIdentity(rv, avps[i]);//(CharstringValue)rv.getField("aVP_Data"));
			
				recOfRoute_Record.appendField(rv);
	        }
			rv_DiameterM.setField("route_Record",recOfRoute_Record);
		}
		else{}
	    return;	
	}
	
	private void decode_Error_Reporting_Host_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Error_Reporting_Host_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue error_Reporting_Host = (RecordValue) rv_DiameterM.getField("error_Reporting_Host").getType().newInstance();
			
			decode_AVPHeader( error_Reporting_Host ,avps[0]);
			decode_DiameterIdentity(error_Reporting_Host, avps[0]);//(CharstringValue)rv.getField("aVP_Data"));
			
			rv_DiameterM.setField("error_Reporting_Host",error_Reporting_Host);
		}
		return;
	}
	
	private void decode_Redirect_Host_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		boolean rv_present=false;
		AVP[] avps=msg.findAVPs(AVP_Redirect_Host_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			try{
			   RecordValue rv = (RecordValue) (RecordValue) rv_DiameterM.getField("redirect_Host").getType().newInstance();
			   
			   decode_AVPHeader( rv ,avps[0]);
			   decode_DiameterIdentity(rv, avps[0]);
			   
			   rv_DiameterM.setField("redirect_Host",rv);
			   rv_present=true;
			   return;
			}
			catch(Exception ex){
				//System.out.print(ex.getMessage());
			}
			if(!rv_present){	
				RecordOfValue recOfRedirect_Host = (RecordOfValue) rv_DiameterM.getField("redirect_Host").getType().newInstance();
				recOfRedirect_Host.setLength(0);
		
		        for(int i=0;i<avps.length;i++){			
		        	RecordValue rv = (RecordValue) recOfRedirect_Host.getElementType().newInstance();
		        	
		        	decode_AVPHeader( rv ,avps[i]);
		        	decode_DiameterIdentity(rv, avps[0]);
				
					recOfRedirect_Host.appendField(rv);
		        }
				rv_DiameterM.setField("redirect_Host",recOfRedirect_Host);
			}
		}
	    return;	
	}
	
	private void decode_Redirect_Host_Usage_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Redirect_Host_Usage_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvRedirect_Host_Usage = (RecordValue) (RecordValue) rv_DiameterM.getField("redirect_Host_Usage").getType().newInstance();
        	
			decode_AVPHeader( rvRedirect_Host_Usage ,avps[0]);
			rvRedirect_Host_Usage.setField("aVP_Data", mkInt(avps[0].int_data));
			
			rv_DiameterM.setField("redirect_Host_Usage",rvRedirect_Host_Usage);
        }
		return;
	}
	
    private void decode_redirect_Max_Cache_Time_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_redirect_Max_Cache_Time_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvredirect_Max_Cache_Time = (RecordValue) (RecordValue) rv_DiameterM.getField("redirect_Max_Cache_Time").getType().newInstance();
        	
			decode_AVPHeader( rvredirect_Max_Cache_Time ,avps[0]);
			rvredirect_Max_Cache_Time.setField("aVP_Data", mkInt64(avps[0].int_data));
			
			rv_DiameterM.setField("redirect_Max_Cache_Time",rvredirect_Max_Cache_Time);
        }
		return;
	}
    
	private void decode_Accounting_Record_Type_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		String s_ac_rt="";
		AVP[] avps=msg.findAVPs(AVP_Accounting_Record_Type_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvAccounting_Record_Type = (RecordValue) (RecordValue) rv_DiameterM.getField("accounting_Record_Type").getType().newInstance();
        	
			decode_AVPHeader( rvAccounting_Record_Type ,avps[0]);
			rvAccounting_Record_Type.setField("aVP_Data", mkInt(avps[0].int_data));
			
			/*switch(avps[0].int_data){
		        case 1: s_ac_rt="EVENT_RECORD_E";break;
		        case 2: s_ac_rt="START_RECORD_E";break;
		        case 3: s_ac_rt="INTERIM_RECORD_E";break;
		        case 4: s_ac_rt="STOP_RECORD_E";break;
		    }
			
		    EnumeratedValue e_acrt= (EnumeratedValue)rvAccounting_Record_Type.getField("aVP_Data");
	 	    e_acrt.setEnum(s_ac_rt);
		    rvAccounting_Record_Type.setField("aVP_Data", e_acrt);*/
			
			rv_DiameterM.setField("accounting_Record_Type",rvAccounting_Record_Type);
        }
		return;
	}
    
	private void decode_Accounting_Record_Number_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Accounting_Record_Number_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvAccounting_Record_Number = (RecordValue) (RecordValue) rv_DiameterM.getField("accounting_Record_Number").getType().newInstance();
        	
			decode_AVPHeader( rvAccounting_Record_Number ,avps[0]);
			rvAccounting_Record_Number.setField("aVP_Data", mkInt(avps[0].int_data));
			
			rv_DiameterM.setField("accounting_Record_Number",rvAccounting_Record_Number);
        }
		return;
	}
	
    private void decode_Accounting_Sub_Session_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Accounting_Sub_Session_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>1){}
		else{
			RecordValue rvAccounting_Sub_Session_Id = (RecordValue) (RecordValue) rv_DiameterM.getField("accounting_Sub_Session_Id").getType().newInstance();
        	
			decode_AVPHeader( rvAccounting_Sub_Session_Id ,avps[0]);
			rvAccounting_Sub_Session_Id.setField("aVP_Data", mkInt64(avps[0].int_data));
			
			rv_DiameterM.setField("accounting_Sub_Session_Id",rvAccounting_Sub_Session_Id);
        }
		return;
	}	

	private void decode_Acct_Session_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Acct_Session_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvAcct_Session_Id = (RecordValue) rv_DiameterM.getField("acct_Session_Id").getType().newInstance();
			
			decode_AVPHeader( rvAcct_Session_Id ,avps[0]);
			rvAcct_Session_Id.setField("aVP_Data", mkOctetstring(avps[0].data));
			
			rv_DiameterM.setField("acct_Session_Id",rvAcct_Session_Id);
		}
	    return;	
	}
	
	private void decode_Acct_Multi_Session_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Acct_Multi_Session_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvAcct_Multi_Session_Id = (RecordValue) rv_DiameterM.getField("acct_Multi_Session_Id").getType().newInstance();
			
			decode_AVPHeader( rvAcct_Multi_Session_Id ,avps[0]);
			rvAcct_Multi_Session_Id.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
						
			rv_DiameterM.setField("acct_Multi_Session_Id",rvAcct_Multi_Session_Id);
		}
	    return;	
	}
	
	private void decode_Acct_Interim_Interval_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Acct_Interim_Interval_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvAcct_Interim_Interval = (RecordValue) rv_DiameterM.getField("acct_Interim_Interval").getType().newInstance();
			
			decode_AVPHeader( rvAcct_Interim_Interval ,avps[0]);
			rvAcct_Interim_Interval.setField("aVP_Data", mkInt(avps[0].int_data));
						
			rv_DiameterM.setField("acct_Interim_Interval",rvAcct_Interim_Interval);
		}
	    return;	
	}
	
	private void decode_Accounting_Realtime_Required_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Accounting_Realtime_Required_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvAccounting_Realtime_Required = (RecordValue) rv_DiameterM.getField("accounting_Realtime_Required").getType().newInstance();
			
			decode_AVPHeader( rvAccounting_Realtime_Required ,avps[0]);
			rvAccounting_Realtime_Required.setField("aVP_Data", mkInt(avps[0].int_data));//TODO:Enumerated!!!
						
			rv_DiameterM.setField("accounting_Realtime_Required",rvAccounting_Realtime_Required);
		}
	    return;	
	}
	
    private void decode_Event_Timestamp_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Event_Timestamp_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvEvent_Timestamp = (RecordValue) rv_DiameterM.getField("event_Timestamp").getType().newInstance();
			
			decode_AVPHeader( rvEvent_Timestamp ,avps[0]);
			//rvEvent_Timestamp.setField("aVP_Data", mkInt64(avps[0].int_data));
			rvEvent_Timestamp.setField("aVP_Data", mkInt(avps[0].long_data));
			
			rv_DiameterM.setField("event_Timestamp",rvEvent_Timestamp);
		}
		return;
	}
    
	private void decode_Termination_Cause_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Termination_Cause_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvTermination_Cause = (RecordValue) rv_DiameterM.getField("termination_Cause").getType().newInstance();
			
			decode_AVPHeader( rvTermination_Cause ,avps[0]);
			rvTermination_Cause.setField("aVP_Data", mkInt(avps[0].int_data));//TODO:Enumerated!!!
						
			rv_DiameterM.setField("termination_Cause",rvTermination_Cause);
		}
	    return;	
	}
	
	private void decode_Class_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Class_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			RecordOfValue recOfClass = (RecordOfValue) rv_DiameterM.getField("class").getType().newInstance();
			recOfClass.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
	        	RecordValue rv = (RecordValue) recOfClass.getElementType().newInstance();
	        	
	        	decode_AVPHeader( rv ,avps[i]);
				rv.setField("aVP_Data", mkCv(avps[i].data.toString()));//TODO:decode octetstring
			
				recOfClass.appendField(rv);
	        }
			rv_DiameterM.setField("class",recOfClass);
		}
	    return;	
	}
	
	private void decode_Disconnect_Cause_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
				
		String cause_str="";
		
		AVP[] avps=msg.findAVPs(AVP_Disconnect_Cause_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvDisconnect_Cause = (RecordValue) rv_DiameterM.getField("disconnect_Cause").getType().newInstance();
			
			decode_AVPHeader( rvDisconnect_Cause ,avps[0]);
			rvDisconnect_Cause.setField("aVP_Data", mkInt(avps[0].int_data));
			
			/*switch(avps[0].int_data){
			    case 0: cause_str="REBOOTING_E";break;
			    case 1: cause_str="BUSY_E";break;
			    case 2: cause_str="DO_NOT_WANT_TO_TALK_TO_YOU_E";break;
			}
			
			EnumeratedValue cause= (EnumeratedValue)rvDisconnect_Cause.getField("aVP_Data");
			cause.setEnum(cause_str);
			rvDisconnect_Cause.setField("aVP_Data", cause);*/
				
			rv_DiameterM.setField("disconnect_Cause",rvDisconnect_Cause);
		}
	    return;	
	}
	/*End RFC3588 Types */
	
	/*Rx Types */
	/*End Rx Types */
	
	/*RFC4005 Types*/
	private void decode_Called_Station_Id_AVP(RecordValue rv_,	AVP avp_){
		AVP[] avp=avp_.findChildAVPs(AVP_Called_Station_Id_AVP);
		
		if (avp==null) return;
		
		if (avp.length>1){} //Should never be more than one AVP
		else{
			RecordValue rv = (RecordValue) rv_.getField("called_Station_Id").getType().newInstance();
			
			decode_AVPHeader( rv ,avp[0]);
			rv.setField("aVP_Data", mkCvUTF8(avp[0].getStringData()));
									
			rv_.setField("called_Station_Id",rv);
		}
	    return;
	}
	
	private void decode_Framed_IP_Address_AVP(RecordValue rv_,	AVP avp_){
		
		AVP[] avps=avp_.findChildAVPs(AVP_Framed_IP_Address_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvframed_IP_Addr = (RecordValue) rv_.getField("framed_IP_Address").getType().newInstance();
			
			decode_AVPHeader( rvframed_IP_Addr ,avps[0]);
			rvframed_IP_Addr.setField("aVP_Data", mkOctetstring(avps[0].data));
			
			rv_.setField("framed_IP_Address",rvframed_IP_Addr);
		}
	    return;	
	}
	
	private void decode_Framed_IP_Address_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Framed_IP_Address_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvframed_IP_Addr = (RecordValue) rv_DiameterM.getField("framed_IP_Address").getType().newInstance();
			
			decode_AVPHeader( rvframed_IP_Addr ,avps[0]);
			rvframed_IP_Addr.setField("aVP_Data", mkOctetstring(avps[0].data));
			
			rv_DiameterM.setField("framed_IP_Address",rvframed_IP_Addr);
		}
	    return;	
	}
	
	private void decode_Framed_IPv6_Prefix_AVP(RecordValue rv_,	AVP avp_){
		
		AVP[] avps=avp_.findChildAVPs(AVP_Framed_IPv6_Prefix_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvframed_IP_AddrPfx = (RecordValue) rv_.getField("framed_IPv6_Prefix").getType().newInstance();
			
			decode_AVPHeader( rvframed_IP_AddrPfx ,avps[0]);
			rvframed_IP_AddrPfx.setField("aVP_Data", mkOctetstring(avps[0].data));
			
			rv_.setField("framed_IPv6_Prefix",rvframed_IP_AddrPfx);
		}
	    return;	
	}
	
	private void decode_Framed_IPv6_Prefix_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Framed_IPv6_Prefix_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvframed_IP_AddrPfx = (RecordValue) rv_DiameterM.getField("framed_IPv6_Prefix").getType().newInstance();
			
			decode_AVPHeader( rvframed_IP_AddrPfx ,avps[0]);
			rvframed_IP_AddrPfx.setField("aVP_Data", mkOctetstring(avps[0].data));
			
			rv_DiameterM.setField("framed_IPv6_Prefix",rvframed_IP_AddrPfx);
		}
	    return;	
	}
	
	private void decode_Framed_Interface_Id_AVP(RecordValue rv_,	AVP avp_){
		
		AVP[] avps=avp_.findChildAVPs(AVP_Framed_Interface_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvframed_IP_AddrPfx = (RecordValue) rv_.getField("framed_Interface_Id").getType().newInstance();
			
			decode_AVPHeader( rvframed_IP_AddrPfx ,avps[0]);
			rvframed_IP_AddrPfx.setField("aVP_Data", mkOctetstring(avps[0].data));
			
			rv_.setField("framed_Interface_Id",rvframed_IP_AddrPfx);
		}
	    return;	
	}
	/*EndRFC4005 Types*/
	
	/*RFC4006 Types*/	
    private void decode_Service_Context_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		 
		AVP[] avps=msg.findAVPs(AVP_Service_Context_Id_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvService_Context_Id = (RecordValue) rv_DiameterM.getField("service_Context_Id").getType().newInstance();
			
			decode_AVPHeader( rvService_Context_Id ,avps[0]);
			rvService_Context_Id.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
						
			rv_DiameterM.setField("service_Context_Id",rvService_Context_Id);
		}
	    return;	
	}
    
    private void decode_CC_Request_Type_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
    	String cc_req="";
    	AVP[] avps=msg.findAVPs(AVP_CC_Request_Type_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvCC_Request_Type = (RecordValue) rv_DiameterM.getField("cC_Request_Type").getType().newInstance();
			
			decode_AVPHeader( rvCC_Request_Type ,avps[0]);
			rvCC_Request_Type.setField("aVP_Data", mkInt(avps[0].int_data));
			
			/*switch(avps[0].int_data){
	 	      //case 0: s_nf="S_CSCF_E";break;
	 	      case 1: cc_req="INITIAL_REQUEST_E";break;
	 	      case 2: cc_req="UPDATE_REQUEST_E";break;
	 	      case 3: cc_req="TERMINATION_REQUEST_E";break;
	 	      case 4: cc_req="EVENT_REQUEST_E";break;
	 	    }
	      	
	 	    EnumeratedValue e_ccreq= (EnumeratedValue)rvCC_Request_Type.getField("aVP_Data");
	 	    e_ccreq.setEnum(cc_req);
	 	    rvCC_Request_Type.setField("aVP_Data", e_ccreq);*/
								
			rv_DiameterM.setField("cC_Request_Type",rvCC_Request_Type);
		}
	    return;	
    }
    
    private void decode_CC_Request_Number_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
    	AVP[] avps=msg.findAVPs(AVP_CC_Request_Number_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvCC_Request_Number = (RecordValue) rv_DiameterM.getField("cC_Request_Number").getType().newInstance();
			
			decode_AVPHeader( rvCC_Request_Number ,avps[0]);
			rvCC_Request_Number.setField("aVP_Data", mkInt(avps[0].int_data));
						
			rv_DiameterM.setField("cC_Request_Number",rvCC_Request_Number);
		}
	    return;		
    }
    
    private void decode_Subscription_Id_Type_AVP(RecordValue rv_sid_type,	AVP sid){

    	String sid_str="";
		
		AVP avp=sid.findChildAVP(AVP_Subscription_Id_Type_AVP);
		if (avp==null) return;
		
		//if (avp.length>1){} //Should never be more than one AVP
		//else{
			RecordValue rvSubscription_Id_Type = (RecordValue) rv_sid_type.getField("subscription_Id_Type").getType().newInstance();
			
			decode_AVPHeader( rvSubscription_Id_Type ,avp);
			rvSubscription_Id_Type.setField("aVP_Data", mkInt(avp.int_data));
			
			/*switch(avp.int_data){
			    case 0: sid_str="END_USER_E164_E";break;
			    case 1: sid_str="END_USER_IMSI_E";break;
			    case 2: sid_str="END_USER_SIP_URI_E";break;
			    case 3: sid_str="END_USER_NAI_E";break;
			    case 4:	sid_str="END_USER_PRIVATE_E";break;
			}
			
			EnumeratedValue e_sid= (EnumeratedValue)rvSubscription_Id_Type.getField("aVP_Data");
			e_sid.setEnum(sid_str);
			rvSubscription_Id_Type.setField("aVP_Data", e_sid);*/
				
			rv_sid_type.setField("subscription_Id_Type",rvSubscription_Id_Type);
		//}
	    return;	
	}
    
    private void decode_Subscription_Id_Data_AVP(RecordValue rv_sid_data,	AVP sid){
	
		AVP avp=sid.findChildAVP(AVP_Subscription_Id_Data_AVP);
		if (avp==null) return;
		
		//if (avp.length>1){} //Should never be more than one AVP
		//else{
			RecordValue rvsubscription_Id_Data = (RecordValue) rv_sid_data.getField("subscription_Id_Data").getType().newInstance();
			
			decode_AVPHeader( rvsubscription_Id_Data ,avp);
			rvsubscription_Id_Data.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
			rv_sid_data.setField("subscription_Id_Data",rvsubscription_Id_Data);
		//}
	    return;	
	}
    
    private void decode_Subscription_Id_AVP(RecordValue rv_sid,	AVP sid){
    	
       AVP[] avps=sid.findChildAVPs(AVP_Subscription_Id_AVP);
	   if (avps==null) return;
    	
	   
	   RecordOfValue rov_Subscription_Id = (RecordOfValue) rv_sid.getField("subscription_Id").getType().newInstance();
	   rov_Subscription_Id.setLength(0);
	   //try{avp.ungroup();}catch(Exception e){return;}
	   
	   for (int i = 0; i < avps.length; i++) {
		    RecordValue rv = (RecordValue) rov_Subscription_Id.getElementType().newInstance();
		    
		    try{avps[0].ungroup();}catch(Exception e){return;}
		    
		    decode_AVPHeader( rv ,avps[i]);
    		decode_Subscription_Id_Type_AVP(rv,avps[i]);
    		decode_Subscription_Id_Data_AVP(rv,avps[i]);

    		rov_Subscription_Id.appendField(rv);
		}
	   
	   rv_sid.setField("subscription_Id",rov_Subscription_Id);
	   return;
 	}
    
    private void decode_Subscription_Id_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
	
    	boolean rv_present=false;
     	AVP[] avps=msg.findAVPs(AVP_Subscription_Id_AVP);
		if (avps==null) return;
    	
        if (avps.length>=1){
        	try{
 			   RecordValue rv_Subscription_Id = (RecordValue) (RecordValue) rv_DiameterM.getField("subscription_Id").getType().newInstance();
 			   
 			   try{avps[0].ungroup();}catch(Exception e){return;}
 			   
 			   decode_AVPHeader( rv_Subscription_Id ,avps[0]);
 			   decode_Subscription_Id_Type_AVP(rv_Subscription_Id,avps[0]);
	     	   decode_Subscription_Id_Data_AVP(rv_Subscription_Id,avps[0]);
 			   
 			   rv_DiameterM.setField("subscription_Id",rv_Subscription_Id);
 			   rv_present=true;
 			   return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
        	        	
				RecordOfValue recOfSubscription_Id = (RecordOfValue) rv_DiameterM.getField("subscription_Id").getType().newInstance();
				recOfSubscription_Id.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfSubscription_Id.getElementType().newInstance();
				    
				    try{avps[0].ungroup();}catch(Exception e){return;}
				    
				    decode_AVPHeader( rv ,avps[i]);
		     		decode_Subscription_Id_Type_AVP(rv,avps[i]);
		     		decode_Subscription_Id_Data_AVP(rv,avps[i]);

		    	    recOfSubscription_Id.appendField(rv);
				}
				rv_DiameterM.setField("subscription_Id",recOfSubscription_Id);
 			}
        }
		return;	
	}
    
    private void decode_Service_Identifier_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
    	boolean rv_present=false;
		AVP[] avps=msg.findAVPs(AVP_Service_Identifier_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			try{
				RecordValue rvService_Identifier = (RecordValue) rv_DiameterM.getField("service_Identifier").getType().newInstance();
				
				decode_AVPHeader( rvService_Identifier ,avps[0]);
				rvService_Identifier.setField("aVP_Data", mkInt(avps[0].int_data));
							
				rv_DiameterM.setField("service_Identifier",rvService_Identifier);
			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
 				RecordOfValue recOfService_Identifier = (RecordOfValue) rv_DiameterM.getField("service_Identifier").getType().newInstance();
				recOfService_Identifier.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfService_Identifier.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
		        	rv.setField("aVP_Data", mkInt(avps[i].int_data));	
		    	   
		    	    recOfService_Identifier.appendField(rv);
				}
				rv_DiameterM.setField("service_Identifier",recOfService_Identifier);
 			}
		}
	    return;	
	}
    
    private void decode_Requested_Action_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
	    //System.out.println("decode_Requested_Action_AVP");
    	String s_ra="";
		AVP avp=msg.findAVP(AVP_Requested_Action_AVP);
		if (avp==null) return;
		
		RecordValue rv_Request_Action = (RecordValue) rv_DiameterM.getField("requested_Action").getType().newInstance();
		
		decode_AVPHeader( rv_Request_Action ,avp);
		rv_Request_Action.setField("aVP_Data", mkInt(avp.int_data));
		
		/*switch(avp.int_data){
 	      case 0: s_ra="DIRECT_DEBITING_E";break;
 	      case 1: s_ra="REFUND_ACCOUNT_E";break;
 	      case 2: s_ra="CHECK_BALANCE_E";break;
 	      case 3: s_ra="PRICE_ENQUIRY_E";break;
 	    }
 	        
 	    EnumeratedValue e_ra= (EnumeratedValue)rv_Request_Action.getField("aVP_Data");
 	    e_ra.setEnum(s_ra);
 	    rv_Request_Action.setField("aVP_Data", e_ra);*/
							
		rv_DiameterM.setField("requested_Action",rv_Request_Action);
		
	    return;	
    }
    
    private void  decode_Multiple_Services_Indicator_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
    	AVP avp=msg.findAVP(AVP_Multiple_Services_Indicator_AVP);
		if (avp==null) return;
			
		RecordValue rv_ = (RecordValue) rv_DiameterM.getField("multiple_Services_Indicator").getType().newInstance();
		
		decode_AVPHeader( rv_ ,avp);
		rv_.setField("aVP_Data", mkInt(avp.int_data));
					
		rv_DiameterM.setField("multiple_Services_Indicator",rv_);
	
        return;
    }    
    
    private void  decode_CC_Session_Failover_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
    	AVP avp=msg.findAVP(AVP_CC_Session_Failover_AVP);
		if (avp==null) return;
		
		RecordValue rv_ = (RecordValue) rv_DiameterM.getField("cC_Session_Failover").getType().newInstance();
		
		decode_AVPHeader( rv_ ,avp);
		rv_.setField("aVP_Data", mkInt(avp.int_data));
					
		rv_DiameterM.setField("cC_Session_Failover",rv_);
		
        return;	
    }
    
    private void  decode_Cost_Unit_AVP(RecordValue rv_sid,	AVP sid){
    	
    	AVP avp=sid.findChildAVP(AVP_Cost_Unit_AVP);
		if (avp==null) return;
		
		RecordValue rv_ = (RecordValue) rv_sid.getField("cost_Unit").getType().newInstance();
		
		decode_AVPHeader( rv_ ,avp);
		rv_.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
					
		rv_sid.setField("cost_Unit",rv_);
		
        return;	
    }
    
    private void  decode_Cost_Information_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
    	AVP avp=msg.findAVP(AVP_Cost_Information_AVP);
		if (avp==null) return;
		
		RecordValue rv_ = (RecordValue) rv_DiameterM.getField("cost_Information").getType().newInstance();
		
		try{avp.ungroup();}catch(Exception e){return;}
		
		decode_AVPHeader( rv_ ,avp);
		decode_Unit_Value_AVP( rv_ ,avp);      //           unit_Value,
		decode_Currency_Code_AVP( rv_ ,avp);     //        currency_Code,
		decode_Cost_Unit_AVP( rv_ ,avp);          //       cost_Unit optional
					
		rv_DiameterM.setField("cost_Information",rv_);
		
        return;	
    }
    
    private void decode_Low_Balance_Indication_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
    	AVP avp=msg.findAVP(AVP_Low_Balance_Indication_AVP);
		if (avp==null) return;
		
		RecordValue rv_ = (RecordValue) rv_DiameterM.getField("low_Balance_Indication").getType().newInstance();
		
		decode_AVPHeader( rv_ ,avp);
		rv_.setField("aVP_Data", mkInt(avp.int_data));
					
		rv_DiameterM.setField("low_Balance_Indication",rv_);
		
        return;	
    }
    
    private void decode_Remaining_Balance_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
    	AVP avp=msg.findAVP(AVP_Remaining_Balance_AVP);
		if (avp==null) return;
		
		RecordValue rv_ = (RecordValue) rv_DiameterM.getField("remaining_Balance").getType().newInstance();
		
		try{avp.ungroup();}catch(Exception e){return;}
		
		decode_AVPHeader( rv_ ,avp);
        decode_Unit_Value_AVP( rv_ ,avp);//      unit_Value,
        decode_Currency_Code_AVP( rv_ ,avp);//   currency_Code
					
		rv_DiameterM.setField("remaining_Balance",rv_);
		
        return;	
    }
    
    private void  decode_Credit_Control_Failure_Handling_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
    	AVP avp=msg.findAVP(AVP_Credit_Control_Failure_Handling_AVP);
		if (avp==null) return;
		
		RecordValue rv_ = (RecordValue) rv_DiameterM.getField("credit_Control_Failure_Handling").getType().newInstance();
		
		decode_AVPHeader( rv_ ,avp);
		rv_.setField("aVP_Data", mkInt(avp.int_data));
		
		rv_DiameterM.setField("credit_Control_Failure_Handling",rv_);
	
        return;	
    }
    
    private void  decode_Direct_Debiting_Failure_Handling_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
    	AVP avp=msg.findAVP(AVP_Direct_Debiting_Failure_Handling_AVP);
		if (avp==null) return;
		
		RecordValue rv_ = (RecordValue) rv_DiameterM.getField("direct_Debiting_Failure_Handling").getType().newInstance();
		
		decode_AVPHeader( rv_ ,avp);
		rv_.setField("aVP_Data", mkInt(avp.int_data));
		
					
		rv_DiameterM.setField("direct_Debiting_Failure_Handling",rv_);
	
        return;	
    }
    
    
    
	/*End RFC 4006 Types*/
    
    /*RFC4740 AVPs*/
    private void decode_Auth_Session_State_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		 
    	String s_ass="";
		AVP[] avps=msg.findAVPs(AVP_Auth_Session_State_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rv_Auth_Session_State = (RecordValue) rv_DiameterM.getField("auth_Session_State").getType().newInstance();
			
			decode_AVPHeader( rv_Auth_Session_State ,avps[0]);
			rv_Auth_Session_State.setField("aVP_Data", mkInt(avps[0].int_data));
			
			/*switch(avps[0].int_data){
	 	      case 0: s_ass="STATE_MAINTAINED_E";break;
	 	      case 1: s_ass="NO_STATE_MAINTAINED_E";break;

	 	    }
	 	        
	 	    EnumeratedValue e_ass= (EnumeratedValue)rv_Auth_Session_State.getField("aVP_Data");
	 	    e_ass.setEnum(s_ass);
	 	    rv_Auth_Session_State.setField("aVP_Data", e_ass);*/
									
			rv_DiameterM.setField("auth_Session_State",rv_Auth_Session_State);
		}
	    return;	
	}
    
    
    private void decode_SIP_Method_AVP(RecordValue rv_sip_m,	AVP sip_m){
		 
		AVP avp=sip_m.findChildAVP(AVP_SIP_Method_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Method = (RecordValue) rv_sip_m.getField("sIP_Method").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Method ,avp);
		rvSIP_Method.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_sip_m.setField("sIP_Method",rvSIP_Method);
		
	    return;	
	}
    
    private void decode_SIP_Method_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		 
		AVP[] avps=msg.findAVPs(AVP_SIP_Method_AVP);
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvSIP_Method = (RecordValue) rv_DiameterM.getField("sIP_Method").getType().newInstance();
			
			decode_AVPHeader( rvSIP_Method ,avps[0]);
			rvSIP_Method.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
						
			rv_DiameterM.setField("sIP_Method",rvSIP_Method);
		}
	    return;	
	}
    
    private void decode_SIP_Number_Auth_Items_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		 
		AVP[] avpsrfc=msg.findAVPs(AVP_SIP_Number_Auth_Items_RFC_AVP);
		AVP[] avps3gpp=msg.findAVPs(AVP_SIP_Number_Auth_Items_3GPP_AVP);
		AVP[] avps=null;
		
		if (avpsrfc!=null) avps=avpsrfc;
		if (avps3gpp!=null) avps=avps3gpp;
		
		if (avps==null) return;
		
		if (avps.length>1){} //Should never be more than one AVP
		else{
			RecordValue rvSIP_AuthItem = (RecordValue) rv_DiameterM.getField("sIP_Number_Auth_Items").getType().newInstance();
			
			decode_AVPHeader( rvSIP_AuthItem ,avps[0]);
			rvSIP_AuthItem.setField("aVP_Data", mkInt(avps[0].int_data));
						
			rv_DiameterM.setField("sIP_Number_Auth_Items",rvSIP_AuthItem);
		}
	    return;	
	}
    
    
    private void decode_SIP_Auth_Data_Item_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    
    	boolean rv_present=false; 
		AVP[] avpsrfc=msg.findAVPs(AVP_SIP_Auth_Data_Item_RFC_AVP);	
		AVP[] avps3gpp=msg.findAVPs(AVP_SIP_Auth_Data_Item_3GPP_AVP);
	
		if ((avpsrfc==null) && (avps3gpp==null)) return;
		
		if ((avpsrfc!=null)&&(avpsrfc.length>0)){
			try{
				UnionValue rvSIP_Auth = (UnionValue) rv_DiameterM.getField("sIP_Auth_Data_Item").getType().newInstance();
				RecordValue rv_Auth = (RecordValue) rvSIP_Auth.getVariant("sIP_Auth_Data_Item_RFC_AVP").getType().newInstance();
				 
				decode_SIP_Auth_Data_Item_RFC_AVP(rv_Auth,	avpsrfc[0]);
							
				rvSIP_Auth.setVariant("sIP_Auth_Data_Item_RFC_AVP", rv_Auth);
				rv_DiameterM.setField("sIP_Auth_Data_Item",rvSIP_Auth);
				rv_present=true;
			}
            catch(Exception ex){}
			if(!rv_present){
				RecordOfValue rov_rt = (RecordOfValue) rv_DiameterM.getField("sIP_Auth_Data_Item").getType().newInstance();
			    rov_rt.setLength(0);
			    
			    for (int i = 0; i < avpsrfc.length; i++) {
				    UnionValue rvSIP_Auth = (UnionValue) rov_rt.getElementType().newInstance();
				    RecordValue rv_Auth = (RecordValue) rvSIP_Auth.getVariant("sIP_Auth_Data_Item_RFC_AVP").getType().newInstance();
					 
					decode_SIP_Auth_Data_Item_RFC_AVP(rv_Auth,	avpsrfc[i]);
								
					rvSIP_Auth.setVariant("sIP_Auth_Data_Item_RFC_AVP", rv_Auth);		           
		            rov_rt.appendField(rvSIP_Auth);
 				}			    	    
			    rv_DiameterM.setField("sIP_Auth_Data_Item",rov_rt);
 			}			
		}
		
		if ((avps3gpp!=null)&&(avps3gpp.length>0)){
            
			try{
	            UnionValue rvSIP_Auth = (UnionValue) rv_DiameterM.getField("sIP_Auth_Data_Item").getType().newInstance();
				RecordValue rv_Auth = (RecordValue) rvSIP_Auth.getVariant("sIP_Auth_Data_Item_3GPP_AVP").getType().newInstance();
				
				decode_SIP_Auth_Data_Item_3GPP_AVP(rv_Auth,	avps3gpp[0]);
				
				rvSIP_Auth.setVariant("sIP_Auth_Data_Item_3GPP_AVP", rv_Auth);
				rv_DiameterM.setField("sIP_Auth_Data_Item",rvSIP_Auth);
				rv_present=true;
            }
            catch(Exception ex){}
			if(!rv_present){
				RecordOfValue rov_rt = (RecordOfValue) rv_DiameterM.getField("sIP_Auth_Data_Item").getType().newInstance();
			    rov_rt.setLength(0);
			    
			    for (int i = 0; i < avps3gpp.length; i++) {
				    UnionValue rvSIP_Auth = (UnionValue) rov_rt.getElementType().newInstance();
				    RecordValue rv_Auth = (RecordValue) rvSIP_Auth.getVariant("sIP_Auth_Data_Item_3GPP_AVP").getType().newInstance();
				   
				    decode_SIP_Auth_Data_Item_3GPP_AVP(rv_Auth,	avps3gpp[i]);
				    		
					rvSIP_Auth.setVariant("sIP_Auth_Data_Item_3GPP_AVP", rv_Auth);	           
		            rov_rt.appendField(rvSIP_Auth);
 				}
			    rv_DiameterM.setField("sIP_Auth_Data_Item",rov_rt);
 			}
		}
	    return;	
	}
    
    private void decode_SIP_Auth_Data_Item_RFC_AVP(RecordValue rv_,	AVP avp_){
		
		decode_AVPHeader( rv_ ,avp_);
        decode_SIP_Authentication_Scheme_AVP( rv_ ,avp_);//      sIP_Authentication_Scheme,//requ.
        decode_SIP_Item_Number_AVP( rv_ ,avp_);//             sIP_Item_Number optional,
        decode_SIP_Authenticate_AVP( rv_ ,avp_);//             sIP_Authenticate optional,
        decode_SIP_Authorization_AVP( rv_ ,avp_);//             sIP_Authorization optional,
        //decode_SIP_Authentication_Info_AVP( rv_ ,avp_);//        sIP_Authentication_Info optional,
        //set of AVP_Type                 aVP_Type optional
	    return;
    }
    
    private void decode_SIP_Authentication_Scheme_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP[] avpsrfc=avp_.findChildAVPs(AVP_SIP_Authentication_Scheme_RFC_AVP);
    	AVP[] avps3gpp=avp_.findChildAVPs(AVP_SIP_Authentication_Scheme_3GPP_AVP);
    	
		if ((avpsrfc==null) && (avps3gpp==null)) return;
    	
		UnionValue rvSIP_AuthScheme = (UnionValue) rv_.getField("sIP_Authentication_Scheme").getType().newInstance();
		
		if ((avpsrfc!=null)&&(avpsrfc.length==1)){
			RecordValue rv_Auth = (RecordValue) rvSIP_AuthScheme.getVariant("sIP_Authentication_Scheme_RFC_AVP").getType().newInstance();
			 
			decode_SIP_Authentication_Scheme_RFC_AVP(rv_Auth,	avpsrfc[0]);
						
			rvSIP_AuthScheme.setVariant("sIP_Authentication_Scheme_RFC_AVP", rv_Auth);
		}
		if ((avps3gpp!=null)&&(avps3gpp.length==1)){
			RecordValue rv_Auth = (RecordValue) rvSIP_AuthScheme.getVariant("sIP_Authentication_Scheme_3GPP_AVP").getType().newInstance();
			
			decode_SIP_Authentication_Scheme_3GPP_AVP(rv_Auth,	avps3gpp[0]);
			
			rvSIP_AuthScheme.setVariant("sIP_Authentication_Scheme_3GPP_AVP", rv_Auth);
		}
		rv_.setField("sIP_Authentication_Scheme",rvSIP_AuthScheme);
    	return;
    }
    
    private void decode_SIP_Authentication_Scheme_RFC_AVP(RecordValue rv_,	AVP avp_){
    	
    	decode_AVPHeader( rv_ ,avp_);
    	rv_.setField("aVP_Data", mkInt(avp_.int_data));
//	        DIGEST_E (0),  // RFC4740 Clause 9.5.1.  SIP-Authentication-Scheme AVP
//	        DIGEST_AKAv1_MD5_E (1), // TODO CHeck the value are correct
//	        NASS_BUNDLE_E (2), // TODO CHeck the value are correct
//	        GIBA_E (3), // TODO CHeck the value are correct
//	        UNKNOWN_E
    	return;
    }
    
    private void decode_SIP_Item_Number_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_SIP_Item_Number_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("sIP_Item_Number").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data",mkInt(avp_.int_data));
						
		rv_.setField("sIP_Item_Number",rvSIP_Item);
	    return;	
	}
    
    private void decode_SIP_Authenticate_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP[] avpsrfc=avp_.findChildAVPs(AVP_SIP_Authenticate_RFC_AVP);
    	AVP[] avps3gpp=avp_.findChildAVPs(AVP_SIP_Authenticate_3GPP_AVP);
    	
		if ((avpsrfc==null) && (avps3gpp==null)) return;
    	
		UnionValue rvSIP_Authenticate = (UnionValue) rv_.getField("sIP_Authenticate").getType().newInstance();
		
		if ((avpsrfc!=null)&&(avpsrfc.length==1)){
			RecordValue rv_Auth = (RecordValue) rvSIP_Authenticate.getVariant("sIP_Authenticate_RFC_AVP").getType().newInstance();
			 
			decode_SIP_Authenticate_RFC_AVP(rv_Auth,	avpsrfc[0]);
						
			rvSIP_Authenticate.setVariant("sIP_Authenticate_RFC_AVP", rv_Auth);
		}
		if ((avps3gpp!=null)&&(avps3gpp.length==1)){
			RecordValue rv_Auth = (RecordValue) rvSIP_Authenticate.getVariant("sIP_Authenticate_3GPP_AVP").getType().newInstance();
			
			decode_SIP_Authenticate_3GPP_AVP(rv_Auth,	avps3gpp[0]);
			
			rvSIP_Authenticate.setVariant("sIP_Authenticate_3GPP_AVP", rv_Auth);
		}
		rv_.setField("sIP_Authenticate",rvSIP_Authenticate);
    	return;
    }
    
    private void decode_SIP_Authenticate_RFC_AVP(RecordValue rv_,	AVP avp_){
    	
    	decode_AVPHeader( rv_ ,avp_);
//         Digest_Realm_AVP               digest_Realm,//requ.
//         Digest_Nonce_AVP               digest_Nonce,//requ.
//         Digest_Domain_AVP              digest_Domain  optional,
//         Digest_Opaque_AVP              digest_Opaque optional,
//         Digest_Stale_AVP               digest_Stale optional,
//         Digest_Algorithm_AVP           digest_Algorithm optional,
//         Digest_QoP_AVP                 digest_QoP optional,
//         Digest_HA1_AVP                 digest_HA1 optional,
//         set of Digest_Auth_Param_AVP   digest_Auth_Param  optional,
//         set of AVP_Type                aVP_Type optional
        
    	return;
    }
    
    private void decode_SIP_Authorization_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP[] avpsrfc=avp_.findChildAVPs(AVP_SIP_Authorization_RFC_AVP);
    	AVP[] avps3gpp=avp_.findChildAVPs(AVP_SIP_Authorization_3GPP_AVP);
    	
		if ((avpsrfc==null) && (avps3gpp==null)) return;
    	
		UnionValue rvSIP_Authenticate = (UnionValue) rv_.getField("sIP_Authorization").getType().newInstance();
		
		if ((avpsrfc!=null)&&(avpsrfc.length==1)){
			RecordValue rv_Auth = (RecordValue) rvSIP_Authenticate.getVariant("sIP_Authorization_RFC_AVP").getType().newInstance();
			 
			decode_SIP_Authorization_RFC_AVP(rv_Auth,	avpsrfc[0]);
						
			rvSIP_Authenticate.setVariant("sIP_Authorization_RFC_AVP", rv_Auth);
		}
		if ((avps3gpp!=null)&&(avps3gpp.length==1)){
			RecordValue rv_Auth = (RecordValue) rvSIP_Authenticate.getVariant("sIP_Authorization_3GPP_AVP").getType().newInstance();
			
			decode_SIP_Authorization_3GPP_AVP(rv_Auth,	avps3gpp[0]);
			
			rvSIP_Authenticate.setVariant("sIP_Authorization_3GPP_AVP", rv_Auth);
		}
		rv_.setField("sIP_Authorization",rvSIP_Authenticate);
    	return;
    }
    
    private void decode_SIP_Authorization_RFC_AVP(RecordValue rv_,	AVP avp_){
    	
    	decode_AVPHeader( rv_ ,avp_);
//    	  Digest_Username_AVP         digest_Username,//requi.
//          Digest_Realm_AVP             digest_Realm,
//          Digest_Nonce_AVP             digest_Nonce,
//          Digest_URI_AVP                 digest_URI,
//          Digest_Response_AVP         digest_Response,//requi.
//          Digest_Algorithm_AVP         digest_Algorithm optional,
//          Digest_CNonce_AVP              digest_CNonce optional,
//          Digest_Opaque_AVP             digest_Opaque optional,
//          Digest_QoP_AVP              digest_QoP optional,
//          Digest_Nonce_Count_AVP         digest_Nonce_Count optional,
//          Digest_Method_AVP             digest_Method optional,
//          Digest_Entity_Body_Hash_AVP     digest_Entity_Body_Hash optional,
//          set of Digest_Auth_Param_AVP     digest_Auth_Param  optional,
//          set of AVP_Type                 aVP_Type optional
        
    	return;
    }
    /*End RFC4740 AVPs*/
    
    /* ******** Digest AVPs ********* */
	private void decode_Digest_Realm_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Realm_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Realm").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Realm",rvSIP_Item);
	    return;	
	}

	private void decode_Digest_Nonce_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Nonce_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Nonce").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Nonce",rvSIP_Item);
	    return;	
	}

	private void decode_Digest_Domain_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Domain_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Domain").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Domain",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Opaque_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Opaque_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("Digest_Opaque").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("Digest_Opaque",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Stale_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Stale_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Stale").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Stale",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Algorithm_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Algorithm_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Algorithm").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Algorithm",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_QoP_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_QoP_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_QoP").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_QoP",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_HA1_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_HA1_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_HA1").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_HA1",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Auth_Param_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Auth_Param_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Auth_Param").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Auth_Param",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Username_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Username_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Username").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Username",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_URI_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_URI_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_URI").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_URI",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Response_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Response_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Response").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Response",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_CNonce_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_CNonce_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_CNonce").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_CNonce",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Nonce_Count_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Nonce_Count_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Nonce_Count").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Nonce_Count",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Method_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Method_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Method").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Method",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Entity_Body_Hash_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Entity_Body_Hash_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Entity_Body_Hash").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Entity_Body_Hash",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Nextnonce_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Nextnonce_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Nextnonce").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Nextnonce",rvSIP_Item);
	    return;	
	}
	
	private void decode_Digest_Response_Auth_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Digest_Response_Auth_AVP);
		if (avp==null) return;
		
		RecordValue rvSIP_Item = (RecordValue) rv_.getField("digest_Response_Auth").getType().newInstance();
			
		decode_AVPHeader( rvSIP_Item ,avp);
		rvSIP_Item.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
						
		rv_.setField("digest_Response_Auth",rvSIP_Item);
	    return;	
	}
    /* ************ end of Digest AVPs************ */
    
    /*TS129_229*/
    private void decode_IP_CAN_Type_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	String s_rt="";
    	AVP avp=rt.findAVP(AVP_IP_CAN_Type_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_rt = (RecordValue) (RecordValue) rv_rid.getField("iP_CAN_Type").getType().newInstance();
	   
	    decode_AVPHeader( rv_rt ,avp);
	    rv_rt.setField("aVP_Data", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	           
	      case 0   : s_rt="ThreeGPP_GPRS_E";break; 
	      case 1   : s_rt="DOCSIS_E";break; 
	      case 2: s_rt="XDSL_E";break; 
	      case 3: s_rt="WIMAX_E";break; 
	      case 4: s_rt="ThreeGPP2_E";break; 
	      case 5: s_rt="ThreeGPP_EPS_E";break; 
	      case 6: s_rt="NON_3GPP_EPS_E";break; 
	 
	    }
	        
	    EnumeratedValue e_rt= (EnumeratedValue)rv_rt.getField("aVP_Data");
	    e_rt.setEnum(s_rt);
	    rv_rt.setField("aVP_Data", e_rt);*/
	    
	    rv_rid.setField("iP_CAN_Type",rv_rt);
    	return;
    }
    
    private void decode_RAT_Type_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	String s_rt="";
    	AVP avp=rt.findAVP(AVP_RAT_Type_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_rt = (RecordValue) (RecordValue) rv_rid.getField("rat_Type").getType().newInstance();
	   
	    decode_AVPHeader( rv_rt ,avp);
	    rv_rt.setField("aVP_Data", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	           
	      case 0   : s_rt="WLAN_E";break; 
	      case 1   : s_rt="VIRTUAL_E";break; 
	      case 1000: s_rt="UTRAN_E";break; 
	      case 1001: s_rt="GERAN_E";break; 
	      case 1002: s_rt="GAN_E";break; 
	      case 1003: s_rt="HSPA_EVOLUTION_E";break; 
	      case 1004: s_rt="EUTRAN_E";break; 
	      case 2000: s_rt="CDMA2000_IX_E";break; 
	      case 2001: s_rt="HRPD_E";break;
	      case 2002: s_rt="UMB_E";break;
	      case 2003: s_rt="EHRPD_E";break;

	    }
	        
	    EnumeratedValue e_rt= (EnumeratedValue)rv_rt.getField("aVP_Data");
	    e_rt.setEnum(s_rt);
	    rv_rt.setField("aVP_Data", e_rt);*/
	    
	    rv_rid.setField("rat_Type",rv_rt);
    	return;
    }
    
    private void decode_EventTrigger(RecordValue r,AVP a){
    	
    	String s_rt="";
    	decode_AVPHeader(r,a);
    	r.setField("aVP_Data", mkInt(a.int_data));
	   
    	/*switch(a.int_data){
	      
	      case 0   : s_rt="SGSN_CHANGE_E";break;
	      case 1   : s_rt="QOS_CHANGE_E";break;
	      case 2   : s_rt="RAT_CHANGE_E";break;
	      case 3   : s_rt="TFT_CHANGE_E";break;
	      case 4   : s_rt="PLMN_CHANGE_E";break;
	      case 5   : s_rt="LOSS_OF_BEARER_E";break;
	      case 6   : s_rt="RECOVERY_OF_BEARER_E";break;
	      case 7   : s_rt="IP_CAN_CHANGE_E";break;
	      case 11  : s_rt="QOS_CHANGE_EXCEEDING_AUTHORIZATION_E";break;
	      case 12  : s_rt="RAI_CHANGE_E";break;
	      case 13  : s_rt="USER_LOCATION_CHANGE_E";break;
	      case 14  : s_rt="NO_EVENT_TRIGGERS_E";break;
	      case 15  : s_rt="OUT_OF_CREDIT_E";break;
	      case 16  : s_rt="REALLOCATION_OF_CREDIT_E";break;
	      case 17  : s_rt="REVALIDATION_TIMEOUT_E";break;
	      case 18  : s_rt="UE_IP_ADDRESS_ALLOCATE_E";break;
	      case 19  : s_rt="UE_IP_ADDRESS_RELEASE_E";break;
	      case 20  : s_rt="DEFAULT_EPS_BEARER_QOS_CHANGE_E";break;
	      case 21  : s_rt="AN_GW_CHANGE_E";break;
	      case 22  : s_rt="SUCCESSFUL_RESOURCE_ALLOCATION_E";break;
	      case 23  : s_rt="RESOURCE_MODIFICATION_REQUEST_E";break;
	      case 24  : s_rt="PGW_TRACE_CONTROL_E";break;
	      case 25  : s_rt="UE_TIME_ZONE_CHANGE_E";break;
	      case 26  : s_rt="TAI_CHANGE_E";break;
	      case 27  : s_rt="ECGI_CHANGE_E";break;
	      case 28  : s_rt="CHARGING_CORRELATION_EXCHANGE_E";break;
	      case 29  : s_rt="APN_AMBR_MODIFICATION_FAILURE_E";break;
	      case 30  : s_rt="USER_CSG_INFORMATION_CHANGE_E";break;
	      case 33  : s_rt="USAGE_REPORT_E";break;
	      case 34  : s_rt="DEFAULT_EPS_BEARER_QOS_MODIFICATION_FAILURE_E";break;
	      case 35  : s_rt="USER_CSG_HYBRID_SUBSCRIBED_INFORMATION_CHANGE_E";break;
	      case 36  : s_rt="USER_CSG_HYBRID_UNSUBSCRIBED_INFORMATION_CHANGE_E";break;
	      case 37  : s_rt="ROUTING_RULE_CHANGE_E";break;
	      case 38  : s_rt="MAX_MBR_APN_AMBR_CHANGE_E";break;
	    }
	        
	    EnumeratedValue e_rt= (EnumeratedValue)r.getField("aVP_Data");
	    e_rt.setEnum(s_rt);
	    r.setField("aVP_Data", e_rt);*/
    	
    	return;
    }
    
    private void decode_Event_Trigger_AVP(RecordValue rv_,	AVP avp_){
		
		AVP[] avps=avp_.findChildAVPs(AVP_Event_Trigger_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			RecordOfValue recOfRoute_Record = (RecordOfValue) rv_.getField("event_Trigger").getType().newInstance();
			recOfRoute_Record.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
	        	RecordValue rv = (RecordValue) recOfRoute_Record.getElementType().newInstance();
	        	//decode_AVPHeader( rv ,avps[i]);
				decode_EventTrigger(rv, avps[i]);//(CharstringValue)rv.getField("aVP_Data"));
			
				recOfRoute_Record.appendField(rv);
	        }
			rv_.setField("event_Trigger",recOfRoute_Record);
		}
		else{}
	    return;	
	}
    
    private void decode_Event_Trigger_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		
		AVP[] avps=msg.findAVPs(AVP_Event_Trigger_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			RecordOfValue recOfRoute_Record = (RecordOfValue) rv_DiameterM.getField("event_Trigger").getType().newInstance();
			recOfRoute_Record.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
	        	RecordValue rv = (RecordValue) recOfRoute_Record.getElementType().newInstance();
	        	//decode_AVPHeader( rv ,avps[i]);
				decode_EventTrigger(rv, avps[i]);//(CharstringValue)rv.getField("aVP_Data"));
			
				recOfRoute_Record.appendField(rv);
	        }
			rv_DiameterM.setField("event_Trigger",recOfRoute_Record);
		}
		else{}
	    return;	
	}
    
    private void decode_Session_Release_Cause_AVP(RecordValue rv_,	AVP avp_){
    	
    	String s_rt="";
    	AVP avp=avp_.findChildAVP(AVP_Session_Release_Cause_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_rt = (RecordValue) (RecordValue) rv_.getField("session_Release_Cause").getType().newInstance();
	   
	    decode_AVPHeader( rv_rt ,avp);
	    rv_rt.setField("aVP_Data", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	      
	      case 0   : s_rt="UNSPECIFIED_REASON_E";break;
	      case 1   : s_rt="UE_SUBSCRIPTION_REASON_E";break;
	      case 2   : s_rt="INSUFFICIENT_SERVER_RESOURCES_E";break;
	      
	    }
	    
	    EnumeratedValue e_rt= (EnumeratedValue)rv_rt.getField("aVP_Data");
	    e_rt.setEnum(s_rt);
	    rv_rt.setField("aVP_Data", e_rt);*/
    	
	    rv_.setField("session_Release_Cause",rv_rt);
    }
    
    private void decode_Session_Release_Cause_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
    	String s_rt="";

    	AVP avp=msg.findAVP(AVP_Session_Release_Cause_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_rt = (RecordValue) (RecordValue) rv_DiameterM.getField("session_Release_Cause").getType().newInstance();
	   
	    decode_AVPHeader( rv_rt ,avp);
	    rv_rt.setField("aVP_Data", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	      
	      case 0   : s_rt="UNSPECIFIED_REASON_E";break;
	      case 1   : s_rt="UE_SUBSCRIPTION_REASON_E";break;
	      case 2   : s_rt="INSUFFICIENT_SERVER_RESOURCES_E";break;
	      
	    }
	    
	    EnumeratedValue e_rt= (EnumeratedValue)rv_rt.getField("aVP_Data");
	    e_rt.setEnum(s_rt);
	    rv_rt.setField("aVP_Data", e_rt);*/
    	
	    rv_DiameterM.setField("session_Release_Cause",rv_rt);
    }
    
    private void decode_Charging_Rule_Name_AVP(RecordValue rv_rid,	AVP rt){											//ref: TS129 212 5a.3.4
    	
    	AVP[] avps=rt.findChildAVPs(AVP_Charging_Rule_Name_AVP);
   	    if (avps==null) return;
       	    
   	    RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("charging_Rule_Name").getType().newInstance();
	    rov_rt.setLength(0);
	    
	    for (int i = 0; i < avps.length; i++) {
			   
		    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();

		    decode_AVPHeader( rv ,avps[i]);
		    rv.setField("aVP_Data", mkOctetstring(avps[i].data));
		    
		    rov_rt.appendField(rv);
		}   
	    rv_rid.setField("charging_Rule_Name",rov_rt);
   	    return;
    }
    
    private void decode_AN_GW_Addr_AVP(RecordValue rv_,	AVP avp_){                                        //ref: 7.3.35
        
    	AVP[] avp=avp_.findChildAVPs(AVP_AN_GW_Address_AVP);
	    if (avp==null) return;
    	
	    RecordOfValue rov_c = (RecordOfValue) rv_.getField("aN_GW_Address").getType().newInstance();
	    rov_c.setLength(0);
	    
	    for(int i=0;i<avp.length;i++){
	      RecordValue rv = (RecordValue) rov_c.getElementType().newInstance();
          
	      decode_AVPHeader( rv ,avp[i]);
          decode_Address( rv ,avp[i]);
          
          rov_c.appendField(rv);
	    }
        rv_.setField("aN_GW_Address",rov_c);
 	   return;
    }
    
    private void decode_AN_GW_Addr_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){                                        //ref: 7.3.35
        
    	AVP[] avp=msg.findAVPs(AVP_AN_GW_Address_AVP);
	    if (avp==null) return;
    	
	    RecordOfValue rov_c = (RecordOfValue) rv_DiameterM.getField("aN_GW_Address").getType().newInstance();
	    rov_c.setLength(0);
	    
	    for(int i=0;i<avp.length;i++){
	      RecordValue rv = (RecordValue) rov_c.getElementType().newInstance();
         
	      decode_AVPHeader( rv ,avp[i]);
          decode_Address( rv ,avp[i]);//                context_Identifier,
          
          rov_c.appendField(rv);
	    }
	    rv_DiameterM.setField("aN_GW_Address",rov_c);  
 	   return;
    }
    
    private void decode_Charging_Rule_Remove_AVP(RecordValue rv_rid,	AVP rt){
    	
    	boolean rv_present=false;
    	AVP[] avps=rt.findChildAVPs(AVP_Charging_Rule_Remove_AVP);
   	    if (avps==null) return;
    	
        if (avps.length>=1){
        	try{
 			   RecordValue rv = (RecordValue) (RecordValue) rv_rid.getField("charging_Rule_Remove").getType().newInstance();
 			   
 			   try{avps[0].ungroup();}catch(Exception e){}
 			   
 			   decode_AVPHeader( rv ,avps[0]);
 			   decode_Charging_Rule_Name_AVP( rv ,avps[0]);//            charging_Rule_Name optional,
//             set of Charging_Rule_Base_Name_AVP        charging_Rule_Base_Name optional,
//             set of AVP_Type                         aVP_Type optional
            
               rv_rid.setField("charging_Rule_Remove",rv);
			   rv_present=true;
			   return;
			}
			catch(Exception ex){
				//System.out.print(ex.getMessage());
			}
			if(!rv_present){
				RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("charging_Rule_Remove").getType().newInstance();
			    rov_rt.setLength(0);
			    
			    for (int i = 0; i < avps.length; i++) {   
				    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
				   
				    try{avps[0].ungroup();}catch(Exception e){}

		        	decode_AVPHeader( rv ,avps[i]);
		        	decode_Charging_Rule_Name_AVP( rv ,avps[i]);//            charging_Rule_Name optional,
//		            set of Charging_Rule_Base_Name_AVP        charging_Rule_Base_Name optional,
//		            set of AVP_Type                         aVP_Type optional		           
		            rov_rt.appendField(rv);
 				}			    	    
 			    rv_rid.setField("charging_Rule_Remove",rov_rt);
 			}
        }
		return;	
    }
    
    private void decode_Charging_Rule_Install_AVP(RecordValue rv_rid,	AVP rt){
    	
    	boolean rv_present=false;
    	AVP[] avps=rt.findChildAVPs(AVP_Charging_Rule_Install_AVP);
   	    if (avps==null) return;
    	
    	
        if (avps.length>=1){
        	try{
 			   RecordValue rv = (RecordValue) (RecordValue) rv_rid.getField("charging_Rule_Install").getType().newInstance();
 			   
 			  try{avps[0].ungroup();}catch(Exception e){}
 			  
 			  decode_AVPHeader( rv ,avps[0]);    	
//            set of Charging_Rule_Definition_AVP        charging_Rule_Definition optional,
              decode_Charging_Rule_Name_AVP( rv ,avps[0]);//            charging_Rule_Name optional,
//            set of Charging_Rule_Base_Name_AVP        charging_Rule_Base_Name optional,
//            Bearer_Identifier_AVP                    bearer_Identifier optional,
//            Rule_Activation_Time_AVP                rule_Activation_Time optional,
//            Rule_DeActivation_Time_AVP                rule_DeActivation_Time optional,
//            Resource_Allocation_Notification_AVP    resource_Allocation_Notification optional,
//            Charging_Correlation_Indicator_AVP        charging_Correlation_Indicator optional,
//            set of AVP_Type                         aVP_Type optional
            
               rv_rid.setField("charging_Rule_Install",rv);
			   rv_present=true;
			   return;
			}
			catch(Exception ex){
				//System.out.print(ex.getMessage());
			}
			if(!rv_present){
				RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("charging_Rule_Install").getType().newInstance();
			    rov_rt.setLength(0);
			    
			    for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
				    
				    try{avps[0].ungroup();}catch(Exception e){}
		        	
		        	decode_AVPHeader( rv ,avps[i]);
//		            set of Charging_Rule_Definition_AVP        charging_Rule_Definition optional,
		        	decode_Charging_Rule_Name_AVP( rv ,avps[i]);//            charging_Rule_Name optional,
//		            set of Charging_Rule_Base_Name_AVP        charging_Rule_Base_Name optional,
//		            Bearer_Identifier_AVP                    bearer_Identifier optional,
//		            Rule_Activation_Time_AVP                rule_Activation_Time optional,
//		            Rule_DeActivation_Time_AVP                rule_DeActivation_Time optional,
//		            Resource_Allocation_Notification_AVP    resource_Allocation_Notification optional,
//		            Charging_Correlation_Indicator_AVP        charging_Correlation_Indicator optional,
//		            set of AVP_Type                         aVP_Type optional
		            rov_rt.appendField(rv);
 				}	    
 			    rv_rid.setField("charging_Rule_Install",rov_rt);
 			}
        }
		return;	
    }
    
   private void decode_Charging_Rule_Report_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	boolean rv_present=false;
    	AVP[] avps=rt.findAVPs(AVP_Charging_Rule_Report_AVP);
   	    if (avps==null) return;
    	
        if (avps.length>=1){
        	try{
 			   RecordValue rv = (RecordValue) (RecordValue) rv_rid.getField("charging_Rule_Report").getType().newInstance();
 			   
 			  try{avps[0].ungroup();}catch(Exception e){}
 			   
 			    decode_AVPHeader( rv ,avps[0]);    	
                decode_Charging_Rule_Name_AVP( rv ,avps[0]); //           charging_Rule_Name optional,
//              set of Charging_Rule_Base_Name_AVP        charging_Rule_Base_Name optional,
//              Bearer_Identifier_AVP                    bearer_Identifier optional,
//              PCC_Rule_Status_AVP                        pCC_Rule_Status optional,
//              Rule_Failure_Code_AVP                    rule_Failure_Code optional,
//              Final_Unit_Indication_AVP                final_Unit_Indication optional,
//              set of AVP_Type                         aVP_Type optional
            
               rv_rid.setField("charging_Rule_Report",rv);
			   rv_present=true;
			   return;
			}
			catch(Exception ex){
				//System.out.print(ex.getMessage());
			}
			if(!rv_present){
				RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("charging_Rule_Report").getType().newInstance();
			    rov_rt.setLength(0);
			    
			    for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();

				    try{avps[0].ungroup();}catch(Exception e){}

		        	decode_AVPHeader( rv ,avps[i]);
		        	decode_Charging_Rule_Name_AVP( rv ,avps[i]); //           charging_Rule_Name optional,
//	                set of Charging_Rule_Base_Name_AVP        charging_Rule_Base_Name optional,
//	                Bearer_Identifier_AVP                    bearer_Identifier optional,
//	                PCC_Rule_Status_AVP                        pCC_Rule_Status optional,
//	                Rule_Failure_Code_AVP                    rule_Failure_Code optional,
//	                Final_Unit_Indication_AVP                final_Unit_Indication optional,
//	                set of AVP_Type                         aVP_Type optional
		           
		            rov_rt.appendField(rv);
 				}    
 			    rv_rid.setField("charging_Rule_Report",rov_rt);
 			}
        }
		return;	
    }
    
    private void decode_QoS_Information_AVP(RecordValue rv_rid,	AVP rt){
    	
    	boolean rv_present=false;
    	AVP[] avps=rt.findChildAVPs(AVP_QoS_Information_AVP);
   	    if (avps==null) return;
    	
        if (avps.length>=1){
        	try{
 			   RecordValue rv = (RecordValue) (RecordValue) rv_rid.getField("qoS_Information").getType().newInstance();
 			   
 			   try{avps[0].ungroup();}catch(Exception e){}
 			  
 			   decode_AVPHeader( rv ,avps[0]); 			   
 			   decode_QoS_Class_Identifier_AVP ( rv ,avps[0]);//               qoS_Class_Identifier optional,
			   decode_Max_Requested_Bandwidth_UL_AVP( rv ,avps[0]);//            max_Requested_Bandwidth_UL optional,
			   decode_Max_Requested_Bandwidth_DL_AVP ( rv ,avps[0]);//           max_Requested_Bandwidth_DL optional,
			   //decode_Guaranteed_Bitrate_UL_AVP ( rv ,avps[i]);//               guaranteed_Bitrate_UL optional,
			   //decode_Guaranteed_Bitrate_DL_AVP( rv ,avps[i]);//                guaranteed_Bitrate_DL optional,
			   //decode_Bearer_Identifier_AVP ( rv ,avps[i]);//                   bearer_Identifier optional,
			   decode_Allocation_Retention_Priority_AVP( rv ,avps[0]);//        allocation_Retention_Priority optional,
			   //decode_APN_Aggregate_Max_Bitrate_UL_AVP( rv ,avps[i]);//        aPN_Aggregate_Max_Bitrate_UL optional,
			   //decode_APN_Aggregate_Max_Bitrate_DL_AVP( rv ,avps[i]);//        aPN_Aggregate_Max_Bitrate_DL optional,
	           //decode_AVP_Type(rv ,avps[i]);//                aVP_Data optional    //other parameters.
	     	   
			   rv_rid.setField("qoS_Information",rv);
 			   rv_present=true;
 			   return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
 				RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("qoS_Information").getType().newInstance();
 			    rov_rt.setLength(0);
 			    
 			    for (int i = 0; i < avps.length; i++) {
 				    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
 		        	
 				    try{avps[0].ungroup();}catch(Exception e){}
 		        	
 				    decode_AVPHeader( rv ,avps[i]);
 				    decode_QoS_Class_Identifier_AVP ( rv ,avps[i]);//               qoS_Class_Identifier optional,
 				    decode_Max_Requested_Bandwidth_UL_AVP( rv ,avps[i]);//            max_Requested_Bandwidth_UL optional,
 				    decode_Max_Requested_Bandwidth_DL_AVP ( rv ,avps[i]);//           max_Requested_Bandwidth_DL optional,
 				    //decode_Guaranteed_Bitrate_UL_AVP ( rv ,avps[i]);//               guaranteed_Bitrate_UL optional,
 				    //decode_Guaranteed_Bitrate_DL_AVP( rv ,avps[i]);//                guaranteed_Bitrate_DL optional,
 				    //decode_Bearer_Identifier_AVP ( rv ,avps[i]);//                   bearer_Identifier optional,
 				    decode_Allocation_Retention_Priority_AVP( rv ,avps[i]);//        allocation_Retention_Priority optional,
 				    //decode_APN_Aggregate_Max_Bitrate_UL_AVP( rv ,avps[i]);//        aPN_Aggregate_Max_Bitrate_UL optional,
 				    //decode_APN_Aggregate_Max_Bitrate_DL_AVP( rv ,avps[i]);//        aPN_Aggregate_Max_Bitrate_DL optional,
 		            
 		            //decode_AVP_Type(rv ,avps[i]);//                aVP_Data optional    //other parameters.
 			
 				    rov_rt.appendField(rv);
 				}    
 			    rv_rid.setField("qoS_Information",rov_rt);
 			}
        }
		return;	
    }
    
    private void decode_QoS_Information_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	boolean rv_present=false;
    	AVP[] avps=rt.findAVPs(AVP_QoS_Information_AVP);
   	    if (avps==null) return;
    	
        if (avps.length>=1){
        	try{
 			    RecordValue rv = (RecordValue) (RecordValue) rv_rid.getField("qoS_Information").getType().newInstance();
 			   
 			    try{avps[0].ungroup();}catch(Exception e){}
 			   
 			    decode_AVPHeader( rv ,avps[0]);
 			    decode_QoS_Class_Identifier_AVP ( rv ,avps[0]);//               qoS_Class_Identifier optional,
			    decode_Max_Requested_Bandwidth_UL_AVP( rv ,avps[0]);//            max_Requested_Bandwidth_UL optional,
			    decode_Max_Requested_Bandwidth_DL_AVP ( rv ,avps[0]);//           max_Requested_Bandwidth_DL optional,
			    //decode_Guaranteed_Bitrate_UL_AVP ( rv ,avps[i]);//               guaranteed_Bitrate_UL optional,
			    //decode_Guaranteed_Bitrate_DL_AVP( rv ,avps[i]);//                guaranteed_Bitrate_DL optional,
			    //decode_Bearer_Identifier_AVP ( rv ,avps[i]);//                   bearer_Identifier optional,
			    decode_Allocation_Retention_Priority_AVP( rv ,avps[0]);//        allocation_Retention_Priority optional,
			    //decode_APN_Aggregate_Max_Bitrate_UL_AVP( rv ,avps[i]);//        aPN_Aggregate_Max_Bitrate_UL optional,
			    //decode_APN_Aggregate_Max_Bitrate_DL_AVP( rv ,avps[i]);//        aPN_Aggregate_Max_Bitrate_DL optional,
	            
	            //decode_AVP_Type(rv ,avps[i]);//                aVP_Data optional    //other parameters.
		
	     	    rv_rid.setField("qoS_Information",rv);
 			    rv_present=true;
 			    return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
 				 RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("qoS_Information").getType().newInstance();
 			     rov_rt.setLength(0);
 			    
 			     for (int i = 0; i < avps.length; i++) {
 					   
 				    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
 				    
 				    try{avps[0].ungroup();}catch(Exception e){}
 		        	
 				    decode_AVPHeader( rv ,avps[i]);
 				    decode_QoS_Class_Identifier_AVP ( rv ,avps[i]);//               qoS_Class_Identifier optional,
 				    decode_Max_Requested_Bandwidth_UL_AVP( rv ,avps[i]);//            max_Requested_Bandwidth_UL optional,
 				    decode_Max_Requested_Bandwidth_DL_AVP ( rv ,avps[i]);//           max_Requested_Bandwidth_DL optional,
 				    //decode_Guaranteed_Bitrate_UL_AVP ( rv ,avps[i]);//               guaranteed_Bitrate_UL optional,
 				    //decode_Guaranteed_Bitrate_DL_AVP( rv ,avps[i]);//                guaranteed_Bitrate_DL optional,
 				    //decode_Bearer_Identifier_AVP ( rv ,avps[i]);//                   bearer_Identifier optional,
 				    decode_Allocation_Retention_Priority_AVP( rv ,avps[i]);//        allocation_Retention_Priority optional,
 				    //decode_APN_Aggregate_Max_Bitrate_UL_AVP( rv ,avps[i]);//        aPN_Aggregate_Max_Bitrate_UL optional,
 				    //decode_APN_Aggregate_Max_Bitrate_DL_AVP( rv ,avps[i]);//        aPN_Aggregate_Max_Bitrate_DL optional,
 		            
 		            //decode_AVP_Type(rv ,avps[i]);//                aVP_Data optional    //other parameters.
 			
 				    rov_rt.appendField(rv);
 				}   
 			    rv_rid.setField("qoS_Information",rov_rt);
 			}
        }
		return;	
    }
    
    private void decode_Server_Capabilities_AVP(RecordValue rv_sid,	AVP sc){
    	
       AVP avp=sc.findChildAVP(AVP_Server_Capabilities_AVP);
  	   if (avp==null) return;
      	
  	   RecordValue rv_sc = (RecordValue) (RecordValue) rv_sid.getField("server_Capabilities").getType().newInstance();
  	   
  	   decode_AVPHeader( rv_sc ,avp);
  	   //decode_Mandatory_Capability_AVP( rv_sc ,avp);
  	   //decode_Optional_Capability_AVP( rv_sc ,avp);
  	   //decode_Server_Name_AVP( rv_sc ,avp);
  	   //decode_AVP_Type( rv_sc ,avp);
  	   
  	   rv_sid.setField("server_Capabilities",rv_sc);
  	   return;
   	}
    
    private void decode_Server_Capabilities_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
       AVP avp=rt.findAVP(AVP_Server_Capabilities_AVP);
   	   if (avp==null) return;
       	
   	   RecordValue rv_sc = (RecordValue) (RecordValue) rv_rid.getField("server_Capabilities").getType().newInstance();
       
   	   try{avp.ungroup();}catch(Exception e){}
   	   decode_AVPHeader( rv_sc ,avp);
   	   //TODO:decode_Mandatory_Capability_AVP( rv_sc ,avp);
   	   //TODO:decode_Optional_Capability_AVP( rv_sc ,avp);
   	   //decode_Server_Name_AVP( rv_sc ,avp);
   	   //decode_AVP_Type( rv_sc ,avp);
   	   
   	   rv_rid.setField("server_Capabilities",rv_sc);
   	   return;
    }
    /*End TS129_229 AVPs*/
    
    //TS129_329 AVPs
    private void decode_Public_Identity_AVP(RecordValue rv_,	AVP avp_){
    	
    	boolean rv_present=false;
    	AVP avps[]=avp_.findChildAVPs(AVP_Public_Identity_AVP);
   	    if (avps==null) return;
   	   
   	    if (avps.length>=1){
   	    	try{
	   	    	RecordValue rv_rt = (RecordValue) rv_.getField("public_Identity_AVP").getType().newInstance();
			   
			    decode_AVPHeader( rv_rt ,avps[0]);
			    rv_rt.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
			    
			    rv_.setField("public_Identity_AVP",rv_rt);
	 			rv_present=true;
	 			return;
   	    	}
   	    	catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
   	    	if(!rv_present){
				RecordOfValue recOfpi = (RecordOfValue) rv_.getField("public_Identity_AVP").getType().newInstance();
				recOfpi.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfpi.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
				    rv.setField("aVP_Data", mkCvUTF8(avps[i].getStringData()));
				    
				    recOfpi.appendField(rv);
				}
				rv_.setField("public_Identity_AVP",recOfpi);
 			}
   	    }
	    return;
    }
    
    private void decode_Public_Identity_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	boolean rv_present=false;
    	AVP avps[]=rt.findAVPs(AVP_Public_Identity_AVP);
   	    if (avps==null) return;
   	   
   	    if (avps.length>=1){
   	    	try{
	   	    	RecordValue rv_rt = (RecordValue) rv_rid.getField("public_Identity").getType().newInstance();
			   
			    decode_AVPHeader( rv_rt ,avps[0]);
			    rv_rt.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
			    
			    rv_rid.setField("public_Identity",rv_rt);
	 			rv_present=true;
	 			return;
   	    	}
   	    	catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
   	    	if(!rv_present){
				RecordOfValue recOfpi = (RecordOfValue) rv_rid.getField("public_Identity").getType().newInstance();
				recOfpi.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfpi.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
				    rv.setField("aVP_Data", mkCvUTF8(avps[i].getStringData()));
				    
				    recOfpi.appendField(rv);
				}
				rv_rid.setField("public_Identity",recOfpi);
 			}
   	    }
	    return;
    }
    
    
    private void decode_User_Identity_AVP(RecordValue rv_rid,	DiameterMessage rt){
        
    	AVP avp=rt.findAVP(AVP_User_Identity_AVP);
   	    if (avp==null) return;
   	    
		RecordValue rv_ui = (RecordValue) (RecordValue) rv_rid.getField("user_Identity").getType().newInstance();
     
 	   try{avp.ungroup();}catch(Exception e){}
 	   
 	   decode_AVPHeader( rv_ui ,avp);
 	   decode_Public_Identity_AVP(rv_ui,avp);
       decode_MSISDN_AVP(rv_ui,avp);
 	   //decode_AVP_Type( rv_ui ,avp);
 	   
 	   rv_rid.setField("user_Identity",rv_ui);
 	   return;
    }
        

    private void decode_MSISDN_AVP(RecordValue rv_,	AVP avp_){
        
        AVP avp=avp_.findChildAVP(AVP_MSISDN_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_rt = (RecordValue) (RecordValue) rv_.getField("mSISDN_AVP").getType().newInstance();
   	    
   	    decode_AVPHeader( rv_rt ,avp);
   	    rv_rt.setField("aVP_Data", mkOctetstring(avp.getData()));
	    
	    rv_.setField("mSISDN_AVP",rv_rt);
    }
   
    private void decode_Data_Reference_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
//    	AVP[] avps=rt.findAVPs(AVP_Data_Reference_AVP);
//   	    if (avps==null) return;
//    	  	    
//   	    RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("data_Reference").getType().newInstance();
//	    rov_rt.setLength(0);
//	    
//	    for (int i = 0; i < avps.length; i++) {   
//		    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
//		   
//		    decode_AVPHeader( rv ,avps[i]);
//		    rv.setField("aVP_Data", mkInt(avps[i].int_data));
//		    
//		    rov_rt.appendField(rv);
//		}	    
//	    rv_rid.setField("data_Reference",rov_rt);
//   	    return;
   	    
   	    
   	    boolean rv_present=false;
	    AVP[] avps=msg.findAVPs(AVP_Data_Reference_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			try{
			   RecordValue rv_dr = (RecordValue) (RecordValue) rv_DiameterM.getField("data_Reference").getType().newInstance();
			   
			   decode_AVPHeader( rv_dr ,avps[0]);
			   decode_Vendor_Specific_Appl_Id_AVP(rv_dr,avps[0]);
			   rv_dr.setField("aVP_Data", mkInt(avps[0].int_data));
			   
			   rv_DiameterM.setField("data_Reference",rv_dr);
			   rv_present=true;
			   return;
			}
			catch(Exception ex){
				//System.out.print(ex.getMessage());
			}
			if(!rv_present){
				try{
					RecordOfValue recOfrv_dr = (RecordOfValue) rv_DiameterM.getField("data_Reference").getType().newInstance();
					recOfrv_dr.setLength(0);
		
		        for(int i=0;i<avps.length;i++){			
		        	RecordValue rv = (RecordValue) recOfrv_dr.getElementType().newInstance();
		        	
		        	decode_AVPHeader( rv ,avps[i]);
				decode_Vendor_Specific_Appl_Id_AVP(rv,avps[i]);
			        rv.setField("aVP_Data", mkInt(avps[i].int_data));
		        	
                                recOfrv_dr.appendField(rv);
		        }
				rv_DiameterM.setField("data_Reference",recOfrv_dr);
				return;
				}catch(Exception ex){
					//System.out.print(ex.getMessage());
				}
			}
		}
		return;
    }
    
	    private void decode_Service_Indication_AVP(RecordValue rv_,	AVP avp_){
	        
	        AVP avp=avp_.findChildAVP(AVP_Service_Indication_AVP);
	   	    if (avp==null) return;
	   	   
	   	    RecordValue rv_rt = (RecordValue) (RecordValue) rv_.getField("service_Indication").getType().newInstance();
	   	    
	   	    decode_AVPHeader( rv_rt ,avp);
	   	    rv_rt.setField("aVP_Data", mkOctetstring(avp.getData()));
		    
		    rv_.setField("service_Indication",rv_rt);
	    }
    
    private void decode_Service_Indication_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	AVP[] avps=rt.findAVPs(AVP_Service_Indication_AVP);
   	    if (avps==null) return;
    	  	    
   	    RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("service_Indication").getType().newInstance();
	    rov_rt.setLength(0);
	    
	    for (int i = 0; i < avps.length; i++) {   
		    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
		   
		    decode_AVPHeader( rv ,avps[i]);
		    rv.setField("aVP_Data", mkOctetstring(avps[i].getData()));
		    
		    rov_rt.appendField(rv);
		}	    
	    rv_rid.setField("service_Indication",rov_rt);
   	    return;
    }
    
    private void decode_Subs_Req_Type_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Subs_Req_Type_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("subs_Req_Type").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
	    rv_uf.setField("aVP_Data", mkInt(avp.int_data));
  
        rv_ufd.setField("subs_Req_Type",rv_uf);
    	return;
    }
    
    private void decode_Requested_Domain_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Requested_Domain_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("requested_Domain").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
	    rv_uf.setField("aVP_Data", mkInt(avp.int_data));
  
        rv_ufd.setField("requested_Domain",rv_uf);
    	return;
    }

    private void decode_Current_Location_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Current_Location_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("current_Location").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
	    rv_uf.setField("aVP_Data", mkInt(avp.int_data));
  
        rv_ufd.setField("current_Location",rv_uf);
    	return;
    }
    
    private void decode_Identity_Set_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	AVP[] avps=rt.findAVPs(AVP_Identity_Set_AVP);
   	    if (avps==null) return;
    	  	    
   	    RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("identity_Set").getType().newInstance();
	    rov_rt.setLength(0);
	    
	    for (int i = 0; i < avps.length; i++) {   
		    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
		   
		    decode_AVPHeader( rv ,avps[i]);
		    rv.setField("aVP_Data", mkInt(avps[i].int_data));
		    
		    rov_rt.appendField(rv);
		}	    
	    rv_rid.setField("identity_Set",rov_rt);
   	    return;
    }
    
    
	private void decode_Expiry_Time_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Expiry_Time_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("expiry_Time").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
	    rv_uf.setField("aVP_Data", mkInt(avp.int_data));
  
        rv_ufd.setField("expiry_Time",rv_uf);
    	return;
    }

	
	private void decode_Send_Data_Indication_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Send_Data_Indication_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("send_Data_Indication").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
	    rv_uf.setField("aVP_Data", mkInt(avp.int_data));
  
        rv_ufd.setField("send_Data_Indication",rv_uf);
    	return;
    }
	
		private void decode_Sequence_Number_AVP(RecordValue rv_,	AVP avp_){
	        
	        AVP avp=avp_.findChildAVP(AVP_Sequence_Number_AVP);
	   	    if (avp==null) return;
	   	   
	   	    RecordValue rv_rt = (RecordValue) (RecordValue) rv_.getField("sequence_Number").getType().newInstance();
	   	    
	   	    decode_AVPHeader( rv_rt ,avp);
	   	    rv_rt.setField("aVP_Data", mkInt(avp.int_data));
		    
		    rv_.setField("sequence_Number",rv_rt);
	    }
	
    private void decode_Repository_Data_ID_AVP(RecordValue rv_rid,	DiameterMessage rt){
        
    	AVP avp=rt.findAVP(AVP_Repository_Data_ID_AVP);
   	    if (avp==null) return;
   	    
		RecordValue rv_ui = (RecordValue) (RecordValue) rv_rid.getField("repository_Data_Id").getType().newInstance();
     
 	    try{avp.ungroup();}catch(Exception e){}
 	   
 	    decode_AVPHeader( rv_ui ,avp);
 	    decode_Service_Indication_AVP(rv_ui,avp);
        decode_Sequence_Number_AVP(rv_ui,avp);
 	    decode_AVP_Type( rv_ui ,avp);
 	   
 	    rv_rid.setField("repository_Data_Id",rv_ui);
 	    return;
    }
    
    
    private void decode_Wildcarded_PSI_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Wildcarded_PSI_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("wildcarded_PSI").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
	    rv_uf.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
  
        rv_ufd.setField("wildcarded_PSI",rv_uf);
    	return;
    }
 
    private void decode_Wildcarded_IMPU_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Wildcarded_IMPU_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("wildcarded_IMPU").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
	    rv_uf.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
  
        rv_ufd.setField("wildcarded_IMPU",rv_uf);
    	return;
    }
      
    private void decode_DSAI_Tag_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	AVP[] avps=rt.findAVPs(AVP_DSAI_Tag_AVP);
   	    if (avps==null) return;
    	  	    
   	    RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("dSAI_Tag").getType().newInstance();
	    rov_rt.setLength(0);
	    
	    for (int i = 0; i < avps.length; i++) {   
		    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
		   
		    decode_AVPHeader( rv ,avps[i]);
		    rv.setField("aVP_Data", mkOctetstring(avps[i].getData()));
		    
		    rov_rt.appendField(rv);
		}	    
	    rv_rid.setField("dSAI_Tag",rov_rt);
   	    return;
    }
    
    private void decode_Experimental_Result_Code_AVP(RecordValue rv,	AVP avp_){
    	
    	String s_erc="";
    	AVP avp=avp_.findChildAVP(AVP_Experimental_Result_Code_AVP);
   	    if (avp==null) return;
   	    
   	    RecordValue rv_erc = (RecordValue) rv.getField("experimental_Result_Code").getType().newInstance();
	        
    	decode_AVPHeader( rv_erc ,avp);
    	rv_erc.setField("aVP_Data", mkInt(avp.int_data));
    	
    	/*switch(avp.int_data){
    		case 2001: s_erc="DIAMETER_FIRST_REGISTRATION_E"; break;
    		case 2002: s_erc="DIAMETER_SUBSEQUENT_REGISTRATION_E"; break;
    		case 2003: s_erc="DIAMETER_UNREGISTERED_SERVICE_E"; break;
    		case 2004: s_erc="DIAMETER_SUCCESS_SERVER_NAME_NOT_STORED_E"; break;
    		case 4181: s_erc="DIAMETER_AUTHENTICATION_DATA_UNAVAILABLE_E"; break;
    		case 5001: s_erc="DIAMETER_ERROR_USER_UNKNOWN_E"; break;
    		case 5002: s_erc="DIAMETER_ERROR_IDENTITIES_DONT_MATCH_E"; break;
    		case 5003: s_erc="DIAMETER_ERROR_IDENTITY_NOT_REGISTERED_E"; break;
    		case 5004: s_erc="DIAMETER_ERROR_ROAMING_NOT_ALLOWED_E"; break;
    		case 5005: s_erc="DIAMETER_ERROR_IDENTITY_ALREADY_REGISTERED_E"; break;
    		case 5006: s_erc="DIAMETER_ERROR_AUTH_SCHEME_NOT_SUPPORTED_E"; break;
    		case 5007: s_erc="DIAMETER_ERROR_IN_ASSIGNMENT_TYPE_E"; break;
    		case 5008: s_erc="DIAMETER_ERROR_TOO_MUCH_DATA_E"; break;
    		case 5009: s_erc="DIAMETER_ERROR_NOT_SUPPORTED_USER_DATA_E"; break;
        // Ref: ETSI TS 129 214 5.5
    		case 5061: s_erc="INVALID_SERVICE_INFORMATION_E"; break;
    		case 5062: s_erc="FILTER_RESTRICTIONS_E"; break;
    		case 5063: s_erc="REQUESTED_SERVICE_NOT_AUTHORIZED_E"; break;
    		case 5064: s_erc="DUPLICATED_AF_SESSION_E"; break;
    		case 5065: s_erc="IP_CAN_SESSION_NOT_AVAILABLE_E"; break;
    		case 5066: s_erc="UNAUTHORIZED_NON_EMERGENCY_SESSION_E"; break;
    		case 5067: s_erc="UNAUTHORIZED_SPONSORED_DATA_CONNECTIVITY_E"; break;
    		case 5140: s_erc="DIAMETER_ERROR_INITIAL_PARAMETERS_E"; break;
    		case 5141: s_erc="DIAMETER_ERROR_TRIGGER_EVENT_E"; break;
    		case 5142: s_erc="DIAMETER_PCC_RULE_EVENT_E"; break;
    		case 5143: s_erc="DIAMETER_ERROR_BEARER_NOT_AUTHORIZED_E"; break;
    		case 5144: s_erc="DIAMETER_ERROR_TRAFFIC_MAPPING_INFO_REJECTED_E"; break;
    		case 5147: s_erc="DIAMETER_ERROR_CONFLICTING_REQUEST_E"; break;
        // Ref: 3GPP TS 29.215 
    		case 5470: s_erc="DIAMETER_ERROR_SUBSESSION_E"; break;
    		case 5420: s_erc="DIAMETER_ERROR_UNKNOWN_EPS_SUBSCRIPTION_E"; break;// (5420),//(9990), // Set the correct error code value - See ETSI TS 129 272 Clause 5.2.1.1.3 Detailed behaviour of the HSS
    		case 5421: s_erc="DIAMETER_ERROR_RAT_NOT_ALLOWED_E"; break;// (5421),//(9991), // Set the correct error code value - See ETSI TS 129 272 Clause 5.2.1.1.3 Detailed behaviour of the HSS
    		case 5422: s_erc="DIAMETER_ERROR_EQUIPMENT_UNKNOWN_E"; break; 
    		case 5423: s_erc="DIAMETER_ERROR_UNKOWN_SERVING_NODE_E"; break;
    		

    		default: s_erc="unknown"; break;
  	    }
    	
    	EnumeratedValue e_rt= (EnumeratedValue)rv_erc.getField("aVP_Data");
	    e_rt.setEnum(s_erc);
	    rv_erc.setField("aVP_Data", e_rt);*/
	    
	    rv.setField("experimental_Result_Code",rv_erc);
		return;
    }
    
    private void decode_Experimental_Result_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Experimental_Result_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_rt = (RecordValue) (RecordValue) rv_.getField("experimental_Result").getType().newInstance();
   	    
   	    try{avp.ungroup();}catch(Exception e){}
	    
   	    decode_AVPHeader( rv_rt ,avp);
	    decode_Vendor_Id_AVP(rv_rt,avp);
	    decode_Experimental_Result_Code_AVP(rv_rt,avp);
	    
	    rv_.setField("experimental_Result",rv_rt);
    }
    
    private void decode_Experimental_Result_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	AVP avp=rt.findAVP(AVP_Experimental_Result_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_rt = (RecordValue) rv_rid.getField("experimental_Result").getType().newInstance();
   	   
   	    try{avp.ungroup();}catch(Exception e){}
	   
   	    decode_AVPHeader( rv_rt ,avp);
	    decode_Vendor_Id_AVP(rv_rt,avp);
	    decode_Experimental_Result_Code_AVP(rv_rt,avp);
	    
	    rv_rid.setField("experimental_Result",rv_rt);
    }
    
    private void decode_User_Data_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_User_Data_AVP);
   	    if (avp==null) avp=uf.findAVP(AVP_User_Data_Sh_AVP);
   	    if (avp==null)return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("user_Data").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
	    rv_uf.setField("aVP_Data", mkOctetstring(avp.data));
  
        rv_ufd.setField("user_Data",rv_uf);
    	return;
    }
    
    private void decode_Server_Name_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Server_Name_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("server_Name").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
	    rv_uf.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
  
        rv_ufd.setField("server_Name",rv_uf);
    	return;
    }
    
    
	    private void  decode_Feature_List_ID_AVP(RecordValue rv_rid,	AVP avp_){
	    
	       	AVP avp=avp_.findChildAVP(AVP_Feature_List_ID_AVP);
			if (avp==null) return;
			
			RecordValue rv = (RecordValue) rv_rid.getField("feature_list_Id").getType().newInstance();
			
			decode_AVPHeader( rv ,avp);
			rv.setField("aVP_Data", mkInt(avp.int_data));
						
			rv_rid.setField("feature_list_Id",rv);
			return;	    	
	    }
	    
	    private void  decode_Feature_List_AVP(RecordValue rv_rid,	AVP avp_){
		    
	       	AVP avp=avp_.findChildAVP(AVP_Feature_List_AVP);
			if (avp==null) return;
			
			RecordValue rv = (RecordValue) rv_rid.getField("feature_list").getType().newInstance();
			
			decode_AVPHeader( rv ,avp);
			rv.setField("aVP_Data", mkInt(avp.int_data));
						
			rv_rid.setField("feature_list",rv);
			return;	    	
	    }
    
    private void decode_Supported_Features_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	AVP[] avps=rt.findAVPs(AVP_Supported_Features_AVP);
   	    if (avps==null) return;
    	  	    
   	    RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("supported_Features").getType().newInstance();
	    rov_rt.setLength(0);
	    
	    for (int i = 0; i < avps.length; i++) {   
		    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
		   
		    try{avps[i].ungroup();}catch(Exception e){}
        	
		    decode_AVPHeader( rv ,avps[i]);
		    decode_Vendor_Id_AVP(rv ,avps[i]);             //     vendor_Id,
            decode_Feature_List_ID_AVP(rv ,avps[i]);//            feature_list_Id optional,
            decode_Feature_List_AVP(rv ,avps[i]); //               feature_list,
            //decode_AVP_Type(rv ,avps[i]);//                aVP_Data optional    //other parameters.
	
		    rov_rt.appendField(rv);
		}	    
	    rv_rid.setField("supported_Features",rov_rt);
   	    return;
    }
    /*End TS129_329 AVPS*/
    
    /*TS129_215 AVPS*/
    private void decode_Routing_Rule_Install_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Routing_Rule_Install_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_.getField("routing_Rule_Install").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        //set of Routing_Rule_Definition_AVP    routing_Rule_Definition optional,
        //set of AVP_Type                     aVP_Type optional
  
        rv_.setField("routing_Rule_Install",rv_uf);
    	return;
    }
    
    private void decode_Routing_Rule_Install_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Routing_Rule_Install_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("routing_Rule_Install").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        //set of Routing_Rule_Definition_AVP    routing_Rule_Definition optional,
        //set of AVP_Type                     aVP_Type optional
  
        rv_ufd.setField("routing_Rule_Install",rv_uf);
    	return;
    }

    private void decode_Routing_Rule_Remove_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP avp=avp_.findChildAVP(AVP_Routing_Rule_Remove_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_.getField("routing_Rule_Remove").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        //set of Routing_Rule_Identifier_AVP    routing_Rule_Identifier optional,
        //set of AVP_Type                     aVP_Type optional
	    
	    rv_.setField("routing_Rule_Remove",rv_uf);
    	return;
    }
    
    private void decode_Routing_Rule_Remove_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Routing_Rule_Remove_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("routing_Rule_Remove").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        //set of Routing_Rule_Identifier_AVP    routing_Rule_Identifier optional,
        //set of AVP_Type                     aVP_Type optional
	    
	    rv_ufd.setField("routing_Rule_Remove",rv_uf);
    	return;
    }

    
    private void decode_QoS_Rule_Name_AVP(RecordValue rv_rid,	AVP rt){											//ref: TS129 212 5a.3.4
    	AVP[] avps=rt.findChildAVPs(AVP_QoS_Rule_Name_AVP);
   	    if (avps==null) return;
       	    
   	    RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("qoS_Rule_Name").getType().newInstance();
	    rov_rt.setLength(0);
	    
	    for (int i = 0; i < avps.length; i++) {
		    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
		    
		    //try{avp.ungroup();}catch(Exception e){}
		    decode_AVPHeader( rv ,avps[i]);
		    rv.setField("aVP_Data", mkOctetstring(avps[i].data));
		    
		    rov_rt.appendField(rv);
		}
	    rv_rid.setField("qoS_Rule_Name",rov_rt);
   	    return;
    }
    
    private void decode_Rule_Failure_Code_AVP(RecordValue rv_,	AVP avp_){
    		  
    	String s_erc="";
    	AVP avp=avp_.findChildAVP(AVP_Rule_Failure_Code_AVP);
   	    if (avp==null) return;
   	    
   	    RecordValue rv = (RecordValue) rv_.getField("rule_Failure_Code").getType().newInstance();
   	
    	decode_AVPHeader( rv ,avp);
    	rv.setField("aVP_Data", mkInt(avp.int_data));
    	
    	/*switch(avp.int_data){
    	
    		case 1: s_erc="UNKNOWN_RULE_NAME_E"; break;
    		case 2: s_erc="RATING_GROUP_ERROR_E"; break;
    		case 3: s_erc="SERVICE_IDENTIFIER_ERROR_E"; break;
    		case 4: s_erc="GW_PCEF_MALFUNCTION_E"; break;
    		case 5: s_erc="RESOURCES_LIMITATION_E"; break;
    		case 6: s_erc="MAX_NR_BEARERS_REACHED_E"; break;
    		case 7: s_erc="UNKNOWN_BEARER_ID_E"; break;
    		case 8: s_erc="MISSING_BEARER_ID_E"; break;
    		case 9: s_erc="MISSING_FLOW_INFORMATION_E"; break;
    		case 10: s_erc="RESOURCE_ALLOCATION_FAILURE_E"; break;
    		case 11: s_erc="UNSUCCESSFUL_QOS_VALIDATION_E"; break;
    		case 12: s_erc="INCORRECT_FLOW_INFORMATION_E"; break;
    		case 13: s_erc="PS_TO_CS_HANDOVER_E"; break;
    		//case 14: s_erc="DIAMETER_ERROR_NOT_SUPPORTED_USER_DATA_E"; break;
        // Ref: ETSI TS 129 214 5.5
    		case 15: s_erc="NO_BEARER_BOUND_E"; break;
    		
  	    }
    	
    	EnumeratedValue e_rt= (EnumeratedValue)rv.getField("aVP_Data");
	    e_rt.setEnum(s_erc);
	    rv.setField("aVP_Data", e_rt);*/
	    
	    rv_.setField("rule_Failure_Code",rv);
		return;
    }
  
    private void decode_PDN_Connection_ID_AVP(RecordValue rv_,	AVP avp_){
		
	    AVP[] avp=avp_.findChildAVPs(AVP_PDN_Connection_ID_AVP);
		
		if (avp==null) return;
		
		if (avp.length>1){} //Should never be more than one AVP
		else{
			RecordValue rv = (RecordValue) rv_.getField("pDN_Connection_ID").getType().newInstance();
			
			decode_AVPHeader( rv ,avp[0]);
			rv.setField("aVP_Data", mkOctetstring(avp[0].getData()));
									
			rv_.setField("pDN_Connection_ID",rv);
		}
	    return;
	}
   
    private void decode_QoS_Rule_Report_AVP(RecordValue rv_rid,	AVP rt){
  	
  	    AVP[] avps=rt.findChildAVPs(AVP_QoS_Rule_Report_AVP);
 	    if (avps==null) return;
  	    
 	    RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("qoS_Rule_Report").getType().newInstance();
	    rov_rt.setLength(0);
	    
	    for (int i = 0; i < avps.length; i++) { 
		    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
            
		    try{avps[i].ungroup();}catch(Exception e){}

		    decode_AVPHeader( rv ,avps[i]);
		    decode_QoS_Rule_Name_AVP( rv ,avps[i]);//			qoS_Rule_Name					optional,
            //decode_QoS_Rule_Base_Name_AVP( rv ,avps[i]);//		qoS_Rule_Base_Name				optional,
            //decode_PCC_Rule_Status_AVP( rv ,avps[i]);//					pCC_Rule_Status					optional,
            decode_Rule_Failure_Code_AVP( rv ,avps[i]);//				rule_Failure_Code				optional,  
            //decode_AVP_Type(rv ,avps[i]);//                aVP_Data optional    //other parameters.
	
		    rov_rt.appendField(rv);
		}	    
	    rv_rid.setField("qoS_Rule_Report",rov_rt);
 	    return;
    }
  
    private void decode_QoS_Rule_Report_AVP(RecordValue rv_rid,	DiameterMessage rt){
    	
    	AVP[] avps=rt.findAVPs(AVP_QoS_Rule_Report_AVP);
   	    if (avps==null) return;

   	    if (useList(rv_rid.getFieldNames(),"qoS_Rule_Report")){
	   	    RecordOfValue rov_rt = (RecordOfValue) rv_rid.getField("qoS_Rule_Report").getType().newInstance();
		    rov_rt.setLength(0);
		    
		    for (int i = 0; i < avps.length; i++) {
				   
			    RecordValue rv = (RecordValue) rov_rt.getElementType().newInstance();
	        	
			    try{avps[i].ungroup();}catch(Exception e){}
			    
			    decode_AVPHeader( rv ,avps[i]);
			    decode_QoS_Rule_Name_AVP( rv ,avps[i]);//			qoS_Rule_Name					optional,
	            //decode_QoS_Rule_Base_Name_AVP( rv ,avps[i]);//		qoS_Rule_Base_Name				optional,
	            //decode_PCC_Rule_Status_AVP( rv ,avps[i]);//					pCC_Rule_Status					optional,
	            decode_Rule_Failure_Code_AVP( rv ,avps[i]);//				rule_Failure_Code				optional,  
	            //decode_AVP_Type(rv ,avps[i]);//                aVP_Data optional    //other parameters.
		
			    rov_rt.appendField(rv);
			} 	    
		    rv_rid.setField("qoS_Rule_Report",rov_rt);
   	    }else{
		    RecordValue rv_avp = (RecordValue)rov_any_AVP.getElementType().newInstance();
		    decode_AVP_Type(rv_avp,avps[0]);
		    rov_any_AVP.appendField(rv_avp);
		}
   	    return;
    }
    
    private void decode_Session_Linking_Indicator_AVP(RecordValue rv_,	AVP avp_){
		  
    	String s_erc="";
    	AVP avp=avp_.findChildAVP(AVP_Session_Linking_Indicator_AVP);
    	if (avp==null) return;
   	   
    	RecordValue rv = (RecordValue) rv_.getField("session_Linking_Indicator").getType().newInstance();
   	    
    	decode_AVPHeader( rv ,avp);
    	rv.setField("session_Linking_Indicator", mkInt(avp.int_data));
    	
    	/*switch(avp.int_data){
    	
    		case 0: s_erc="SESSION_LINKING_IMMEDIATE_E"; break;
    		case 1: s_erc="SESSION_LINKING_DEFERRED_E"; break;
  	    }
    	
    	EnumeratedValue e_rt= (EnumeratedValue)rv.getField("session_Linking_Indicator");
	    e_rt.setEnum(s_erc);
	    rv.setField("session_Linking_Indicator", e_rt);*/
	    
	    rv_.setField("session_Linking_Indicator",rv);
		return;
    }
    
    private void decode_Subses_Decision_Info_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
	   
    	AVP[] avps=msg.findAVPs(AVP_Subses_Decision_Info_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			RecordOfValue recOf = (RecordOfValue) rv_DiameterM.getField("subsession_Decision_Info").getType().newInstance();
			recOf.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
	        	RecordValue rv = (RecordValue) recOf.getElementType().newInstance();
	        	
	        	try{avps[i].ungroup();}catch(Exception e){}

	        	decode_AVPHeader( rv ,avps[i]);
	        	decode_Subsession_Id_AVP(rv ,avps[i]);//					subsession_Id,
//                set length (0..2) of AN_GW_Addr_AVP	aN_GW_Address					optional,
                decode_Result_Code_AVP(rv ,avps[i]);//						result_Code						optional,
                decode_Experimental_Result_AVP(rv ,avps[i]);//				experimental_Result				optional,
                decode_Charging_Rule_Remove_AVP(rv ,avps[i]);//    	charging_Rule_Remove			optional,
                decode_Charging_Rule_Install_AVP(rv ,avps[i]);//    charging_Rule_Install			optional,
                decode_QoS_Rule_Install_AVP(rv ,avps[i]);//			qoS_Rule_Install				optional,
//                set of QoS_Rule_Remove_AVP			qoS_Rule_Remove					optional,
//                Default_EPS_Bearer_QoS_AVP       	default_EPS_Bearer_QoS			optional,
//                set of Usage_Monitoring_Info_AVP    usage_Monitoring_Information	optional,
                decode_Session_Release_Cause_AVP(rv ,avps[i]);//           session_Release_Cause			optional,
//                Bearer_Control_Mode_AVP            	bearer_Control_Mode				optional,
//                set of Event_Trigger_AVP        	event_Trigger					optional,
//                Revalidation_Time_AVP            	revalidation_Time				optional,
//                Online_AVP                        	online							optional,
//                Offline_AVP                        	offline							optional,
//                QoS_Information_AVP                	qoS_Information					optional,
//                set of AVP_Type                    	aVP_Type						optional

				recOf.appendField(rv);
	        }
			rv_DiameterM.setField("subsession_Decision_Info",recOf);
		}
		else{}
	    return;	
	}
    
    private void decode_Subses_Enforcement_Info_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
	    
    	AVP[] avps=msg.findAVPs(AVP_Subses_Enforcement_Info_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			RecordOfValue recOf = (RecordOfValue) rv_DiameterM.getField("subsesion_Enforcement_Info").getType().newInstance();
			recOf.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
	        	RecordValue rv = (RecordValue) recOf.getElementType().newInstance();
	        	
	        	try{avps[i].ungroup();}catch(Exception e){}

	        	decode_AVPHeader( rv ,avps[i]);
	        	decode_Subsession_Id_AVP(rv ,avps[i]);//					subsession_Id,
                decode_Subsession_Operation_AVP(rv ,avps[i]);//			subsession_Operation			optional,
                decode_AN_GW_Addr_AVP(rv ,avps[i]);//	aN_GW_Address					optional,
//                Bearer_Identifier_AVP            	bearer_Identifier				optional,
//                Bearer_Operation_AVP            	bearer_Operation				optional,
//                set of Packet_Filter_Info_AVP		packet_Filter_Information		optional,
//                Packet_Filter_Operation_AVP        	packet_Filter_Operation			optional,
                decode_QoS_Information_AVP(rv ,avps[i]);    //            	qoS_Information					optional,
//                Framed_IP_Address_AVP            	framed_IP_Address				optional,
//                Framed_IPv6_Prefix_AVP            	framed_IPv6_Prefix				optional,
//                set of CoA_Information_AVP        	coA_Information					optional,
                decode_Called_Station_Id_AVP(rv ,avps[i]);//            	called_Station_Id				optional,
                decode_PDN_Connection_ID_AVP(rv ,avps[i]);//				pDN_Connection_ID				optional,
//                Bearer_Usage_AVP                	bearer_Usage					optional,
//                set of TFT_Packet_Filter_Info_AVP   tFT_Packet_Filter_Information	optional,
//                Online_AVP                        	online							optional,
//                Offline_AVP                        	offline							optional,
                decode_Result_Code_AVP(rv ,avps[i]);//						result_Code						optional,
                decode_Experimental_Result_AVP(rv ,avps[i]);//				experimental_Result				optional,
//                set of Charging_Rule_Report_AVP    	charging_Rule_Report			optional,
                decode_QoS_Rule_Report_AVP(rv ,avps[i]);//			qoS_Rule_Report					optional,
//                IP_CAN_Type_AVP                    	iP_CAN_Type						optional,
//                RAT_Type_AVP                    	rAT_Type						optional,
//                ThreeGPP_SGSN_MCC_MNC_AVP        	threeGPP_SGSN_MCC_MNC			optional,
//                ThreeGPP_SGSN_Address_AVP        	threeGPP_SGSN_Address			optional,
//                ThreeGPP_SGSN_IPv6_Address_AVP    	threeGPP_SGSN_IPv6_Address		optional,
//                RAI_AVP                            	rAI								optional,
//                ThreeGPP_User_Location_Info_AVP    	threeGPP_User_Location_Info		optional,
//                ThreeGPP2_BSID_AVP					threeGPP2_BSID					optional,
//                User_CSG_Information_AVP			user_CSG_Information			optional,
//                Default_EPS_Bearer_QoS_AVP       	default_EPS_Bearer_QoS			optional,
//                Network_Request_Support_AVP        	network_Request_Support			optional,
                  decode_Routing_Rule_Install_AVP(rv ,avps[i]);//        	routing_Rule_Install			optional,
                  decode_Routing_Rule_Remove_AVP(rv ,avps[i]);//            	routing_Rule_Remove				optional,
//                Logical_Access_ID_AVP            	logical_Access_ID				optional,
//                Physical_Access_ID_AVP            	physical_Access_ID				optional,
//                set of Usage_Monitoring_Info_AVP    usage_Monitoring_Information	optional,
                  decode_Multiple_BBERF_Action_AVP(rv ,avps[i]);//			multiple_BBERF_Action			optional,
                  decode_Event_Trigger_AVP(rv ,avps[i]);//        	event_Trigger					optional,
//                Access_Netw_Charging_Addr_AVP    	access_Network_Charging_Address optional,
//                set of Access_Netw_Charg_Id_Gx_AVP  access_Network_Charging_Id_Gx	optional,
                  decode_Session_Linking_Indicator_AVP(rv ,avps[i]);//		session_Linking_Indicator		optional,
//                set of AVP_Type                    	aVP_Type						optional

				recOf.appendField(rv);
	        }
			rv_DiameterM.setField("subsesion_Enforcement_Info",recOf);
		}
		else{}
	    return;	
	}
    
    private void decode_Subsession_Id_AVP(RecordValue rv_,	AVP avp_){
    	
	       AVP avp=avp_.findChildAVP(AVP_Subsession_Id_AVP);
	   	   if (avp==null) return;
	       	
	   	   RecordValue rv_c = (RecordValue) (RecordValue) rv_.getField("subsession_Id").getType().newInstance();
	   	   
	   	   decode_AVPHeader( rv_c ,avp);
	   	   rv_c.setField("aVP_Data", mkInt(avp.int_data));
	   	   
	   	   rv_.setField("subsession_Id",rv_c);
	   	   return;
 	}
    
    private void decode_Subsession_Operation_AVP(RecordValue rv_,	AVP avp_){
    	
    	String s_ue="";
    	AVP avp=avp_.findChildAVP(AVP_Subsession_Operation_AVP);
    	if (avp==null) return;
    	   
    	RecordValue rv_ue = (RecordValue) (RecordValue) rv_.getField("subsession_Operation").getType().newInstance();
       
        decode_AVPHeader( rv_ue ,avp);
        rv_ue.setField("subsession_Operation", mkInt(avp.int_data));
       
        /*switch(avp.int_data){
               
          case 0   : s_ue="TERMINATION_E";break; 
          case 1   : s_ue="ESTABLISHMENT_E";break; 
          case 2   : s_ue="MODIFICATION_E";break; 
         
        }
            
        EnumeratedValue e_ue= (EnumeratedValue)rv_ue.getField("subsession_Operation");
        e_ue.setEnum(s_ue);
        rv_ue.setField("subsession_Operation", e_ue);*/
        
        rv_.setField("subsession_Operation",rv_ue);
    	return;
    }
    
    private void decode_Multiple_BBERF_Action_AVP(RecordValue rv_rid,	AVP avp_){
		String s_ue="";
		AVP avp=avp_.findChildAVP(AVP_Multiple_BBERF_Action_AVP);
		    if (avp==null) return;
		   
		    RecordValue rv_ue = (RecordValue) (RecordValue) rv_rid.getField("multiple_BBERF_Action").getType().newInstance();
	   
	    decode_AVPHeader( rv_ue ,avp);
	    rv_ue.setField("multiple_BBERF_Action", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	           
	      case 0   : s_ue="ESTABLISHMENT_E";break; 
	      case 1   : s_ue="TERMINATION_E";break; 
	    }
	        
	    EnumeratedValue e_ue= (EnumeratedValue)rv_ue.getField("multiple_BBERF_Action");
	    e_ue.setEnum(s_ue);
	    rv_ue.setField("multiple_BBERF_Action", e_ue);*/
	    
	    rv_rid.setField("multiple_BBERF_Action",rv_ue);
		return;
	}

    private void decode_Multiple_BBERF_Action_AVP(RecordValue rv_rid,	DiameterMessage rt){
		
    	String s_ue="";
		AVP avp=rt.findAVP(AVP_Multiple_BBERF_Action_AVP);
		if (avp==null) return;
		   
		RecordValue rv_ue = (RecordValue) (RecordValue) rv_rid.getField("multiple_BBERF_Action").getType().newInstance();
	   
	    decode_AVPHeader( rv_ue ,avp);
	    rv_ue.setField("multiple_BBERF_Action", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	           
	      case 0   : s_ue="ESTABLISHMENT_E";break; 
	      case 1   : s_ue="TERMINATION_E";break; 
	    }
	        
	    EnumeratedValue e_ue= (EnumeratedValue)rv_ue.getField("multiple_BBERF_Action");
	    e_ue.setEnum(s_ue);
	    rv_ue.setField("multiple_BBERF_Action", e_ue);*/
	    
	    rv_rid.setField("multiple_BBERF_Action",rv_ue);
		return;
	}
    
    private void decode_QoS_Rule_Install_AVP(RecordValue rv_,	AVP avp_){
	   
    	AVP[] avps=avp_.findChildAVPs(AVP_QoS_Rule_Install_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			RecordOfValue recOf = (RecordOfValue) rv_.getField("qoS_Rule_Install").getType().newInstance();
			recOf.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
	        	RecordValue rv = (RecordValue) recOf.getElementType().newInstance();
                
	        	try{avps[i].ungroup();}catch(Exception e){}

	        	decode_AVPHeader( rv ,avps[i]);
	        	//TODO:
//                set of QoS_Rule_Definition_AVP		qoS_Rule_Definition				optional,
//                set of QoS_Rule_Name_AVP			qoS_Rule_Name					optional,
//                set of QoS_Rule_Base_Name_AVP		qoS_Rule_Base_Name				optional,
//                Tunnel_Information_AVP    			tunnel_Information				optional,
//                ANw_Charging_Id_Value_AVP			acc_Net_Charging_Id_Val			optional,
//                Resource_Alloc_Notification_AVP		resource_Alloc_Notification		optional,
//                Rule_Activation_Time_AVP            rule_Activation_Time			optional,
//                Rule_DeActivation_Time_AVP          rule_DeActivation_Time			optional,
//                ThreeGPP_GGSN_Address_AVP			threeGPP_GGSN_Address			optional,
//                ThreeGPP_GGSN_IPv6_Address_AVP		threeGPP_GGSN_IPv6_Address		optional,
//                set length (0..2) of AN_GW_Addr_AVP	aN_GW_Address					optional,
//                UDP_Source_Port_AVP					uDP_Source_Port					optional,
//                set of AVP_Type                    	aVP_Type						optional
	        	
				recOf.appendField(rv);
	        }
			rv_.setField("qoS_Rule_Install",recOf);
		}
		else{}
	    return;	
	}
    
    private void decode_QoS_Rule_Install_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
	    
    	AVP[] avps=msg.findAVPs(AVP_QoS_Rule_Install_AVP);
		if (avps==null) return;
		
		if (avps.length>=1){
			RecordOfValue recOf = (RecordOfValue) rv_DiameterM.getField("qoS_Rule_Install").getType().newInstance();
			recOf.setLength(0);
	
	        for(int i=0;i<avps.length;i++){			
	        	RecordValue rv = (RecordValue) recOf.getElementType().newInstance();
	        	
	        	decode_AVPHeader( rv ,avps[i]);
			    decode_QoS_Rule_Name_AVP( rv ,avps[i]);//			qoS_Rule_Name					optional,

				recOf.appendField(rv);
	        }
			rv_DiameterM.setField("qoS_Rule_Install",recOf);
		}
		else{}
	    return;	
	}
    /*End TS129_215 AVPS*/
    
    //TS129_272 AVP codes
    private void decode_ULR_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_ULR_Flags_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("ulr_Flags").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("ulr_Flags",rv_uf);
    	return;
    }
    
    private void decode_ULA_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_ULA_Flags_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("ula_Flags").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("ula_Flags",rv_uf);
    	return;
    }
    
    private void decode_Visited_PLMN_Id_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
		AVP avp=uf.findAVP(AVP_Visited_PLMN_Id_AVP);
		if (avp==null) return;
		
		RecordValue rv_visitedPLMN_Id = (RecordValue) rv_ufd.getField("visited_PLMN_Id").getType().newInstance();
			
		decode_AVPHeader( rv_visitedPLMN_Id ,avp);
		rv_visitedPLMN_Id.setField("aVP_Data", mkOctetstring(avp.data));
			
		rv_ufd.setField("visited_PLMN_Id",rv_visitedPLMN_Id);
    	return;
    }
    
    private void decode_UE_SRVCC_Capability_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	String s_ue="";
    	AVP avp=uf.findAVP(AVP_UE_SRVCC_Capability_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ue = (RecordValue) (RecordValue) rv_ufd.getField("ue_SRVCC_Capability").getType().newInstance();
	   
	    decode_AVPHeader( rv_ue ,avp);
	    rv_ue.setField("aVP_Data", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	           
	      case 0   : s_ue="UE_SRVCC_NOT_SUPPORTED_E";break; 
	      case 1   : s_ue="UE_SRVCC_SUPPORTED_E";break; 
	    }
	        
	    EnumeratedValue e_ue= (EnumeratedValue)rv_ue.getField("aVP_Data");
	    e_ue.setEnum(s_ue);
	    rv_ue.setField("aVP_Data", e_ue);*/
	    
	    rv_ufd.setField("ue_SRVCC_Capability",rv_ue);
    	return;
    }
    
    private void decode_SGSN_Number_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
		AVP avp=uf.findAVP(AVP_SGSN_Number_AVP);
		if (avp==null) return;
		
		RecordValue rv_SGSN_Number_AVP = (RecordValue) rv_ufd.getField("sgsn_Number").getType().newInstance();
			
		decode_AVPHeader( rv_SGSN_Number_AVP ,avp);
		rv_SGSN_Number_AVP.setField("aVP_Data", mkOctetstring(avp.data));
			
		rv_ufd.setField("sgsn_Number",rv_SGSN_Number_AVP);
    	return;
    }
    
    private void decode_Homogeneous_Support_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	String s_hs="";
    	AVP avp=uf.findAVP(AVP_Homogeneous_Support_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_hs = (RecordValue) (RecordValue) rv_ufd.getField("homogeneous_Support").getType().newInstance();
	   
	    decode_AVPHeader( rv_hs ,avp);
	    rv_hs.setField("aVP_Data", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	           
	      case 0   : s_hs="NOT_SUPPORTED_E";break; 
	      case 1   : s_hs="SUPPORTED_E";break; 
	    }
	        
	    EnumeratedValue e_hs= (EnumeratedValue)rv_hs.getField("aVP_Data");
	    e_hs.setEnum(s_hs);
	    rv_hs.setField("aVP_Data", e_hs);*/
	    
	    rv_ufd.setField("homogeneous_Support",rv_hs);
    	return;
    }
    
    
        private void decode_Context_Identifier_AVP(RecordValue rv_,	AVP avp_){
    	
	       AVP avp=avp_.findChildAVP(AVP_Context_Identifier_AVP);
	   	   if (avp==null) return;
	       	
	   	   RecordValue rv_c = (RecordValue) (RecordValue) rv_.getField("context_Identifier").getType().newInstance();
	   	   
	   	   decode_AVPHeader( rv_c ,avp);
	   	   rv_c.setField("aVP_Data", mkInt(avp.int_data));
	   	   
	   	   rv_.setField("context_Identifier",rv_c);
	   	   return;
    	}
   
        private void decode_Service_Selection_AVP(RecordValue rv_,	AVP avp_){
        	
	       AVP avp=avp_.findChildAVP(AVP_Service_Selection_AVP);
	   	   if (avp==null) return;
	       	
	   	   RecordValue rv_c = (RecordValue) (RecordValue) rv_.getField("service_Selection").getType().newInstance();
	   	   
	   	   decode_AVPHeader( rv_c ,avp);
	   	   rv_c.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
	   	   
	   	   rv_.setField("service_Selection",rv_c);
	   	   return;
    	}
        
	        private void decode_MIP_Home_Agent_Address_AVP(RecordValue rv_,	AVP avp_){
	        	
		       AVP[] avp=avp_.findChildAVPs(AVP_MIP_Home_Agent_Address_AVP);
		   	   if (avp==null) return;
		       	
		   	   if (avp.length <= 2){
			   	   RecordValue rv_c = (RecordValue) rv_.getField("mip_Home_Agent_Address1").getType().newInstance();
			   	   decode_AVPHeader( rv_c ,avp[0]);
			   	   decode_Address(rv_c,avp[0]);
			   	   rv_.setField("mip_Home_Agent_Address1",rv_c);
		   	   }
		   	   if (avp.length==2){
			   	   RecordValue rv_c = (RecordValue) rv_.getField("mip_Home_Agent_Address2").getType().newInstance();
			   	   decode_AVPHeader( rv_c ,avp[1]);
			   	   decode_Address(rv_c,avp[1]);
			   	   rv_.setField("mip_Home_Agent_Address2",rv_c); 
		   	   }
		   	   return;
	    	}
	        
	        private void decode_MIP_Home_Agent_Host_AVP(RecordValue rv_,	AVP avp_){
	        	
	 	       AVP avp=avp_.findChildAVP(AVP_MIP_Home_Agent_Host_AVP);
	 	   	   if (avp==null) return;
	 	       	
	 	   	   RecordValue rv_c = (RecordValue) (RecordValue) rv_.getField("mip_Home_Agent_Host").getType().newInstance();
	 	   	   
	 	   	   decode_AVPHeader( rv_c ,avp);
              
               decode_Destination_Realm_AVP( rv_c ,avp);    
               decode_Destination_Host_AVP( rv_c ,avp);      
               //decode_AVP_Type( rv_c ,avp);  //optional
	 	   	   
	 	   	   rv_.setField("mip_Home_Agent_Host",rv_c);
	 	   	   return;
	     	}
        
	        private void decode_MIP6_Home_Link_Prefix_AVP(RecordValue rv_,	AVP avp_){
        	
	 	       AVP avp=avp_.findChildAVP(AVP_MIP6_Home_Link_Prefix_AVP);
	 	   	   if (avp==null) return;
	 	       	
	 	   	   RecordValue rv_c = (RecordValue) (RecordValue) rv_.getField("mip6_Home_Link_Prefix").getType().newInstance();
	 	   	   
	 	   	   decode_AVPHeader( rv_c ,avp);
	 	       rv_c.setField("aVP_Data", mkOctetstring(avp.data));
               	 	   	   
	 	   	   rv_.setField("mip6_Home_Link_Prefix",rv_c);
	 	   	   return;
	     	}
	        
        private void decode_MIP6_Agent_Info_AVP(RecordValue rv_,	AVP avp_){
        	
 	       AVP avp=avp_.findChildAVP(AVP_MIP6_Agent_Info_AVP);
 	   	   if (avp==null) return;
 	       	
 	   	   RecordValue rv_c = (RecordValue) (RecordValue) rv_.getField("mip6_Agent_Info").getType().newInstance();
 	   	   
 	   	   decode_AVPHeader( rv_c ,avp);
 	   	   decode_MIP_Home_Agent_Address_AVP(rv_c,avp);         //  mip_Home_Agent_Address1            optional,
	       //decode_MIP_Home_Agent_Address_AVP(rv_c,avp);         //  mip_Home_Agent_Address2            optional,
	       decode_MIP_Home_Agent_Host_AVP(rv_c,avp);           //   mip_Home_Agent_Host                optional,
	       decode_MIP6_Home_Link_Prefix_AVP(rv_c,avp);         //   mip6_Home_Link_Prefix              optional,
	       //decode_AVP_Type(rv_c,avp);  //  optional
	 	   	   
 	   	   rv_.setField("mip6_Agent_Info",rv_c);
 	   	   return;
     	}
 
        
        private void decode_Visited_Network_Identifier_AVP(RecordValue rv_,	AVP avp_){
        	
 	       AVP avp=avp_.findChildAVP(AVP_Visited_Network_Identifier_AVP);
 	   	   if (avp==null) return;
 	       	
 	   	   RecordValue rv_c = (RecordValue) (RecordValue) rv_.getField("visited_Network_Identifier").getType().newInstance();
 	   	   
 	   	   decode_AVPHeader( rv_c ,avp);
 	   	   rv_c.setField("aVP_Data", mkOctetstring(avp.data));
 	   	   
 	   	   rv_.setField("visited_Network_Identifier",rv_c);
 	   	   return;
     	}
        
        private void decode_Visited_Network_Identifier_AVP(RecordValue rv_ufd,	DiameterMessage uf){
        	
  	       AVP avp=uf.findAVP(AVP_Visited_Network_Identifier_AVP);
  	   	   if (avp==null) return;
  	       	
  	   	   RecordValue rv_c = (RecordValue) rv_ufd.getField("visited_Network_Identifier").getType().newInstance();
  	   	   
  	   	   decode_AVPHeader( rv_c ,avp);
  	   	   rv_c.setField("aVP_Data", mkOctetstring(avp.data));
  	   	   
  	   	   rv_ufd.setField("visited_Network_Identifier",rv_c);
  	   	   return;
      	}
        
        private void decode_SIP_Authentication_Context_AVP(RecordValue rv_,	AVP avp_){
        	
   	       AVP avp=avp_.findChildAVP(AVP_SIP_Authentication_Context_AVP);
   	   	   if (avp==null) return;
   	       	
   	   	   RecordValue rv_c = (RecordValue) rv_.getField("sIP_Authentication_Context").getType().newInstance();
   	   	   
   	   	   decode_AVPHeader( rv_c ,avp);
   	   	   rv_c.setField("aVP_Data", mkOctetstring(avp.data));
   	   	   
   	   	   rv_.setField("sIP_Authentication_Context",rv_c);
   	   	   return;
       	}
        
        private void decode_UAR_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
        	
        	AVP avp=uf.findAVP(AVP_UAR_Flags_AVP);
       	    if (avp==null) return;
       	   
       	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("uAR_Flags").getType().newInstance();
    	   
    	    decode_AVPHeader( rv_uf ,avp);
            rv_uf.setField("aVP_Data", mkInt(avp.int_data));
    	    
    	    rv_ufd.setField("uAR_Flags",rv_uf);
        	return;
        }

        private void decode_User_Authorization_Type_AVP(RecordValue rv_ufd,	DiameterMessage uf){
        	
        	AVP avp=uf.findAVP(AVP_User_Authorization_Type_AVP);
       	    if (avp==null) return;
       	   
       	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("user_Authorization_Type").getType().newInstance();
    	   
    	    decode_AVPHeader( rv_uf ,avp);
            rv_uf.setField("aVP_Data", mkInt(avp.int_data));
    	    
    	    rv_ufd.setField("user_Authorization_Type",rv_uf);
        	return;
        }
        
        private void decode_Server_Assignment_Type_AVP_AVP(RecordValue rv_ufd,	DiameterMessage uf){
        	
        	AVP avp=uf.findAVP(AVP_Server_Assignment_Type_AVP);
       	    if (avp==null) return;
       	   
       	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("server_Assignment_Type").getType().newInstance();
    	   
    	    decode_AVPHeader( rv_uf ,avp);
            rv_uf.setField("aVP_Data", mkInt(avp.int_data));
    	    
    	    rv_ufd.setField("server_Assignment_Type",rv_uf);
        	return;
        }
        
        private void decode_User_Data_Already_Available_AVP(RecordValue rv_ufd,	DiameterMessage uf){
        	
        	AVP avp=uf.findAVP(AVP_User_Data_Already_Available_AVP);
       	    if (avp==null) return;
       	   
       	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("user_Data_Already_Available").getType().newInstance();
    	   
    	    decode_AVPHeader( rv_uf ,avp);
            rv_uf.setField("aVP_Data", mkInt(avp.int_data));
    	    
    	    rv_ufd.setField("user_Data_Already_Available",rv_uf);
        	return;
        }
        
        private void decode_SIP_Digest_Authenticate_AVP(RecordValue rv_,	AVP avp_){
        	
	       AVP avp=avp_.findChildAVP(AVP_SIP_Digest_Authenticate_AVP);
	   	   if (avp==null) return;
	       	
	   	   RecordValue rv_c = (RecordValue) rv_.getField("sIP_Digest_Authenticate").getType().newInstance();
	   	   
	   	   try {avp.ungroup();}catch(Exception ex){}
	   	    
	   	   decode_AVPHeader( rv_c ,avp);
	   	   decode_Digest_Realm_AVP( rv_c ,avp);//            digest_Realm,//requ.
           decode_Digest_Algorithm_AVP( rv_c ,avp);//         digest_Algorithm optional,
           decode_Digest_QoP_AVP( rv_c ,avp);//                 digest_QoP optional,
           decode_Digest_HA1_AVP( rv_c ,avp);//                 digest_HA1 optional,
           //set of AVP_Type                 aVP_Type optional
	   	   
	   	   rv_.setField("sIP_Digest_Authenticate",rv_c);
	   	   return;
    	}
        
        
	    	private void decode_Path_AVP(RecordValue rv_,	AVP avp_){
	    		
	    		AVP avp=avp_.findChildAVP(AVP_Path_AVP);
	       	    if (avp==null) return;
	       	   
	       	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("path").getType().newInstance();
	    	   
	    	    decode_AVPHeader( rv_ed ,avp);
	    	    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
	    	    
	    	    rv_.setField("path",rv_ed);
	    	    return;
	    	}
	    	
	    	private void decode_Contact_AVP(RecordValue rv_,	AVP avp_){
	    		
	    		AVP avp=avp_.findChildAVP(AVP_Contact_AVP);
	       	    if (avp==null) return;
	       	   
	       	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("contact").getType().newInstance();
	    	   
	    	    decode_AVPHeader( rv_ed ,avp);
	    	    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
	    	    
	    	    rv_.setField("contact",rv_ed);
	    	    return;
	    	}
	    	
	    	private void decode_Subscription_Info_AVP(RecordValue rv_,	AVP avp_){
	    		
	    		AVP avp=avp_.findChildAVP(AVP_Subscription_Info_AVP);
	       	    if (avp==null) return;
	       	   
	       	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("subscription_Info").getType().newInstance();
	    	   
	    	    decode_AVPHeader( rv_ed ,avp);
//	    	    decode_Call_ID_SIP_Header_AVP( rv_ed ,avp);//        call_ID_SIP_Header,
//	    	    decode_From_SIP_Header_AVP( rv_ed ,avp);//           from_SIP_Header,
//	    	    decode_To_SIP_Header_AVP( rv_ed ,avp);//             to_SIP_Header,
//	    	    decode_Record_Route_AVP( rv_ed ,avp);//              record_Route,
//	    	    decode_Contact_AVP( rv_ed ,avp);//                   contact,
                //set of AVP_Type               aVP_Type optional
	    	    
	    	    rv_.setField("subscription_Info",rv_ed);
	    	    return;
	    	}
        
        
        private void decode_Restoration_Info_AVP(RecordValue rv_,	AVP avp_){
        	
	       AVP[] avps=avp_.findChildAVPs(AVP_Restoration_Info_AVP);
	   	   if (avps==null) return;
	       	
	   	   RecordOfValue rov_c = (RecordOfValue) rv_.getField("restoration_Info").getType().newInstance();
	   	   rov_c.setLength(0);

	   	   for(int i=0;i<avps.length;i++){
	           RecordValue rv = (RecordValue) rov_c.getElementType().newInstance();
        	
               try{avps[i].ungroup();}catch(Exception e){}
		   	
               decode_AVPHeader( rv ,avps[i]);
	           decode_Path_AVP ( rv ,avps[i]);//                     path,
	           decode_Contact_AVP( rv ,avps[i]);     //              contact,
	           //decode_Subscription_Info_AVP( rv ,avps[i]);//         subscription_Info optional,
	           //set of AVP_Type               aVP_Type optional
	           
	           rov_c.appendField(rv);
	   	   }
	   	   rv_.setField("restoration_Info",rov_c);
	   	   return;
 	    }
        
        private void decode_SCSCF_Restoration_Info_AVP(RecordValue rv_ufd,	DiameterMessage uf){
        	
        	boolean rv_present = false;
        	
        	AVP[] avps=uf.findAVPs(AVP_SCSCF_Restoration_Info_AVP);
       	    if (avps==null) return;
       
       	    
       	    if (avps.length>=1){
 			  try{	
 				 RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("sCSCF_Restoration_Info").getType().newInstance();
 	       	    
 	       	    try {avps[0].ungroup();}catch(Exception ex){}
 	       	    
 	       	    decode_AVPHeader( rv_uf ,avps[0]);
 	            decode_Restoration_Info_AVP( rv_uf ,avps[0]);//    restoration_Info,
 	            decode_SIP_Authentication_Scheme_AVP( rv_uf ,avps[0]);//                       sIP_Authentication_Scheme,
 	            //set of AVP_Type                                     aVP_Type optional
 	            
 	            rv_ufd.setField("sCSCF_Restoration_Info",rv_uf);
 				
 	            rv_present=true;
	 			}catch(Exception ex){
	 				//System.out.print(ex.getMessage());	
	 			}
	 			if(!rv_present){
	 				
	 				RecordOfValue recOfpi = (RecordOfValue) rv_ufd.getField("sCSCF_Restoration_Info").getType().newInstance();
					recOfpi.setLength(0);
				
					for (int i = 0; i < avps.length; i++) {
					    RecordValue rv = (RecordValue) recOfpi.getElementType().newInstance();
			        	
					    try {avps[i].ungroup();}catch(Exception ex){}
		 	       	    
		 	       	    decode_AVPHeader( rv ,avps[i]);
		 	            decode_Restoration_Info_AVP( rv ,avps[i]);//    restoration_Info,
		 	            decode_SIP_Authentication_Scheme_AVP( rv ,avps[i]);//                       sIP_Authentication_Scheme,
		 	            //set of AVP_Type                                     aVP_Type optional
					    
					    recOfpi.appendField(rv);
					}
					rv_ufd.setField("sCSCF_Restoration_Info",recOfpi);	
	 	 		}
       	   }
       	   return;
       	}
        
        private void decode_Originating_Request_AVP(RecordValue rv_ufd,	DiameterMessage uf){
        	
        	AVP avp=uf.findAVP(AVP_Originating_Request_AVP);
       	    if (avp==null) return;
       	   
       	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("originating_Request").getType().newInstance();
    	   
    	    decode_AVPHeader( rv_uf ,avp);
            rv_uf.setField("aVP_Data", mkInt(avp.int_data));
    	    
    	    rv_ufd.setField("originating_Request",rv_uf);
        	return;
        }
        
        	private void decode_Reason_Code_AVP(RecordValue rv_,	AVP avp_){
        	
	 	       AVP avp=avp_.findChildAVP(AVP_Reason_Code_AVP);
	 	   	   if (avp==null) return;
	 	       
	 	   	   RecordValue rv_c = (RecordValue) (RecordValue) rv_.getField("reason_Code").getType().newInstance();
	 	   	   
	 	   	   decode_AVPHeader( rv_c ,avp);
	 	       rv_c.setField("aVP_Data", mkInt(avp.int_data));
            	 	   	   
	 	   	   rv_.setField("reason_Code",rv_c);
	 	   	   return;
	     	}
        	
        	private void decode_Reason_Info_AVP(RecordValue rv_,	AVP avp_){
            	
 	 	       AVP avp=avp_.findChildAVP(AVP_Reason_Info_AVP);
 	 	   	   if (avp==null) return;
 	 	   	   
 	 	   	   RecordValue rv_c = (RecordValue) (RecordValue) rv_.getField("reason_Info").getType().newInstance();
 	 	   	   
 	 	   	   decode_AVPHeader( rv_c ,avp);
 	 	       rv_c.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
             	 	   	   
 	 	   	   rv_.setField("reason_Info",rv_c);
 	 	   	   return;
 	     	}
        
        private void decode_Deregistration_Reason_AVP(RecordValue rv_ufd,	DiameterMessage uf){
        	
        	AVP avp=uf.findAVP(AVP_Deregistration_Reason_AVP);
       	    if (avp==null) return;
       	   
       	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("deregistration_Reason").getType().newInstance();
            
       	    try{avp.ungroup();}catch(Exception e){}

    	    decode_AVPHeader( rv_uf ,avp);    	    
            decode_Reason_Code_AVP( rv_uf ,avp);//         reason_Code,//requi.
            decode_Reason_Info_AVP( rv_uf ,avp);//         reason_Info optional,
            //set of AVP_Type         aVP_Type optional
    	    
    	    rv_ufd.setField("deregistration_Reason",rv_uf);
        	return;
        }
         
        private void decode_Specific_APN_Info_AVP(RecordValue rv_,	AVP avp_){
        	
  	       AVP avp=avp_.findChildAVP(AVP_Specific_APN_Info_AVP);
  	   	   if (avp==null) return;
  	       	
  	   	   RecordValue rv_c = (RecordValue) (RecordValue) rv_.getField("specific_APN_Info").getType().newInstance();
  	   	   
  	   	   decode_AVPHeader( rv_c ,avp);
	       
  	   	   decode_Service_Selection_AVP(rv_c,avp);
	       decode_MIP6_Agent_Info_AVP(rv_c,avp);
	       decode_Visited_Network_Identifier_AVP(rv_c,avp);
	      // decode_AVP_Type(rv_c,avp);//     optional
  	   	   
  	   	   rv_.setField("specific_APN_Info",rv_c);
  	   	   return;
      	}
        
    private void decode_Active_APN_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Active_APN_AVP);
		if (avp==null) return;
		
		RecordValue rv_ActiveAPN_AVP = (RecordValue) rv_ufd.getField("active_APN").getType().newInstance();
			
		decode_AVPHeader( rv_ActiveAPN_AVP ,avp);
	    decode_Context_Identifier_AVP(rv_ActiveAPN_AVP,avp);
        decode_Service_Selection_AVP(rv_ActiveAPN_AVP,avp);//optional
        decode_MIP6_Agent_Info_AVP(rv_ActiveAPN_AVP,avp);//optional,
        decode_Visited_Network_Identifier_AVP(rv_ActiveAPN_AVP,avp); //optional,
        decode_Specific_APN_Info_AVP(rv_ActiveAPN_AVP,avp);//                optional,
        //decode_AVP_Type(rv_ActiveAPN_AVP,avp);//     optional
			
		rv_ufd.setField("active_APN",rv_ActiveAPN_AVP);
    	return;
    }
    
    private void decode_Error_Diagnostic_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	String s_ed="";
    	AVP avp=uf.findAVP(AVP_Error_Diagnostic_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_ufd.getField("error_Diagnostic").getType().newInstance();
	   
	    decode_AVPHeader( rv_ed ,avp);
	    rv_ed.setField("error_Diagnostic_Code", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	           
	      case 0   : s_ed="GPRS_DATA_SUBSCRIBED_E";break;       
	      case 1   : s_ed="NO_GPRS_DATA_SUBSCRIBED_E";break;    
	      case 2   : s_ed="ODB_ALL_APN_E";break;                
	      case 3   : s_ed="ODB_HPLMN_APN_E";break;              
	      case 4   : s_ed="ODB_VPLMN_APN_E";break;              
	    }
	        
	    EnumeratedValue e_ed= (EnumeratedValue)rv_ed.getField("error_Diagnostic_Code");
	    e_ed.setEnum(s_ed);
	    rv_ed.setField("error_Diagnostic_Code", e_ed);*/
	    
	    rv_ufd.setField("error_Diagnostic",rv_ed);
    	return;
    }
    
    private void decode_Subscriber_Status_AVP(RecordValue rv_ufd,	AVP avp_){
    	
    	String s_ed="";
    	AVP avp=avp_.findChildAVP(AVP_Subscriber_Status_AVP);
     	if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) rv_ufd.getField("subscriber_Status").getType().newInstance();
	   
	    decode_AVPHeader(rv_ed,avp);
	    rv_ed.setField("subscriber_Status_Code", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	           
	      case 0   : s_ed="SERVICE_GRANTED_E";break;       
	      case 1   : s_ed="OPERATOR_DETERMINED_BARRING_E";break;    
	    }
	        
	    EnumeratedValue e_ed= (EnumeratedValue)rv_ed.getField("subscriber_Status_Code");
	    e_ed.setEnum(s_ed);
	    rv_ed.setField("subscriber_Status_Code", e_ed);*/
	    
	    rv_ufd.setField("subscriber_Status",rv_ed);
    	return;
    }
    
    private void decode_Operator_Determined_Barring_AVP(RecordValue rv_,	AVP avp_){
    	
       AVP avp=avp_.findChildAVP(AVP_Operator_Determined_Barring_AVP);
   	   if (avp==null) return;
       	
   	   RecordValue rv_c = (RecordValue) rv_.getField("operator_Determined_Barring").getType().newInstance();
   	   
   	   decode_AVPHeader( rv_c ,avp);
   	   rv_c.setField("aVP_Data", mkInt(avp.int_data));
   	   
   	   rv_.setField("operator_Determined_Barring",rv_c);
   	   return;
	}
    
    private void decode_HPLMN_ODB_AVP(RecordValue rv_,	AVP avp_){
    	
       AVP avp=avp_.findChildAVP(AVP_HPLMN_ODB_AVP);
   	   if (avp==null) return;
       	
   	   RecordValue rv_c = (RecordValue) rv_.getField("hPLMN_ODB").getType().newInstance();
   	   
   	   decode_AVPHeader( rv_c ,avp);
   	   rv_c.setField("aVP_Data", mkInt(avp.int_data));
   	   
   	   rv_.setField("hPLMN_ODB",rv_c);
   	   return;
	}
    
    private void decode_Access_Restriction_Data_AVP(RecordValue rv_,	AVP avp_){
    	
        AVP avp=avp_.findChildAVP(AVP_Access_Restriction_Data_AVP);
	    if (avp==null) return;
    	
	    RecordValue rv_c = (RecordValue) rv_.getField("access_Restriction_Data").getType().newInstance();
	   
	    decode_AVPHeader( rv_c ,avp);
	    rv_c.setField("aVP_Data", mkInt(avp.int_data));
	   
	    rv_.setField("access_Restriction_Data",rv_c);
	    return;
 	}
    
    private void decode_APN_OI_Replacement_AVP(RecordValue rv_,	AVP avp_){
    	
        AVP avp=avp_.findChildAVP(AVP_APN_OI_Replacement_AVP);
	    if (avp==null) return;
    	
	    RecordValue rv_c = (RecordValue) rv_.getField("aPN_OI_Replacement").getType().newInstance();
	   
	    decode_AVPHeader( rv_c ,avp);
	    rv_c.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
	   
	    rv_.setField("aPN_OI_Replacement",rv_c);
	    return;
 	}
    
	private void decode_Max_Requested_Bandwidth_UL_AVP(RecordValue rv_,	AVP avp_){
		
		AVP avp=avp_.findChildAVP(AVP_Max_Requested_Bandwidth_UL_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("max_Requested_Bandwidth_UL").getType().newInstance();
	   
	    decode_AVPHeader( rv_ed ,avp);
	    rv_ed.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_.setField("max_Requested_Bandwidth_UL",rv_ed);
	    return;
	}
	
	private void decode_Max_Requested_Bandwidth_DL_AVP(RecordValue rv_,	AVP avp_){
		
		AVP avp=avp_.findChildAVP(AVP_Max_Requested_Bandwidth_DL_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("max_Requested_Bandwidth_DL").getType().newInstance();
	   
	    decode_AVPHeader( rv_ed ,avp);
	    rv_ed.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_.setField("max_Requested_Bandwidth_DL",rv_ed);
	    return;
	}
    
    private void decode_AMBR_AVP(RecordValue rv_, AVP avp_) {                                //ref: 7.3.33
    	
    	String _str="";
    	AVP avp=avp_.findChildAVP(AVP_AMBR_AVP);
     	if (avp==null) return;
     	
        RecordValue rv_c = (RecordValue) rv_.getField("aMBR").getType().newInstance();
        
        try{avp.ungroup();}catch(Exception e){}

     	decode_AVPHeader( rv_c ,avp);
 	    decode_Max_Requested_Bandwidth_UL_AVP( rv_c ,avp);//max_Requested_Bandwidth_UL,
        decode_Max_Requested_Bandwidth_DL_AVP( rv_c ,avp);//max_Requested_Bandwidth_DL,
        //set of AVP_Type                     aVP_Type                        optional
        
        rv_.setField("aMBR",rv_c);
     	return;
    }
    
    private void decode_All_APN_Conf_Included_Id_AVP(RecordValue rv_, AVP avp_) {                                //ref: 7.3.33
    	
    	String _str="";
    	AVP avp=avp_.findChildAVP(AVP_All_APN_Conf_Included_Id_AVP);
     	if (avp==null) return;
         	
     	RecordValue rv_c = (RecordValue) rv_.getField("all_APN_Conf_Included_Id").getType().newInstance();
        
        decode_AVPHeader( rv_c ,avp);
        rv_c.setField("all_APN_Conf_Included_Indicator", mkInt(avp.int_data));
        
        /*switch(avp.int_data){
		    case 0: _str="All_APN_CONFIGURATIONS_INCLUDED_E";break;
		    case 1: _str="MODIFIED_ADDED_APN_CONFIGURATIONS_INCLUDED_E";break;
		    default:break;
	    }
        
		EnumeratedValue e= (EnumeratedValue)rv_c.getField("all_APN_Conf_Included_Indicator");
		e.setEnum(_str);
		rv_c.setField("all_APN_Conf_Included_Indicator", e);*/
        
        rv_.setField("all_APN_Conf_Included_Id",rv_c);
        return;
    }
    
    private void decode_APN_Configuration_Profile_AVP(RecordValue rv_,	AVP avp_){                                //ref: 7.3.34
    	
    	AVP avp=avp_.findChildAVP(AVP_APN_Configuration_Profile_AVP);
     	if (avp==null) return;
         	
     	RecordValue rv_c = (RecordValue) rv_.getField("aPN_Configuration_Profile").getType().newInstance();
        
     	try{avp.ungroup();}catch(Exception e){}

     	decode_AVPHeader( rv_c ,avp); 
        decode_Context_Identifier_AVP ( rv_c ,avp);//               context_Identifier,
        decode_All_APN_Conf_Included_Id_AVP( rv_c ,avp);//      all_APN_Conf_Included_Id,
        /*set length (1 .. infinity) of*/ decode_APN_Configuration_AVP( rv_c ,avp);//    aPN_Configuration,
        //set of AVP_Type                     aVP_Type                        optional
        
        rv_.setField("aPN_Configuration_Profile",rv_c);
        return;
    }
    
   private void decode_PDN_Type_AVP(RecordValue rv_, AVP avp_) {                                //ref: 7.3.33
    	
    	String _str="";
    	AVP avp=avp_.findChildAVP(AVP_PDN_Type_AVP);
     	if (avp==null) return;
         	
     	RecordValue rv_c = (RecordValue) rv_.getField("pDN_Type").getType().newInstance();
        
        decode_AVPHeader( rv_c ,avp);
        rv_c.setField("pDN_Type_Code", mkInt(avp.int_data));
        
        /*switch(avp.int_data){
		    case 0: _str="IPv4_E";break;
		    case 1: _str="IPv6_E";break;
		    case 2: _str="IPv4v6_E";break;
		    case 3: _str="IPv4_OR_IPv6_E";break;
		      
		    default:break;
	    }
        
		EnumeratedValue e= (EnumeratedValue)rv_c.getField("pDN_Type_Code");
		e.setEnum(_str);
		rv_c.setField("pDN_Type_Code", e);*/
        
        rv_.setField("pDN_Type",rv_c);
        return;
    }    
   
   		private void decode_QoS_Class_Identifier_AVP(RecordValue rv_,	AVP avp_){                                //ref: 7.3.34
   	    	
   			String _str="";

   		   	AVP avp=avp_.findChildAVP(AVP_QoS_Class_Identifier_AVP);
   		    if (avp==null) return;
   		        	
   		    RecordValue rv_c = (RecordValue) rv_.getField("qoS_Class_Identifier").getType().newInstance();
   		       
   		    decode_AVPHeader( rv_c ,avp);
   		    
   	        switch(avp.int_data){
	   	        case 0: _str="Reserved1_E";break;//                (  0),
	    		case 1: _str="QCI_1_E";break;//                 (  1),
	    		case 2: _str="QCI_2_E";break;//                 (  2),
	    		case 3: _str="QCI_3_E";break;//                 (  3),
	    		case 4: _str="QCI_4_E";break;//                 (  4),
	    		case 5: _str="QCI_5_E";break;//                 (  5),
	    		case 6: _str="QCI_6_E";break;//                 (  6),
	    		case 7: _str="QCI_7_E";break;//                 (  7),
	    		case 8: _str="QCI_8_E";break;//                 (  8),
	    		case 9: _str="QCI_9_E";break;//                 (  9),
	    		
	   			default:if (avp.int_data>=10  & avp.int_data<=127) _str="Reserved2_E";//                ( 10),    // .. 127
	   			   else if (avp.int_data>=128 & avp.int_data<=254) _str="OperatorSpecific_E";//        (128),    // .. 254
	   			   else if (avp.int_data==255 )  _str="Reserved3_E";//                (255)break;
   		    }
   	        
   			EnumeratedValue e= (EnumeratedValue)rv_c.getField("qoS_Class_Identifier_Code");
   			e.setEnum(_str);
   			rv_c.setField("qoS_Class_Identifier_Code", e);
   			
   		    rv_.setField("qoS_Class_Identifier",rv_c);
   			return;
   		}
   		
	   		private void decode_Priority_Level_AVP(RecordValue rv_,	AVP avp_){
	   			
	   			AVP avp=avp_.findChildAVP(AVP_Priority_Level_AVP);
	   	   	    if (avp==null) return;
	   	   	   
	   	   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("priority_Level").getType().newInstance();
	   		   
	   		    decode_AVPHeader( rv_ed ,avp);
	   		    rv_ed.setField("aVP_Data", mkInt(avp.int_data));
	   		    
	   		    rv_.setField("priority_Level",rv_ed);
	   		    return;
	   		}
   		
   		private void decode_Allocation_Retention_Priority_AVP(RecordValue rv_,	AVP avp_){                                //ref: 7.3.34
   		   	
   		   	AVP avp=avp_.findChildAVP(AVP_Allocation_Retention_Priority_AVP);
	    	if (avp==null) return;
	        	
	    	RecordValue rv_c = (RecordValue) rv_.getField("allocation_Retention_Priority").getType().newInstance();
	       
            try{avp.ungroup();}catch(Exception e){}

	    	decode_AVPHeader( rv_c ,avp);
            decode_Priority_Level_AVP( rv_c ,avp);//                    priority_Level,
            //Pre_emption_Capability_AVP            pre_emption_Capability            optional,
            //Pre_emption_Vulnerability_AVP        pre_emption_Vulnerability        optional
	    	
	    	rv_.setField("allocation_Retention_Priority",rv_c);
	        return;
   		}   
   		
	private void  decode_EPS_Subscribed_QoS_Profile_AVP(RecordValue rv_,	AVP avp_){                                //ref: 7.3.34
	   	
	   	AVP avp=avp_.findChildAVP(AVP_EPS_Subscribed_QoS_Profile_AVP);
    	if (avp==null) return;
        	
    	RecordValue rv_c = (RecordValue) rv_.getField("ePS_Subscribed_QoS_Profile").getType().newInstance();
        
    	try{avp.ungroup();}catch(Exception e){}

    	decode_AVPHeader( rv_c ,avp);
		decode_QoS_Class_Identifier_AVP( rv_c ,avp);   //         qoS_Class_Identifier,
        decode_Allocation_Retention_Priority_AVP( rv_c ,avp);  //  allocation_Retention_Priority,
       //set of AVP_Type                     aVP_Type                        optional
       
       rv_.setField("ePS_Subscribed_QoS_Profile",rv_c);
       return;
	}   
   
    private void decode_APN_Configuration_AVP(RecordValue rv_,	AVP avp_){                                        //ref: 7.3.35
        
    	AVP[] avp=avp_.findChildAVPs(AVP_APN_Configuration_AVP);
	    if (avp==null) return;
    	
	    RecordOfValue rov_c = (RecordOfValue) rv_.getField("aPN_Configuration").getType().newInstance();
	    rov_c.setLength(0);
	    
	    for(int i=0;i<avp.length;i++){
	    	RecordValue rv = (RecordValue) rov_c.getElementType().newInstance();
        	
            try{avp[i].ungroup();}catch(Exception e){}
	    	
	    	decode_AVPHeader( rv ,avp[i]);
            decode_Context_Identifier_AVP( rv ,avp[i]);//                context_Identifier,
//          set length (2) of Served_Party_IP_Address_AVP    served_Party        optional,
            decode_PDN_Type_AVP( rv ,avp[i]); //                       pDN_Type,
            decode_Service_Selection_AVP( rv ,avp[i]);//                service_Selection,
            decode_EPS_Subscribed_QoS_Profile_AVP( rv ,avp[i]);//       ePS_Subscribed_QoS_Profile        optional,
//          VPLMN_Dynamic_Address_Allowed_AVP    vPLMN_Dynamic_Address_Allowed    optional,
//          MIP6_Agent_Info_AVP                    mIP6_Agent_Info                    optional,
//          Visited_Network_Identifier_AVP        visited_Network_Identifier        optional,
//          PDN_GW_Allocation_Type_AVP            pDN_GW_Allocation_Type            optional,
//          ThreeGPP_Charging_Char_AVP            threeGPP_Charging_Char            optional,
            decode_AMBR_AVP( rv ,avp[i]);//                            aMBR                            optional,
//          set of Specific_APN_Info_AVP        specific_APN_Info                optional,
//          APN_OI_Replacement_AVP                aPN_OI_Replacement                optional,
//          SIPTO_Permission_AVP                sIPTO_Permission                optional,
//          LIPA_Permission_AVP                    lIPA_Permission                    optional,
//          set of AVP_Type                     aVP_Type                        optional
          
            rov_c.appendField(rv);
	    }
        rv_.setField("aPN_Configuration",rov_c);
 	    return;
    }
    
    private void decode_GMLC_Address_AVP(RecordValue rv_mb,	DiameterMessage dm){
    	
    	AVP avp=dm.findAVP(AVP_GMLC_Address_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ga = (RecordValue) (RecordValue) rv_mb.getField("gmlc_Address").getType().newInstance();
	   
	    decode_AVPHeader( rv_ga ,avp);
	    decode_Address(rv_ga,avp);

 		rv_mb.setField("gmlc_Address",rv_ga);
    	return;
    }
    
    private void decode_Complete_DL_Included_Indicator_AVP(RecordValue rv_, AVP avp_) {                                //ref: 7.3.33
    	
    	String _str="";
    	AVP avp=avp_.findChildAVP(AVP_Complete_DL_Included_Indicator_AVP);
     	if (avp==null) return;
         	
     	RecordValue rv_c = (RecordValue) rv_.getField("complete_Data_List_Included_Indicator").getType().newInstance();
        
        decode_AVPHeader( rv_c ,avp);
        rv_c.setField("complete_DL_Included_Indicator", mkInt(avp.int_data));
        
        /*switch(avp.int_data){
		    case 0: _str="All_PDP_CONTEXTS_INCLUDED_E";break;
		    case 1: _str="MODIFIED_ADDED_PDP_CONTEXTS_INCLUDED_E";break;		      
		    default:break;
	    }
        
		EnumeratedValue e= (EnumeratedValue)rv_c.getField("complete_DL_Included_Indicator");
		e.setEnum(_str);
		rv_c.setField("complete_DL_Included_Indicator", e);*/
		
	    rv_.setField("complete_Data_List_Included_Indicator",rv_c);
		
		return;
    }
    
	private void decode_PDP_Type_AVP(RecordValue rv_,	AVP avp_){
		
		AVP avp=avp_.findChildAVP(AVP_PDP_Type_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("pDP_Type").getType().newInstance();
	   
	    decode_AVPHeader( rv_ed ,avp);
	    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
	    
	    rv_.setField("pDP_Type",rv_ed);
	    return;
	}
	
	private void decode_QoS_Subscribed_AVP(RecordValue rv_,	AVP avp_){
		
		AVP avp=avp_.findChildAVP(AVP_QoS_Subscribed_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("qoS_Subscribed").getType().newInstance();
	   
	    decode_AVPHeader( rv_ed ,avp);
	    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
	    
	    rv_.setField("qoS_Subscribed",rv_ed);
	    return;
	}
    
    private void decode_PDP_Context_AVP(RecordValue rv_,	AVP avp_){
    	
    	AVP[] avp=avp_.findChildAVPs(AVP_PDP_Context_AVP);
	    if (avp==null) return;
    	
	    RecordOfValue rov_c = (RecordOfValue) rv_.getField("pDP_Context").getType().newInstance();
	    rov_c.setLength(0);
	    
	    for(int i=0;i<avp.length;i++){
	    	RecordValue rv = (RecordValue) rov_c.getElementType().newInstance();
        	
	    	try{avp[i].ungroup();}catch(Exception e){}
	    	
	    	decode_AVPHeader( rv ,avp[i]);
        	decode_Context_Identifier_AVP(rv, avp[i]);//                context_Identifier,
            decode_PDP_Type_AVP(rv, avp[i]);//                         pDP_Type,
            //PDP_Address_AVP                        pDP_Address                        optional,
            decode_QoS_Subscribed_AVP(rv, avp[i]);//                     qoS_Subscribed,
            //VPLMN_Dynamic_Address_Allowed_AVP    vPLMN_Dynamic_Address_Allowed    optional,
            decode_Service_Selection_AVP(rv, avp[i]);//                service_Selection,
            //ThreeGPP_Charging_Char_AVP            threeGPP_Charging_Char            optional,
            //Ext_PDP_Type_AVP                    ext_PDP_Type                    optional,
            //Ext_PDP_Address_AVP                    ext_PDP_Address                    optional,
            //AMBR_AVP                            aMBR                            optional,
            //SIPTO_Permission_AVP                sIPTO_Permission                optional,
            //LIPA_Permission_AVP                    lIPA_Permission                    optional,
            //set of AVP_Type                     aVP_Type                        optional
	        
            rov_c.appendField(rv);
	    }
        rv_.setField("pDP_Context",rov_c);
    	return;
    }
    
    private void decode_GPRS_Subscription_Data_AVP(RecordValue rv_,	AVP avp_){                                        //ref: 7.3.35
        
    	AVP avp=avp_.findChildAVP(AVP_GPRS_Subscription_Data_AVP);
	    if (avp==null) return;
    	
	    RecordValue rv_c = (RecordValue) rv_.getField("gPRS_Subscription_Data").getType().newInstance();
	   
	    try{avp.ungroup();}catch(Exception e){}
    	
	    decode_AVPHeader( rv_c ,avp);
    	decode_Complete_DL_Included_Indicator_AVP(rv_c,avp);//    complete_Data_List_Included_Indicator,
        decode_PDP_Context_AVP(rv_c,avp);//    pDP_Context,
        //set of AVP_Type                     aVP_Type   optional
    	
        rv_.setField("gPRS_Subscription_Data",rv_c);
        return;	
	}
    
    private void decode_Subscription_Data_AVP(RecordValue rv_mb,	DiameterMessage dm){
    	
    	AVP avp=dm.findAVP(AVP_Subscription_Data_AVP);
   	    if (avp==null) return;
    
   	    RecordValue rv_sd = (RecordValue) (RecordValue) rv_mb.getField("subscription_Data").getType().newInstance();
	   
   	    try{avp.ungroup();}catch(Exception e){}
   	    
	    decode_AVPHeader( rv_sd ,avp);        
        decode_Subscriber_Status_AVP(rv_sd,avp);//.findChildAVP(AVP_Subscriber_Status_AVP));             //subscriber_Status                optional,
//        decode_MSISDN_AVP                            mSISDN                            optional,
//        decode_STN_SR_AVP                            sTN_SR                            optional,
//        decode_ICS_Indicator_AVP                    iCS_Indicator                    optional,
//        decode_Network_Access_Mode_AVP                network_Access_Mode                optional,
        decode_Operator_Determined_Barring_AVP(rv_sd,avp);//        operator_Determined_Barring        optional,
        decode_HPLMN_ODB_AVP(rv_sd,avp);//                        hPLMN_ODB                        optional,
//        set length (10) of decode_Regional_Subscription_Zone_Code_AVP    zone_Code    optional,
        decode_Access_Restriction_Data_AVP(rv_sd,avp);//            access_Restriction_Data            optional,
        decode_APN_OI_Replacement_AVP(rv_sd,avp);//                aPN_OI_Replacement                optional,
//        decode_LCS_Info_AVP                        lCS_Info                        optional,
//        decode_Teleservice_List_AVP                teleservice_List                optional,
//        set of decode_Call_Barring_Info_AVP        call_Barring_Info                optional,
//        decode_ThreeGPP_Charging_Char_AVP            threeGPP_Charging_Char            optional,
//        decode_AMBR_AVP                            aMBR                            optional,
        decode_APN_Configuration_Profile_AVP(rv_sd,avp);//        aPN_Configuration_Profile        optional,
//        decode_RAT_Freq_Sel_Priority_ID_AVP        rAT_Freq_Sel_Priority_ID        optional,
//        decode_Trace_Data_AVP                        trace_Data                        optional,
        decode_GPRS_Subscription_Data_AVP(rv_sd,avp);//            gPRS_Subscription_Data            optional,
//        decode_CSG_Subscription_Data_AVP            cSG_Subscription_Data            optional,
//        decode_Roaming_Rest_Unsupp_Feature_AVP        roaming_Rest_Unsupp_Feature        optional,
//        decode_RAU_TAU_Timer_AVP                    subscribed_Perio_RAUTAU_Timer    optional,
//        decode_MPS_Priority_AVP                    mPS_Priority                    optional,
//        decode_VPLMN_LIPA_Allowed_AVP                vPLMN_LIPA_Allowed                optional,
//        decode_Relay_Node_Indicator_AVP            relay_Node_Indicator            optional,
//        decode_MDT_User_Consent_AVP                mDT_User_Consent                optional,
//        set of decode_AVP_Type                     aVP_Type                        optional
	    
	    rv_mb.setField("subscription_Data",rv_sd);
    }
    
    
    	private void decode_Immediate_Response_Preferred_AVP(RecordValue rv_,	AVP avp_){
    		
    		AVP avp=avp_.findChildAVP(AVP_Immediate_Response_Preferred_AVP);
       	    if (avp==null) return;
       	   
       	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("immediate_Response_Preferred").getType().newInstance();
    	   
    	    decode_AVPHeader( rv_ed ,avp);
    	    rv_ed.setField("aVP_Data", mkInt(avp.int_data));
    	    
    	    rv_.setField("immediate_Response_Preferred",rv_ed);
    	    return;
    	}
    	
    	private void decode_Re_synchronization_Info_AVP(RecordValue rv_,	AVP avp_){
    		
    		AVP avp=avp_.findChildAVP(AVP_Re_synchronization_Info_AVP);
       	    if (avp==null) return;
       	   
       	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("re_synchronization_Info").getType().newInstance();
    	   
    	    decode_AVPHeader( rv_ed ,avp);
    	    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
    	    
    	    rv_.setField("re_synchronization_Info",rv_ed);
    	    return;
    	}
    	
    private void decode_Req_EUTRAN_Auth_Info_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Req_EUTRAN_Auth_Info_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_ufd.getField("req_EUTRAN_Auth_Info").getType().newInstance();
	   
   	    try{avp.ungroup();}catch(Exception e){}
   	    
    	decode_AVPHeader( rv_ed ,avp); 	
    	//Number_Of_Requested_Vectors_AVP        number_Of_Requested_Vectors        optional,
        decode_Immediate_Response_Preferred_AVP(rv_ed,avp);//    immediate_Response_Preferred    optional,
        decode_Re_synchronization_Info_AVP(rv_ed,avp);//            re_synchronization_Info            optional,
        //set of AVP_Type                     aVP_Type                        optional
	    
	    rv_ufd.setField("req_EUTRAN_Auth_Info",rv_ed);
    	return;
    }
    
    
    private void decode_Req_UTRAN_GERAN_Auth_Info_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Req_UTRAN_GERAN_Auth_Info_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_ufd.getField("req_UTRAN_GERAN_Auth_Info").getType().newInstance();
	   
   	    try{avp.ungroup();}catch(Exception e){}
    	
   	    decode_AVPHeader( rv_ed ,avp);  	
    	//Number_Of_Requested_Vectors_AVP        number_Of_Requested_Vectors        optional,
        decode_Immediate_Response_Preferred_AVP(rv_ed,avp);//    immediate_Response_Preferred    optional,
        decode_Re_synchronization_Info_AVP(rv_ed,avp);//            re_synchronization_Info            optional,
        //set of AVP_Type                     aVP_Type                        optional
	    
	    rv_ufd.setField("req_UTRAN_GERAN_Auth_Info",rv_ed);
    	return;
    }
    
			private void decode_Item_Number_AVP(RecordValue rv_,	AVP avp_){
				
				AVP avp=avp_.findChildAVP(AVP_Item_Number_AVP);
		   	    if (avp==null) return;
		   	   
		   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("item_Number").getType().newInstance();
			   
			    decode_AVPHeader( rv_ed ,avp);
			    rv_ed.setField("aVP_Data", mkInt(avp.int_data));
			    
			    rv_.setField("item_Number",rv_ed);
			    return;
			}
    
		    private void decode_RAND_AVP(RecordValue rv_,	AVP avp_){
				
		    	AVP avp=avp_.findChildAVP(AVP_RAND_AVP);
		   	    if (avp==null) return;
		   	   
		   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("rAND").getType().newInstance();
			   
			    decode_AVPHeader( rv_ed ,avp);
			    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
			    
			    rv_.setField("rAND",rv_ed);
			    return;
			}
		    
		    private void decode_XRES_AVP(RecordValue rv_,	AVP avp_){
				
		    	AVP avp=avp_.findChildAVP(AVP_XRES_AVP);
		   	    if (avp==null) return;
		   	   
		   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("xRES").getType().newInstance();
			   
			    decode_AVPHeader( rv_ed ,avp);
			    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
			    
			    rv_.setField("xRES",rv_ed);
			    return;
			}
		    
		    private void decode_AUTN_AVP(RecordValue rv_,	AVP avp_){
				
		    	AVP avp=avp_.findChildAVP(AVP_AUTN_AVP);
		   	    if (avp==null) return;
		   	   
		   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("aUTN").getType().newInstance();
			   
			    decode_AVPHeader( rv_ed ,avp);
			    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
			    
			    rv_.setField("aUTN",rv_ed);
			    return;
			}
		    
		    private void decode_KASME_AVP(RecordValue rv_,	AVP avp_){
				
		    	AVP avp=avp_.findChildAVP(AVP_KASME_AVP);
		   	    if (avp==null) return;
		   	   
		   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("kASME").getType().newInstance();
			   
			    decode_AVPHeader( rv_ed ,avp);
			    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
			    
			    rv_.setField("kASME",rv_ed);
			    return;
			}
    
	    private void decode_E_UTRAN_Vector_AVP(RecordValue rv_,	AVP avp_){
			
	    	AVP[] avps=avp_.findChildAVPs(AVP_E_UTRAN_Vector_AVP);
	   	    if (avps==null) return;
	   	   
	   	    RecordOfValue rv_ed = (RecordOfValue) rv_.getField("e_UTRAN_Vector").getType().newInstance();
		    rv_ed.setLength(0);
	   	    
		    for (int i = 0; i < avps.length; i++) {
				   
			    RecordValue rv = (RecordValue) rv_ed.getElementType().newInstance();
	        	
			    try{avps[i].ungroup();}catch(Exception e){}
		   	    
			    decode_AVPHeader( rv ,avps[i]);                                        //ref: 7.3.18
		        decode_Item_Number_AVP( rv ,avps[i]);//                       item_Number                        optional,
		        decode_RAND_AVP( rv ,avps[i]);       //                     rAND,
		        decode_XRES_AVP( rv ,avps[i]);     //                       xRES,
		        decode_AUTN_AVP( rv ,avps[i]);   //                         aUTN,
		        decode_KASME_AVP( rv ,avps[i]);//                            kASME,
		        //set of AVP_Type                     aVP_Type                        optional

		        rv_ed.appendField(rv);
			}
	        rv_.setField("e_UTRAN_Vector",rv_ed);
	        return;
	    }
	    
	    private void decode_SIP_Auth_Data_Item_3GPP_AVP(RecordValue rv_,	AVP avp_){
			
	    	try{avp_.ungroup();}catch(Exception e){}
	    	
			decode_AVPHeader( rv_ ,avp_);			
            decode_SIP_Authentication_Scheme_AVP(rv_,avp_);   //    sIP_Authentication_Scheme optional,
            decode_SIP_Item_Number_AVP(rv_,avp_);   //                 sIP_Item_Number optional,
            decode_SIP_Authenticate_AVP(rv_,avp_);//                sIP_Authenticate optional,
            decode_SIP_Authorization_AVP(rv_,avp_);//               sIP_Authorization optional,
            decode_SIP_Authentication_Context_AVP(rv_,avp_);//      sIP_Authentication_Context optional,
            decode_SIP_Digest_Authenticate_AVP(rv_,avp_);//         sIP_Digest_Authenticate optional,
            decode_Confidentiality_Key_AVP(rv_,avp_);//             confidentiality_Key optional,
            decode_Integrity_Key_AVP(rv_,avp_);//                   integrity_Key optional,
            decode_Framed_IP_Address_AVP(rv_,avp_);//               framed_IP_Address optional,
            decode_Framed_IPv6_Prefix_AVP(rv_,avp_);//              framed_IPv6_Prefix optional,
            decode_Framed_Interface_Id_AVP(rv_,avp_);//             framed_Interface_Id optional,
            decode_Line_Identifier_AVP(rv_,avp_);//                 line_Identifier optional,
//            set of AVP_Type                     aVP_Type optional
		    return;
	    }
	    
	    private void decode_SIP_Authentication_Scheme_3GPP_AVP(RecordValue rv_,	AVP avp_){
	    	
	    	decode_AVPHeader( rv_ ,avp_);
	        rv_.setField("aVP_Data", mkCvUTF8(avp_.getStringData()));
	    	return;
	    }
	    
	    private void decode_SIP_Authenticate_3GPP_AVP(RecordValue rv_,	AVP avp_){
	    	
	    	decode_AVPHeader( rv_ ,avp_);
	    	rv_.setField("aVP_Data", mkOctetstring(avp_.data));
	    	return;
	    }
        
	    private void decode_SIP_Authorization_3GPP_AVP(RecordValue rv_,	AVP avp_){
	    	
	    	decode_AVPHeader( rv_ ,avp_);
	    	rv_.setField("aVP_Data", mkOctetstring(avp_.data));
	    	return;
	    }
	    
	    
	    private void decode_Identity_with_Emergency_Registration_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
			
			AVP[] avps=msg.findAVPs(AVP_Identity_with_Emergency_Registration_AVP);
			if (avps==null) return;
			
			if (avps.length>=1){
			
				RecordOfValue recOf = (RecordOfValue) rv_DiameterM.getField("identity_with_Emergency_Registration").getType().newInstance();
				recOf.setLength(0);
		
		        for(int i=0;i<avps.length;i++){			
		        	RecordValue rv = (RecordValue) recOf.getElementType().newInstance();
		        	
		        	try{avps[i].ungroup();}catch(Exception e){}
		        	
		        	decode_AVPHeader( rv ,avps[i]);
		        	decode_User_Name_AVP( rv ,avps[i]);
		        	decode_Public_Identity_AVP( rv ,avps[i]);         
		            //set of AVP_Type             aVP_Type optional
					
					recOf.appendField(rv);
		        }
				rv_DiameterM.setField("identity_with_Emergency_Registration",recOf);
			}
			return;
	    }
	    
		    private void decode_Confidentiality_Key_AVP(RecordValue rv_,	AVP avp_){
				
		    	AVP avp=avp_.findChildAVP(AVP_Confidentiality_Key_AVP);
		   	    if (avp==null) return;
		   	   
		   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("confidentiality_Key").getType().newInstance();
			   
			    decode_AVPHeader( rv_ed ,avp);
			    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
			    
			    rv_.setField("confidentiality_Key",rv_ed);
			    return;
			}
		    
		    private void decode_Integrity_Key_AVP(RecordValue rv_,	AVP avp_){
				
		    	AVP avp=avp_.findChildAVP(AVP_Integrity_Key_AVP);
		   	    if (avp==null) return;
		   	   
		   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("integrity_Key").getType().newInstance();
			   
			    decode_AVPHeader( rv_ed ,avp);
			    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
			    
			    rv_.setField("integrity_Key",rv_ed);
			    return;
			}
		    
		    private void decode_Line_Identifier_AVP(RecordValue rv_,	AVP avp_){
				
		    	AVP avp=avp_.findChildAVP(AVP_Line_Identifier_AVP);
		   	    if (avp==null) return;
		   	   
		   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("line_Identifier").getType().newInstance();
			   
			    decode_AVPHeader( rv_ed ,avp);
			    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
			    
			    rv_.setField("line_Identifier",rv_ed);
			    return;
			}
    
	    private void decode_UTRAN_Vector_AVP(RecordValue rv_,	AVP avp_){
			
	    	AVP[] avps=avp_.findChildAVPs(AVP_UTRAN_Vector_AVP);
	   	    if (avps==null) return;
	   	   
	   	    RecordOfValue rv_ed = (RecordOfValue) rv_.getField("uTRAN_Vector").getType().newInstance();
		    rv_ed.setLength(0);
	   	    
		    for (int i = 0; i < avps.length; i++) {
			    RecordValue rv = (RecordValue) rv_ed.getElementType().newInstance();
	        	
			    try{avps[i].ungroup();}catch(Exception e){}
			    
			    decode_AVPHeader( rv ,avps[i]);                                        //ref: 7.3.18
		        decode_Item_Number_AVP( rv ,avps[i]);//                       item_Number                        optional,
		        decode_RAND_AVP( rv ,avps[i]);       //                     rAND,
		        decode_XRES_AVP( rv ,avps[i]);     //                       xRES,
		        decode_AUTN_AVP( rv ,avps[i]);   //                         aUTN,
		        decode_Confidentiality_Key_AVP( rv ,avps[i]);
		        decode_Integrity_Key_AVP( rv ,avps[i]);
		        //set of AVP_Type                     aVP_Type                        optional

		        rv_ed.appendField(rv);
			}
	        rv_.setField("uTRAN_Vector",rv_ed);
	        return;
	    }
	    
		    private void decode_SRES_AVP(RecordValue rv_,	AVP avp_){
				
		    	AVP avp=avp_.findChildAVP(AVP_SRES_AVP);
		   	    if (avp==null) return;
		   	   
		   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("sRES").getType().newInstance();
			   
			    decode_AVPHeader( rv_ed ,avp);
			    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
			    
			    rv_.setField("sRES",rv_ed);
			    return;
			}
		    
		    private void decode_Kc_AVP(RecordValue rv_,	AVP avp_){
				
		    	AVP avp=avp_.findChildAVP(AVP_Kc_AVP);
		   	    if (avp==null) return;
		   	   
		   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_.getField("kc").getType().newInstance();
			   
			    decode_AVPHeader( rv_ed ,avp);
			    rv_ed.setField("aVP_Data", mkOctetstring(avp.data));
			    
			    rv_.setField("kc",rv_ed);
			    return;
			}
	    
	    private void decode_GERAN_Vector_AVP(RecordValue rv_,	AVP avp_){
			
	    	AVP[] avps=avp_.findChildAVPs(AVP_GERAN_Vector_AVP);
	   	    if (avps==null) return;
	   	   
	   	    RecordOfValue rv_ed = (RecordOfValue) rv_.getField("gERAN_Vector").getType().newInstance();
		    rv_ed.setLength(0);
	   	    
		    for (int i = 0; i < avps.length; i++) {
			    RecordValue rv = (RecordValue) rv_ed.getElementType().newInstance();
	        	
			    try{avps[i].ungroup();}catch(Exception e){}
		   	    
			    decode_AVPHeader( rv ,avps[i]);                                        //ref: 7.3.18
		        decode_Item_Number_AVP( rv ,avps[i]);//                       item_Number                        optional,
		        decode_RAND_AVP( rv ,avps[i]);       //                     rAND,
		        decode_SRES_AVP( rv ,avps[i]);    
		        decode_Kc_AVP( rv ,avps[i]);  
		        //set of AVP_Type                     aVP_Type                        optional
	     	
		        rv_ed.appendField(rv);
			} 
	        rv_.setField("gERAN_Vector",rv_ed);
	        return;
	    }
	    
	    
    private void decode_Authentication_Info_AVP(RecordValue rv_ufd,	DiameterMessage uf){                                    //ref: 7.3.17
        
    	AVP avp=uf.findAVP(AVP_Authentication_Info_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_ufd.getField("authentication_Info").getType().newInstance();
	   
   	    try{avp.ungroup();}catch(Exception e){}
	    
   	    decode_AVPHeader( rv_ed ,avp);
        decode_E_UTRAN_Vector_AVP(rv_ed,avp);  //       e_UTRAN_Vector                    optional,
        decode_UTRAN_Vector_AVP(rv_ed,avp);     //    uTRAN_Vector                    optional,
        decode_GERAN_Vector_AVP(rv_ed,avp);     //    gERAN_Vector                    optional,
        //set of decode_AVP_Type                     aVP_Type                        optional
        
        rv_ufd.setField("authentication_Info",rv_ed);
        return;
    }

    
    private void decode_Cancellation_Type_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	String s_ct="";
    	AVP avp=uf.findAVP(AVP_Cancellation_Type_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_ufd.getField("cancellation_Type").getType().newInstance();
	   
	    decode_AVPHeader( rv_ed ,avp);
	    rv_ed.setField("cancellation_Type", mkInt(avp.int_data));
	    
	    /*switch(avp.int_data){
	           
	      case 0   : s_ct="MME_UPDATE_PROCEDURE_E";break;       
	      case 1   : s_ct="SGSN_UPDATE_PROCEDURE_E";break;    
	      case 2   : s_ct="SUBSCRIPTION_WITHDRAWAL_E";break;                
	      case 3   : s_ct="UPDATE_PROCEDURE_IWF_E";break;              
	      case 4   : s_ct="INITIAL_ATTACH_PROCEDURE_E";break;      

	    }
	        
	    EnumeratedValue e_ct= (EnumeratedValue)rv_ed.getField("cancellation_Type");
	    e_ct.setEnum(s_ct);
	    rv_ed.setField("cancellation_Type", e_ct);*/
	    
	    rv_ufd.setField("cancellation_Type",rv_ed);
    	return;
    }
    
    private void decode_CLR_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_CLR_Flags_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("cLR_Flags").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("cLR_Flags",rv_uf);
    	return;
    }
    
    private void decode_IDR_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_IDR_Flags_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("iDR_Flags").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("iDR_Flags",rv_uf);
    	return;
    }
    
    private void decode_DSR_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_DSR_Flags_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("dSR_Flags").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("dSR_Flags",rv_uf);
    	return;
    }
    
    private void decode_DSA_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_DSA_Flags_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("dSA_Flags").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("dSA_Flags",rv_uf);
    	return;
    }
    
    private void decode_PUER_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_PUER_Flags_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("pUER_Flags").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("pUER_Flags",rv_uf);
    	return;
    }
    
    private void decode_PUEA_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_PUEA_Flags_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("pUEA_Flags").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("pUEA_Flags",rv_uf);
    	return;
    }
    
    private void decode_IDA_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_IDA_Flags_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("iDA_Flags").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("iDA_Flags",rv_uf);
    	return;
    }
    
    private void decode_NOR_Flags_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_NOR_Flags_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("nOR_Flags").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("nOR_Flags",rv_uf);
    	return;
    }
    
    private void decode_IMS_Voice_PS_Sessions_Support_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	String s_ct="";
    	AVP avp=uf.findAVP(AVP_IMS_Voice_PS_Sessions_Support_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_ufd.getField("iMS_Voice_PS_Sessions_Support").getType().newInstance();
	   
	    decode_AVPHeader( rv_ed ,avp);
	    rv_ed.setField("iMS_Voice_Over_PS_Sessions_Supported", mkInt(avp.int_data));

	    /*switch(avp.int_data){
	           
	      case 0   : s_ct="NOT_SUPPORTED_E";break;       
	      case 1   : s_ct="SUPPORTED_E";break;    
	    }
	        
	    EnumeratedValue e_ct= (EnumeratedValue)rv_ed.getField("iMS_Voice_Over_PS_Sessions_Supported");
	    e_ct.setEnum(s_ct);
	    rv_ed.setField("iMS_Voice_Over_PS_Sessions_Supported", e_ct);*/
	    
	    rv_ufd.setField("iMS_Voice_PS_Sessions_Support",rv_ed);
    	return;
    }
    
    private void decode_Last_UE_Activity_Time_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_Last_UE_Activity_Time_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_uf = (RecordValue) (RecordValue) rv_ufd.getField("last_UE_Activity_Time_AVP").getType().newInstance();
	   
	    decode_AVPHeader( rv_uf ,avp);
        rv_uf.setField("aVP_Data", mkInt(avp.int_data));
	    
	    rv_ufd.setField("last_UE_Activity_Time_AVP",rv_uf);
    	return;
    }
    
    private void decode_EPS_User_State_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_EPS_User_State_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_ufd.getField("ePS_User_State").getType().newInstance();
        
   	    try{avp.ungroup();}catch(Exception e){}

    	decode_AVPHeader( rv_ed ,avp);
 //TODO:   	
        //MME_User_State_AVP                    mME_User_State                    optional,
        //SGSN_User_State_AVP                    sGSN_User_State                    optional,
        //set of AVP_Type                     aVP_Type                        optional
	    
	    rv_ufd.setField("ePS_User_State",rv_ed);
    	return;
    }
    
    private void decode_EPS_Location_Information_AVP(RecordValue rv_ufd,	DiameterMessage uf){
    	
    	AVP avp=uf.findAVP(AVP_EPS_Location_Information_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_ufd.getField("ePS_Location_Information").getType().newInstance();
       
   	    try{avp.ungroup();}catch(Exception e){}

    	decode_AVPHeader( rv_ed ,avp);
 //TODO:   	
    	 //MME_Location_Information_AVP        mME_Location_Information        optional,
         //SGSN_Location_Information_AVP        sGSN_Location_Information        optional,
        //set of AVP_Type                     aVP_Type                        optional
	    
	    rv_ufd.setField("ePS_Location_Information",rv_ed);
    	return;
    }    
    //*End TS129_272 AVP codes
    
    //TS 129 299
	    private void decode_Primary_Event_Charging_Function_Name_AVP(RecordValue rv_sid,	AVP sid){
	    	
	       AVP avp=sid.findChildAVP(AVP_Primary_Event_Charging_Function_Name_AVP);
	  	   if (avp==null) return;
	      	
	  	   RecordValue rv_ = (RecordValue) (RecordValue) rv_sid.getField("primary_Event_Charging_Function_Name").getType().newInstance();
	  	   
	  	   decode_AVPHeader( rv_ ,avp);
	  	   //decode_DiameterURI(rv_,avp);
	  	   rv_.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
	  	   
	  	   rv_sid.setField("primary_Event_Charging_Function_Name",rv_);
	  	   return;
	   	}
	    
	    private void decode_Secondary_Event_Charging_Function_Name_AVP(RecordValue rv_sid,	AVP sid){
	    	
	       AVP avp=sid.findChildAVP(AVP_Secondary_Event_Charging_Function_Name_AVP);
	  	   if (avp==null) return;
	      	
	  	   RecordValue rv_ = (RecordValue) (RecordValue) rv_sid.getField("secondary_Event_Charging_Function_Name").getType().newInstance();
	  	   
	  	   decode_AVPHeader( rv_ ,avp);
	  	   //decode_DiameterURI(rv_,avp);
	  	   rv_.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
	  	   
	  	   rv_sid.setField("secondary_Event_Charging_Function_Name",rv_);
	  	   return;
	   	}
    
	    private void decode_Primary_Charging_Collection_Function_Name_AVP(RecordValue rv_sid,	AVP sid){
	    	
	       AVP avp=sid.findChildAVP(AVP_Primary_Charging_Collection_Function_Name_AVP);
	  	   if (avp==null) return;
	      	
	  	   RecordValue rv_ = (RecordValue) (RecordValue) rv_sid.getField("primary_Charging_Collection_Function_Name").getType().newInstance();
	  	   
	  	   decode_AVPHeader( rv_ ,avp);
	  	   //decode_DiameterURI(rv_,avp);
	  	   rv_.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
	  	   
	  	   rv_sid.setField("primary_Charging_Collection_Function_Name",rv_);
	  	   return;
	   	}
	    
	    private void decode_Secondary_Charging_Collection_Function_Name_AVP(RecordValue rv_sid,	AVP sid){
	    	
	       AVP avp=sid.findChildAVP(AVP_Secondary_Charging_Collection_Function_Name_AVP);
	  	   if (avp==null) return;
	      	
	  	   RecordValue rv_ = (RecordValue) (RecordValue) rv_sid.getField("secondary_Charging_Collection_Function_Name").getType().newInstance();
	  	   
	  	   decode_AVPHeader( rv_ ,avp);
	  	   //decode_DiameterURI(rv_,avp);
	  	   rv_.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
	  	   
	  	   rv_sid.setField("secondary_Charging_Collection_Function_Name",rv_);
	  	   return;
	   	}
	    
    private void decode_Charging_Information_AVP(RecordValue rv_ufd,	DiameterMessage uf){

    	AVP avp=uf.findAVP(AVP_Charging_Information_AVP);
   	    if (avp==null) return;
   	   
   	    RecordValue rv_ed = (RecordValue) (RecordValue) rv_ufd.getField("charging_Information").getType().newInstance();
	   
        try{avp.ungroup();}catch(Exception e){}

    	decode_AVPHeader( rv_ed ,avp);  	
    	decode_Primary_Event_Charging_Function_Name_AVP(rv_ed ,avp);// optional,
    	decode_Secondary_Event_Charging_Function_Name_AVP(rv_ed ,avp);// optional,
    	decode_Primary_Charging_Collection_Function_Name_AVP(rv_ed ,avp);// optional,
    	decode_Secondary_Charging_Collection_Function_Name_AVP(rv_ed ,avp);// optional,
        //set of AVP_Type
	    
	    rv_ufd.setField("charging_Information",rv_ed);
    	return;
    }
    //*End TS129_299 AVP codes
    
    /*TS132_299*/
    		private void decode_Tariff_Time_Change_AVP(RecordValue rv_sid,	AVP sid){
    	    	
 		       AVP avp=sid.findChildAVP(AVP_Tariff_Time_Change_AVP);
 		  	   if (avp==null) return;
 		      	
 		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("tariff_Time_Change").getType().newInstance();
 			    
 		  	   decode_AVPHeader( rv_ ,avp);
 		  	   rv_.setField("aVP_Data", mkInt(avp.int_data));
 		  	   
 		  	   rv_sid.setField("tariff_Time_Change",rv_);
		  	   return;
		    }
    		
    		private void decode_CC_Time_AVP(RecordValue rv_sid,	AVP sid){
    	    	
 		       AVP avp=sid.findChildAVP(AVP_CC_Time_AVP);
 		  	   if (avp==null) return;
 		      	
 		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("cC_Time").getType().newInstance();
 			    
 		  	   decode_AVPHeader( rv_ ,avp);
 		  	   rv_.setField("aVP_Data", mkInt(avp.int_data));
 		  	   
 		  	   rv_sid.setField("cC_Time",rv_);
		  	   return;
		    } 
    		
	    		private void decode_Value_Digits_AVP(RecordValue rv_sid,	AVP sid){
	    	    	
	  		       AVP avp=sid.findChildAVP(AVP_Value_Digits_AVP);
	  		  	   if (avp==null) return;
	  		      	
	  		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("value_Digits").getType().newInstance();
	  			    
	  		  	   decode_AVPHeader( rv_ ,avp);
	  		  	   rv_.setField("aVP_Data", mkInt64(avp.int_data));
	  		  	   
	  		  	   rv_sid.setField("value_Digits",rv_);
	 		  	   return;
	 		    }
	    		
	    		private void decode_Exponent_AVP(RecordValue rv_sid,	AVP sid){
	    	    	
		  		       AVP avp=sid.findChildAVP(AVP_Exponent_AVP);
		  		  	   if (avp==null) return;
		  		      	
		  		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("exponent").getType().newInstance();
		  			    
		  		  	   decode_AVPHeader( rv_ ,avp);
		  		  	   rv_.setField("aVP_Data", mkInt(avp.int_data));
		  		  	   
		  		  	   rv_sid.setField("exponent",rv_);
		 		  	   return;
		 		    }
    		
    		private void decode_Unit_Value_AVP(RecordValue rv_sid,	AVP sid){
    	    	
  		       AVP avp=sid.findChildAVP(AVP_Unit_Value_AVP);
  		  	   if (avp==null) return;
  		      	
  		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("unit_Value").getType().newInstance();
  			   
  		  	   try{avp.ungroup();}catch(Exception e){}
  		  	
  		  	   decode_AVPHeader( rv_ ,avp);
  		  	   decode_Value_Digits_AVP( rv_ ,avp); //            value_Digits,
  		       decode_Exponent_AVP( rv_ ,avp);     //            exponent optional
  		  	   
  		  	   rv_sid.setField("unit_Value",rv_);
 		  	   return;
 		    }
    		
    		private void decode_Currency_Code_AVP(RecordValue rv_sid,	AVP sid){
    	    	
  		       AVP avp=sid.findChildAVP(AVP_Currency_Code_AVP);
  		  	   if (avp==null) return;
  		      	
  		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("currency_Code").getType().newInstance();
  			    
  		  	   decode_AVPHeader( rv_ ,avp);
  		  	   rv_.setField("aVP_Data", mkInt(avp.int_data));
  		  	   
  		  	   rv_sid.setField("currency_Code",rv_);
 		  	   return;
 		    }
    		
    		private void decode_CC_Money_AVP(RecordValue rv_sid,	AVP sid){
    	    	
 		       AVP avp=sid.findChildAVP(AVP_CC_Money_AVP);
 		  	   if (avp==null) return;
 		      	
 		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("cC_Money").getType().newInstance();
 			   
 		  	   try{avp.ungroup();}catch(Exception e){}
 		  	   
 		  	   decode_AVPHeader( rv_ ,avp);
 		  	   decode_Unit_Value_AVP( rv_ ,avp);//                unit_Value,
 		  	   decode_Currency_Code_AVP( rv_ ,avp);//             currency_Code optional
 		  	   
 		  	   rv_sid.setField("cC_Money",rv_);
		  	   return;
		    }
    		
    		private void decode_CC_Total_Octets_AVP(RecordValue rv_sid,	AVP sid){
    	    	
  		       AVP avp=sid.findChildAVP(AVP_CC_Total_Octets_AVP);
  		  	   if (avp==null) return;
  		      	
  		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("cC_Total_Octets").getType().newInstance();
  			    
  		  	   decode_AVPHeader( rv_ ,avp);
  		  	   rv_.setField("aVP_Data", mkInt64(avp.int_data));
  		  	   
  		  	   rv_sid.setField("cC_Total_Octets",rv_);
 		  	   return;
 		    }
    		
    		private void decode_CC_Input_Octets_AVP(RecordValue rv_sid,	AVP sid){
    	    	
   		       AVP avp=sid.findChildAVP(AVP_CC_Input_Octets_AVP);
   		  	   if (avp==null) return;
   		      	
   		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("cC_Input_Octets").getType().newInstance();
   			    
   		  	   decode_AVPHeader( rv_ ,avp);
   		  	   rv_.setField("aVP_Data", mkInt64(avp.int_data));
   		  	   
   		  	   rv_sid.setField("cC_Input_Octets",rv_);
  		  	   return;
  		    }
    		
    		private void decode_CC_Output_Octets_AVP(RecordValue rv_sid,	AVP sid){
    	    	
    		       AVP avp=sid.findChildAVP(AVP_CC_Output_Octets_AVP);
    		  	   if (avp==null) return;
    		      	
    		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("cC_Output_Octets").getType().newInstance();
    			    
    		  	   decode_AVPHeader( rv_ ,avp);
    		  	   rv_.setField("aVP_Data", mkInt64(avp.int_data));
    		  	   
    		  	   rv_sid.setField("cC_Output_Octets",rv_);
   		  	   return;
   		    }
    		
    		private void decode_CC_Service_Specific_Units_AVP(RecordValue rv_sid,	AVP sid){
    	    	
 		       AVP avp=sid.findChildAVP(AVP_CC_Service_Specific_Units_AVP);
 		  	   if (avp==null) return;
 		      	
 		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("cC_Service_Specific_Units").getType().newInstance();
 			    
 		  	   decode_AVPHeader( rv_ ,avp);
 		  	   rv_.setField("aVP_Data", mkInt64(avp.int_data));
 		  	   
 		  	   rv_sid.setField("cC_Service_Specific_Units",rv_);
		  	   return;
		    }
 		  	   
    		private void decode_Granted_Service_Unit_AVP(RecordValue rv_sid,	AVP sid){
	    	
		       AVP avp=sid.findChildAVP(AVP_Granted_Service_Unit_AVP);
		  	   if (avp==null) return;
		      	
		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("granted_Service_Unit").getType().newInstance();
			   
		  	   try{avp.ungroup();}catch(Exception e){}
		  	 
		  	   decode_AVPHeader( rv_ ,avp);
		  	   decode_Tariff_Time_Change_AVP( rv_ ,avp);//         tariff_Time_Change optional,
		  	   decode_CC_Time_AVP( rv_ ,avp);//                 cC_Time optional,
		  	   decode_CC_Money_AVP( rv_ ,avp);//                 cC_Money optional,
		  	   decode_CC_Total_Octets_AVP( rv_ ,avp);//         cC_Total_Octets optional,
		  	   decode_CC_Input_Octets_AVP( rv_ ,avp);//         cC_Input_Octets optional,
		  	   decode_CC_Output_Octets_AVP( rv_ ,avp);//         cC_Output_Octets optional,
		  	   decode_CC_Service_Specific_Units_AVP( rv_ ,avp);//     cC_Service_Specific_Units optional,
		  	   //decode_AVP_Type( rv_ ,avp);//                 aVP_Type optional
		  	   
		  	   rv_sid.setField("granted_Service_Unit",rv_);
		  	   return;
		    }
    	
    		private void decode_Requested_Service_Unit_AVP(RecordValue rv_sid,	AVP sid){
	    	
		       AVP avp=sid.findChildAVP(AVP_Requested_Service_Unit_AVP);
		  	   if (avp==null) return;
		      	
		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("requested_Service_Unit").getType().newInstance();
		  	   
		  	   try{avp.ungroup();}catch(Exception e){} 
		  	   
		  	   decode_AVPHeader( rv_ ,avp);
		  	   decode_CC_Time_AVP( rv_ ,avp);//                 cC_Time optional,
		  	   decode_CC_Money_AVP( rv_ ,avp);//                 cC_Money optional,
		  	   decode_CC_Total_Octets_AVP( rv_ ,avp);//         cC_Total_Octets optional,
		  	   decode_CC_Input_Octets_AVP( rv_ ,avp);//         cC_Input_Octets optional,
		  	   decode_CC_Output_Octets_AVP( rv_ ,avp);//         cC_Output_Octets optional,
		  	   decode_CC_Service_Specific_Units_AVP( rv_ ,avp);//     cC_Service_Specific_Units optional,
		  	   //decode_AVP_Type( rv_ ,avp);//                 aVP_Type optional
		  	   
		  	   rv_sid.setField("requested_Service_Unit",rv_);
		  	   return;
		    }
    		
    		private void decode_Requested_Service_Unit_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	    	
 		       AVP avp=msg.findAVP(AVP_Requested_Service_Unit_AVP);
 		  	   if (avp==null) return;
 		      	
 		  	   RecordValue rv_ = (RecordValue)rv_DiameterM.getField("requested_Service_Unit").getType().newInstance();
 			   
 		  	   try{avp.ungroup();}catch(Exception e){}
 		  	   
 		  	   decode_AVPHeader( rv_ ,avp);
 		  	   decode_CC_Time_AVP( rv_ ,avp);//                 cC_Time optional,
 		  	   decode_CC_Money_AVP( rv_ ,avp);//                 cC_Money optional,
 		  	   decode_CC_Total_Octets_AVP( rv_ ,avp);//         cC_Total_Octets optional,
 		  	   decode_CC_Input_Octets_AVP( rv_ ,avp);//         cC_Input_Octets optional,
 		  	   decode_CC_Output_Octets_AVP( rv_ ,avp);//         cC_Output_Octets optional,
 		  	   decode_CC_Service_Specific_Units_AVP( rv_ ,avp);//     cC_Service_Specific_Units optional,
 		  	   //decode_AVP_Type( rv_ ,avp);//                 aVP_Type optional
 		  	   
 		  	   rv_DiameterM.setField("requested_Service_Unit",rv_);
 		  	   return;
 		    }
    		
    		private void decode_Reporting_Reason_AVP(RecordValue rv_sid,	AVP sid){
    	    			  	   
	  		  	boolean rv_present=false;
	  	        AVP avps[]=sid.findChildAVPs(AVP_Reporting_Reason_AVP);
	  	        if (avps==null) return;
	  	         
	  	        if (avps.length>=1){
	  	        	try{
	  	 			   RecordValue rv_ = (RecordValue) rv_sid.getField("reporting_Reason").getType().newInstance();
	  	 			   
	  	 			   decode_AVPHeader( rv_ ,avps[0]);
	  	 		       rv_.setField("aVP_Data",  mkInt(avps[0].int_data));
	
	  	 		       rv_sid.setField("reporting_Reason",rv_);
	  	 			   rv_present=true;
	  	 			   return;
	  	 			}
	  	 			catch(Exception ex){
	  	 				//System.out.print(ex.getMessage());
	  	 			}
	  	 			if(!rv_present){
	  					RecordOfValue recOfrr = (RecordOfValue) rv_sid.getField("reporting_Reason").getType().newInstance();
	  					recOfrr.setLength(0);
	  				
	  					for (int i = 0; i < avps.length; i++) {
	  					    RecordValue rv = (RecordValue) recOfrr.getElementType().newInstance();
	  			        	
	  					    decode_AVPHeader( rv ,avps[i]);
	  					    rv.setField("aVP_Data", mkInt(avps[i].int_data));	
	  			     	
	  			    	    recOfrr.appendField(rv);
	  					}
	  					rv_sid.setField("reporting_Reason",recOfrr);
	  	 			}
	  	        }
	  	        return;    
  		    }           
             
    		 private void decode_Trigger_Type_AVP(RecordValue rv_sid,	AVP sid){
    		   
    		   AVP[] avps=sid.findChildAVPs(AVP_Trigger_Type_AVP);
  		  	   if (avps==null){ 
  		  		   
  		  		   RecordOfValue rov_ = (RecordOfValue)rv_sid.getField("trigger_Type").getType().newInstance();
  			       rov_.setLength(0);
  			       rv_sid.setField("trigger_Type",rov_);
  		  		   return;
  		  	   }
  		  	   
  		  	   RecordOfValue rov_ = (RecordOfValue)rv_sid.getField("trigger_Type").getType().newInstance();
  			   rov_.setLength(0);
  			   for(int i=0;i<avps.length;i++){
  			   
  			       RecordValue rvin= (RecordValue) rov_.getElementType().newInstance();
  		  	       
  			       decode_AVPHeader( rvin ,avps[i]);
  			       rvin.setField("aVP_Data", mkInt(avps[i].int_data));
  		  	       rov_.appendField(rvin);
  			   }
  			   rv_sid.setField("trigger_Type",rov_);
    		 }
    		 
	         private void decode_Trigger_AVP(RecordValue rv_sid,	AVP sid){
		    	
		       AVP avp=sid.findChildAVP(AVP_Trigger_AVP);
		  	   if (avp==null) return;
		  	   
		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("trigger_").getType().newInstance();
			   
			   try{avp.ungroup();}catch(Exception e){}

		  	   decode_AVPHeader( rv_ ,avp);
		  	   decode_Trigger_Type_AVP( rv_ ,avp);
			   
		  	   rv_sid.setField("trigger_",rv_);
		  	   return;
  		    }
	         
	         private void decode_Refund_Information_AVP(RecordValue rv_sid,	AVP sid){
	    	    	
	   		       AVP avp=sid.findChildAVP(AVP_Refund_Information_AVP);
	   		  	   if (avp==null) return;
	   		      	
	   		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("refund_Information").getType().newInstance();
	   			    
	   		  	   decode_AVPHeader( rv_ ,avp);
	   		  	   rv_.setField("aVP_Data", mkOctetstring(avp.data));
	   		  	   
	   		  	   rv_sid.setField("refund_Information",rv_);
	  		  	   return;
	  		    }
	       
    		
    		private void decode_Tariff_Change_Usage_AVP(RecordValue rv_sid,	AVP sid){
    	    	
  		       AVP avp=sid.findChildAVP(AVP_Tariff_Change_Usage_AVP);
  		  	   if (avp==null) return;
  		      	
  		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("tariff_Change_Usage").getType().newInstance();
  			    
  		  	   decode_AVPHeader( rv_ ,avp);
  		  	   rv_.setField("aVP_Data", mkInt(avp.int_data));
  		  	   
  		  	   rv_sid.setField("tariff_Change_Usage",rv_);
 		  	   return;
 		    }
    		
    		private void decode_Used_Service_Unit_AVP(RecordValue rv_sid,	AVP sid){
    	    	
 		       AVP[] avps=sid.findChildAVPs(AVP_Used_Service_Unit_AVP);
 		  	   if (avps==null) return;
 		      	
 		  	   RecordOfValue rov_ = (RecordOfValue)rv_sid.getField("used_Service_Unit").getType().newInstance();
 			   rov_.setLength(0);
 		  	   
 			   for(int i=0;i<avps.length;i++){
			    	
			    RecordValue rv_ = (RecordValue) rov_.getElementType().newInstance();
		        
			    try{avps[i].ungroup();}catch(Exception e){}
 			   
 		  	    decode_AVPHeader( rv_ ,avps[i]);
 		  	    decode_Reporting_Reason_AVP( rv_ ,avps[i]);//        reporting_Reason optional, // TS 132 299
 		  	    decode_Tariff_Change_Usage_AVP( rv_ ,avps[i]);//     tariff_Change_Usage optional,
 		  	    //decode_Tariff_Time_Change_AVP( rv_ ,avps[i]);//         tariff_Time_Change optional,
 		  	    decode_CC_Time_AVP( rv_ ,avps[i]);//                 cC_Time optional,
 		  	    decode_CC_Money_AVP( rv_ ,avps[i]);//                 cC_Money optional,
 		  	    decode_CC_Total_Octets_AVP( rv_ ,avps[i]);//         cC_Total_Octets optional,
 		  	    decode_CC_Input_Octets_AVP( rv_ ,avps[i]);//         cC_Input_Octets optional,
 		  	    decode_CC_Output_Octets_AVP( rv_ ,avps[i]);//         cC_Output_Octets optional,
 		  	    decode_CC_Service_Specific_Units_AVP( rv_ ,avps[i]);//     cC_Service_Specific_Units optional,
 		  	    //decode_Event_Charging_TimeStamp_AVP( rv_ ,avps[i]);//      event_Charging_TimeStamp optional,
 		  	    //decode_AVP_Type( rv_ ,avps[i]);//                 aVP_Type optional
 		  	   
 		  	    rov_.appendField(rv_);
            
 			   }
 		  	   
 		  	   rv_sid.setField("used_Service_Unit",rov_);
 		  	   return;
 		    }
    		
    		private void decode_Used_Service_Unit_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	    	
  		       AVP[] avps=msg.findAVPs(AVP_Used_Service_Unit_AVP);
  		  	   if (avps==null) return;
  		      	
  		  	   RecordOfValue rov_ = (RecordOfValue)rv_DiameterM.getField("used_Service_Unit").getType().newInstance();
  			   rov_.setLength(0);
  		  	   
  			   for(int i=0;i<avps.length;i++){
 			    	
 			    RecordValue rv_ = (RecordValue) rov_.getElementType().newInstance();
 		        
 			    try{avps[i].ungroup();}catch(Exception e){}
  			   
  		  	    decode_AVPHeader( rv_ ,avps[i]);
  		  	    decode_Reporting_Reason_AVP( rv_ ,avps[i]);//        reporting_Reason optional, // TS 132 299
  		  	    decode_Tariff_Change_Usage_AVP( rv_ ,avps[i]);//     tariff_Change_Usage optional,
  		  	    //decode_Tariff_Time_Change_AVP( rv_ ,avps[i]);//         tariff_Time_Change optional,
  		  	    decode_CC_Time_AVP( rv_ ,avps[i]);//                 cC_Time optional,
  		  	    decode_CC_Money_AVP( rv_ ,avps[i]);//                 cC_Money optional,
  		  	    decode_CC_Total_Octets_AVP( rv_ ,avps[i]);//         cC_Total_Octets optional,
  		  	    decode_CC_Input_Octets_AVP( rv_ ,avps[i]);//         cC_Input_Octets optional,
  		  	    decode_CC_Output_Octets_AVP( rv_ ,avps[i]);//         cC_Output_Octets optional,
  		  	    decode_CC_Service_Specific_Units_AVP( rv_ ,avps[i]);//     cC_Service_Specific_Units optional,
  		  	    //decode_Event_Charging_TimeStamp_AVP( rv_ ,avps[i]);//      event_Charging_TimeStamp optional,
  		  	    //decode_AVP_Type( rv_ ,avps[i]);//                 aVP_Type optional
  		  	   
  		  	    rov_.appendField(rv_);
  			   }
  		  	   
  			   rv_DiameterM.setField("used_Service_Unit",rov_);
  		  	   return;
  		    }
    		
    		
    		private void decode_Service_Identifier_AVP(RecordValue rv_sid,	AVP sid){
		    	
  		       AVP[] avps=sid.findChildAVPs(AVP_Service_Identifier_AVP);
  		  	   if (avps==null) return;
  		      	
  		  	   RecordOfValue rov_ = (RecordOfValue)rv_sid.getField("service_Identifier").getType().newInstance();
  			   rov_.setLength(0);
  		  	   
  			   for(int i=0;i<avps.length;i++){
			    	
 			    RecordValue rv_ = (RecordValue) rov_.getElementType().newInstance();
 		        
 			    //try{avps[i].ungroup();}catch(Exception e){}
  			   
  		  	    decode_AVPHeader( rv_ ,avps[i]);
  		  	    rv_.setField("aVP_Data", mkInt(avps[i].int_data));
  			   
  			    rov_.appendField(rv_);
  			   }
  			   
  		  	   rv_sid.setField("service_Identifier",rov_);
  		  	   return;
  		    }
    		
    		private void decode_Rating_Group_AVP(RecordValue rv_sid,	AVP sid){
		    	
 		       AVP avp=sid.findChildAVP(AVP_Rating_Group_AVP);
 		  	   if (avp==null) return;
 		      	
 		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("rating_Group").getType().newInstance();
 			    
 		  	   decode_AVPHeader( rv_ ,avp);
 		       rv_.setField("aVP_Data", mkInt(avp.int_data));
 		  	   
 		  	   rv_sid.setField("rating_Group",rv_);
 		  	   return;
 		    }
    		
	    		private void decode_G_S_U_Pool_Identifier_AVP(RecordValue rv_sid,	AVP sid){
			    	
	  		       AVP avp=sid.findChildAVP(AVP_G_S_U_Pool_Identifier_AVP);
	  		  	   if (avp==null) return;
	  		      	
	  		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("g_S_U_Pool_Identifier").getType().newInstance();
	  			    
	  		  	   decode_AVPHeader( rv_ ,avp);
	  		       rv_.setField("aVP_Data", mkInt(avp.int_data));
	  		  	   
	  		  	   rv_sid.setField("g_S_U_Pool_Identifier",rv_);
	  		  	   return;
	  		    }
	    		
	    		private void decode_CC_Unit_Type_AVP(RecordValue rv_sid,	AVP sid){
			    	
	  		       AVP avp=sid.findChildAVP(AVP_CC_Unit_Type_AVP);
	  		  	   if (avp==null) return;
	  		      	
	  		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("cC_Unit_Type").getType().newInstance();
	  			    
	  		  	   decode_AVPHeader( rv_ ,avp);
	  		       rv_.setField("aVP_Data", mkInt(avp.int_data));
	  		  	   
	  		  	   rv_sid.setField("cC_Unit_Type",rv_);
	  		  	   return;
	  		    }
    		
    		private void decode_G_S_U_Pool_Reference_AVP(RecordValue rv_sid,	AVP sid){
		    	
   		       AVP[] avps=sid.findChildAVPs(AVP_G_S_U_Pool_Reference_AVP);
   		  	   if (avps==null) return;
   		      	
   		  	   RecordOfValue rov_ = (RecordOfValue)rv_sid.getField("g_S_U_Pool_Reference").getType().newInstance();
   			   rov_.setLength(0);
   		  	   
   			   for(int i=0;i<avps.length;i++){
 			    	
  			    RecordValue rv_ = (RecordValue) rov_.getElementType().newInstance();
  		        
  			    try{avps[i].ungroup();}catch(Exception e){}
   			   
   		  	    decode_AVPHeader( rv_ ,avps[i]);
   		  	    decode_G_S_U_Pool_Identifier_AVP( rv_ ,avps[i]);//    g_S_U_Pool_Identifier,
   		  	    decode_CC_Unit_Type_AVP( rv_ ,avps[i]);//            cC_Unit_Type,
   		  	    decode_Unit_Value_AVP( rv_ ,avps[i]);//                unit_Value
   			   
   			    rov_.appendField(rv_);
   			   }
   			   
   		  	   rv_sid.setField("g_S_U_Pool_Reference",rov_);
   		  	   return;
   		    }             
           
    		private void decode_Final_Unit_Action_AVP(RecordValue rv_sid,	AVP sid){
		    	
  		       AVP avp=sid.findChildAVP(AVP_Final_Unit_Action_AVP);
  		  	   if (avp==null) return;
  		      	
  		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("final_Unit_Action").getType().newInstance();
  			    
  		  	   decode_AVPHeader( rv_ ,avp);
  		       rv_.setField("aVP_Data", mkInt(avp.int_data));
  		  	   
  		  	   rv_sid.setField("final_Unit_Action",rv_);
  		  	   return;
  		    }
    		
    		private void decode_Restriction_Filter_Rule_AVP(RecordValue rv_sid,	AVP sid){
		    	
		       AVP[] avps=sid.findChildAVPs(AVP_Restriction_Filter_Rule_AVP);
		  	   if (avps==null) return;
		      	
		  	   RecordOfValue rov_ = (RecordOfValue)rv_sid.getField("restriction_Filter_Rule").getType().newInstance();
			   rov_.setLength(0);
			   
		  	   for(int i=0;i<avps.length;i++){
			    	
	  			    RecordValue rv_ = (RecordValue) rov_.getElementType().newInstance();
	  		        
	  			    try{avps[i].ungroup();}catch(Exception e){}
	   			   
	   		  	    decode_AVPHeader( rv_ ,avps[i]);
	   		  	    //decode_IPFilterRule_Type( rv_ ,avps[i]);
	   		  	    
	   		  	    rov_.appendField(rv_);
		  	   }    
		  	   
		  	   rv_sid.setField("restriction_Filter_Rule",rov_);
		  	   return;
		    }
    		
    		private void decode_Filter_Id_AVP(RecordValue rv_sid,	AVP sid){
		    	
   		       AVP[] avps=sid.findChildAVPs(AVP_Filter_Id_AVP);
   		  	   if (avps==null) return;
   		      	
   		  	   RecordOfValue rov_ = (RecordOfValue)rv_sid.getField("filter_Id").getType().newInstance();
   			   rov_.setLength(0);
   			   
   			   for(int i=0;i<avps.length;i++){
		    	
  			    RecordValue rv_ = (RecordValue) rov_.getElementType().newInstance();
  		        
  			    //try{avps[i].ungroup();}catch(Exception e){}
   			   
   		  	    decode_AVPHeader( rv_ ,avps[i]);
   		  	    rv_.setField("aVP_Data", mkCvUTF8(avps[i].getStringData()));
   		  	    
   		  	    rov_.appendField(rv_);
   			   	}    		  	   
   		  	   rv_sid.setField("filter_Id",rov_);
   		  	   return;
   		    }
    		
	    		private void decode_Redirect_Address_Type_AVP(RecordValue rv_sid,	AVP sid){
				    	
	   		       AVP avp=sid.findChildAVP(AVP_Redirect_Address_Type_AVP);
	   		  	   if (avp==null) return;
	   		      	
	   		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("redirect_Address_Type").getType().newInstance();
	   			    
	   		  	   decode_AVPHeader( rv_ ,avp);
	   		       rv_.setField("aVP_Data", mkInt(avp.int_data));
	   		  	   
	   		  	   rv_sid.setField("redirect_Address_Type",rv_);
	   		  	   return;
	   		    }
	    		
	    		private void decode_Redirect_Server_Address_AVP(RecordValue rv_sid,	AVP sid){
			    	
	   		       AVP avp=sid.findChildAVP(AVP_Redirect_Server_Address_AVP);
	   		  	   if (avp==null) return;
	   		      	
	   		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("redirect_Server_Address").getType().newInstance();
	   			    
	   		  	   decode_AVPHeader( rv_ ,avp);
	   		       rv_.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
	   		  	   
	   		  	   rv_sid.setField("redirect_Server_Address",rv_);
	   		  	   return;
	   		    }
    		   	                
            private void decode_Redirect_Server_AVP(RecordValue rv_sid,	AVP sid){
		    	
   		       AVP avp=sid.findChildAVP(AVP_Redirect_Server_AVP);
   		  	   if (avp==null) return;
   		      	
   		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("redirect_Server").getType().newInstance();
   			    
   		  	   try{avp.ungroup();}catch(Exception e){}
   		  	   
   		  	   decode_AVPHeader( rv_ ,avp);
   		  	   decode_Redirect_Address_Type_AVP( rv_ ,avp);//     redirect_Address_Type,
   		  	   decode_Redirect_Server_Address_AVP( rv_ ,avp);// redirect_Server_Address
   		  	   
   		  	   rv_sid.setField("redirect_Server",rv_);
   		  	   return;
   		    }
    		
    		
           private void decode_Final_Unit_Indication_AVP(RecordValue rv_sid,	AVP sid){
		    	
 		       AVP avp=sid.findChildAVP(AVP_Final_Unit_Indication_AVP);
 		  	   if (avp==null) return;
 		      	
 		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("final_Unit_Indication").getType().newInstance();
 		  	   
 		  	   try{avp.ungroup();}catch(Exception e){}
 		  	   
 		  	   decode_AVPHeader( rv_ ,avp);
 		  	   decode_Final_Unit_Action_AVP( rv_ ,avp);//         final_Unit_Action,
 		  	   decode_Restriction_Filter_Rule_AVP( rv_ ,avp);// restriction_Filter_Rule optional,
 		  	   decode_Filter_Id_AVP( rv_ ,avp);//         filter_Id optional,
 		  	   decode_Redirect_Server_AVP( rv_ ,avp);//         redirect_Server optional
 		  	   
 		  	   rv_sid.setField("final_Unit_Indication",rv_);
 		  	   return;
 		    } 
             
    		
    		private void decode_Time_Quota_Threshold_AVP(RecordValue rv_sid,	AVP sid){
		    	
		       AVP avp=sid.findChildAVP(AVP_Time_Quota_Threshold_AVP);
		  	   if (avp==null) return;
		      	
		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("time_Quota_Threshold").getType().newInstance();
			    
		  	   decode_AVPHeader( rv_ ,avp);
		       rv_.setField("aVP_Data", mkInt(avp.int_data));
		  	   
		  	   rv_sid.setField("time_Quota_Threshold",rv_);
		  	   return;
		    }
    		
    		private void decode_Volume_Quota_Threshold_AVP(RecordValue rv_sid,	AVP sid){
		    	
   		       AVP avp=sid.findChildAVP(AVP_Volume_Quota_Threshold_AVP);
   		  	   if (avp==null) return;
   		      	
   		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("volume_Quota_Threshold").getType().newInstance();
   			    
   		  	   decode_AVPHeader( rv_ ,avp);
   		       rv_.setField("aVP_Data", mkInt(avp.int_data));
   		  	   
   		  	   rv_sid.setField("volume_Quota_Threshold",rv_);
   		  	   return;
   		    }
    		
    		private void decode_Unit_Quota_Threshold_AVP(RecordValue rv_sid,	AVP sid){
		    	
  		       AVP avp=sid.findChildAVP(AVP_Unit_Quota_Threshold_AVP);
  		  	   if (avp==null) return;
  		      	
  		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("unit_Quota_Threshold").getType().newInstance();
  			    
  		  	   decode_AVPHeader( rv_ ,avp);
  		       rv_.setField("aVP_Data", mkInt(avp.int_data));
  		  	   
  		  	   rv_sid.setField("unit_Quota_Threshold",rv_);
  		  	   return;
  		    }
    		
    		private void decode_Quota_Holding_Time_AVP(RecordValue rv_sid,	AVP sid){
		    	
 		       AVP avp=sid.findChildAVP(AVP_Quota_Holding_Time_AVP);
 		  	   if (avp==null) return;
 		      	
 		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("quota_Holding_Time").getType().newInstance();
 			    
 		  	   decode_AVPHeader( rv_ ,avp);
 		       rv_.setField("aVP_Data", mkInt(avp.int_data));
 		  	   
 		  	   rv_sid.setField("quota_Holding_Time",rv_);
 		  	   return;
 		    }
    		
		    private void decode_Quota_Consumption_Time_AVP(RecordValue rv_sid,	AVP sid){
		    	
		       AVP avp=sid.findChildAVP(AVP_Quota_Consumption_Time_AVP);
		  	   if (avp==null) return;
		      	
		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("quota_Consumption_Time").getType().newInstance();
			    
		  	   decode_AVPHeader( rv_ ,avp);
		       rv_.setField("aVP_Data", mkInt(avp.int_data));
		  	   
		  	   rv_sid.setField("quota_Consumption_Time",rv_);
		  	   return;
		    }
		    
		    
		    private void decode_Time_Quota_Type_AVP(RecordValue rv_sid,	AVP sid){    //time_Quota_Type optional,
		       
		    	AVP avp=sid.findChildAVP(AVP_Time_Quota_Type_AVP);
		  	   if (avp==null) return;
		      	
		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("time_Quota_Type").getType().newInstance();
			    
		  	   decode_AVPHeader( rv_ ,avp);
		       rv_.setField("aVP_Data", mkInt(avp.int_data));
		  	   
		  	   rv_sid.setField("time_Quota_Type",rv_);
		  	   return;
		    }
		    
		    private void decode_Base_Time_Interval_AVP(RecordValue rv_sid,	AVP sid){// base_Time_Interval optional
		    	
		    	AVP avp=sid.findChildAVP(AVP_Base_Time_Interval_AVP);
		  	    if (avp==null) return;
		      	
		  	    RecordValue rv_ = (RecordValue)rv_sid.getField("base_Time_Interval").getType().newInstance();
			    
		  	    decode_AVPHeader( rv_ ,avp);
		        rv_.setField("aVP_Data", mkInt(avp.int_data));
		  	   
		  	    rv_sid.setField("base_Time_Interval",rv_);
		  	    return;
		    }
		    
		    private void decode_Time_Quota_Mechanism_AVP(RecordValue rv_sid,	AVP sid){
		    	
		       AVP avp=sid.findChildAVP(AVP_Time_Quota_Mechanism_AVP);
		  	   if (avp==null) return;
		      	
		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("time_Quota_Mechanism").getType().newInstance();
		  	   
		  	   try{avp.ungroup();}catch(Exception e){}
		  	   
		  	   decode_AVPHeader( rv_ ,avp);
		  	   decode_Time_Quota_Type_AVP( rv_ ,avp);    //time_Quota_Type optional,
	           decode_Base_Time_Interval_AVP( rv_ ,avp);// base_Time_Interval optional 
		  	   
		  	   rv_sid.setField("time_Quota_Mechanism",rv_);
		  	   return;
		    }
		    
		    private void decode_Envelope_Start_Time_AVP(RecordValue rv_sid,	AVP sid){// base_Time_Interval optional
		    	
		    	AVP avp=sid.findChildAVP(AVP_Envelope_Start_Time_AVP);
		  	    if (avp==null) return;
		      	
		  	    RecordValue rv_ = (RecordValue)rv_sid.getField("envelope_Start_Time").getType().newInstance();
			    
		  	    decode_AVPHeader( rv_ ,avp);
		        rv_.setField("aVP_Data", mkInt(avp.int_data));
		  	   
		  	    rv_sid.setField("envelope_Start_Time",rv_);
		  	    return;
		    }
		    
		    private void decode_Envelope_End_Time_AVP(RecordValue rv_sid,	AVP sid){// base_Time_Interval optional
		    	
		    	AVP avp=sid.findChildAVP(AVP_Envelope_End_Time_AVP);
		  	    if (avp==null) return;
		      	
		  	    RecordValue rv_ = (RecordValue)rv_sid.getField("envelope_End_Time").getType().newInstance();
			    
		  	    decode_AVPHeader( rv_ ,avp);
		        rv_.setField("aVP_Data", mkInt(avp.int_data));
		  	   
		  	    rv_sid.setField("envelope_End_Time",rv_);
		  	    return;
		    }
		   
		    private void decode_Envelope_AVP(RecordValue rv_sid,	AVP sid){
		    	
		       AVP avps[]=sid.findChildAVPs(AVP_Envelope_AVP);
		  	   if (avps==null) return;
		      	
		  	   RecordOfValue rov_ = (RecordOfValue)rv_sid.getField("envelope").getType().newInstance();
		  	   rov_.setLength(0);
		  	   
		  	   for(int i=0;i<avps.length;i++){
			    	
			       RecordValue rv_ = (RecordValue) rov_.getElementType().newInstance();
		        
			       try{avps[i].ungroup();}catch(Exception e){}
		  	   
			  	   decode_AVPHeader( rv_ ,avps[i]);
				   decode_Envelope_Start_Time_AVP( rv_ ,avps[i]);//         envelope_Start_Time,
				   decode_Envelope_End_Time_AVP( rv_ ,avps[i]);//           envelope_End_Time optional,
				   decode_CC_Total_Octets_AVP( rv_ ,avps[i]);//             cc_Total_Octets optional,
				   decode_CC_Input_Octets_AVP( rv_ ,avps[i]);//             cc_Input_Octets optional,
				   decode_CC_Output_Octets_AVP( rv_ ,avps[i]);//            cc_Output_Octets optional,
				   decode_CC_Service_Specific_Units_AVP( rv_ ,avps[i]);//   cc_Service_Specific_Units optional,
				   //decode_AVP_Type( rv_ ,avps[i]);//                 aVP_Type optional
		  	       
				   rov_.appendField(rv_);
		  	   }
		       
		  	   rv_sid.setField("envelope",rov_);
		  	   return;
		    }
		    
		    private void decode_Envelope_Reporting_AVP(RecordValue rv_sid,	AVP sid){
		    	
		       AVP avp=sid.findChildAVP(AVP_Envelope_Reporting_AVP);
		  	   if (avp==null) return;
		      	
		  	   RecordValue rv_ = (RecordValue)rv_sid.getField("envelope_Reporting").getType().newInstance();
		  	 
		  	   decode_AVPHeader( rv_ ,avp);
		       rv_.setField("aVP_Data", mkInt(avp.int_data));
		       
		  	   rv_sid.setField("envelope_Reporting",rv_);
		  	   return;
		    }
		    
		    private void decode_Validity_Time_AVP(RecordValue rv_sid,	AVP sid){// base_Time_Interval optional
		    	
		    	AVP avp=sid.findChildAVP(AVP_Validity_Time_AVP);
		  	    if (avp==null) return;
		      	
		  	    RecordValue rv_ = (RecordValue)rv_sid.getField("validity_Time").getType().newInstance();
			    
		  	    decode_AVPHeader( rv_ ,avp);
		        rv_.setField("aVP_Data", mkInt(avp.int_data));
		  	   
		  	    rv_sid.setField("validity_Time",rv_);
		  	    return;
		    }
		    
		    private void decode_Multiple_Services_Credit_Control_AVP(RecordValue rv_sid,	AVP sid){
		    	
		       AVP[] avps=sid.findChildAVPs(AVP_Multiple_Services_Credit_Control_AVP);
		  	   if (avps==null) return;
			    if (avps.length<1){}
				else{
					RecordOfValue rov_c = (RecordOfValue) rv_sid.getField("multiple_Services_Credit_Control").getType().newInstance();
					rov_c.setLength(0);
				    
			    for(int i=0;i<avps.length;i++){
			    	
			    	RecordValue rv_ = (RecordValue) rov_c.getElementType().newInstance();
		        
			    	try{avps[i].ungroup();}catch(Exception e){}
			    	
       		  	    decode_AVPHeader( rv_ ,avps[i]);
       		  	    decode_Granted_Service_Unit_AVP( rv_ ,avps[i]);//                granted_Service_Unit optional,
       		  	    decode_Requested_Service_Unit_AVP( rv_ ,avps[i]);//              requested_Service_Unit optional,
       		  	    decode_Used_Service_Unit_AVP( rv_ ,avps[i]);//            used_Service_Unit optional,
       		     	decode_Tariff_Change_Usage_AVP( rv_ ,avps[i]);//                 tariff_Change_Usage optional,
       		     	decode_Service_Identifier_AVP( rv_ ,avps[i]);//           service_Identifier optional,
       		     	decode_Rating_Group_AVP( rv_ ,avps[i]);//                        rating_Group optional,
       		     	decode_G_S_U_Pool_Reference_AVP( rv_ ,avps[i]);//         g_S_U_Pool_Reference optional,
       		     	decode_Validity_Time_AVP( rv_ ,avps[i]);//                       validity_Time optional,
       		     	decode_Result_Code_AVP( rv_ ,avps[i]);//                         result_Codeoptional optional,
       		     	decode_Final_Unit_Indication_AVP( rv_ ,avps[i]);//               final_Unit_Indication optional,
       		     	decode_Time_Quota_Threshold_AVP( rv_ ,avps[i]);//                time_Quota_Threshold optional, // TS 132 299
       		        decode_Volume_Quota_Threshold_AVP( rv_ ,avps[i]);//              volume_Quota_Threshold optional, // TS 132 299 
       		        decode_Unit_Quota_Threshold_AVP( rv_ ,avps[i]);//                unit_Quota_Threshold optional, // TS 132 299
       		        decode_Quota_Holding_Time_AVP( rv_ ,avps[i]);//                  quota_Holding_Time optional, // TS 132 299
       		        decode_Quota_Consumption_Time_AVP( rv_ ,avps[i]);//              quota_Consumption_Time optional, // TS 132 299
       		        decode_Reporting_Reason_AVP( rv_ ,avps[i]);//             reporting_Reason optional, // TS 132 299
       		     	decode_Trigger_AVP( rv_ ,avps[i]);//                              trigger_ optional, // TS 132 299
//                PS_Furnish_Charging_Information_AVP     ps_Furnish_Charging_Information optional, // TS 132 299
//                Refund_Information_AVP                  refund_Information optional, // TS 132 299
//                set of AF_Correlation_Information_AVP   aF_Correlation_Information optional, // TS 132 299
       		     	decode_Envelope_AVP( rv_ ,avps[i]);//                     envelope optional, // TS 132 299
       		     	decode_Envelope_Reporting_AVP( rv_ ,avps[i]);//                  envelope_Reporting optional, // TS 132 299
       		     	decode_Time_Quota_Mechanism_AVP( rv_ ,avps[i]);//                time_Quota_Mechanism optional, // TS 132 299
//                set of Service_Specific_Info_AVP        service_Specific_Info optional, // TS 132 299
       		     	decode_QoS_Information_AVP( rv_ ,avps[i]);//                     qoS_Information optional, // TS 132 299
       		     	//decode_AVP_Type( rv_ ,avps[i]);//                         aVP_Type optional									
				
       		     	rov_c.appendField(rv_);
			    }
			    rv_sid.setField("multiple_Services_Credit_Control",rov_c);
			}
		  	   return;
		    }
		    
		    private void  decode_Multiple_Services_Credit_Control_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
		    	
		    	AVP[] avps=msg.findAVPs(AVP_Multiple_Services_Credit_Control_AVP);
				if (avps==null) return;
				
				if (avps.length<1){}
				else{
					RecordOfValue rov_c = (RecordOfValue) rv_DiameterM.getField("multiple_Services_Credit_Control").getType().newInstance();
					rov_c.setLength(0);
				    
				    for(int i=0;i<avps.length;i++){
				    	
				    	RecordValue rv_ = (RecordValue) rov_c.getElementType().newInstance();
			        
				    	try{avps[i].ungroup();}catch(Exception e){}
				    	
           		  	    decode_AVPHeader( rv_ ,avps[i]);
           		  	    decode_Granted_Service_Unit_AVP( rv_ ,avps[i]);//                granted_Service_Unit optional,
           		  	    decode_Requested_Service_Unit_AVP( rv_ ,avps[i]);//              requested_Service_Unit optional,
           		  	    decode_Used_Service_Unit_AVP( rv_ ,avps[i]);//            used_Service_Unit optional,
           		     	decode_Tariff_Change_Usage_AVP( rv_ ,avps[i]);//                 tariff_Change_Usage optional,
           		     	decode_Service_Identifier_AVP( rv_ ,avps[i]);//           service_Identifier optional,
           		     	decode_Rating_Group_AVP( rv_ ,avps[i]);//                        rating_Group optional,
           		     	decode_G_S_U_Pool_Reference_AVP( rv_ ,avps[i]);//         g_S_U_Pool_Reference optional,
           		     	decode_Validity_Time_AVP( rv_ ,avps[i]);//                       validity_Time optional,
           		     	decode_Result_Code_AVP( rv_ ,avps[i]);//                         result_Codeoptional optional,
           		     	decode_Final_Unit_Indication_AVP( rv_ ,avps[i]);//               final_Unit_Indication optional,
           		     	decode_Time_Quota_Threshold_AVP( rv_ ,avps[i]);//                time_Quota_Threshold optional, // TS 132 299
           		        decode_Volume_Quota_Threshold_AVP( rv_ ,avps[i]);//              volume_Quota_Threshold optional, // TS 132 299 
           		        decode_Unit_Quota_Threshold_AVP( rv_ ,avps[i]);//                unit_Quota_Threshold optional, // TS 132 299
           		        decode_Quota_Holding_Time_AVP( rv_ ,avps[i]);//                  quota_Holding_Time optional, // TS 132 299
           		        decode_Quota_Consumption_Time_AVP( rv_ ,avps[i]);//              quota_Consumption_Time optional, // TS 132 299
           		        decode_Reporting_Reason_AVP( rv_ ,avps[i]);//             reporting_Reason optional, // TS 132 299
           		     	decode_Trigger_AVP( rv_ ,avps[i]);//                              trigger_ optional, // TS 132 299
//	                PS_Furnish_Charging_Information_AVP     ps_Furnish_Charging_Information optional, // TS 132 299
	                    decode_Refund_Information_AVP( rv_ ,avps[i]);//                  refund_Information optional, // TS 132 299
//	                set of AF_Correlation_Information_AVP   aF_Correlation_Information optional, // TS 132 299
           		     	decode_Envelope_AVP( rv_ ,avps[i]);//                     envelope optional, // TS 132 299
           		     	decode_Envelope_Reporting_AVP( rv_ ,avps[i]);//                  envelope_Reporting optional, // TS 132 299
           		     	decode_Time_Quota_Mechanism_AVP( rv_ ,avps[i]);//                time_Quota_Mechanism optional, // TS 132 299
//	                set of Service_Specific_Info_AVP        service_Specific_Info optional, // TS 132 299
           		     	decode_QoS_Information_AVP( rv_ ,avps[i]);//                     qoS_Information optional, // TS 132 299
           		     	//decode_AVP_Type( rv_ ,avps[i]);//                         aVP_Type optional									
					
           		     	rov_c.appendField(rv_);
				    }
					rv_DiameterM.setField("multiple_Services_Credit_Control",rov_c);
				}
		        return;	
		    }
	    
        private void decode_Offline_Charging_AVP(RecordValue rv_sid,	AVP sid){
	    	
	       AVP avp=sid.findChildAVP(AVP_Offline_Charging_AVP);
	  	   if (avp==null) return;
	      	
	  	   RecordValue rv_ = (RecordValue)rv_sid.getField("offline_Charging").getType().newInstance();
	  	   
	  	   try{avp.ungroup();}catch(Exception e){}
	  	 
	  	   decode_AVPHeader( rv_ ,avp);
	       decode_Quota_Consumption_Time_AVP( rv_ ,avp);                  //quota_Consumption_Time optional,
	       decode_Time_Quota_Mechanism_AVP( rv_ ,avp);                    //time_Quota_Mechanism optional,
	       decode_Envelope_Reporting_AVP( rv_ ,avp);                      //envelope_Reporting optional,
	       decode_Multiple_Services_Credit_Control_AVP( rv_ ,avp);// multiple_Services_Credit_Control( rv_ ,avp);// optional,
	       //set of AVP_Type                             aVP_Type optional
	     
	      rv_sid.setField("offline_Charging",rv_);
	  	  return;
	    }
    
    private void decode_PS_Information_AVP(RecordValue rv_sid,	AVP sid){
    	
       AVP avp=sid.findChildAVP(AVP_PS_Information_AVP);
 	   if (avp==null) return;
     	
 	   RecordValue rv_PS_Information = (RecordValue) (RecordValue) rv_sid.getField("ps_Information").getType().newInstance();

 	   try{avp.ungroup();}catch(Exception e){}
 	   
 	   decode_AVPHeader( rv_PS_Information ,avp);
//		 ThreeGPP_Charging_Id_AVP                       threeGPP_Charging_Id optional,
//     PDN_Connection_Charging_ID_AVP                 pdn_Connection_Charging_ID optional,
       //decode_Node_Id_AVP( rv_PS_Information ,avp);//                                    node_Id optional,
//     ThreeGPP_PDP_Type_AVP                          threeGPP_PDP_Type optional,
//     set of PDP_Address_AVP                         pdp_Address optional,
//     PDP_Address_Prefix_Length_AVP                  pdp_Address_Prefix_Length optional,
//     Dynamic_Address_Flag_AVP                       dynamic_Address_Flag optional,
//     Dynamic_Address_Flag_Extension_AVP             dynamic_Address_Flag_Extension optional,
       decode_QoS_Information_AVP( rv_PS_Information ,avp);//    qos_Information optional,
//     set of SGSN_Address_AVP                        sgsn_Address optional,
//     set of GGSN_Address_AVP                        ggsn_Address optional,
//     set of SGW_Address_AVP                         sgw_Address optional,
//     CG_Address_AVP                                 cg_Address optional,
//     Serving_Node_Type_AVP                          serving_Node_Type optional,
//     SGW_Change_AVP                                 sgw_Change optional,
//     ThreeGPP_IMSI_MCC_MNC_AVP                      threeGPP_IMSI_MCC_MNC optional,
//     IMSI_Unauthenticated_Flag_AVP                  imsi_Unauthenticated_Flag optional,
//     ThreeGPP_GGSN_MCC_MNC_AVP                      threeGPP_GGSN_MCC_MNC optional,
//     ThreeGPP_NSAPI_AVP                             threeGPP_NSAPI optional,
//     Called_Station_Id_AVP                          called_Station_Id optional,
//     ThreeGPP_Session_Stop_Indicator_AVP            threeGPP_Session_Stop_Indicator optional,
//     ThreeGPP_Selection_Mode_AVP                    threeGPP_Selection_Mode optional,
//     ThreeGPP_Charging_Characteristics_AVP          threeGPP_Charging_Characteristics optional,
//     Charging_Characteristics_Selection_Mode_AVP    charging_Characteristics_Selection_Mode optional,
//     ThreeGPP_SGSN_MCC_MNC_AVP                      threeGPP_SGSN_MCC_MNC optional,
//     ThreeGPP_MS_TimeZone_AVP                       threeGPP_MS_TimeZone optional,
//     Charging_Rule_Base_Name_AVP                    charging_Rule_Base_Name optional,
//     ThreeGPP_User_Location_Info_AVP                threeGPP_User_Location_Info optional,
//     User_CSG_Information_AVP                       user_CSG_Information optional,
//     ThreeGPP2_BSID_AVP                             threeGPP2_BSID optional,
//     ThreeGPP_RAT_Type_AVP                          threeGPP_RAT_Type optional,
//     PS_Furnish_Charging_Information_AVP            ps_Furnish_Charging_Information optional,
//     PDP_Context_Type_AVP                           pdp_Context_Type optional,
       decode_Offline_Charging_AVP(rv_PS_Information ,avp);
//     set of Traffic_Data_Volumes_AVP                traffic_Data_Volumes optional,
//     set of Service_Data_Container_AVP              service_Data_Container optional,
//     User_Equipment_Info_AVP                        user_Equipment_Info optional,
//     Terminal_Information_AVP                       terminal_Information optional,
//     Start_Time_AVP                                 start_Time optional,
//     Stop_Time_AVP                                  stop_Time optional,
//     Change_Condition_AVP                           change_Condition optional,
//     Diagnostics_AVP                                diagnostics optional,
//     Low_Priority_Indicator_AVP                     low_Priority_Indicator optional
 	   
 	   rv_sid.setField("ps_Information",rv_PS_Information);
 	   return;
  	}
    
    private void decode_WLAN_Information_AVP(RecordValue rv_sid,	AVP sid){
    	
       AVP avp=sid.findChildAVP(AVP_WLAN_Information_AVP);
  	   if (avp==null) return;
      	
  	   RecordValue rv_WLAN_Information = (RecordValue) (RecordValue) rv_sid.getField("wlan_Information").getType().newInstance();
  	   
  	   try{avp.ungroup();}catch(Exception e){}
  	   
  	   decode_AVPHeader( rv_WLAN_Information ,avp);
  	   //...
  	   
  	   rv_sid.setField("wlan_Information",rv_WLAN_Information);
  	   return;
   	}
    
    private void decode_Event_AVP(RecordValue rv_et,	AVP e_){
    	
       AVP avp=e_.findChildAVP(AVP_Event_AVP);
       if (avp==null) return;
        	
       RecordValue rv_event = (RecordValue) (RecordValue) rv_et.getField("event").getType().newInstance();
    	   
       decode_AVPHeader( rv_event ,avp);
       rv_event.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
    	   
   	   rv_et.setField("event",rv_event); 
   	   return;
    }
    
    private void decode_Expires_AVP(RecordValue rv_expires,	AVP e_){
    	
       AVP avp=e_.findChildAVP(AVP_Expires_AVP);
       if (avp==null) return;
         	
       RecordValue rv_expire = (RecordValue) (RecordValue) rv_expires.getField("expires").getType().newInstance();
     	   
       decode_AVPHeader( rv_expire ,avp);
       rv_expire.setField("aVP_Data", mkInt(avp.int_data));
     	   
       rv_expires.setField("expires",rv_expire);
       return;
    }
    
    private void decode_Event_Type_AVP(RecordValue rv_ims_i,	AVP e_t){
    	
       AVP avp=e_t.findChildAVP(AVP_Event_Type_AVP);
   	   if (avp==null) return;
       	
   	   RecordValue rv_event_type = (RecordValue) (RecordValue) rv_ims_i.getField("event_Type").getType().newInstance();
   	   
   	   try{avp.ungroup();}catch(Exception e){}
   	   
   	   decode_AVPHeader( rv_event_type ,avp);
   	   decode_SIP_Method_AVP( rv_event_type ,avp);
   	   decode_Event_AVP( rv_event_type ,avp);
   	   decode_Expires_AVP( rv_event_type ,avp);
   	   
   	   rv_ims_i.setField("event_Type",rv_event_type);
   	   return;
    }
    
    private void decode_Role_Of_Node_AVP(RecordValue rv_ims_i,	AVP ron){
    	
       String s_ron="";
       AVP avp=ron.findChildAVP(AVP_Role_Of_Node_AVP);
       if (avp==null) return;
        	
       RecordValue rv_ron = (RecordValue) (RecordValue) rv_ims_i.getField("role_Of_Node").getType().newInstance();
    	   
       decode_AVPHeader( rv_ron ,avp);
       rv_ron.setField("aVP_Data", mkInt(avp.int_data));
       
       /*switch(avp.int_data){
	     case 0: s_ron="ORIGINATING_ROLE_E";break;
	     case 1: s_ron="TERMINATING_ROLE_E";break;
	     case 2: s_ron="PROXY_ROLE_E";break;
	     case 3: s_ron="B2BUA_ROLE_E";break;
	   }
	
	   EnumeratedValue e_ron= (EnumeratedValue)rv_ron.getField("aVP_Data");
	   e_ron.setEnum(s_ron);
	   rv_ron.setField("aVP_Data", e_ron);*/
    	   
       rv_ims_i.setField("role_Of_Node",rv_ron);   
       return;
    }
    
    private void decode_Node_Functionality_AVP(RecordValue rv_ims_i,	AVP nf){
    	
       String s_nf="";
       AVP avp=nf.findChildAVP(AVP_Node_Functionality_AVP);
       if (avp==null) return;
         	
       RecordValue rv_nf = (RecordValue) (RecordValue) rv_ims_i.getField("node_Functionality").getType().newInstance();
     	   
       decode_AVPHeader( rv_nf ,avp);
       rv_nf.setField("aVP_Data", mkInt(avp.int_data));
       
       /*switch(avp.int_data){
 	     case 0: s_nf="S_CSCF_E";break;
 	     case 1: s_nf="P_CSCF_E";break;
 	     case 2: s_nf="I_CSCF_E";break;
 	     case 3: s_nf="MRFC_E";break;
 	     case 4: s_nf="MGCF_E";break;
 	     case 5: s_nf="BGCF_E";break;
 	     case 6: s_nf="AS_E";break;
 	   }
    	
 	   EnumeratedValue e_nf= (EnumeratedValue)rv_nf.getField("aVP_Data");
 	   e_nf.setEnum(s_nf);
 	   rv_nf.setField("aVP_Data", e_nf);*/
     	   
       rv_ims_i.setField("node_Functionality",rv_nf);   
       return;
    }
    
    private void decode_User_Session_ID_AVP(RecordValue rv_ims_i,	AVP usid){
    	
       AVP avp=usid.findChildAVP(AVP_User_Session_ID_AVP);
       if (avp==null) return;
        	
       RecordValue rv_usid = (RecordValue) (RecordValue) rv_ims_i.getField("user_Session_Id").getType().newInstance();
     	   
       decode_AVPHeader( rv_usid ,avp);
       rv_usid.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
     	   
       rv_ims_i.setField("user_Session_Id",rv_usid);
       return;
    }
    
    private void decode_Calling_Party_Address_AVP(RecordValue rv_ims_i,	AVP cgpa){
    	
    	boolean rv_present=false;
        AVP avps[]=cgpa.findChildAVPs(AVP_Calling_Party_Address_AVP);
        if (avps==null) return;
         
        if (avps.length>=1){
        	try{
 			   RecordValue rv_cgpa = (RecordValue) (RecordValue) rv_ims_i.getField("calling_Party_Address").getType().newInstance();
 			   
 			   decode_AVPHeader( rv_cgpa ,avps[0]);
 		       rv_cgpa.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));

 			   rv_ims_i.setField("calling_Party_Address",rv_cgpa);
 			   rv_present=true;
 			   return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
				RecordOfValue recOfcgpa = (RecordOfValue) rv_ims_i.getField("calling_Party_Address").getType().newInstance();
				recOfcgpa.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfcgpa.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
				    rv.setField("aVP_Data", mkCvUTF8(avps[i].getStringData()));	
		     	
		    	    recOfcgpa.appendField(rv);
				}
				rv_ims_i.setField("calling_Party_Address",recOfcgpa);
 			}
        }
        return;
    }
    
    private void decode_Called_Party_Address_AVP(RecordValue rv_ims_i,	AVP cdpa){
    	
        AVP avp=cdpa.findChildAVP(AVP_Called_Party_Address_AVP);
        if (avp==null) return;
      
 		RecordValue rv_cdpa = (RecordValue) (RecordValue) rv_ims_i.getField("called_Party_Address").getType().newInstance();
 			   
 		decode_AVPHeader( rv_cdpa ,avp);
 		rv_cdpa.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

 		rv_ims_i.setField("called_Party_Address",rv_cdpa);
 		return;
    }
    
    private void decode_Called_Asserted_Identity_AVP(RecordValue rv_ims_i,	AVP cdai){
    	
    	boolean rv_present=false;
        AVP avps[]=cdai.findChildAVPs(AVP_Called_Asserted_Identity_AVP);
        if (avps==null) return;
         
        if (avps.length>=1){
        	try{
 			   RecordValue rv_cdai = (RecordValue) (RecordValue) rv_ims_i.getField("called_Asserted_Identity").getType().newInstance();
 			   
 			   decode_AVPHeader( rv_cdai ,avps[0]);
 		       rv_cdai.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));

 			   rv_ims_i.setField("called_Asserted_Identity",rv_cdai);
 			   rv_present=true;
 			   return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
				RecordOfValue recOfcdai = (RecordOfValue) rv_ims_i.getField("called_Asserted_Identity").getType().newInstance();
				recOfcdai.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfcdai.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
				    rv.setField("aVP_Data", mkCvUTF8(avps[i].getStringData()));	
		     	
		    	    recOfcdai.appendField(rv);
				}
				rv_ims_i.setField("called_Asserted_Identity",recOfcdai);
 			}
        }
        return;
    }
    
    private void decode_Alternate_Charged_Party_Address_AVP(RecordValue rv_ims_i,	AVP acpa){
    	
        AVP avp=acpa.findChildAVP(AVP_Alternate_Charged_Party_Address_AVP);
        if (avp==null) return;
      
 		RecordValue rv_acpa = (RecordValue) (RecordValue) rv_ims_i.getField("alternate_Charged_Party_Address").getType().newInstance();
 			   
 		decode_AVPHeader( rv_acpa ,avp);
 		rv_acpa.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

 		rv_ims_i.setField("alternate_Charged_Party_Address",rv_acpa);
 		return;
    }
    
    private void decode_Requested_Party_Address_AVP(RecordValue rv_ims_i,	AVP rpa){
    	
        AVP avp=rpa.findChildAVP(AVP_Requested_Party_Address_AVP);
        if (avp==null) return;
      
        RecordValue rv_rpa = (RecordValue) (RecordValue) rv_ims_i.getField("requested_Party_Address").getType().newInstance();
 			   
 		decode_AVPHeader( rv_rpa ,avp);
 		rv_rpa.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

 		rv_ims_i.setField("requested_Party_Address",rv_rpa);
 		return;
    }
    
    private void decode_Associated_URI_AVP(RecordValue rv_ims_i,	AVP auri){
    	
    	boolean rv_present=false;
        AVP avps[]=auri.findChildAVPs(AVP_Associated_URI_AVP);
        if (avps==null) return;
         
        if (avps.length>=1){
        	try{
 			   RecordValue rv_auri = (RecordValue) (RecordValue) rv_ims_i.getField("associated_URI").getType().newInstance();
 			   
 			   decode_AVPHeader( rv_auri ,avps[0]);
 		       rv_auri.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));

 			   rv_ims_i.setField("associated_URI",rv_auri);
 			   rv_present=true;
 			   return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
				RecordOfValue recOfauri = (RecordOfValue) rv_ims_i.getField("associated_URI").getType().newInstance();
				recOfauri.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfauri.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
				    rv.setField("aVP_Data", mkCvUTF8(avps[i].getStringData()));	
		     	
		    	    recOfauri.appendField(rv);
				}
				rv_ims_i.setField("associated_URI",recOfauri);
 			}
        }
        return;
    }
    
    private void decode_SIP_Request_Timestamp_AVP(RecordValue rv_ims_i,	AVP ts){
    	
        AVP avp=ts.findChildAVP(AVP_SIP_Request_Timestamp_AVP);
        if (avp==null) return;
         	
        RecordValue rv_ts = (RecordValue) (RecordValue) rv_ims_i.getField("sIP_Request_Timestamp").getType().newInstance();
      	   
        decode_AVPHeader( rv_ts ,avp);              
	    rv_ts.setField("aVP_Data", mkInt(avp.int_data));
      	   
        rv_ims_i.setField("sIP_Request_Timestamp",rv_ts);
        return;
    }

    private void decode_SIP_Response_Timestamp_AVP(RecordValue rv_ims_i,	AVP ts){
	
	    AVP avp=ts.findChildAVP(AVP_SIP_Response_Timestamp_AVP);
	    if (avp==null) return;
	     	
	    RecordValue rv_ts = (RecordValue) (RecordValue) rv_ims_i.getField("sIP_Response_Timestamp").getType().newInstance();
	  	   
	    decode_AVPHeader( rv_ts ,avp);
	    rv_ts.setField("aVP_Data", mkInt(avp.int_data));
	  	   
	    rv_ims_i.setField("sIP_Response_Timestamp",rv_ts);
	    return;
    }
    
    private void decode_Time_Stamps_AVP(RecordValue rv_ims_i,	AVP ts){
    	
        AVP avp=ts.findChildAVP(AVP_Time_Stamps_AVP);
        if (avp==null) return;
         	
        RecordValue rv_ts = (RecordValue) (RecordValue) rv_ims_i.getField("time_Stamps").getType().newInstance();
      	   
        decode_AVPHeader( rv_ts ,avp);
        decode_SIP_Request_Timestamp_AVP( rv_ts ,avp); 
        decode_SIP_Response_Timestamp_AVP( rv_ts ,avp);     

        rv_ims_i.setField("time_Stamps",rv_ts);
        return;
     }
    
    private void decode_Application_Server_AVP(RecordValue rv_ims_i,	AVP as){
    	
        AVP avp=as.findChildAVP(AVP_Application_Server_AVP);
        if (avp==null) return;
      
 		RecordValue rv_as = (RecordValue) rv_ims_i.getField("application_Server").getType().newInstance();
 			   
 		decode_AVPHeader( rv_as ,avp);
 		rv_as.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

 		rv_ims_i.setField("application_Server",rv_as);
 		return;
    }
    
    private void decode_Application_Provided_Called_Party_Address_AVP(RecordValue rv_ims_i,	AVP cgpa){
    	
    	boolean rv_present=false;
        AVP avps[]=cgpa.findChildAVPs(AVP_Application_Provided_Called_Party_Address_AVP);
        if (avps==null) return;
         
        if (avps.length>=1){
        	try{
 			   RecordValue rv_cgpa = (RecordValue) (RecordValue) rv_ims_i.getField("application_Provided_Called_Party_Address").getType().newInstance();
 			   
 			   decode_AVPHeader( rv_cgpa ,avps[0]);
 		       rv_cgpa.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));

 			   rv_ims_i.setField("application_Provided_Called_Party_Address",rv_cgpa);
 			   rv_present=true;
 			   return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
				RecordOfValue recOfcgpa = (RecordOfValue) rv_ims_i.getField("application_Provided_Called_Party_Address").getType().newInstance();
				recOfcgpa.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfcgpa.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
				    rv.setField("aVP_Data", mkCvUTF8(avps[i].getStringData()));	
		     	
		    	    recOfcgpa.appendField(rv);
				}
				rv_ims_i.setField("application_Provided_Called_Party_Address",recOfcgpa);
 			}
        }
        return;
    }
    
    private void decode_Application_Server_Information_AVP(RecordValue rv_ims_i,	AVP asi){
    	
    	boolean rv_present=false;
        AVP avps[]=asi.findChildAVPs(AVP_Application_Server_Information_AVP);
        if (avps==null) return;
         
        if (avps.length>=1){
        	try{
 			   RecordValue rv_asi = (RecordValue) (RecordValue) rv_ims_i.getField("application_Server_Information").getType().newInstance();
 			   
 			   decode_AVPHeader( rv_asi ,avps[0]);
 			   decode_Application_Server_AVP(rv_asi,avps[0]);
 			   decode_Application_Provided_Called_Party_Address_AVP(rv_asi,avps[0]);
 		       
 			   rv_ims_i.setField("application_Server_Information",rv_asi);
 			   rv_present=true;
 			   return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
				RecordOfValue recOfasi = (RecordOfValue) rv_ims_i.getField("application_Server_Information").getType().newInstance();
				recOfasi.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfasi.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
				    decode_Application_Server_AVP(rv,avps[i]);
		 			decode_Application_Provided_Called_Party_Address_AVP(rv,avps[i]);
			
		    	    recOfasi.appendField(rv);
				}
				rv_ims_i.setField("application_Server_Information",recOfasi);
 			}
        }
        return;
    }
    
    private void decode_Originating_IOI_AVP(RecordValue rv_ims_i,	AVP ioi){
    	
        AVP avp=ioi.findChildAVP(AVP_Originating_IOI_AVP);
        if (avp==null) return;
      
 		RecordValue rv_ioi = (RecordValue) rv_ims_i.getField("originating_IOI").getType().newInstance();
 			   
 		decode_AVPHeader( rv_ioi ,avp);
 		rv_ioi.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

 		rv_ims_i.setField("originating_IOI",rv_ioi);
 		return;
    }
    
    private void decode_Terminating_IOI_AVP(RecordValue rv_ims_i,	AVP ioi){
    	
        AVP avp=ioi.findChildAVP(AVP_Terminating_IOI_AVP);
        if (avp==null) return;
      
 		RecordValue rv_ioi = (RecordValue) rv_ims_i.getField("terminating_IOI").getType().newInstance();
 			   
 		decode_AVPHeader( rv_ioi ,avp);
 		rv_ioi.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

 		rv_ims_i.setField("terminating_IOI",rv_ioi);
 		return;
    }
    
    private void decode_Inter_Operator_Identifier_AVP(RecordValue rv_ims_i,	AVP ioi){
    	
    	boolean rv_present=false;
        AVP avps[]=ioi.findChildAVPs(AVP_Inter_Operator_Identifier_AVP);
        if (avps==null) return;
         
        if (avps.length>=1){
        	try{
 			   RecordValue rv_ioi = (RecordValue) (RecordValue) rv_ims_i.getField("inter_Operator_Identifier").getType().newInstance();
 			   
 			   decode_AVPHeader( rv_ioi ,avps[0]);
 			   decode_Originating_IOI_AVP(rv_ioi,avps[0]);
 			   decode_Terminating_IOI_AVP(rv_ioi,avps[0]);
 			 		       
 			   rv_ims_i.setField("inter_Operator_Identifier",rv_ioi);
 			   rv_present=true;
 			   return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
				RecordOfValue recOfioi = (RecordOfValue) rv_ims_i.getField("inter_Operator_Identifier").getType().newInstance();
				recOfioi.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfioi.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
				    decode_Originating_IOI_AVP(rv,avps[i]);
				    decode_Terminating_IOI_AVP(rv,avps[i]);
			
		    	    recOfioi.appendField(rv);
				}
				rv_ims_i.setField("inter_Operator_Identifier",recOfioi);
 			}
        }
        return;
    }
    
    private void decode_IMS_Charging_Identifier_AVP(RecordValue rv_ims_i,	AVP ici){
    	
        AVP avp=ici.findChildAVP(AVP_IMS_Charging_Identifier_AVP);
        if (avp==null) return;
      
 		RecordValue rv_ici = (RecordValue) rv_ims_i.getField("ims_Charging_Identifier").getType().newInstance();
 			   
 		decode_AVPHeader( rv_ici ,avp);
 		rv_ici.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

 		rv_ims_i.setField("ims_Charging_Identifier",rv_ici);
 		return;
    }
    
    private void decode_SDP_Session_Description_AVP(RecordValue rv_ims_i,	AVP sdp){
    	
 	   	boolean rv_present=false;
        AVP avps[]=sdp.findChildAVPs(AVP_SDP_Session_Description_AVP);
        if (avps==null) return;
         
        if (avps.length>=1){
        	try{
 			   RecordValue rv_sdp = (RecordValue) (RecordValue) rv_ims_i.getField("sdp_Session_Description").getType().newInstance();
 			   
 			   decode_AVPHeader( rv_sdp ,avps[0]);
 			   rv_sdp.setField("aVP_Data", mkCvUTF8(avps[0].getStringData()));
 			 		       
 			   rv_ims_i.setField("sdp_Session_Description",rv_sdp);
 			   rv_present=true;
 			   return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
				RecordOfValue recOfioi = (RecordOfValue) rv_ims_i.getField("sdp_Session_Description").getType().newInstance();
				recOfioi.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfioi.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
				    rv.setField("aVP_Data", mkCvUTF8(avps[i].getStringData()));
		    	    
				    recOfioi.appendField(rv);
				}
				rv_ims_i.setField("sdp_Session_Description",recOfioi);
 			}
        }
        return;
    }
    
    private void decode_SDP_Media_Component_AVP(RecordValue rv_ims_i,	AVP sdp){
    	
 	   	boolean rv_present=false;
        AVP avps[]=sdp.findChildAVPs(AVP_SDP_Media_Component_AVP);
        if (avps==null) return;
         
        if (avps.length>=1){
        	try{
 			   RecordValue rv_sdp = (RecordValue) (RecordValue) rv_ims_i.getField("sdp_Media_Component").getType().newInstance();
 			   
 			   decode_AVPHeader( rv_sdp ,avps[0]);
 			   //decode_SDP_Media_Name_AVP(rv_sdp ,avps[0]);
               //decode_SDP_Media_Description_AVP(rv_sdp ,avps[0]);
               //decode_Media_Initiator_Flag_AVP(rv_sdp ,avps[0]);
               //decode_Media_Initiator_Party_AVP(rv_sdp ,avps[0]);
               //decode_Authorized_QoS_AVP(rv_sdp ,avps[0]);
               //decode_ThreeGPP_Charging_Id_AVP(rv_sdp ,avps[0]);
               //decode_Access_Network_Charging_Identifier_Value_AVP(rv_sdp ,avps[0]);
 			 		       
 			   rv_ims_i.setField("sdp_Media_Component",rv_sdp);
 			   rv_present=true;
 			   return;
 			}
 			catch(Exception ex){
 				//System.out.print(ex.getMessage());
 			}
 			if(!rv_present){
				RecordOfValue recOfioi = (RecordOfValue) rv_ims_i.getField("sdp_Media_Component").getType().newInstance();
				recOfioi.setLength(0);
			
				for (int i = 0; i < avps.length; i++) {
				    RecordValue rv = (RecordValue) recOfioi.getElementType().newInstance();
		        	
				    decode_AVPHeader( rv ,avps[i]);
				    //decode_SDP_Media_Name_AVP(rv_sdp ,avps[i]);
		            //decode_SDP_Media_Description_AVP(rv ,avps[i]);
		            //decode_Media_Initiator_Flag_AVP(rv ,avps[i]);
		            //decode_Media_Initiator_Party_AVP(rv ,avps[i]);
		            //decode_Authorized_QoS_AVP(rv ,avps[i]);
		            //decode_ThreeGPP_Charging_Id_AVP(rv ,avps[i]);
		            //decode_Access_Network_Charging_Identifier_Value_AVP(rv ,avps[i]);;
		    	    
				    recOfioi.appendField(rv);
				}
				rv_ims_i.setField("sdp_Media_Component",recOfioi);
 			}
        }
        return;
    }
    
    private void decode_Served_Party_IP_Address_AVP(RecordValue rv_ims_i,	AVP spip){
    	
        AVP avp=spip.findChildAVP(AVP_Served_Party_IP_Address_AVP);
        if (avp==null) return;
      
 		RecordValue rv_spip = (RecordValue) rv_ims_i.getField("served_Party_IP_Address").getType().newInstance();
 			   
 		decode_AVPHeader( rv_spip ,avp);
 		decode_Address(rv_spip,avp);

 		rv_ims_i.setField("served_Party_IP_Address",rv_spip);
 		return;
    }
    
    private void decode_Incoming_Trunk_Group_ID_AVP(RecordValue rv_ims_i,	AVP tgid){
    	
        AVP avp=tgid.findChildAVP(AVP_Incoming_Trunk_Group_ID_AVP);
        if (avp==null) return;
      
 		RecordValue rv_tg = (RecordValue) rv_ims_i.getField("incoming_Trunk_Group_ID").getType().newInstance();
 			   
 		decode_AVPHeader( rv_tg ,avp);
 		rv_tg.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

 		rv_ims_i.setField("incoming_Trunk_Group_ID",rv_tg);
 		return;
    }
    
    private void decode_Outgoing_Trunk_Group_ID_AVP(RecordValue rv_ims_i,	AVP tgid){
    	
        AVP avp=tgid.findChildAVP(AVP_Outgoing_Trunk_Group_ID_AVP);
        if (avp==null) return;
      
 		RecordValue rv_tg = (RecordValue) rv_ims_i.getField("outgoing_Trunk_Group_ID").getType().newInstance();
 			   
 		decode_AVPHeader( rv_tg ,avp);
 		rv_tg.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

 		rv_ims_i.setField("outgoing_Trunk_Group_ID",rv_tg);
 		return;
    }
    
    private void decode_Trunk_Group_ID_AVP(RecordValue rv_ims_i,	AVP tgid){
    	
    	AVP avp=tgid.findChildAVP(AVP_Trunk_Group_ID_AVP);
        if (avp==null) return;
         
        RecordValue rv_tg = (RecordValue) (RecordValue) rv_ims_i.getField("trunk_Group_ID").getType().newInstance();
 			   
 		decode_AVPHeader( rv_tg ,avp);
 		decode_Incoming_Trunk_Group_ID_AVP(rv_tg,avp);
 		decode_Outgoing_Trunk_Group_ID_AVP(rv_tg,avp);
 			 		       
 		rv_ims_i.setField("trunk_Group_ID",rv_tg);	   
 		return;
    }
        
    private void decode_IMS_Information_AVP(RecordValue rv_sid,	AVP sid){
    	
       AVP avp=sid.findChildAVP(AVP_IMS_Information_AVP);
  	   if (avp==null) return;
      	
  	   RecordValue rv_IMS_Information = (RecordValue) (RecordValue) rv_sid.getField("ims_Information").getType().newInstance();
  	   
  	   try{avp.ungroup();}catch(Exception e){return;}
  	   
  	   decode_AVPHeader( rv_IMS_Information ,avp);
       decode_Event_Type_AVP(rv_IMS_Information ,avp);
       decode_Role_Of_Node_AVP(rv_IMS_Information ,avp);
       decode_Node_Functionality_AVP(rv_IMS_Information ,avp);
       decode_User_Session_ID_AVP(rv_IMS_Information ,avp);
//       Outgoing_Session_Id_AVP                     outgoing_Session_Id optional,
//       Session_Priority_AVP                        session_Priority optional,
       decode_Calling_Party_Address_AVP(rv_IMS_Information ,avp);
       decode_Called_Party_Address_AVP(rv_IMS_Information ,avp);
       decode_Called_Asserted_Identity_AVP(rv_IMS_Information ,avp);
//       Number_Portability_Routing_Information_AVP  number_Portability_Routing_Information optional,
//       Carrier_Select_Routing_Information_AVP      carrier_Select_Routing_Information optional,
       decode_Alternate_Charged_Party_Address_AVP(rv_IMS_Information ,avp);
       decode_Requested_Party_Address_AVP(rv_IMS_Information ,avp);
       decode_Associated_URI_AVP(rv_IMS_Information ,avp);
       decode_Time_Stamps_AVP(rv_IMS_Information ,avp);
       decode_Application_Server_Information_AVP(rv_IMS_Information ,avp);
       decode_Inter_Operator_Identifier_AVP(rv_IMS_Information ,avp);
       decode_IMS_Charging_Identifier_AVP(rv_IMS_Information ,avp);
       decode_SDP_Session_Description_AVP(rv_IMS_Information ,avp);
       decode_SDP_Media_Component_AVP(rv_IMS_Information ,avp);
       decode_Served_Party_IP_Address_AVP(rv_IMS_Information ,avp);
       decode_Server_Capabilities_AVP(rv_IMS_Information ,avp);
       decode_Trunk_Group_ID_AVP(rv_IMS_Information ,avp);
//       Bearer_Service_AVP                          bearer_Service optional,
//       Service_Id_AVP                              service_Id optional,
//       set of Service_Specific_Info_AVP            service_Specific_Info optional,
//       set of Message_Body_AVP                     message_Body optional,
//       Cause_Code_AVP                              cause_Code optional,
//       Access_Network_Information_AVP              access_Network_Information optional,
//       set of Early_Media_Description_AVP          early_Media_Description optional, 
//       IMS_Communication_Service_Identifier_AVP    ims_Communication_Service_Identifier optional, 
//       Online_Charging_Flag_AVP                    online_Charging_Flag optional,
//       Real_Time_Tariff_Information_AVP            real_Time_Tariff_Information optional,
//       Account_Expiration_AVP                      account_Expiration optional,
//       Initial_IMS_Charging_Identifier_AVP         initial_IMS_Charging_Identifier optional, 
//       IMS_Emergency_Indicator_AVP                 ims_Emergency_Indicator optional
       
  	   
  	   rv_sid.setField("ims_Information",rv_IMS_Information);   
  	   return;
   	}
    
	    private void decode_Application_Server_ID_AVP(RecordValue rv_sid,	AVP sid){
			 
	       AVP avp=sid.findChildAVP(AVP_Application_Server_ID_AVP);
	   	   if (avp==null) return;
	       	
	   	   RecordValue rv_ = (RecordValue) (RecordValue) rv_sid.getField("application_Server_ID").getType().newInstance();
	   	   
	   	   decode_AVPHeader( rv_ ,avp);
	   	   rv_.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

 		   rv_sid.setField("application_Server_ID",rv_);
 		   return;
		}
	
	    private void decode_Application_Service_Type_AVP(RecordValue rv_sid,	AVP sid){
			 
	       AVP avp=sid.findChildAVP(AVP_Application_Service_Type_AVP);
	   	   if (avp==null) return;
	       	
	   	   RecordValue rv_ = (RecordValue) (RecordValue) rv_sid.getField("application_Service_Type").getType().newInstance();
	   	   
	   	   decode_AVPHeader( rv_ ,avp);
	   	   rv_.setField("aVP_Data", mkInt(avp.int_data));

	 	   rv_sid.setField("application_Service_Type",rv_);
	 	   return;
		}
	    
	    private void decode_Application_Session_ID_AVP(RecordValue rv_sid,	AVP sid){
			 
	       AVP avp=sid.findChildAVP(AVP_Application_Session_ID_AVP);
	   	   if (avp==null) return;
	       	
	   	   RecordValue rv_ = (RecordValue) (RecordValue) rv_sid.getField("application_Session_ID").getType().newInstance();
	   	   
	   	   decode_AVPHeader( rv_ ,avp);
	   	   //rv_.setField("aVP_Data", mkCvUTF8(avp.getStringData()));
	   	   rv_.setField("aVP_Data", mkInt(avp.int_data));

		   rv_sid.setField("application_Session_ID",rv_);
		   return;
		}
	    
	    private void decode_Delivery_Status_AVP(RecordValue rv_sid,	AVP sid){
			 
	       AVP avp=sid.findChildAVP(AVP_Delivery_Status_AVP);
	   	   if (avp==null) return;
	       	
	   	   RecordValue rv_ = (RecordValue) (RecordValue) rv_sid.getField("delivery_Status").getType().newInstance();
	   	   
	   	   decode_AVPHeader( rv_ ,avp);
	   	   rv_.setField("aVP_Data", mkCvUTF8(avp.getStringData()));

		   rv_sid.setField("delivery_Status",rv_);
		   return;
		}
    
    
    private void decode_Service_Generic_Information_AVP(RecordValue rv_sid,	AVP sid){
    	
       AVP avp=sid.findChildAVP(AVP_Service_Generic_Information_AVP);
   	   if (avp==null) return;
       	
   	   RecordValue rv_sgi = (RecordValue) (RecordValue) rv_sid.getField("service_Generic_Information").getType().newInstance();
   	  
   	   try{avp.ungroup();}catch(Exception e){return;}
   	   
   	   decode_AVPHeader( rv_sgi ,avp);
   	   decode_Application_Server_ID_AVP( rv_sgi ,avp);//       application_Server_ID optional,
   	   decode_Application_Service_Type_AVP( rv_sgi ,avp);//    application_Service_Type optional,
   	   decode_Application_Session_ID_AVP( rv_sgi ,avp);//      application_Session_ID optional,
   	   decode_Delivery_Status_AVP( rv_sgi ,avp);//             delivery_Status optional
   	   
   	   rv_sid.setField("service_Generic_Information",rv_sgi);   
	   return;
	}
         
    private void decode_Service_Information_AVP(RecordValue rv_DiameterM,	DiameterMessage msg){
    	
     	AVP[] avps=msg.findAVPs(AVP_Service_Information_AVP);
		if (avps==null) return;
    	
        if (avps.length>1){}
        else{
		   RecordValue rv_Service_Information = (RecordValue) (RecordValue) rv_DiameterM.getField("service_Information").getType().newInstance();
		   
		   try{avps[0].ungroup();}catch(Exception e){return;}		   
		   
		   decode_AVPHeader( rv_Service_Information ,avps[0]);
     	   decode_Subscription_Id_AVP(rv_Service_Information,avps[0]);
//           AoC_Information_AVP             aoc_Information optional,
     	   decode_PS_Information_AVP(rv_Service_Information,avps[0]);
     	   decode_WLAN_Information_AVP(rv_Service_Information,avps[0]);
     	   decode_IMS_Information_AVP(rv_Service_Information,avps[0]);
//           MMS_Information_AVP             mms_Information optional,
//           LCS_Information_AVP             lcs_Information optional,
//           PoC_Information_AVP             poc_Information optional,
//           MBMS_Information_AVP            mbms_Information optional,
//           SMS_Information_AVP             sms_Information optional,
//           MMTel_Information_AVP           mmtel_Information optional,
           decode_Service_Generic_Information_AVP(rv_Service_Information,avps[0]);
//           IM_Information_AVP              im_Information optional,
//           DCD_Information_AVP             dcd_Information optional
     	   
		   rv_DiameterM.setField("service_Information",rv_Service_Information);
		   return;
        }	
	}
    /*End TS132_299 AVPs*/
	
	//TODO:Continue WITH AVPs
	

	private RecordOfValue decode_AVP_Type(byte[] pdata){
	    //TODO: decode AVPs.....
		RecordOfValue rov;
		rov=null;
		return rov;
	}
	
	private void decode_AVP_Type(RecordValue rv_avp,	AVP pavp){
		
		decode_AVPHeader( rv_avp ,pavp);	
		UnionValue avp_d = (UnionValue)rv_avp.getField("aVP_Data");
 		avp_d.setVariant("oCTETSTRING", mkOctetstring(pavp.data));
		
		rv_avp.setField("aVP_Data", avp_d);
		return;
	}
	   
    private void decode_AVP_Type(RecordValue rv_DiameterM,	DiameterMessage msg){
		
    
		return;
    }
    
	private void decode_IntegerValue ( RecordValue rvAvp, AVP avp) {
		
		IntegerValue AvpData = (IntegerValue)rvAvp.getField("aVP_Data").getType().newInstance();
		AvpData=mkInt(avp.int_data);
		rvAvp.setField("aVP_Data",AvpData);
		return;
	}
  
    private BitString BufferToBitString(byte[] buf, int SizeOfBitstring/*,int UsedBitsInByte*/) {
		try {
			BitString bitStr = new BitString(RB, SizeOfBitstring, new byte[1]);
			
			int Count = 0;
			for (Count = 0; Count < SizeOfBitstring; Count++) {
				if (((buf[(Count + UsedBitsInByte) / 8]) & (1 << (7 - (UsedBitsInByte + Count) % 8))) != 0) {
					bitStr.setBit(Count, 1);
				} else {
					bitStr.setBit(Count, 0);
				}
			}
		    UsedBitsInByte = (UsedBitsInByte + SizeOfBitstring) % 8;
			return bitStr;
		} catch (Exception ex) {
		      //ex.printStackTrace();
		  return null;
		}
    }
  
	private CharstringValue mkCv(String theString) {
		
		CharstringValue cv = (CharstringValue) charstringType.newInstance();
		cv.setString(theString);
	    return cv;
		
	}

	private UniversalCharstringValue mkCvUTF8(String theString) {
		
		UniversalCharstringValue cv = (UniversalCharstringValue) universalcharstringType.newInstance();
		cv.setString(theString);
	    return cv;
		
	}
	
	private OctetstringValue mkOctetstring(byte[] octetBytes){
		
	    OctetstringValue ov = (OctetstringValue) octetstringType.newInstance();
	    //ov.setString(octetBytes.toString());
	    ov.setLength(octetBytes.length);
		for (int i = 0; i < octetBytes.length; i++) {
			ov.setOctet(i, octetBytes[i]);
	    }
        return ov;
	}
	  	
	private IntegerValue mkInt(int theInt) {
		
		IntegerValue iv = (IntegerValue) integerType.newInstance();
		iv.setInt(theInt);
		return iv;
	}
	
	private IntegerValue mkInt(long theInt) {
		
		IntegerValue iv = (IntegerValue) integerType.newInstance();//(IntegerValue) IntegerValue.class.newInstance();
		iv.setBigInt(BigInteger.valueOf(theInt& 0xffffffffL));
		return iv;
	}
	
	private IntegerValue mkInt64(int theInt) {
		
		IntegerValue in = (IntegerValue) integerType.newInstance();
		in.setInt(theInt);
		return in;
	}
	
	public static boolean useList(String[] arr, String targetValue) {
	    return Arrays.asList(arr).contains(targetValue);
	}

}
/* ######################################################################### */
/* # End of file                                                           # */
/* ######################################################################### */
